import { ResolverFactory } from 'npa-core';
import chalk from 'chalk';
import require$$0, { promises } from 'fs';
import express from 'express';
import figlet from 'figlet';
import require$$0$1 from 'os';
import require$$2 from 'net';
import path$6, { dirname } from 'path';
import require$$1 from 'tty';
import require$$1$1 from 'util';
import * as process$3 from 'process';
import process__default from 'process';
import process$4 from 'node:process';
import { Buffer as Buffer$1 } from 'node:buffer';
import path$7 from 'node:path';
import url, { fileURLToPath } from 'node:url';
import childProcess$2, { ChildProcess } from 'node:child_process';
import fs$6 from 'node:fs/promises';
import fs$5, { createWriteStream, createReadStream, constants as constants$1 } from 'node:fs';
import require$$1$2 from 'child_process';
import require$$0$2 from 'assert';
import require$$2$1 from 'events';
import require$$0$4 from 'buffer';
import require$$0$3 from 'stream';
import os$3, { constants } from 'node:os';
import { debuglog } from 'node:util';
import { fileURLToPath as fileURLToPath$1 } from 'url';

function AnalyzerFactory(root, depthLimit) {
  const factory = new ResolverFactory();
  const resolver = factory.getDependencyGraph(root, depthLimit);
  const dependencyGraph = resolver.resolveDependencies();
  const packages = resolver.getPackageSetOfPackageJson();
  return new PackageAnalyzer(dependencyGraph, packages);
}
class PackageAnalyzer {
  constructor(dependencyGraph, packages) {
    this.dependencyGraph = dependencyGraph;
    this.packages = packages;
  }
  //
  getGraphData() {
    return {
      nodes: this.dependencyGraph.exportPackages(),
      edges: this.dependencyGraph.exportEdges()
    };
  }
  // 对导出的图的数据做进一步的统计分析
  graphStatistics(data) {
    const licenseNum = /* @__PURE__ */ new Map();
    data.nodes.forEach((e) => {
      const packageJson = this.packages[e.path];
      if (packageJson.license && typeof packageJson.license === "string") {
        if (licenseNum.get(packageJson.license) === void 0) {
          licenseNum.set(packageJson.license, 1);
        } else {
          licenseNum.set(packageJson.license, licenseNum.get(packageJson.license) + 1);
        }
      }
    });
    data.licenses = Object.fromEntries(licenseNum);
    return data;
  }
  getPackageDependencies(id, depth) {
    return this.graphStatistics(this.dependencyGraph.getSpecifiedPackageDependencies(id, depth));
  }
  //
  whyInstalledIt(id) {
    return this.graphStatistics(this.dependencyGraph.whyInstalledIt(id));
  }
}

function error$2(program, msg) {
  program.error(chalk.red("ERROR   ") + msg);
}
function warning(msg) {
  console.log(chalk.yellow("WARNING   ") + msg);
}
function success(msg) {
  console.log(chalk.green("INFO   ") + msg);
}

var n = function (n) {
    return 1e3 * n[0] + n[1] / 1e6;
  },
  r = function (n) {
    return Number(n.toFixed(2));
  },
  e = function (n, e) {
    var t = function () {
      return r(e());
    };
    return t.startedAt = function () {
      return n;
    }, t.seconds = t.s = function () {
      return r(e() / 1e3);
    }, t.milliseconds = t.ms = function () {
      return r(e());
    }, t.microseconds = t.us = function () {
      return r(1e3 * e());
    }, t.nanoseconds = t.ns = function () {
      return r(1e6 * e());
    }, t;
  },
  t = "undefined" != typeof process && process.hrtime ? function () {
    var r = process.hrtime();
    return e(n(r), function () {
      return n(process.hrtime(r));
    });
  } : "undefined" != typeof window && window.performance ? function () {
    var n = window.performance.now();
    return e(n, function () {
      return window.performance.now() - n;
    });
  } : function () {
    var n = Date.now();
    return e(n, function () {
      return Date.now() - n;
    });
  };

function measureExecutionTime(run, callback) {
  const getElapsed = t();
  const ret = run();
  callback(getElapsed(), ret);
}

function saveJsonData(pth, data) {
  require$$0.writeFileSync(pth, JSON.stringify(data, null, 2));
}

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

var portfinder$1 = {};

var async = {exports: {}};

async.exports;

(function (module, exports) {
	(function (global, factory) {
	  factory(exports) ;
	})(commonjsGlobal, function (exports) {

	  function slice(arrayLike, start) {
	    start = start | 0;
	    var newLen = Math.max(arrayLike.length - start, 0);
	    var newArr = Array(newLen);
	    for (var idx = 0; idx < newLen; idx++) {
	      newArr[idx] = arrayLike[start + idx];
	    }
	    return newArr;
	  }

	  /**
	   * Creates a continuation function with some arguments already applied.
	   *
	   * Useful as a shorthand when combined with other control flow functions. Any
	   * arguments passed to the returned function are added to the arguments
	   * originally passed to apply.
	   *
	   * @name apply
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @category Util
	   * @param {Function} fn - The function you want to eventually apply all
	   * arguments to. Invokes with (arguments...).
	   * @param {...*} arguments... - Any number of arguments to automatically apply
	   * when the continuation is called.
	   * @returns {Function} the partially-applied function
	   * @example
	   *
	   * // using apply
	   * async.parallel([
	   *     async.apply(fs.writeFile, 'testfile1', 'test1'),
	   *     async.apply(fs.writeFile, 'testfile2', 'test2')
	   * ]);
	   *
	   *
	   * // the same process without using apply
	   * async.parallel([
	   *     function(callback) {
	   *         fs.writeFile('testfile1', 'test1', callback);
	   *     },
	   *     function(callback) {
	   *         fs.writeFile('testfile2', 'test2', callback);
	   *     }
	   * ]);
	   *
	   * // It's possible to pass any number of additional arguments when calling the
	   * // continuation:
	   *
	   * node> var fn = async.apply(sys.puts, 'one');
	   * node> fn('two', 'three');
	   * one
	   * two
	   * three
	   */
	  var apply = function (fn /*, ...args*/) {
	    var args = slice(arguments, 1);
	    return function /*callArgs*/
	    () {
	      var callArgs = slice(arguments);
	      return fn.apply(null, args.concat(callArgs));
	    };
	  };
	  var initialParams = function (fn) {
	    return function /*...args, callback*/
	    () {
	      var args = slice(arguments);
	      var callback = args.pop();
	      fn.call(this, args, callback);
	    };
	  };

	  /**
	   * Checks if `value` is the
	   * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
	   * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
	   *
	   * @static
	   * @memberOf _
	   * @since 0.1.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is an object, else `false`.
	   * @example
	   *
	   * _.isObject({});
	   * // => true
	   *
	   * _.isObject([1, 2, 3]);
	   * // => true
	   *
	   * _.isObject(_.noop);
	   * // => true
	   *
	   * _.isObject(null);
	   * // => false
	   */
	  function isObject(value) {
	    var type = typeof value;
	    return value != null && (type == 'object' || type == 'function');
	  }
	  var hasSetImmediate = typeof setImmediate === 'function' && setImmediate;
	  var hasNextTick = typeof process === 'object' && typeof process.nextTick === 'function';
	  function fallback(fn) {
	    setTimeout(fn, 0);
	  }
	  function wrap(defer) {
	    return function (fn /*, ...args*/) {
	      var args = slice(arguments, 1);
	      defer(function () {
	        fn.apply(null, args);
	      });
	    };
	  }
	  var _defer;
	  if (hasSetImmediate) {
	    _defer = setImmediate;
	  } else if (hasNextTick) {
	    _defer = process.nextTick;
	  } else {
	    _defer = fallback;
	  }
	  var setImmediate$1 = wrap(_defer);

	  /**
	   * Take a sync function and make it async, passing its return value to a
	   * callback. This is useful for plugging sync functions into a waterfall,
	   * series, or other async functions. Any arguments passed to the generated
	   * function will be passed to the wrapped function (except for the final
	   * callback argument). Errors thrown will be passed to the callback.
	   *
	   * If the function passed to `asyncify` returns a Promise, that promises's
	   * resolved/rejected state will be used to call the callback, rather than simply
	   * the synchronous return value.
	   *
	   * This also means you can asyncify ES2017 `async` functions.
	   *
	   * @name asyncify
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @alias wrapSync
	   * @category Util
	   * @param {Function} func - The synchronous function, or Promise-returning
	   * function to convert to an {@link AsyncFunction}.
	   * @returns {AsyncFunction} An asynchronous wrapper of the `func`. To be
	   * invoked with `(args..., callback)`.
	   * @example
	   *
	   * // passing a regular synchronous function
	   * async.waterfall([
	   *     async.apply(fs.readFile, filename, "utf8"),
	   *     async.asyncify(JSON.parse),
	   *     function (data, next) {
	   *         // data is the result of parsing the text.
	   *         // If there was a parsing error, it would have been caught.
	   *     }
	   * ], callback);
	   *
	   * // passing a function returning a promise
	   * async.waterfall([
	   *     async.apply(fs.readFile, filename, "utf8"),
	   *     async.asyncify(function (contents) {
	   *         return db.model.create(contents);
	   *     }),
	   *     function (model, next) {
	   *         // `model` is the instantiated model object.
	   *         // If there was an error, this function would be skipped.
	   *     }
	   * ], callback);
	   *
	   * // es2017 example, though `asyncify` is not needed if your JS environment
	   * // supports async functions out of the box
	   * var q = async.queue(async.asyncify(async function(file) {
	   *     var intermediateStep = await processFile(file);
	   *     return await somePromise(intermediateStep)
	   * }));
	   *
	   * q.push(files);
	   */
	  function asyncify(func) {
	    return initialParams(function (args, callback) {
	      var result;
	      try {
	        result = func.apply(this, args);
	      } catch (e) {
	        return callback(e);
	      }
	      // if result is Promise object
	      if (isObject(result) && typeof result.then === 'function') {
	        result.then(function (value) {
	          invokeCallback(callback, null, value);
	        }, function (err) {
	          invokeCallback(callback, err.message ? err : new Error(err));
	        });
	      } else {
	        callback(null, result);
	      }
	    });
	  }
	  function invokeCallback(callback, error, value) {
	    try {
	      callback(error, value);
	    } catch (e) {
	      setImmediate$1(rethrow, e);
	    }
	  }
	  function rethrow(error) {
	    throw error;
	  }
	  var supportsSymbol = typeof Symbol === 'function';
	  function isAsync(fn) {
	    return supportsSymbol && fn[Symbol.toStringTag] === 'AsyncFunction';
	  }
	  function wrapAsync(asyncFn) {
	    return isAsync(asyncFn) ? asyncify(asyncFn) : asyncFn;
	  }
	  function applyEach$1(eachfn) {
	    return function (fns /*, ...args*/) {
	      var args = slice(arguments, 1);
	      var go = initialParams(function (args, callback) {
	        var that = this;
	        return eachfn(fns, function (fn, cb) {
	          wrapAsync(fn).apply(that, args.concat(cb));
	        }, callback);
	      });
	      if (args.length) {
	        return go.apply(this, args);
	      } else {
	        return go;
	      }
	    };
	  }

	  /** Detect free variable `global` from Node.js. */
	  var freeGlobal = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

	  /** Detect free variable `self`. */
	  var freeSelf = typeof self == 'object' && self && self.Object === Object && self;

	  /** Used as a reference to the global object. */
	  var root = freeGlobal || freeSelf || Function('return this')();

	  /** Built-in value references. */
	  var Symbol$1 = root.Symbol;

	  /** Used for built-in method references. */
	  var objectProto = Object.prototype;

	  /** Used to check objects for own properties. */
	  var hasOwnProperty = objectProto.hasOwnProperty;

	  /**
	   * Used to resolve the
	   * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
	   * of values.
	   */
	  var nativeObjectToString = objectProto.toString;

	  /** Built-in value references. */
	  var symToStringTag$1 = Symbol$1 ? Symbol$1.toStringTag : undefined;

	  /**
	   * A specialized version of `baseGetTag` which ignores `Symbol.toStringTag` values.
	   *
	   * @private
	   * @param {*} value The value to query.
	   * @returns {string} Returns the raw `toStringTag`.
	   */
	  function getRawTag(value) {
	    var isOwn = hasOwnProperty.call(value, symToStringTag$1),
	      tag = value[symToStringTag$1];
	    try {
	      value[symToStringTag$1] = undefined;
	      var unmasked = true;
	    } catch (e) {}
	    var result = nativeObjectToString.call(value);
	    if (unmasked) {
	      if (isOwn) {
	        value[symToStringTag$1] = tag;
	      } else {
	        delete value[symToStringTag$1];
	      }
	    }
	    return result;
	  }

	  /** Used for built-in method references. */
	  var objectProto$1 = Object.prototype;

	  /**
	   * Used to resolve the
	   * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
	   * of values.
	   */
	  var nativeObjectToString$1 = objectProto$1.toString;

	  /**
	   * Converts `value` to a string using `Object.prototype.toString`.
	   *
	   * @private
	   * @param {*} value The value to convert.
	   * @returns {string} Returns the converted string.
	   */
	  function objectToString(value) {
	    return nativeObjectToString$1.call(value);
	  }

	  /** `Object#toString` result references. */
	  var nullTag = '[object Null]';
	  var undefinedTag = '[object Undefined]';

	  /** Built-in value references. */
	  var symToStringTag = Symbol$1 ? Symbol$1.toStringTag : undefined;

	  /**
	   * The base implementation of `getTag` without fallbacks for buggy environments.
	   *
	   * @private
	   * @param {*} value The value to query.
	   * @returns {string} Returns the `toStringTag`.
	   */
	  function baseGetTag(value) {
	    if (value == null) {
	      return value === undefined ? undefinedTag : nullTag;
	    }
	    return symToStringTag && symToStringTag in Object(value) ? getRawTag(value) : objectToString(value);
	  }

	  /** `Object#toString` result references. */
	  var asyncTag = '[object AsyncFunction]';
	  var funcTag = '[object Function]';
	  var genTag = '[object GeneratorFunction]';
	  var proxyTag = '[object Proxy]';

	  /**
	   * Checks if `value` is classified as a `Function` object.
	   *
	   * @static
	   * @memberOf _
	   * @since 0.1.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is a function, else `false`.
	   * @example
	   *
	   * _.isFunction(_);
	   * // => true
	   *
	   * _.isFunction(/abc/);
	   * // => false
	   */
	  function isFunction(value) {
	    if (!isObject(value)) {
	      return false;
	    }
	    // The use of `Object#toString` avoids issues with the `typeof` operator
	    // in Safari 9 which returns 'object' for typed arrays and other constructors.
	    var tag = baseGetTag(value);
	    return tag == funcTag || tag == genTag || tag == asyncTag || tag == proxyTag;
	  }

	  /** Used as references for various `Number` constants. */
	  var MAX_SAFE_INTEGER = 9007199254740991;

	  /**
	   * Checks if `value` is a valid array-like length.
	   *
	   * **Note:** This method is loosely based on
	   * [`ToLength`](http://ecma-international.org/ecma-262/7.0/#sec-tolength).
	   *
	   * @static
	   * @memberOf _
	   * @since 4.0.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is a valid length, else `false`.
	   * @example
	   *
	   * _.isLength(3);
	   * // => true
	   *
	   * _.isLength(Number.MIN_VALUE);
	   * // => false
	   *
	   * _.isLength(Infinity);
	   * // => false
	   *
	   * _.isLength('3');
	   * // => false
	   */
	  function isLength(value) {
	    return typeof value == 'number' && value > -1 && value % 1 == 0 && value <= MAX_SAFE_INTEGER;
	  }

	  /**
	   * Checks if `value` is array-like. A value is considered array-like if it's
	   * not a function and has a `value.length` that's an integer greater than or
	   * equal to `0` and less than or equal to `Number.MAX_SAFE_INTEGER`.
	   *
	   * @static
	   * @memberOf _
	   * @since 4.0.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is array-like, else `false`.
	   * @example
	   *
	   * _.isArrayLike([1, 2, 3]);
	   * // => true
	   *
	   * _.isArrayLike(document.body.children);
	   * // => true
	   *
	   * _.isArrayLike('abc');
	   * // => true
	   *
	   * _.isArrayLike(_.noop);
	   * // => false
	   */
	  function isArrayLike(value) {
	    return value != null && isLength(value.length) && !isFunction(value);
	  }

	  // A temporary value used to identify if the loop should be broken.
	  // See #1064, #1293
	  var breakLoop = {};

	  /**
	   * This method returns `undefined`.
	   *
	   * @static
	   * @memberOf _
	   * @since 2.3.0
	   * @category Util
	   * @example
	   *
	   * _.times(2, _.noop);
	   * // => [undefined, undefined]
	   */
	  function noop() {
	    // No operation performed.
	  }
	  function once(fn) {
	    return function () {
	      if (fn === null) return;
	      var callFn = fn;
	      fn = null;
	      callFn.apply(this, arguments);
	    };
	  }
	  var iteratorSymbol = typeof Symbol === 'function' && Symbol.iterator;
	  var getIterator = function (coll) {
	    return iteratorSymbol && coll[iteratorSymbol] && coll[iteratorSymbol]();
	  };

	  /**
	   * The base implementation of `_.times` without support for iteratee shorthands
	   * or max array length checks.
	   *
	   * @private
	   * @param {number} n The number of times to invoke `iteratee`.
	   * @param {Function} iteratee The function invoked per iteration.
	   * @returns {Array} Returns the array of results.
	   */
	  function baseTimes(n, iteratee) {
	    var index = -1,
	      result = Array(n);
	    while (++index < n) {
	      result[index] = iteratee(index);
	    }
	    return result;
	  }

	  /**
	   * Checks if `value` is object-like. A value is object-like if it's not `null`
	   * and has a `typeof` result of "object".
	   *
	   * @static
	   * @memberOf _
	   * @since 4.0.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
	   * @example
	   *
	   * _.isObjectLike({});
	   * // => true
	   *
	   * _.isObjectLike([1, 2, 3]);
	   * // => true
	   *
	   * _.isObjectLike(_.noop);
	   * // => false
	   *
	   * _.isObjectLike(null);
	   * // => false
	   */
	  function isObjectLike(value) {
	    return value != null && typeof value == 'object';
	  }

	  /** `Object#toString` result references. */
	  var argsTag = '[object Arguments]';

	  /**
	   * The base implementation of `_.isArguments`.
	   *
	   * @private
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is an `arguments` object,
	   */
	  function baseIsArguments(value) {
	    return isObjectLike(value) && baseGetTag(value) == argsTag;
	  }

	  /** Used for built-in method references. */
	  var objectProto$3 = Object.prototype;

	  /** Used to check objects for own properties. */
	  var hasOwnProperty$2 = objectProto$3.hasOwnProperty;

	  /** Built-in value references. */
	  var propertyIsEnumerable = objectProto$3.propertyIsEnumerable;

	  /**
	   * Checks if `value` is likely an `arguments` object.
	   *
	   * @static
	   * @memberOf _
	   * @since 0.1.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is an `arguments` object,
	   *  else `false`.
	   * @example
	   *
	   * _.isArguments(function() { return arguments; }());
	   * // => true
	   *
	   * _.isArguments([1, 2, 3]);
	   * // => false
	   */
	  var isArguments = baseIsArguments(function () {
	    return arguments;
	  }()) ? baseIsArguments : function (value) {
	    return isObjectLike(value) && hasOwnProperty$2.call(value, 'callee') && !propertyIsEnumerable.call(value, 'callee');
	  };

	  /**
	   * Checks if `value` is classified as an `Array` object.
	   *
	   * @static
	   * @memberOf _
	   * @since 0.1.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is an array, else `false`.
	   * @example
	   *
	   * _.isArray([1, 2, 3]);
	   * // => true
	   *
	   * _.isArray(document.body.children);
	   * // => false
	   *
	   * _.isArray('abc');
	   * // => false
	   *
	   * _.isArray(_.noop);
	   * // => false
	   */
	  var isArray = Array.isArray;

	  /**
	   * This method returns `false`.
	   *
	   * @static
	   * @memberOf _
	   * @since 4.13.0
	   * @category Util
	   * @returns {boolean} Returns `false`.
	   * @example
	   *
	   * _.times(2, _.stubFalse);
	   * // => [false, false]
	   */
	  function stubFalse() {
	    return false;
	  }

	  /** Detect free variable `exports`. */
	  var freeExports = typeof exports == 'object' && exports && !exports.nodeType && exports;

	  /** Detect free variable `module`. */
	  var freeModule = freeExports && 'object' == 'object' && module && !module.nodeType && module;

	  /** Detect the popular CommonJS extension `module.exports`. */
	  var moduleExports = freeModule && freeModule.exports === freeExports;

	  /** Built-in value references. */
	  var Buffer = moduleExports ? root.Buffer : undefined;

	  /* Built-in method references for those with the same name as other `lodash` methods. */
	  var nativeIsBuffer = Buffer ? Buffer.isBuffer : undefined;

	  /**
	   * Checks if `value` is a buffer.
	   *
	   * @static
	   * @memberOf _
	   * @since 4.3.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is a buffer, else `false`.
	   * @example
	   *
	   * _.isBuffer(new Buffer(2));
	   * // => true
	   *
	   * _.isBuffer(new Uint8Array(2));
	   * // => false
	   */
	  var isBuffer = nativeIsBuffer || stubFalse;

	  /** Used as references for various `Number` constants. */
	  var MAX_SAFE_INTEGER$1 = 9007199254740991;

	  /** Used to detect unsigned integer values. */
	  var reIsUint = /^(?:0|[1-9]\d*)$/;

	  /**
	   * Checks if `value` is a valid array-like index.
	   *
	   * @private
	   * @param {*} value The value to check.
	   * @param {number} [length=MAX_SAFE_INTEGER] The upper bounds of a valid index.
	   * @returns {boolean} Returns `true` if `value` is a valid index, else `false`.
	   */
	  function isIndex(value, length) {
	    var type = typeof value;
	    length = length == null ? MAX_SAFE_INTEGER$1 : length;
	    return !!length && (type == 'number' || type != 'symbol' && reIsUint.test(value)) && value > -1 && value % 1 == 0 && value < length;
	  }

	  /** `Object#toString` result references. */
	  var argsTag$1 = '[object Arguments]';
	  var arrayTag = '[object Array]';
	  var boolTag = '[object Boolean]';
	  var dateTag = '[object Date]';
	  var errorTag = '[object Error]';
	  var funcTag$1 = '[object Function]';
	  var mapTag = '[object Map]';
	  var numberTag = '[object Number]';
	  var objectTag = '[object Object]';
	  var regexpTag = '[object RegExp]';
	  var setTag = '[object Set]';
	  var stringTag = '[object String]';
	  var weakMapTag = '[object WeakMap]';
	  var arrayBufferTag = '[object ArrayBuffer]';
	  var dataViewTag = '[object DataView]';
	  var float32Tag = '[object Float32Array]';
	  var float64Tag = '[object Float64Array]';
	  var int8Tag = '[object Int8Array]';
	  var int16Tag = '[object Int16Array]';
	  var int32Tag = '[object Int32Array]';
	  var uint8Tag = '[object Uint8Array]';
	  var uint8ClampedTag = '[object Uint8ClampedArray]';
	  var uint16Tag = '[object Uint16Array]';
	  var uint32Tag = '[object Uint32Array]';

	  /** Used to identify `toStringTag` values of typed arrays. */
	  var typedArrayTags = {};
	  typedArrayTags[float32Tag] = typedArrayTags[float64Tag] = typedArrayTags[int8Tag] = typedArrayTags[int16Tag] = typedArrayTags[int32Tag] = typedArrayTags[uint8Tag] = typedArrayTags[uint8ClampedTag] = typedArrayTags[uint16Tag] = typedArrayTags[uint32Tag] = true;
	  typedArrayTags[argsTag$1] = typedArrayTags[arrayTag] = typedArrayTags[arrayBufferTag] = typedArrayTags[boolTag] = typedArrayTags[dataViewTag] = typedArrayTags[dateTag] = typedArrayTags[errorTag] = typedArrayTags[funcTag$1] = typedArrayTags[mapTag] = typedArrayTags[numberTag] = typedArrayTags[objectTag] = typedArrayTags[regexpTag] = typedArrayTags[setTag] = typedArrayTags[stringTag] = typedArrayTags[weakMapTag] = false;

	  /**
	   * The base implementation of `_.isTypedArray` without Node.js optimizations.
	   *
	   * @private
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is a typed array, else `false`.
	   */
	  function baseIsTypedArray(value) {
	    return isObjectLike(value) && isLength(value.length) && !!typedArrayTags[baseGetTag(value)];
	  }

	  /**
	   * The base implementation of `_.unary` without support for storing metadata.
	   *
	   * @private
	   * @param {Function} func The function to cap arguments for.
	   * @returns {Function} Returns the new capped function.
	   */
	  function baseUnary(func) {
	    return function (value) {
	      return func(value);
	    };
	  }

	  /** Detect free variable `exports`. */
	  var freeExports$1 = typeof exports == 'object' && exports && !exports.nodeType && exports;

	  /** Detect free variable `module`. */
	  var freeModule$1 = freeExports$1 && 'object' == 'object' && module && !module.nodeType && module;

	  /** Detect the popular CommonJS extension `module.exports`. */
	  var moduleExports$1 = freeModule$1 && freeModule$1.exports === freeExports$1;

	  /** Detect free variable `process` from Node.js. */
	  var freeProcess = moduleExports$1 && freeGlobal.process;

	  /** Used to access faster Node.js helpers. */
	  var nodeUtil = function () {
	    try {
	      // Use `util.types` for Node.js 10+.
	      var types = freeModule$1 && freeModule$1.require && freeModule$1.require('util').types;
	      if (types) {
	        return types;
	      }

	      // Legacy `process.binding('util')` for Node.js < 10.
	      return freeProcess && freeProcess.binding && freeProcess.binding('util');
	    } catch (e) {}
	  }();

	  /* Node.js helper references. */
	  var nodeIsTypedArray = nodeUtil && nodeUtil.isTypedArray;

	  /**
	   * Checks if `value` is classified as a typed array.
	   *
	   * @static
	   * @memberOf _
	   * @since 3.0.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is a typed array, else `false`.
	   * @example
	   *
	   * _.isTypedArray(new Uint8Array);
	   * // => true
	   *
	   * _.isTypedArray([]);
	   * // => false
	   */
	  var isTypedArray = nodeIsTypedArray ? baseUnary(nodeIsTypedArray) : baseIsTypedArray;

	  /** Used for built-in method references. */
	  var objectProto$2 = Object.prototype;

	  /** Used to check objects for own properties. */
	  var hasOwnProperty$1 = objectProto$2.hasOwnProperty;

	  /**
	   * Creates an array of the enumerable property names of the array-like `value`.
	   *
	   * @private
	   * @param {*} value The value to query.
	   * @param {boolean} inherited Specify returning inherited property names.
	   * @returns {Array} Returns the array of property names.
	   */
	  function arrayLikeKeys(value, inherited) {
	    var isArr = isArray(value),
	      isArg = !isArr && isArguments(value),
	      isBuff = !isArr && !isArg && isBuffer(value),
	      isType = !isArr && !isArg && !isBuff && isTypedArray(value),
	      skipIndexes = isArr || isArg || isBuff || isType,
	      result = skipIndexes ? baseTimes(value.length, String) : [],
	      length = result.length;
	    for (var key in value) {
	      if ((inherited || hasOwnProperty$1.call(value, key)) && !(skipIndexes && (
	      // Safari 9 has enumerable `arguments.length` in strict mode.
	      key == 'length' ||
	      // Node.js 0.10 has enumerable non-index properties on buffers.
	      isBuff && (key == 'offset' || key == 'parent') ||
	      // PhantomJS 2 has enumerable non-index properties on typed arrays.
	      isType && (key == 'buffer' || key == 'byteLength' || key == 'byteOffset') ||
	      // Skip index properties.
	      isIndex(key, length)))) {
	        result.push(key);
	      }
	    }
	    return result;
	  }

	  /** Used for built-in method references. */
	  var objectProto$5 = Object.prototype;

	  /**
	   * Checks if `value` is likely a prototype object.
	   *
	   * @private
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is a prototype, else `false`.
	   */
	  function isPrototype(value) {
	    var Ctor = value && value.constructor,
	      proto = typeof Ctor == 'function' && Ctor.prototype || objectProto$5;
	    return value === proto;
	  }

	  /**
	   * Creates a unary function that invokes `func` with its argument transformed.
	   *
	   * @private
	   * @param {Function} func The function to wrap.
	   * @param {Function} transform The argument transform.
	   * @returns {Function} Returns the new function.
	   */
	  function overArg(func, transform) {
	    return function (arg) {
	      return func(transform(arg));
	    };
	  }

	  /* Built-in method references for those with the same name as other `lodash` methods. */
	  var nativeKeys = overArg(Object.keys, Object);

	  /** Used for built-in method references. */
	  var objectProto$4 = Object.prototype;

	  /** Used to check objects for own properties. */
	  var hasOwnProperty$3 = objectProto$4.hasOwnProperty;

	  /**
	   * The base implementation of `_.keys` which doesn't treat sparse arrays as dense.
	   *
	   * @private
	   * @param {Object} object The object to query.
	   * @returns {Array} Returns the array of property names.
	   */
	  function baseKeys(object) {
	    if (!isPrototype(object)) {
	      return nativeKeys(object);
	    }
	    var result = [];
	    for (var key in Object(object)) {
	      if (hasOwnProperty$3.call(object, key) && key != 'constructor') {
	        result.push(key);
	      }
	    }
	    return result;
	  }

	  /**
	   * Creates an array of the own enumerable property names of `object`.
	   *
	   * **Note:** Non-object values are coerced to objects. See the
	   * [ES spec](http://ecma-international.org/ecma-262/7.0/#sec-object.keys)
	   * for more details.
	   *
	   * @static
	   * @since 0.1.0
	   * @memberOf _
	   * @category Object
	   * @param {Object} object The object to query.
	   * @returns {Array} Returns the array of property names.
	   * @example
	   *
	   * function Foo() {
	   *   this.a = 1;
	   *   this.b = 2;
	   * }
	   *
	   * Foo.prototype.c = 3;
	   *
	   * _.keys(new Foo);
	   * // => ['a', 'b'] (iteration order is not guaranteed)
	   *
	   * _.keys('hi');
	   * // => ['0', '1']
	   */
	  function keys(object) {
	    return isArrayLike(object) ? arrayLikeKeys(object) : baseKeys(object);
	  }
	  function createArrayIterator(coll) {
	    var i = -1;
	    var len = coll.length;
	    return function next() {
	      return ++i < len ? {
	        value: coll[i],
	        key: i
	      } : null;
	    };
	  }
	  function createES2015Iterator(iterator) {
	    var i = -1;
	    return function next() {
	      var item = iterator.next();
	      if (item.done) return null;
	      i++;
	      return {
	        value: item.value,
	        key: i
	      };
	    };
	  }
	  function createObjectIterator(obj) {
	    var okeys = keys(obj);
	    var i = -1;
	    var len = okeys.length;
	    return function next() {
	      var key = okeys[++i];
	      if (key === '__proto__') {
	        return next();
	      }
	      return i < len ? {
	        value: obj[key],
	        key: key
	      } : null;
	    };
	  }
	  function iterator(coll) {
	    if (isArrayLike(coll)) {
	      return createArrayIterator(coll);
	    }
	    var iterator = getIterator(coll);
	    return iterator ? createES2015Iterator(iterator) : createObjectIterator(coll);
	  }
	  function onlyOnce(fn) {
	    return function () {
	      if (fn === null) throw new Error("Callback was already called.");
	      var callFn = fn;
	      fn = null;
	      callFn.apply(this, arguments);
	    };
	  }
	  function _eachOfLimit(limit) {
	    return function (obj, iteratee, callback) {
	      callback = once(callback || noop);
	      if (limit <= 0 || !obj) {
	        return callback(null);
	      }
	      var nextElem = iterator(obj);
	      var done = false;
	      var running = 0;
	      var looping = false;
	      function iterateeCallback(err, value) {
	        running -= 1;
	        if (err) {
	          done = true;
	          callback(err);
	        } else if (value === breakLoop || done && running <= 0) {
	          done = true;
	          return callback(null);
	        } else if (!looping) {
	          replenish();
	        }
	      }
	      function replenish() {
	        looping = true;
	        while (running < limit && !done) {
	          var elem = nextElem();
	          if (elem === null) {
	            done = true;
	            if (running <= 0) {
	              callback(null);
	            }
	            return;
	          }
	          running += 1;
	          iteratee(elem.value, elem.key, onlyOnce(iterateeCallback));
	        }
	        looping = false;
	      }
	      replenish();
	    };
	  }

	  /**
	   * The same as [`eachOf`]{@link module:Collections.eachOf} but runs a maximum of `limit` async operations at a
	   * time.
	   *
	   * @name eachOfLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.eachOf]{@link module:Collections.eachOf}
	   * @alias forEachOfLimit
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - An async function to apply to each
	   * item in `coll`. The `key` is the item's key, or index in the case of an
	   * array.
	   * Invoked with (item, key, callback).
	   * @param {Function} [callback] - A callback which is called when all
	   * `iteratee` functions have finished, or an error occurs. Invoked with (err).
	   */
	  function eachOfLimit(coll, limit, iteratee, callback) {
	    _eachOfLimit(limit)(coll, wrapAsync(iteratee), callback);
	  }
	  function doLimit(fn, limit) {
	    return function (iterable, iteratee, callback) {
	      return fn(iterable, limit, iteratee, callback);
	    };
	  }

	  // eachOf implementation optimized for array-likes
	  function eachOfArrayLike(coll, iteratee, callback) {
	    callback = once(callback || noop);
	    var index = 0,
	      completed = 0,
	      length = coll.length;
	    if (length === 0) {
	      callback(null);
	    }
	    function iteratorCallback(err, value) {
	      if (err) {
	        callback(err);
	      } else if (++completed === length || value === breakLoop) {
	        callback(null);
	      }
	    }
	    for (; index < length; index++) {
	      iteratee(coll[index], index, onlyOnce(iteratorCallback));
	    }
	  }

	  // a generic version of eachOf which can handle array, object, and iterator cases.
	  var eachOfGeneric = doLimit(eachOfLimit, Infinity);

	  /**
	   * Like [`each`]{@link module:Collections.each}, except that it passes the key (or index) as the second argument
	   * to the iteratee.
	   *
	   * @name eachOf
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @alias forEachOf
	   * @category Collection
	   * @see [async.each]{@link module:Collections.each}
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - A function to apply to each
	   * item in `coll`.
	   * The `key` is the item's key, or index in the case of an array.
	   * Invoked with (item, key, callback).
	   * @param {Function} [callback] - A callback which is called when all
	   * `iteratee` functions have finished, or an error occurs. Invoked with (err).
	   * @example
	   *
	   * var obj = {dev: "/dev.json", test: "/test.json", prod: "/prod.json"};
	   * var configs = {};
	   *
	   * async.forEachOf(obj, function (value, key, callback) {
	   *     fs.readFile(__dirname + value, "utf8", function (err, data) {
	   *         if (err) return callback(err);
	   *         try {
	   *             configs[key] = JSON.parse(data);
	   *         } catch (e) {
	   *             return callback(e);
	   *         }
	   *         callback();
	   *     });
	   * }, function (err) {
	   *     if (err) console.error(err.message);
	   *     // configs is now a map of JSON data
	   *     doSomethingWith(configs);
	   * });
	   */
	  var eachOf = function (coll, iteratee, callback) {
	    var eachOfImplementation = isArrayLike(coll) ? eachOfArrayLike : eachOfGeneric;
	    eachOfImplementation(coll, wrapAsync(iteratee), callback);
	  };
	  function doParallel(fn) {
	    return function (obj, iteratee, callback) {
	      return fn(eachOf, obj, wrapAsync(iteratee), callback);
	    };
	  }
	  function _asyncMap(eachfn, arr, iteratee, callback) {
	    callback = callback || noop;
	    arr = arr || [];
	    var results = [];
	    var counter = 0;
	    var _iteratee = wrapAsync(iteratee);
	    eachfn(arr, function (value, _, callback) {
	      var index = counter++;
	      _iteratee(value, function (err, v) {
	        results[index] = v;
	        callback(err);
	      });
	    }, function (err) {
	      callback(err, results);
	    });
	  }

	  /**
	   * Produces a new collection of values by mapping each value in `coll` through
	   * the `iteratee` function. The `iteratee` is called with an item from `coll`
	   * and a callback for when it has finished processing. Each of these callback
	   * takes 2 arguments: an `error`, and the transformed item from `coll`. If
	   * `iteratee` passes an error to its callback, the main `callback` (for the
	   * `map` function) is immediately called with the error.
	   *
	   * Note, that since this function applies the `iteratee` to each item in
	   * parallel, there is no guarantee that the `iteratee` functions will complete
	   * in order. However, the results array will be in the same order as the
	   * original `coll`.
	   *
	   * If `map` is passed an Object, the results will be an Array.  The results
	   * will roughly be in the order of the original Objects' keys (but this can
	   * vary across JavaScript engines).
	   *
	   * @name map
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * The iteratee should complete with the transformed item.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. Results is an Array of the
	   * transformed items from the `coll`. Invoked with (err, results).
	   * @example
	   *
	   * async.map(['file1','file2','file3'], fs.stat, function(err, results) {
	   *     // results is now an array of stats for each file
	   * });
	   */
	  var map = doParallel(_asyncMap);

	  /**
	   * Applies the provided arguments to each function in the array, calling
	   * `callback` after all functions have completed. If you only provide the first
	   * argument, `fns`, then it will return a function which lets you pass in the
	   * arguments as if it were a single function call. If more arguments are
	   * provided, `callback` is required while `args` is still optional.
	   *
	   * @name applyEach
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {Array|Iterable|Object} fns - A collection of {@link AsyncFunction}s
	   * to all call with the same arguments
	   * @param {...*} [args] - any number of separate arguments to pass to the
	   * function.
	   * @param {Function} [callback] - the final argument should be the callback,
	   * called when all functions have completed processing.
	   * @returns {Function} - If only the first argument, `fns`, is provided, it will
	   * return a function which lets you pass in the arguments as if it were a single
	   * function call. The signature is `(..args, callback)`. If invoked with any
	   * arguments, `callback` is required.
	   * @example
	   *
	   * async.applyEach([enableSearch, updateSchema], 'bucket', callback);
	   *
	   * // partial application example:
	   * async.each(
	   *     buckets,
	   *     async.applyEach([enableSearch, updateSchema]),
	   *     callback
	   * );
	   */
	  var applyEach = applyEach$1(map);
	  function doParallelLimit(fn) {
	    return function (obj, limit, iteratee, callback) {
	      return fn(_eachOfLimit(limit), obj, wrapAsync(iteratee), callback);
	    };
	  }

	  /**
	   * The same as [`map`]{@link module:Collections.map} but runs a maximum of `limit` async operations at a time.
	   *
	   * @name mapLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.map]{@link module:Collections.map}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * The iteratee should complete with the transformed item.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. Results is an array of the
	   * transformed items from the `coll`. Invoked with (err, results).
	   */
	  var mapLimit = doParallelLimit(_asyncMap);

	  /**
	   * The same as [`map`]{@link module:Collections.map} but runs only a single async operation at a time.
	   *
	   * @name mapSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.map]{@link module:Collections.map}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * The iteratee should complete with the transformed item.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. Results is an array of the
	   * transformed items from the `coll`. Invoked with (err, results).
	   */
	  var mapSeries = doLimit(mapLimit, 1);

	  /**
	   * The same as [`applyEach`]{@link module:ControlFlow.applyEach} but runs only a single async operation at a time.
	   *
	   * @name applyEachSeries
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.applyEach]{@link module:ControlFlow.applyEach}
	   * @category Control Flow
	   * @param {Array|Iterable|Object} fns - A collection of {@link AsyncFunction}s to all
	   * call with the same arguments
	   * @param {...*} [args] - any number of separate arguments to pass to the
	   * function.
	   * @param {Function} [callback] - the final argument should be the callback,
	   * called when all functions have completed processing.
	   * @returns {Function} - If only the first argument is provided, it will return
	   * a function which lets you pass in the arguments as if it were a single
	   * function call.
	   */
	  var applyEachSeries = applyEach$1(mapSeries);

	  /**
	   * A specialized version of `_.forEach` for arrays without support for
	   * iteratee shorthands.
	   *
	   * @private
	   * @param {Array} [array] The array to iterate over.
	   * @param {Function} iteratee The function invoked per iteration.
	   * @returns {Array} Returns `array`.
	   */
	  function arrayEach(array, iteratee) {
	    var index = -1,
	      length = array == null ? 0 : array.length;
	    while (++index < length) {
	      if (iteratee(array[index], index, array) === false) {
	        break;
	      }
	    }
	    return array;
	  }

	  /**
	   * Creates a base function for methods like `_.forIn` and `_.forOwn`.
	   *
	   * @private
	   * @param {boolean} [fromRight] Specify iterating from right to left.
	   * @returns {Function} Returns the new base function.
	   */
	  function createBaseFor(fromRight) {
	    return function (object, iteratee, keysFunc) {
	      var index = -1,
	        iterable = Object(object),
	        props = keysFunc(object),
	        length = props.length;
	      while (length--) {
	        var key = props[fromRight ? length : ++index];
	        if (iteratee(iterable[key], key, iterable) === false) {
	          break;
	        }
	      }
	      return object;
	    };
	  }

	  /**
	   * The base implementation of `baseForOwn` which iterates over `object`
	   * properties returned by `keysFunc` and invokes `iteratee` for each property.
	   * Iteratee functions may exit iteration early by explicitly returning `false`.
	   *
	   * @private
	   * @param {Object} object The object to iterate over.
	   * @param {Function} iteratee The function invoked per iteration.
	   * @param {Function} keysFunc The function to get the keys of `object`.
	   * @returns {Object} Returns `object`.
	   */
	  var baseFor = createBaseFor();

	  /**
	   * The base implementation of `_.forOwn` without support for iteratee shorthands.
	   *
	   * @private
	   * @param {Object} object The object to iterate over.
	   * @param {Function} iteratee The function invoked per iteration.
	   * @returns {Object} Returns `object`.
	   */
	  function baseForOwn(object, iteratee) {
	    return object && baseFor(object, iteratee, keys);
	  }

	  /**
	   * The base implementation of `_.findIndex` and `_.findLastIndex` without
	   * support for iteratee shorthands.
	   *
	   * @private
	   * @param {Array} array The array to inspect.
	   * @param {Function} predicate The function invoked per iteration.
	   * @param {number} fromIndex The index to search from.
	   * @param {boolean} [fromRight] Specify iterating from right to left.
	   * @returns {number} Returns the index of the matched value, else `-1`.
	   */
	  function baseFindIndex(array, predicate, fromIndex, fromRight) {
	    var length = array.length,
	      index = fromIndex + (fromRight ? 1 : -1);
	    while (fromRight ? index-- : ++index < length) {
	      if (predicate(array[index], index, array)) {
	        return index;
	      }
	    }
	    return -1;
	  }

	  /**
	   * The base implementation of `_.isNaN` without support for number objects.
	   *
	   * @private
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is `NaN`, else `false`.
	   */
	  function baseIsNaN(value) {
	    return value !== value;
	  }

	  /**
	   * A specialized version of `_.indexOf` which performs strict equality
	   * comparisons of values, i.e. `===`.
	   *
	   * @private
	   * @param {Array} array The array to inspect.
	   * @param {*} value The value to search for.
	   * @param {number} fromIndex The index to search from.
	   * @returns {number} Returns the index of the matched value, else `-1`.
	   */
	  function strictIndexOf(array, value, fromIndex) {
	    var index = fromIndex - 1,
	      length = array.length;
	    while (++index < length) {
	      if (array[index] === value) {
	        return index;
	      }
	    }
	    return -1;
	  }

	  /**
	   * The base implementation of `_.indexOf` without `fromIndex` bounds checks.
	   *
	   * @private
	   * @param {Array} array The array to inspect.
	   * @param {*} value The value to search for.
	   * @param {number} fromIndex The index to search from.
	   * @returns {number} Returns the index of the matched value, else `-1`.
	   */
	  function baseIndexOf(array, value, fromIndex) {
	    return value === value ? strictIndexOf(array, value, fromIndex) : baseFindIndex(array, baseIsNaN, fromIndex);
	  }

	  /**
	   * Determines the best order for running the {@link AsyncFunction}s in `tasks`, based on
	   * their requirements. Each function can optionally depend on other functions
	   * being completed first, and each function is run as soon as its requirements
	   * are satisfied.
	   *
	   * If any of the {@link AsyncFunction}s pass an error to their callback, the `auto` sequence
	   * will stop. Further tasks will not execute (so any other functions depending
	   * on it will not run), and the main `callback` is immediately called with the
	   * error.
	   *
	   * {@link AsyncFunction}s also receive an object containing the results of functions which
	   * have completed so far as the first argument, if they have dependencies. If a
	   * task function has no dependencies, it will only be passed a callback.
	   *
	   * @name auto
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {Object} tasks - An object. Each of its properties is either a
	   * function or an array of requirements, with the {@link AsyncFunction} itself the last item
	   * in the array. The object's key of a property serves as the name of the task
	   * defined by that property, i.e. can be used when specifying requirements for
	   * other tasks. The function receives one or two arguments:
	   * * a `results` object, containing the results of the previously executed
	   *   functions, only passed if the task has any dependencies,
	   * * a `callback(err, result)` function, which must be called when finished,
	   *   passing an `error` (which can be `null`) and the result of the function's
	   *   execution.
	   * @param {number} [concurrency=Infinity] - An optional `integer` for
	   * determining the maximum number of tasks that can be run in parallel. By
	   * default, as many as possible.
	   * @param {Function} [callback] - An optional callback which is called when all
	   * the tasks have been completed. It receives the `err` argument if any `tasks`
	   * pass an error to their callback. Results are always returned; however, if an
	   * error occurs, no further `tasks` will be performed, and the results object
	   * will only contain partial results. Invoked with (err, results).
	   * @returns undefined
	   * @example
	   *
	   * async.auto({
	   *     // this function will just be passed a callback
	   *     readData: async.apply(fs.readFile, 'data.txt', 'utf-8'),
	   *     showData: ['readData', function(results, cb) {
	   *         // results.readData is the file's contents
	   *         // ...
	   *     }]
	   * }, callback);
	   *
	   * async.auto({
	   *     get_data: function(callback) {
	   *         console.log('in get_data');
	   *         // async code to get some data
	   *         callback(null, 'data', 'converted to array');
	   *     },
	   *     make_folder: function(callback) {
	   *         console.log('in make_folder');
	   *         // async code to create a directory to store a file in
	   *         // this is run at the same time as getting the data
	   *         callback(null, 'folder');
	   *     },
	   *     write_file: ['get_data', 'make_folder', function(results, callback) {
	   *         console.log('in write_file', JSON.stringify(results));
	   *         // once there is some data and the directory exists,
	   *         // write the data to a file in the directory
	   *         callback(null, 'filename');
	   *     }],
	   *     email_link: ['write_file', function(results, callback) {
	   *         console.log('in email_link', JSON.stringify(results));
	   *         // once the file is written let's email a link to it...
	   *         // results.write_file contains the filename returned by write_file.
	   *         callback(null, {'file':results.write_file, 'email':'user@example.com'});
	   *     }]
	   * }, function(err, results) {
	   *     console.log('err = ', err);
	   *     console.log('results = ', results);
	   * });
	   */
	  var auto = function (tasks, concurrency, callback) {
	    if (typeof concurrency === 'function') {
	      // concurrency is optional, shift the args.
	      callback = concurrency;
	      concurrency = null;
	    }
	    callback = once(callback || noop);
	    var keys$$1 = keys(tasks);
	    var numTasks = keys$$1.length;
	    if (!numTasks) {
	      return callback(null);
	    }
	    if (!concurrency) {
	      concurrency = numTasks;
	    }
	    var results = {};
	    var runningTasks = 0;
	    var hasError = false;
	    var listeners = Object.create(null);
	    var readyTasks = [];

	    // for cycle detection:
	    var readyToCheck = []; // tasks that have been identified as reachable
	    // without the possibility of returning to an ancestor task
	    var uncheckedDependencies = {};
	    baseForOwn(tasks, function (task, key) {
	      if (!isArray(task)) {
	        // no dependencies
	        enqueueTask(key, [task]);
	        readyToCheck.push(key);
	        return;
	      }
	      var dependencies = task.slice(0, task.length - 1);
	      var remainingDependencies = dependencies.length;
	      if (remainingDependencies === 0) {
	        enqueueTask(key, task);
	        readyToCheck.push(key);
	        return;
	      }
	      uncheckedDependencies[key] = remainingDependencies;
	      arrayEach(dependencies, function (dependencyName) {
	        if (!tasks[dependencyName]) {
	          throw new Error('async.auto task `' + key + '` has a non-existent dependency `' + dependencyName + '` in ' + dependencies.join(', '));
	        }
	        addListener(dependencyName, function () {
	          remainingDependencies--;
	          if (remainingDependencies === 0) {
	            enqueueTask(key, task);
	          }
	        });
	      });
	    });
	    checkForDeadlocks();
	    processQueue();
	    function enqueueTask(key, task) {
	      readyTasks.push(function () {
	        runTask(key, task);
	      });
	    }
	    function processQueue() {
	      if (readyTasks.length === 0 && runningTasks === 0) {
	        return callback(null, results);
	      }
	      while (readyTasks.length && runningTasks < concurrency) {
	        var run = readyTasks.shift();
	        run();
	      }
	    }
	    function addListener(taskName, fn) {
	      var taskListeners = listeners[taskName];
	      if (!taskListeners) {
	        taskListeners = listeners[taskName] = [];
	      }
	      taskListeners.push(fn);
	    }
	    function taskComplete(taskName) {
	      var taskListeners = listeners[taskName] || [];
	      arrayEach(taskListeners, function (fn) {
	        fn();
	      });
	      processQueue();
	    }
	    function runTask(key, task) {
	      if (hasError) return;
	      var taskCallback = onlyOnce(function (err, result) {
	        runningTasks--;
	        if (arguments.length > 2) {
	          result = slice(arguments, 1);
	        }
	        if (err) {
	          var safeResults = {};
	          baseForOwn(results, function (val, rkey) {
	            safeResults[rkey] = val;
	          });
	          safeResults[key] = result;
	          hasError = true;
	          listeners = Object.create(null);
	          callback(err, safeResults);
	        } else {
	          results[key] = result;
	          taskComplete(key);
	        }
	      });
	      runningTasks++;
	      var taskFn = wrapAsync(task[task.length - 1]);
	      if (task.length > 1) {
	        taskFn(results, taskCallback);
	      } else {
	        taskFn(taskCallback);
	      }
	    }
	    function checkForDeadlocks() {
	      // Kahn's algorithm
	      // https://en.wikipedia.org/wiki/Topological_sorting#Kahn.27s_algorithm
	      // http://connalle.blogspot.com/2013/10/topological-sortingkahn-algorithm.html
	      var currentTask;
	      var counter = 0;
	      while (readyToCheck.length) {
	        currentTask = readyToCheck.pop();
	        counter++;
	        arrayEach(getDependents(currentTask), function (dependent) {
	          if (--uncheckedDependencies[dependent] === 0) {
	            readyToCheck.push(dependent);
	          }
	        });
	      }
	      if (counter !== numTasks) {
	        throw new Error('async.auto cannot execute tasks due to a recursive dependency');
	      }
	    }
	    function getDependents(taskName) {
	      var result = [];
	      baseForOwn(tasks, function (task, key) {
	        if (isArray(task) && baseIndexOf(task, taskName, 0) >= 0) {
	          result.push(key);
	        }
	      });
	      return result;
	    }
	  };

	  /**
	   * A specialized version of `_.map` for arrays without support for iteratee
	   * shorthands.
	   *
	   * @private
	   * @param {Array} [array] The array to iterate over.
	   * @param {Function} iteratee The function invoked per iteration.
	   * @returns {Array} Returns the new mapped array.
	   */
	  function arrayMap(array, iteratee) {
	    var index = -1,
	      length = array == null ? 0 : array.length,
	      result = Array(length);
	    while (++index < length) {
	      result[index] = iteratee(array[index], index, array);
	    }
	    return result;
	  }

	  /** `Object#toString` result references. */
	  var symbolTag = '[object Symbol]';

	  /**
	   * Checks if `value` is classified as a `Symbol` primitive or object.
	   *
	   * @static
	   * @memberOf _
	   * @since 4.0.0
	   * @category Lang
	   * @param {*} value The value to check.
	   * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
	   * @example
	   *
	   * _.isSymbol(Symbol.iterator);
	   * // => true
	   *
	   * _.isSymbol('abc');
	   * // => false
	   */
	  function isSymbol(value) {
	    return typeof value == 'symbol' || isObjectLike(value) && baseGetTag(value) == symbolTag;
	  }

	  /** Used as references for various `Number` constants. */
	  var INFINITY = 1 / 0;

	  /** Used to convert symbols to primitives and strings. */
	  var symbolProto = Symbol$1 ? Symbol$1.prototype : undefined;
	  var symbolToString = symbolProto ? symbolProto.toString : undefined;

	  /**
	   * The base implementation of `_.toString` which doesn't convert nullish
	   * values to empty strings.
	   *
	   * @private
	   * @param {*} value The value to process.
	   * @returns {string} Returns the string.
	   */
	  function baseToString(value) {
	    // Exit early for strings to avoid a performance hit in some environments.
	    if (typeof value == 'string') {
	      return value;
	    }
	    if (isArray(value)) {
	      // Recursively convert values (susceptible to call stack limits).
	      return arrayMap(value, baseToString) + '';
	    }
	    if (isSymbol(value)) {
	      return symbolToString ? symbolToString.call(value) : '';
	    }
	    var result = value + '';
	    return result == '0' && 1 / value == -INFINITY ? '-0' : result;
	  }

	  /**
	   * The base implementation of `_.slice` without an iteratee call guard.
	   *
	   * @private
	   * @param {Array} array The array to slice.
	   * @param {number} [start=0] The start position.
	   * @param {number} [end=array.length] The end position.
	   * @returns {Array} Returns the slice of `array`.
	   */
	  function baseSlice(array, start, end) {
	    var index = -1,
	      length = array.length;
	    if (start < 0) {
	      start = -start > length ? 0 : length + start;
	    }
	    end = end > length ? length : end;
	    if (end < 0) {
	      end += length;
	    }
	    length = start > end ? 0 : end - start >>> 0;
	    start >>>= 0;
	    var result = Array(length);
	    while (++index < length) {
	      result[index] = array[index + start];
	    }
	    return result;
	  }

	  /**
	   * Casts `array` to a slice if it's needed.
	   *
	   * @private
	   * @param {Array} array The array to inspect.
	   * @param {number} start The start position.
	   * @param {number} [end=array.length] The end position.
	   * @returns {Array} Returns the cast slice.
	   */
	  function castSlice(array, start, end) {
	    var length = array.length;
	    end = end === undefined ? length : end;
	    return !start && end >= length ? array : baseSlice(array, start, end);
	  }

	  /**
	   * Used by `_.trim` and `_.trimEnd` to get the index of the last string symbol
	   * that is not found in the character symbols.
	   *
	   * @private
	   * @param {Array} strSymbols The string symbols to inspect.
	   * @param {Array} chrSymbols The character symbols to find.
	   * @returns {number} Returns the index of the last unmatched string symbol.
	   */
	  function charsEndIndex(strSymbols, chrSymbols) {
	    var index = strSymbols.length;
	    while (index-- && baseIndexOf(chrSymbols, strSymbols[index], 0) > -1) {}
	    return index;
	  }

	  /**
	   * Used by `_.trim` and `_.trimStart` to get the index of the first string symbol
	   * that is not found in the character symbols.
	   *
	   * @private
	   * @param {Array} strSymbols The string symbols to inspect.
	   * @param {Array} chrSymbols The character symbols to find.
	   * @returns {number} Returns the index of the first unmatched string symbol.
	   */
	  function charsStartIndex(strSymbols, chrSymbols) {
	    var index = -1,
	      length = strSymbols.length;
	    while (++index < length && baseIndexOf(chrSymbols, strSymbols[index], 0) > -1) {}
	    return index;
	  }

	  /**
	   * Converts an ASCII `string` to an array.
	   *
	   * @private
	   * @param {string} string The string to convert.
	   * @returns {Array} Returns the converted array.
	   */
	  function asciiToArray(string) {
	    return string.split('');
	  }

	  /** Used to compose unicode character classes. */
	  var rsAstralRange = '\\ud800-\\udfff';
	  var rsComboMarksRange = '\\u0300-\\u036f';
	  var reComboHalfMarksRange = '\\ufe20-\\ufe2f';
	  var rsComboSymbolsRange = '\\u20d0-\\u20ff';
	  var rsComboRange = rsComboMarksRange + reComboHalfMarksRange + rsComboSymbolsRange;
	  var rsVarRange = '\\ufe0e\\ufe0f';

	  /** Used to compose unicode capture groups. */
	  var rsZWJ = '\\u200d';

	  /** Used to detect strings with [zero-width joiners or code points from the astral planes](http://eev.ee/blog/2015/09/12/dark-corners-of-unicode/). */
	  var reHasUnicode = RegExp('[' + rsZWJ + rsAstralRange + rsComboRange + rsVarRange + ']');

	  /**
	   * Checks if `string` contains Unicode symbols.
	   *
	   * @private
	   * @param {string} string The string to inspect.
	   * @returns {boolean} Returns `true` if a symbol is found, else `false`.
	   */
	  function hasUnicode(string) {
	    return reHasUnicode.test(string);
	  }

	  /** Used to compose unicode character classes. */
	  var rsAstralRange$1 = '\\ud800-\\udfff';
	  var rsComboMarksRange$1 = '\\u0300-\\u036f';
	  var reComboHalfMarksRange$1 = '\\ufe20-\\ufe2f';
	  var rsComboSymbolsRange$1 = '\\u20d0-\\u20ff';
	  var rsComboRange$1 = rsComboMarksRange$1 + reComboHalfMarksRange$1 + rsComboSymbolsRange$1;
	  var rsVarRange$1 = '\\ufe0e\\ufe0f';

	  /** Used to compose unicode capture groups. */
	  var rsAstral = '[' + rsAstralRange$1 + ']';
	  var rsCombo = '[' + rsComboRange$1 + ']';
	  var rsFitz = '\\ud83c[\\udffb-\\udfff]';
	  var rsModifier = '(?:' + rsCombo + '|' + rsFitz + ')';
	  var rsNonAstral = '[^' + rsAstralRange$1 + ']';
	  var rsRegional = '(?:\\ud83c[\\udde6-\\uddff]){2}';
	  var rsSurrPair = '[\\ud800-\\udbff][\\udc00-\\udfff]';
	  var rsZWJ$1 = '\\u200d';

	  /** Used to compose unicode regexes. */
	  var reOptMod = rsModifier + '?';
	  var rsOptVar = '[' + rsVarRange$1 + ']?';
	  var rsOptJoin = '(?:' + rsZWJ$1 + '(?:' + [rsNonAstral, rsRegional, rsSurrPair].join('|') + ')' + rsOptVar + reOptMod + ')*';
	  var rsSeq = rsOptVar + reOptMod + rsOptJoin;
	  var rsSymbol = '(?:' + [rsNonAstral + rsCombo + '?', rsCombo, rsRegional, rsSurrPair, rsAstral].join('|') + ')';

	  /** Used to match [string symbols](https://mathiasbynens.be/notes/javascript-unicode). */
	  var reUnicode = RegExp(rsFitz + '(?=' + rsFitz + ')|' + rsSymbol + rsSeq, 'g');

	  /**
	   * Converts a Unicode `string` to an array.
	   *
	   * @private
	   * @param {string} string The string to convert.
	   * @returns {Array} Returns the converted array.
	   */
	  function unicodeToArray(string) {
	    return string.match(reUnicode) || [];
	  }

	  /**
	   * Converts `string` to an array.
	   *
	   * @private
	   * @param {string} string The string to convert.
	   * @returns {Array} Returns the converted array.
	   */
	  function stringToArray(string) {
	    return hasUnicode(string) ? unicodeToArray(string) : asciiToArray(string);
	  }

	  /**
	   * Converts `value` to a string. An empty string is returned for `null`
	   * and `undefined` values. The sign of `-0` is preserved.
	   *
	   * @static
	   * @memberOf _
	   * @since 4.0.0
	   * @category Lang
	   * @param {*} value The value to convert.
	   * @returns {string} Returns the converted string.
	   * @example
	   *
	   * _.toString(null);
	   * // => ''
	   *
	   * _.toString(-0);
	   * // => '-0'
	   *
	   * _.toString([1, 2, 3]);
	   * // => '1,2,3'
	   */
	  function toString(value) {
	    return value == null ? '' : baseToString(value);
	  }

	  /** Used to match leading and trailing whitespace. */
	  var reTrim = /^\s+|\s+$/g;

	  /**
	   * Removes leading and trailing whitespace or specified characters from `string`.
	   *
	   * @static
	   * @memberOf _
	   * @since 3.0.0
	   * @category String
	   * @param {string} [string=''] The string to trim.
	   * @param {string} [chars=whitespace] The characters to trim.
	   * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
	   * @returns {string} Returns the trimmed string.
	   * @example
	   *
	   * _.trim('  abc  ');
	   * // => 'abc'
	   *
	   * _.trim('-_-abc-_-', '_-');
	   * // => 'abc'
	   *
	   * _.map(['  foo  ', '  bar  '], _.trim);
	   * // => ['foo', 'bar']
	   */
	  function trim(string, chars, guard) {
	    string = toString(string);
	    if (string && (guard || chars === undefined)) {
	      return string.replace(reTrim, '');
	    }
	    if (!string || !(chars = baseToString(chars))) {
	      return string;
	    }
	    var strSymbols = stringToArray(string),
	      chrSymbols = stringToArray(chars),
	      start = charsStartIndex(strSymbols, chrSymbols),
	      end = charsEndIndex(strSymbols, chrSymbols) + 1;
	    return castSlice(strSymbols, start, end).join('');
	  }
	  var FN_ARGS = /^(?:async\s+)?(function)?\s*[^\(]*\(\s*([^\)]*)\)/m;
	  var FN_ARG_SPLIT = /,/;
	  var FN_ARG = /(=.+)?(\s*)$/;
	  var STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
	  function parseParams(func) {
	    func = func.toString().replace(STRIP_COMMENTS, '');
	    func = func.match(FN_ARGS)[2].replace(' ', '');
	    func = func ? func.split(FN_ARG_SPLIT) : [];
	    func = func.map(function (arg) {
	      return trim(arg.replace(FN_ARG, ''));
	    });
	    return func;
	  }

	  /**
	   * A dependency-injected version of the [async.auto]{@link module:ControlFlow.auto} function. Dependent
	   * tasks are specified as parameters to the function, after the usual callback
	   * parameter, with the parameter names matching the names of the tasks it
	   * depends on. This can provide even more readable task graphs which can be
	   * easier to maintain.
	   *
	   * If a final callback is specified, the task results are similarly injected,
	   * specified as named parameters after the initial error parameter.
	   *
	   * The autoInject function is purely syntactic sugar and its semantics are
	   * otherwise equivalent to [async.auto]{@link module:ControlFlow.auto}.
	   *
	   * @name autoInject
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.auto]{@link module:ControlFlow.auto}
	   * @category Control Flow
	   * @param {Object} tasks - An object, each of whose properties is an {@link AsyncFunction} of
	   * the form 'func([dependencies...], callback). The object's key of a property
	   * serves as the name of the task defined by that property, i.e. can be used
	   * when specifying requirements for other tasks.
	   * * The `callback` parameter is a `callback(err, result)` which must be called
	   *   when finished, passing an `error` (which can be `null`) and the result of
	   *   the function's execution. The remaining parameters name other tasks on
	   *   which the task is dependent, and the results from those tasks are the
	   *   arguments of those parameters.
	   * @param {Function} [callback] - An optional callback which is called when all
	   * the tasks have been completed. It receives the `err` argument if any `tasks`
	   * pass an error to their callback, and a `results` object with any completed
	   * task results, similar to `auto`.
	   * @example
	   *
	   * //  The example from `auto` can be rewritten as follows:
	   * async.autoInject({
	   *     get_data: function(callback) {
	   *         // async code to get some data
	   *         callback(null, 'data', 'converted to array');
	   *     },
	   *     make_folder: function(callback) {
	   *         // async code to create a directory to store a file in
	   *         // this is run at the same time as getting the data
	   *         callback(null, 'folder');
	   *     },
	   *     write_file: function(get_data, make_folder, callback) {
	   *         // once there is some data and the directory exists,
	   *         // write the data to a file in the directory
	   *         callback(null, 'filename');
	   *     },
	   *     email_link: function(write_file, callback) {
	   *         // once the file is written let's email a link to it...
	   *         // write_file contains the filename returned by write_file.
	   *         callback(null, {'file':write_file, 'email':'user@example.com'});
	   *     }
	   * }, function(err, results) {
	   *     console.log('err = ', err);
	   *     console.log('email_link = ', results.email_link);
	   * });
	   *
	   * // If you are using a JS minifier that mangles parameter names, `autoInject`
	   * // will not work with plain functions, since the parameter names will be
	   * // collapsed to a single letter identifier.  To work around this, you can
	   * // explicitly specify the names of the parameters your task function needs
	   * // in an array, similar to Angular.js dependency injection.
	   *
	   * // This still has an advantage over plain `auto`, since the results a task
	   * // depends on are still spread into arguments.
	   * async.autoInject({
	   *     //...
	   *     write_file: ['get_data', 'make_folder', function(get_data, make_folder, callback) {
	   *         callback(null, 'filename');
	   *     }],
	   *     email_link: ['write_file', function(write_file, callback) {
	   *         callback(null, {'file':write_file, 'email':'user@example.com'});
	   *     }]
	   *     //...
	   * }, function(err, results) {
	   *     console.log('err = ', err);
	   *     console.log('email_link = ', results.email_link);
	   * });
	   */
	  function autoInject(tasks, callback) {
	    var newTasks = {};
	    baseForOwn(tasks, function (taskFn, key) {
	      var params;
	      var fnIsAsync = isAsync(taskFn);
	      var hasNoDeps = !fnIsAsync && taskFn.length === 1 || fnIsAsync && taskFn.length === 0;
	      if (isArray(taskFn)) {
	        params = taskFn.slice(0, -1);
	        taskFn = taskFn[taskFn.length - 1];
	        newTasks[key] = params.concat(params.length > 0 ? newTask : taskFn);
	      } else if (hasNoDeps) {
	        // no dependencies, use the function as-is
	        newTasks[key] = taskFn;
	      } else {
	        params = parseParams(taskFn);
	        if (taskFn.length === 0 && !fnIsAsync && params.length === 0) {
	          throw new Error("autoInject task functions require explicit parameters.");
	        }

	        // remove callback param
	        if (!fnIsAsync) params.pop();
	        newTasks[key] = params.concat(newTask);
	      }
	      function newTask(results, taskCb) {
	        var newArgs = arrayMap(params, function (name) {
	          return results[name];
	        });
	        newArgs.push(taskCb);
	        wrapAsync(taskFn).apply(null, newArgs);
	      }
	    });
	    auto(newTasks, callback);
	  }

	  // Simple doubly linked list (https://en.wikipedia.org/wiki/Doubly_linked_list) implementation
	  // used for queues. This implementation assumes that the node provided by the user can be modified
	  // to adjust the next and last properties. We implement only the minimal functionality
	  // for queue support.
	  function DLL() {
	    this.head = this.tail = null;
	    this.length = 0;
	  }
	  function setInitial(dll, node) {
	    dll.length = 1;
	    dll.head = dll.tail = node;
	  }
	  DLL.prototype.removeLink = function (node) {
	    if (node.prev) node.prev.next = node.next;else this.head = node.next;
	    if (node.next) node.next.prev = node.prev;else this.tail = node.prev;
	    node.prev = node.next = null;
	    this.length -= 1;
	    return node;
	  };
	  DLL.prototype.empty = function () {
	    while (this.head) this.shift();
	    return this;
	  };
	  DLL.prototype.insertAfter = function (node, newNode) {
	    newNode.prev = node;
	    newNode.next = node.next;
	    if (node.next) node.next.prev = newNode;else this.tail = newNode;
	    node.next = newNode;
	    this.length += 1;
	  };
	  DLL.prototype.insertBefore = function (node, newNode) {
	    newNode.prev = node.prev;
	    newNode.next = node;
	    if (node.prev) node.prev.next = newNode;else this.head = newNode;
	    node.prev = newNode;
	    this.length += 1;
	  };
	  DLL.prototype.unshift = function (node) {
	    if (this.head) this.insertBefore(this.head, node);else setInitial(this, node);
	  };
	  DLL.prototype.push = function (node) {
	    if (this.tail) this.insertAfter(this.tail, node);else setInitial(this, node);
	  };
	  DLL.prototype.shift = function () {
	    return this.head && this.removeLink(this.head);
	  };
	  DLL.prototype.pop = function () {
	    return this.tail && this.removeLink(this.tail);
	  };
	  DLL.prototype.toArray = function () {
	    var arr = Array(this.length);
	    var curr = this.head;
	    for (var idx = 0; idx < this.length; idx++) {
	      arr[idx] = curr.data;
	      curr = curr.next;
	    }
	    return arr;
	  };
	  DLL.prototype.remove = function (testFn) {
	    var curr = this.head;
	    while (!!curr) {
	      var next = curr.next;
	      if (testFn(curr)) {
	        this.removeLink(curr);
	      }
	      curr = next;
	    }
	    return this;
	  };
	  function queue(worker, concurrency, payload) {
	    if (concurrency == null) {
	      concurrency = 1;
	    } else if (concurrency === 0) {
	      throw new Error('Concurrency must not be zero');
	    }
	    var _worker = wrapAsync(worker);
	    var numRunning = 0;
	    var workersList = [];
	    var processingScheduled = false;
	    function _insert(data, insertAtFront, callback) {
	      if (callback != null && typeof callback !== 'function') {
	        throw new Error('task callback must be a function');
	      }
	      q.started = true;
	      if (!isArray(data)) {
	        data = [data];
	      }
	      if (data.length === 0 && q.idle()) {
	        // call drain immediately if there are no tasks
	        return setImmediate$1(function () {
	          q.drain();
	        });
	      }
	      for (var i = 0, l = data.length; i < l; i++) {
	        var item = {
	          data: data[i],
	          callback: callback || noop
	        };
	        if (insertAtFront) {
	          q._tasks.unshift(item);
	        } else {
	          q._tasks.push(item);
	        }
	      }
	      if (!processingScheduled) {
	        processingScheduled = true;
	        setImmediate$1(function () {
	          processingScheduled = false;
	          q.process();
	        });
	      }
	    }
	    function _next(tasks) {
	      return function (err) {
	        numRunning -= 1;
	        for (var i = 0, l = tasks.length; i < l; i++) {
	          var task = tasks[i];
	          var index = baseIndexOf(workersList, task, 0);
	          if (index === 0) {
	            workersList.shift();
	          } else if (index > 0) {
	            workersList.splice(index, 1);
	          }
	          task.callback.apply(task, arguments);
	          if (err != null) {
	            q.error(err, task.data);
	          }
	        }
	        if (numRunning <= q.concurrency - q.buffer) {
	          q.unsaturated();
	        }
	        if (q.idle()) {
	          q.drain();
	        }
	        q.process();
	      };
	    }
	    var isProcessing = false;
	    var q = {
	      _tasks: new DLL(),
	      concurrency: concurrency,
	      payload: payload,
	      saturated: noop,
	      unsaturated: noop,
	      buffer: concurrency / 4,
	      empty: noop,
	      drain: noop,
	      error: noop,
	      started: false,
	      paused: false,
	      push: function (data, callback) {
	        _insert(data, false, callback);
	      },
	      kill: function () {
	        q.drain = noop;
	        q._tasks.empty();
	      },
	      unshift: function (data, callback) {
	        _insert(data, true, callback);
	      },
	      remove: function (testFn) {
	        q._tasks.remove(testFn);
	      },
	      process: function () {
	        // Avoid trying to start too many processing operations. This can occur
	        // when callbacks resolve synchronously (#1267).
	        if (isProcessing) {
	          return;
	        }
	        isProcessing = true;
	        while (!q.paused && numRunning < q.concurrency && q._tasks.length) {
	          var tasks = [],
	            data = [];
	          var l = q._tasks.length;
	          if (q.payload) l = Math.min(l, q.payload);
	          for (var i = 0; i < l; i++) {
	            var node = q._tasks.shift();
	            tasks.push(node);
	            workersList.push(node);
	            data.push(node.data);
	          }
	          numRunning += 1;
	          if (q._tasks.length === 0) {
	            q.empty();
	          }
	          if (numRunning === q.concurrency) {
	            q.saturated();
	          }
	          var cb = onlyOnce(_next(tasks));
	          _worker(data, cb);
	        }
	        isProcessing = false;
	      },
	      length: function () {
	        return q._tasks.length;
	      },
	      running: function () {
	        return numRunning;
	      },
	      workersList: function () {
	        return workersList;
	      },
	      idle: function () {
	        return q._tasks.length + numRunning === 0;
	      },
	      pause: function () {
	        q.paused = true;
	      },
	      resume: function () {
	        if (q.paused === false) {
	          return;
	        }
	        q.paused = false;
	        setImmediate$1(q.process);
	      }
	    };
	    return q;
	  }

	  /**
	   * A cargo of tasks for the worker function to complete. Cargo inherits all of
	   * the same methods and event callbacks as [`queue`]{@link module:ControlFlow.queue}.
	   * @typedef {Object} CargoObject
	   * @memberOf module:ControlFlow
	   * @property {Function} length - A function returning the number of items
	   * waiting to be processed. Invoke like `cargo.length()`.
	   * @property {number} payload - An `integer` for determining how many tasks
	   * should be process per round. This property can be changed after a `cargo` is
	   * created to alter the payload on-the-fly.
	   * @property {Function} push - Adds `task` to the `queue`. The callback is
	   * called once the `worker` has finished processing the task. Instead of a
	   * single task, an array of `tasks` can be submitted. The respective callback is
	   * used for every task in the list. Invoke like `cargo.push(task, [callback])`.
	   * @property {Function} saturated - A callback that is called when the
	   * `queue.length()` hits the concurrency and further tasks will be queued.
	   * @property {Function} empty - A callback that is called when the last item
	   * from the `queue` is given to a `worker`.
	   * @property {Function} drain - A callback that is called when the last item
	   * from the `queue` has returned from the `worker`.
	   * @property {Function} idle - a function returning false if there are items
	   * waiting or being processed, or true if not. Invoke like `cargo.idle()`.
	   * @property {Function} pause - a function that pauses the processing of tasks
	   * until `resume()` is called. Invoke like `cargo.pause()`.
	   * @property {Function} resume - a function that resumes the processing of
	   * queued tasks when the queue is paused. Invoke like `cargo.resume()`.
	   * @property {Function} kill - a function that removes the `drain` callback and
	   * empties remaining tasks from the queue forcing it to go idle. Invoke like `cargo.kill()`.
	   */

	  /**
	   * Creates a `cargo` object with the specified payload. Tasks added to the
	   * cargo will be processed altogether (up to the `payload` limit). If the
	   * `worker` is in progress, the task is queued until it becomes available. Once
	   * the `worker` has completed some tasks, each callback of those tasks is
	   * called. Check out [these](https://camo.githubusercontent.com/6bbd36f4cf5b35a0f11a96dcd2e97711ffc2fb37/68747470733a2f2f662e636c6f75642e6769746875622e636f6d2f6173736574732f313637363837312f36383130382f62626330636662302d356632392d313165322d393734662d3333393763363464633835382e676966) [animations](https://camo.githubusercontent.com/f4810e00e1c5f5f8addbe3e9f49064fd5d102699/68747470733a2f2f662e636c6f75642e6769746875622e636f6d2f6173736574732f313637363837312f36383130312f38346339323036362d356632392d313165322d383134662d3964336430323431336266642e676966)
	   * for how `cargo` and `queue` work.
	   *
	   * While [`queue`]{@link module:ControlFlow.queue} passes only one task to one of a group of workers
	   * at a time, cargo passes an array of tasks to a single worker, repeating
	   * when the worker is finished.
	   *
	   * @name cargo
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.queue]{@link module:ControlFlow.queue}
	   * @category Control Flow
	   * @param {AsyncFunction} worker - An asynchronous function for processing an array
	   * of queued tasks. Invoked with `(tasks, callback)`.
	   * @param {number} [payload=Infinity] - An optional `integer` for determining
	   * how many tasks should be processed per round; if omitted, the default is
	   * unlimited.
	   * @returns {module:ControlFlow.CargoObject} A cargo object to manage the tasks. Callbacks can
	   * attached as certain properties to listen for specific events during the
	   * lifecycle of the cargo and inner queue.
	   * @example
	   *
	   * // create a cargo object with payload 2
	   * var cargo = async.cargo(function(tasks, callback) {
	   *     for (var i=0; i<tasks.length; i++) {
	   *         console.log('hello ' + tasks[i].name);
	   *     }
	   *     callback();
	   * }, 2);
	   *
	   * // add some items
	   * cargo.push({name: 'foo'}, function(err) {
	   *     console.log('finished processing foo');
	   * });
	   * cargo.push({name: 'bar'}, function(err) {
	   *     console.log('finished processing bar');
	   * });
	   * cargo.push({name: 'baz'}, function(err) {
	   *     console.log('finished processing baz');
	   * });
	   */
	  function cargo(worker, payload) {
	    return queue(worker, 1, payload);
	  }

	  /**
	   * The same as [`eachOf`]{@link module:Collections.eachOf} but runs only a single async operation at a time.
	   *
	   * @name eachOfSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.eachOf]{@link module:Collections.eachOf}
	   * @alias forEachOfSeries
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * Invoked with (item, key, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. Invoked with (err).
	   */
	  var eachOfSeries = doLimit(eachOfLimit, 1);

	  /**
	   * Reduces `coll` into a single value using an async `iteratee` to return each
	   * successive step. `memo` is the initial state of the reduction. This function
	   * only operates in series.
	   *
	   * For performance reasons, it may make sense to split a call to this function
	   * into a parallel map, and then use the normal `Array.prototype.reduce` on the
	   * results. This function is for situations where each step in the reduction
	   * needs to be async; if you can get the data before reducing it, then it's
	   * probably a good idea to do so.
	   *
	   * @name reduce
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @alias inject
	   * @alias foldl
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {*} memo - The initial state of the reduction.
	   * @param {AsyncFunction} iteratee - A function applied to each item in the
	   * array to produce the next step in the reduction.
	   * The `iteratee` should complete with the next state of the reduction.
	   * If the iteratee complete with an error, the reduction is stopped and the
	   * main `callback` is immediately called with the error.
	   * Invoked with (memo, item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Result is the reduced value. Invoked with
	   * (err, result).
	   * @example
	   *
	   * async.reduce([1,2,3], 0, function(memo, item, callback) {
	   *     // pointless async:
	   *     process.nextTick(function() {
	   *         callback(null, memo + item)
	   *     });
	   * }, function(err, result) {
	   *     // result is now equal to the last value of memo, which is 6
	   * });
	   */
	  function reduce(coll, memo, iteratee, callback) {
	    callback = once(callback || noop);
	    var _iteratee = wrapAsync(iteratee);
	    eachOfSeries(coll, function (x, i, callback) {
	      _iteratee(memo, x, function (err, v) {
	        memo = v;
	        callback(err);
	      });
	    }, function (err) {
	      callback(err, memo);
	    });
	  }

	  /**
	   * Version of the compose function that is more natural to read. Each function
	   * consumes the return value of the previous function. It is the equivalent of
	   * [compose]{@link module:ControlFlow.compose} with the arguments reversed.
	   *
	   * Each function is executed with the `this` binding of the composed function.
	   *
	   * @name seq
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.compose]{@link module:ControlFlow.compose}
	   * @category Control Flow
	   * @param {...AsyncFunction} functions - the asynchronous functions to compose
	   * @returns {Function} a function that composes the `functions` in order
	   * @example
	   *
	   * // Requires lodash (or underscore), express3 and dresende's orm2.
	   * // Part of an app, that fetches cats of the logged user.
	   * // This example uses `seq` function to avoid overnesting and error
	   * // handling clutter.
	   * app.get('/cats', function(request, response) {
	   *     var User = request.models.User;
	   *     async.seq(
	   *         _.bind(User.get, User),  // 'User.get' has signature (id, callback(err, data))
	   *         function(user, fn) {
	   *             user.getCats(fn);      // 'getCats' has signature (callback(err, data))
	   *         }
	   *     )(req.session.user_id, function (err, cats) {
	   *         if (err) {
	   *             console.error(err);
	   *             response.json({ status: 'error', message: err.message });
	   *         } else {
	   *             response.json({ status: 'ok', message: 'Cats found', data: cats });
	   *         }
	   *     });
	   * });
	   */
	  function seq( /*...functions*/
	  ) {
	    var _functions = arrayMap(arguments, wrapAsync);
	    return function /*...args*/
	    () {
	      var args = slice(arguments);
	      var that = this;
	      var cb = args[args.length - 1];
	      if (typeof cb == 'function') {
	        args.pop();
	      } else {
	        cb = noop;
	      }
	      reduce(_functions, args, function (newargs, fn, cb) {
	        fn.apply(that, newargs.concat(function (err /*, ...nextargs*/) {
	          var nextargs = slice(arguments, 1);
	          cb(err, nextargs);
	        }));
	      }, function (err, results) {
	        cb.apply(that, [err].concat(results));
	      });
	    };
	  }

	  /**
	   * Creates a function which is a composition of the passed asynchronous
	   * functions. Each function consumes the return value of the function that
	   * follows. Composing functions `f()`, `g()`, and `h()` would produce the result
	   * of `f(g(h()))`, only this version uses callbacks to obtain the return values.
	   *
	   * Each function is executed with the `this` binding of the composed function.
	   *
	   * @name compose
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {...AsyncFunction} functions - the asynchronous functions to compose
	   * @returns {Function} an asynchronous function that is the composed
	   * asynchronous `functions`
	   * @example
	   *
	   * function add1(n, callback) {
	   *     setTimeout(function () {
	   *         callback(null, n + 1);
	   *     }, 10);
	   * }
	   *
	   * function mul3(n, callback) {
	   *     setTimeout(function () {
	   *         callback(null, n * 3);
	   *     }, 10);
	   * }
	   *
	   * var add1mul3 = async.compose(mul3, add1);
	   * add1mul3(4, function (err, result) {
	   *     // result now equals 15
	   * });
	   */
	  var compose = function /*...args*/
	  () {
	    return seq.apply(null, slice(arguments).reverse());
	  };
	  var _concat = Array.prototype.concat;

	  /**
	   * The same as [`concat`]{@link module:Collections.concat} but runs a maximum of `limit` async operations at a time.
	   *
	   * @name concatLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.concat]{@link module:Collections.concat}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - A function to apply to each item in `coll`,
	   * which should use an array as its result. Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished, or an error occurs. Results is an array
	   * containing the concatenated results of the `iteratee` function. Invoked with
	   * (err, results).
	   */
	  var concatLimit = function (coll, limit, iteratee, callback) {
	    callback = callback || noop;
	    var _iteratee = wrapAsync(iteratee);
	    mapLimit(coll, limit, function (val, callback) {
	      _iteratee(val, function (err /*, ...args*/) {
	        if (err) return callback(err);
	        return callback(null, slice(arguments, 1));
	      });
	    }, function (err, mapResults) {
	      var result = [];
	      for (var i = 0; i < mapResults.length; i++) {
	        if (mapResults[i]) {
	          result = _concat.apply(result, mapResults[i]);
	        }
	      }
	      return callback(err, result);
	    });
	  };

	  /**
	   * Applies `iteratee` to each item in `coll`, concatenating the results. Returns
	   * the concatenated list. The `iteratee`s are called in parallel, and the
	   * results are concatenated as they return. There is no guarantee that the
	   * results array will be returned in the original order of `coll` passed to the
	   * `iteratee` function.
	   *
	   * @name concat
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - A function to apply to each item in `coll`,
	   * which should use an array as its result. Invoked with (item, callback).
	   * @param {Function} [callback(err)] - A callback which is called after all the
	   * `iteratee` functions have finished, or an error occurs. Results is an array
	   * containing the concatenated results of the `iteratee` function. Invoked with
	   * (err, results).
	   * @example
	   *
	   * async.concat(['dir1','dir2','dir3'], fs.readdir, function(err, files) {
	   *     // files is now a list of filenames that exist in the 3 directories
	   * });
	   */
	  var concat = doLimit(concatLimit, Infinity);

	  /**
	   * The same as [`concat`]{@link module:Collections.concat} but runs only a single async operation at a time.
	   *
	   * @name concatSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.concat]{@link module:Collections.concat}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - A function to apply to each item in `coll`.
	   * The iteratee should complete with an array an array of results.
	   * Invoked with (item, callback).
	   * @param {Function} [callback(err)] - A callback which is called after all the
	   * `iteratee` functions have finished, or an error occurs. Results is an array
	   * containing the concatenated results of the `iteratee` function. Invoked with
	   * (err, results).
	   */
	  var concatSeries = doLimit(concatLimit, 1);

	  /**
	   * Returns a function that when called, calls-back with the values provided.
	   * Useful as the first function in a [`waterfall`]{@link module:ControlFlow.waterfall}, or for plugging values in to
	   * [`auto`]{@link module:ControlFlow.auto}.
	   *
	   * @name constant
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @category Util
	   * @param {...*} arguments... - Any number of arguments to automatically invoke
	   * callback with.
	   * @returns {AsyncFunction} Returns a function that when invoked, automatically
	   * invokes the callback with the previous given arguments.
	   * @example
	   *
	   * async.waterfall([
	   *     async.constant(42),
	   *     function (value, next) {
	   *         // value === 42
	   *     },
	   *     //...
	   * ], callback);
	   *
	   * async.waterfall([
	   *     async.constant(filename, "utf8"),
	   *     fs.readFile,
	   *     function (fileData, next) {
	   *         //...
	   *     }
	   *     //...
	   * ], callback);
	   *
	   * async.auto({
	   *     hostname: async.constant("https://server.net/"),
	   *     port: findFreePort,
	   *     launchServer: ["hostname", "port", function (options, cb) {
	   *         startServer(options, cb);
	   *     }],
	   *     //...
	   * }, callback);
	   */
	  var constant = function /*...values*/
	  () {
	    var values = slice(arguments);
	    var args = [null].concat(values);
	    return function /*...ignoredArgs, callback*/
	    () {
	      var callback = arguments[arguments.length - 1];
	      return callback.apply(this, args);
	    };
	  };

	  /**
	   * This method returns the first argument it receives.
	   *
	   * @static
	   * @since 0.1.0
	   * @memberOf _
	   * @category Util
	   * @param {*} value Any value.
	   * @returns {*} Returns `value`.
	   * @example
	   *
	   * var object = { 'a': 1 };
	   *
	   * console.log(_.identity(object) === object);
	   * // => true
	   */
	  function identity(value) {
	    return value;
	  }
	  function _createTester(check, getResult) {
	    return function (eachfn, arr, iteratee, cb) {
	      cb = cb || noop;
	      var testPassed = false;
	      var testResult;
	      eachfn(arr, function (value, _, callback) {
	        iteratee(value, function (err, result) {
	          if (err) {
	            callback(err);
	          } else if (check(result) && !testResult) {
	            testPassed = true;
	            testResult = getResult(true, value);
	            callback(null, breakLoop);
	          } else {
	            callback();
	          }
	        });
	      }, function (err) {
	        if (err) {
	          cb(err);
	        } else {
	          cb(null, testPassed ? testResult : getResult(false));
	        }
	      });
	    };
	  }
	  function _findGetResult(v, x) {
	    return x;
	  }

	  /**
	   * Returns the first value in `coll` that passes an async truth test. The
	   * `iteratee` is applied in parallel, meaning the first iteratee to return
	   * `true` will fire the detect `callback` with that result. That means the
	   * result might not be the first item in the original `coll` (in terms of order)
	   * that passes the test.
	  
	   * If order within the original `coll` is important, then look at
	   * [`detectSeries`]{@link module:Collections.detectSeries}.
	   *
	   * @name detect
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @alias find
	   * @category Collections
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - A truth test to apply to each item in `coll`.
	   * The iteratee must complete with a boolean value as its result.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called as soon as any
	   * iteratee returns `true`, or after all the `iteratee` functions have finished.
	   * Result will be the first item in the array that passes the truth test
	   * (iteratee) or the value `undefined` if none passed. Invoked with
	   * (err, result).
	   * @example
	   *
	   * async.detect(['file1','file2','file3'], function(filePath, callback) {
	   *     fs.access(filePath, function(err) {
	   *         callback(null, !err)
	   *     });
	   * }, function(err, result) {
	   *     // result now equals the first file in the list that exists
	   * });
	   */
	  var detect = doParallel(_createTester(identity, _findGetResult));

	  /**
	   * The same as [`detect`]{@link module:Collections.detect} but runs a maximum of `limit` async operations at a
	   * time.
	   *
	   * @name detectLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.detect]{@link module:Collections.detect}
	   * @alias findLimit
	   * @category Collections
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - A truth test to apply to each item in `coll`.
	   * The iteratee must complete with a boolean value as its result.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called as soon as any
	   * iteratee returns `true`, or after all the `iteratee` functions have finished.
	   * Result will be the first item in the array that passes the truth test
	   * (iteratee) or the value `undefined` if none passed. Invoked with
	   * (err, result).
	   */
	  var detectLimit = doParallelLimit(_createTester(identity, _findGetResult));

	  /**
	   * The same as [`detect`]{@link module:Collections.detect} but runs only a single async operation at a time.
	   *
	   * @name detectSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.detect]{@link module:Collections.detect}
	   * @alias findSeries
	   * @category Collections
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - A truth test to apply to each item in `coll`.
	   * The iteratee must complete with a boolean value as its result.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called as soon as any
	   * iteratee returns `true`, or after all the `iteratee` functions have finished.
	   * Result will be the first item in the array that passes the truth test
	   * (iteratee) or the value `undefined` if none passed. Invoked with
	   * (err, result).
	   */
	  var detectSeries = doLimit(detectLimit, 1);
	  function consoleFunc(name) {
	    return function (fn /*, ...args*/) {
	      var args = slice(arguments, 1);
	      args.push(function (err /*, ...args*/) {
	        var args = slice(arguments, 1);
	        if (typeof console === 'object') {
	          if (err) {
	            if (console.error) {
	              console.error(err);
	            }
	          } else if (console[name]) {
	            arrayEach(args, function (x) {
	              console[name](x);
	            });
	          }
	        }
	      });
	      wrapAsync(fn).apply(null, args);
	    };
	  }

	  /**
	   * Logs the result of an [`async` function]{@link AsyncFunction} to the
	   * `console` using `console.dir` to display the properties of the resulting object.
	   * Only works in Node.js or in browsers that support `console.dir` and
	   * `console.error` (such as FF and Chrome).
	   * If multiple arguments are returned from the async function,
	   * `console.dir` is called on each argument in order.
	   *
	   * @name dir
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @category Util
	   * @param {AsyncFunction} function - The function you want to eventually apply
	   * all arguments to.
	   * @param {...*} arguments... - Any number of arguments to apply to the function.
	   * @example
	   *
	   * // in a module
	   * var hello = function(name, callback) {
	   *     setTimeout(function() {
	   *         callback(null, {hello: name});
	   *     }, 1000);
	   * };
	   *
	   * // in the node repl
	   * node> async.dir(hello, 'world');
	   * {hello: 'world'}
	   */
	  var dir = consoleFunc('dir');

	  /**
	   * The post-check version of [`during`]{@link module:ControlFlow.during}. To reflect the difference in
	   * the order of operations, the arguments `test` and `fn` are switched.
	   *
	   * Also a version of [`doWhilst`]{@link module:ControlFlow.doWhilst} with asynchronous `test` function.
	   * @name doDuring
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.during]{@link module:ControlFlow.during}
	   * @category Control Flow
	   * @param {AsyncFunction} fn - An async function which is called each time
	   * `test` passes. Invoked with (callback).
	   * @param {AsyncFunction} test - asynchronous truth test to perform before each
	   * execution of `fn`. Invoked with (...args, callback), where `...args` are the
	   * non-error args from the previous callback of `fn`.
	   * @param {Function} [callback] - A callback which is called after the test
	   * function has failed and repeated execution of `fn` has stopped. `callback`
	   * will be passed an error if one occurred, otherwise `null`.
	   */
	  function doDuring(fn, test, callback) {
	    callback = onlyOnce(callback || noop);
	    var _fn = wrapAsync(fn);
	    var _test = wrapAsync(test);
	    function next(err /*, ...args*/) {
	      if (err) return callback(err);
	      var args = slice(arguments, 1);
	      args.push(check);
	      _test.apply(this, args);
	    }
	    function check(err, truth) {
	      if (err) return callback(err);
	      if (!truth) return callback(null);
	      _fn(next);
	    }
	    check(null, true);
	  }

	  /**
	   * The post-check version of [`whilst`]{@link module:ControlFlow.whilst}. To reflect the difference in
	   * the order of operations, the arguments `test` and `iteratee` are switched.
	   *
	   * `doWhilst` is to `whilst` as `do while` is to `while` in plain JavaScript.
	   *
	   * @name doWhilst
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.whilst]{@link module:ControlFlow.whilst}
	   * @category Control Flow
	   * @param {AsyncFunction} iteratee - A function which is called each time `test`
	   * passes. Invoked with (callback).
	   * @param {Function} test - synchronous truth test to perform after each
	   * execution of `iteratee`. Invoked with any non-error callback results of
	   * `iteratee`.
	   * @param {Function} [callback] - A callback which is called after the test
	   * function has failed and repeated execution of `iteratee` has stopped.
	   * `callback` will be passed an error and any arguments passed to the final
	   * `iteratee`'s callback. Invoked with (err, [results]);
	   */
	  function doWhilst(iteratee, test, callback) {
	    callback = onlyOnce(callback || noop);
	    var _iteratee = wrapAsync(iteratee);
	    var next = function (err /*, ...args*/) {
	      if (err) return callback(err);
	      var args = slice(arguments, 1);
	      if (test.apply(this, args)) return _iteratee(next);
	      callback.apply(null, [null].concat(args));
	    };
	    _iteratee(next);
	  }

	  /**
	   * Like ['doWhilst']{@link module:ControlFlow.doWhilst}, except the `test` is inverted. Note the
	   * argument ordering differs from `until`.
	   *
	   * @name doUntil
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.doWhilst]{@link module:ControlFlow.doWhilst}
	   * @category Control Flow
	   * @param {AsyncFunction} iteratee - An async function which is called each time
	   * `test` fails. Invoked with (callback).
	   * @param {Function} test - synchronous truth test to perform after each
	   * execution of `iteratee`. Invoked with any non-error callback results of
	   * `iteratee`.
	   * @param {Function} [callback] - A callback which is called after the test
	   * function has passed and repeated execution of `iteratee` has stopped. `callback`
	   * will be passed an error and any arguments passed to the final `iteratee`'s
	   * callback. Invoked with (err, [results]);
	   */
	  function doUntil(iteratee, test, callback) {
	    doWhilst(iteratee, function () {
	      return !test.apply(this, arguments);
	    }, callback);
	  }

	  /**
	   * Like [`whilst`]{@link module:ControlFlow.whilst}, except the `test` is an asynchronous function that
	   * is passed a callback in the form of `function (err, truth)`. If error is
	   * passed to `test` or `fn`, the main callback is immediately called with the
	   * value of the error.
	   *
	   * @name during
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.whilst]{@link module:ControlFlow.whilst}
	   * @category Control Flow
	   * @param {AsyncFunction} test - asynchronous truth test to perform before each
	   * execution of `fn`. Invoked with (callback).
	   * @param {AsyncFunction} fn - An async function which is called each time
	   * `test` passes. Invoked with (callback).
	   * @param {Function} [callback] - A callback which is called after the test
	   * function has failed and repeated execution of `fn` has stopped. `callback`
	   * will be passed an error, if one occurred, otherwise `null`.
	   * @example
	   *
	   * var count = 0;
	   *
	   * async.during(
	   *     function (callback) {
	   *         return callback(null, count < 5);
	   *     },
	   *     function (callback) {
	   *         count++;
	   *         setTimeout(callback, 1000);
	   *     },
	   *     function (err) {
	   *         // 5 seconds have passed
	   *     }
	   * );
	   */
	  function during(test, fn, callback) {
	    callback = onlyOnce(callback || noop);
	    var _fn = wrapAsync(fn);
	    var _test = wrapAsync(test);
	    function next(err) {
	      if (err) return callback(err);
	      _test(check);
	    }
	    function check(err, truth) {
	      if (err) return callback(err);
	      if (!truth) return callback(null);
	      _fn(next);
	    }
	    _test(check);
	  }
	  function _withoutIndex(iteratee) {
	    return function (value, index, callback) {
	      return iteratee(value, callback);
	    };
	  }

	  /**
	   * Applies the function `iteratee` to each item in `coll`, in parallel.
	   * The `iteratee` is called with an item from the list, and a callback for when
	   * it has finished. If the `iteratee` passes an error to its `callback`, the
	   * main `callback` (for the `each` function) is immediately called with the
	   * error.
	   *
	   * Note, that since this function applies `iteratee` to each item in parallel,
	   * there is no guarantee that the iteratee functions will complete in order.
	   *
	   * @name each
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @alias forEach
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async function to apply to
	   * each item in `coll`. Invoked with (item, callback).
	   * The array index is not passed to the iteratee.
	   * If you need the index, use `eachOf`.
	   * @param {Function} [callback] - A callback which is called when all
	   * `iteratee` functions have finished, or an error occurs. Invoked with (err).
	   * @example
	   *
	   * // assuming openFiles is an array of file names and saveFile is a function
	   * // to save the modified contents of that file:
	   *
	   * async.each(openFiles, saveFile, function(err){
	   *   // if any of the saves produced an error, err would equal that error
	   * });
	   *
	   * // assuming openFiles is an array of file names
	   * async.each(openFiles, function(file, callback) {
	   *
	   *     // Perform operation on file here.
	   *     console.log('Processing file ' + file);
	   *
	   *     if( file.length > 32 ) {
	   *       console.log('This file name is too long');
	   *       callback('File name too long');
	   *     } else {
	   *       // Do work to process file here
	   *       console.log('File processed');
	   *       callback();
	   *     }
	   * }, function(err) {
	   *     // if any of the file processing produced an error, err would equal that error
	   *     if( err ) {
	   *       // One of the iterations produced an error.
	   *       // All processing will now stop.
	   *       console.log('A file failed to process');
	   *     } else {
	   *       console.log('All files have been processed successfully');
	   *     }
	   * });
	   */
	  function eachLimit(coll, iteratee, callback) {
	    eachOf(coll, _withoutIndex(wrapAsync(iteratee)), callback);
	  }

	  /**
	   * The same as [`each`]{@link module:Collections.each} but runs a maximum of `limit` async operations at a time.
	   *
	   * @name eachLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.each]{@link module:Collections.each}
	   * @alias forEachLimit
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * The array index is not passed to the iteratee.
	   * If you need the index, use `eachOfLimit`.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called when all
	   * `iteratee` functions have finished, or an error occurs. Invoked with (err).
	   */
	  function eachLimit$1(coll, limit, iteratee, callback) {
	    _eachOfLimit(limit)(coll, _withoutIndex(wrapAsync(iteratee)), callback);
	  }

	  /**
	   * The same as [`each`]{@link module:Collections.each} but runs only a single async operation at a time.
	   *
	   * @name eachSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.each]{@link module:Collections.each}
	   * @alias forEachSeries
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async function to apply to each
	   * item in `coll`.
	   * The array index is not passed to the iteratee.
	   * If you need the index, use `eachOfSeries`.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called when all
	   * `iteratee` functions have finished, or an error occurs. Invoked with (err).
	   */
	  var eachSeries = doLimit(eachLimit$1, 1);

	  /**
	   * Wrap an async function and ensure it calls its callback on a later tick of
	   * the event loop.  If the function already calls its callback on a next tick,
	   * no extra deferral is added. This is useful for preventing stack overflows
	   * (`RangeError: Maximum call stack size exceeded`) and generally keeping
	   * [Zalgo](http://blog.izs.me/post/59142742143/designing-apis-for-asynchrony)
	   * contained. ES2017 `async` functions are returned as-is -- they are immune
	   * to Zalgo's corrupting influences, as they always resolve on a later tick.
	   *
	   * @name ensureAsync
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @category Util
	   * @param {AsyncFunction} fn - an async function, one that expects a node-style
	   * callback as its last argument.
	   * @returns {AsyncFunction} Returns a wrapped function with the exact same call
	   * signature as the function passed in.
	   * @example
	   *
	   * function sometimesAsync(arg, callback) {
	   *     if (cache[arg]) {
	   *         return callback(null, cache[arg]); // this would be synchronous!!
	   *     } else {
	   *         doSomeIO(arg, callback); // this IO would be asynchronous
	   *     }
	   * }
	   *
	   * // this has a risk of stack overflows if many results are cached in a row
	   * async.mapSeries(args, sometimesAsync, done);
	   *
	   * // this will defer sometimesAsync's callback if necessary,
	   * // preventing stack overflows
	   * async.mapSeries(args, async.ensureAsync(sometimesAsync), done);
	   */
	  function ensureAsync(fn) {
	    if (isAsync(fn)) return fn;
	    return initialParams(function (args, callback) {
	      var sync = true;
	      args.push(function () {
	        var innerArgs = arguments;
	        if (sync) {
	          setImmediate$1(function () {
	            callback.apply(null, innerArgs);
	          });
	        } else {
	          callback.apply(null, innerArgs);
	        }
	      });
	      fn.apply(this, args);
	      sync = false;
	    });
	  }
	  function notId(v) {
	    return !v;
	  }

	  /**
	   * Returns `true` if every element in `coll` satisfies an async test. If any
	   * iteratee call returns `false`, the main `callback` is immediately called.
	   *
	   * @name every
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @alias all
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async truth test to apply to each item
	   * in the collection in parallel.
	   * The iteratee must complete with a boolean result value.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Result will be either `true` or `false`
	   * depending on the values of the async tests. Invoked with (err, result).
	   * @example
	   *
	   * async.every(['file1','file2','file3'], function(filePath, callback) {
	   *     fs.access(filePath, function(err) {
	   *         callback(null, !err)
	   *     });
	   * }, function(err, result) {
	   *     // if result is true then every file exists
	   * });
	   */
	  var every = doParallel(_createTester(notId, notId));

	  /**
	   * The same as [`every`]{@link module:Collections.every} but runs a maximum of `limit` async operations at a time.
	   *
	   * @name everyLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.every]{@link module:Collections.every}
	   * @alias allLimit
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - An async truth test to apply to each item
	   * in the collection in parallel.
	   * The iteratee must complete with a boolean result value.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Result will be either `true` or `false`
	   * depending on the values of the async tests. Invoked with (err, result).
	   */
	  var everyLimit = doParallelLimit(_createTester(notId, notId));

	  /**
	   * The same as [`every`]{@link module:Collections.every} but runs only a single async operation at a time.
	   *
	   * @name everySeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.every]{@link module:Collections.every}
	   * @alias allSeries
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async truth test to apply to each item
	   * in the collection in series.
	   * The iteratee must complete with a boolean result value.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Result will be either `true` or `false`
	   * depending on the values of the async tests. Invoked with (err, result).
	   */
	  var everySeries = doLimit(everyLimit, 1);

	  /**
	   * The base implementation of `_.property` without support for deep paths.
	   *
	   * @private
	   * @param {string} key The key of the property to get.
	   * @returns {Function} Returns the new accessor function.
	   */
	  function baseProperty(key) {
	    return function (object) {
	      return object == null ? undefined : object[key];
	    };
	  }
	  function filterArray(eachfn, arr, iteratee, callback) {
	    var truthValues = new Array(arr.length);
	    eachfn(arr, function (x, index, callback) {
	      iteratee(x, function (err, v) {
	        truthValues[index] = !!v;
	        callback(err);
	      });
	    }, function (err) {
	      if (err) return callback(err);
	      var results = [];
	      for (var i = 0; i < arr.length; i++) {
	        if (truthValues[i]) results.push(arr[i]);
	      }
	      callback(null, results);
	    });
	  }
	  function filterGeneric(eachfn, coll, iteratee, callback) {
	    var results = [];
	    eachfn(coll, function (x, index, callback) {
	      iteratee(x, function (err, v) {
	        if (err) {
	          callback(err);
	        } else {
	          if (v) {
	            results.push({
	              index: index,
	              value: x
	            });
	          }
	          callback();
	        }
	      });
	    }, function (err) {
	      if (err) {
	        callback(err);
	      } else {
	        callback(null, arrayMap(results.sort(function (a, b) {
	          return a.index - b.index;
	        }), baseProperty('value')));
	      }
	    });
	  }
	  function _filter(eachfn, coll, iteratee, callback) {
	    var filter = isArrayLike(coll) ? filterArray : filterGeneric;
	    filter(eachfn, coll, wrapAsync(iteratee), callback || noop);
	  }

	  /**
	   * Returns a new array of all the values in `coll` which pass an async truth
	   * test. This operation is performed in parallel, but the results array will be
	   * in the same order as the original.
	   *
	   * @name filter
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @alias select
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {Function} iteratee - A truth test to apply to each item in `coll`.
	   * The `iteratee` is passed a `callback(err, truthValue)`, which must be called
	   * with a boolean argument once it has completed. Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Invoked with (err, results).
	   * @example
	   *
	   * async.filter(['file1','file2','file3'], function(filePath, callback) {
	   *     fs.access(filePath, function(err) {
	   *         callback(null, !err)
	   *     });
	   * }, function(err, results) {
	   *     // results now equals an array of the existing files
	   * });
	   */
	  var filter = doParallel(_filter);

	  /**
	   * The same as [`filter`]{@link module:Collections.filter} but runs a maximum of `limit` async operations at a
	   * time.
	   *
	   * @name filterLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.filter]{@link module:Collections.filter}
	   * @alias selectLimit
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {Function} iteratee - A truth test to apply to each item in `coll`.
	   * The `iteratee` is passed a `callback(err, truthValue)`, which must be called
	   * with a boolean argument once it has completed. Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Invoked with (err, results).
	   */
	  var filterLimit = doParallelLimit(_filter);

	  /**
	   * The same as [`filter`]{@link module:Collections.filter} but runs only a single async operation at a time.
	   *
	   * @name filterSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.filter]{@link module:Collections.filter}
	   * @alias selectSeries
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {Function} iteratee - A truth test to apply to each item in `coll`.
	   * The `iteratee` is passed a `callback(err, truthValue)`, which must be called
	   * with a boolean argument once it has completed. Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Invoked with (err, results)
	   */
	  var filterSeries = doLimit(filterLimit, 1);

	  /**
	   * Calls the asynchronous function `fn` with a callback parameter that allows it
	   * to call itself again, in series, indefinitely.
	  
	   * If an error is passed to the callback then `errback` is called with the
	   * error, and execution stops, otherwise it will never be called.
	   *
	   * @name forever
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {AsyncFunction} fn - an async function to call repeatedly.
	   * Invoked with (next).
	   * @param {Function} [errback] - when `fn` passes an error to it's callback,
	   * this function will be called, and execution stops. Invoked with (err).
	   * @example
	   *
	   * async.forever(
	   *     function(next) {
	   *         // next is suitable for passing to things that need a callback(err [, whatever]);
	   *         // it will result in this function being called again.
	   *     },
	   *     function(err) {
	   *         // if next is called with a value in its first parameter, it will appear
	   *         // in here as 'err', and execution will stop.
	   *     }
	   * );
	   */
	  function forever(fn, errback) {
	    var done = onlyOnce(errback || noop);
	    var task = wrapAsync(ensureAsync(fn));
	    function next(err) {
	      if (err) return done(err);
	      task(next);
	    }
	    next();
	  }

	  /**
	   * The same as [`groupBy`]{@link module:Collections.groupBy} but runs a maximum of `limit` async operations at a time.
	   *
	   * @name groupByLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.groupBy]{@link module:Collections.groupBy}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * The iteratee should complete with a `key` to group the value under.
	   * Invoked with (value, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. Result is an `Object` whoses
	   * properties are arrays of values which returned the corresponding key.
	   */
	  var groupByLimit = function (coll, limit, iteratee, callback) {
	    callback = callback || noop;
	    var _iteratee = wrapAsync(iteratee);
	    mapLimit(coll, limit, function (val, callback) {
	      _iteratee(val, function (err, key) {
	        if (err) return callback(err);
	        return callback(null, {
	          key: key,
	          val: val
	        });
	      });
	    }, function (err, mapResults) {
	      var result = {};
	      // from MDN, handle object having an `hasOwnProperty` prop
	      var hasOwnProperty = Object.prototype.hasOwnProperty;
	      for (var i = 0; i < mapResults.length; i++) {
	        if (mapResults[i]) {
	          var key = mapResults[i].key;
	          var val = mapResults[i].val;
	          if (hasOwnProperty.call(result, key)) {
	            result[key].push(val);
	          } else {
	            result[key] = [val];
	          }
	        }
	      }
	      return callback(err, result);
	    });
	  };

	  /**
	   * Returns a new object, where each value corresponds to an array of items, from
	   * `coll`, that returned the corresponding key. That is, the keys of the object
	   * correspond to the values passed to the `iteratee` callback.
	   *
	   * Note: Since this function applies the `iteratee` to each item in parallel,
	   * there is no guarantee that the `iteratee` functions will complete in order.
	   * However, the values for each key in the `result` will be in the same order as
	   * the original `coll`. For Objects, the values will roughly be in the order of
	   * the original Objects' keys (but this can vary across JavaScript engines).
	   *
	   * @name groupBy
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * The iteratee should complete with a `key` to group the value under.
	   * Invoked with (value, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. Result is an `Object` whoses
	   * properties are arrays of values which returned the corresponding key.
	   * @example
	   *
	   * async.groupBy(['userId1', 'userId2', 'userId3'], function(userId, callback) {
	   *     db.findById(userId, function(err, user) {
	   *         if (err) return callback(err);
	   *         return callback(null, user.age);
	   *     });
	   * }, function(err, result) {
	   *     // result is object containing the userIds grouped by age
	   *     // e.g. { 30: ['userId1', 'userId3'], 42: ['userId2']};
	   * });
	   */
	  var groupBy = doLimit(groupByLimit, Infinity);

	  /**
	   * The same as [`groupBy`]{@link module:Collections.groupBy} but runs only a single async operation at a time.
	   *
	   * @name groupBySeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.groupBy]{@link module:Collections.groupBy}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * The iteratee should complete with a `key` to group the value under.
	   * Invoked with (value, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. Result is an `Object` whoses
	   * properties are arrays of values which returned the corresponding key.
	   */
	  var groupBySeries = doLimit(groupByLimit, 1);

	  /**
	   * Logs the result of an `async` function to the `console`. Only works in
	   * Node.js or in browsers that support `console.log` and `console.error` (such
	   * as FF and Chrome). If multiple arguments are returned from the async
	   * function, `console.log` is called on each argument in order.
	   *
	   * @name log
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @category Util
	   * @param {AsyncFunction} function - The function you want to eventually apply
	   * all arguments to.
	   * @param {...*} arguments... - Any number of arguments to apply to the function.
	   * @example
	   *
	   * // in a module
	   * var hello = function(name, callback) {
	   *     setTimeout(function() {
	   *         callback(null, 'hello ' + name);
	   *     }, 1000);
	   * };
	   *
	   * // in the node repl
	   * node> async.log(hello, 'world');
	   * 'hello world'
	   */
	  var log = consoleFunc('log');

	  /**
	   * The same as [`mapValues`]{@link module:Collections.mapValues} but runs a maximum of `limit` async operations at a
	   * time.
	   *
	   * @name mapValuesLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.mapValues]{@link module:Collections.mapValues}
	   * @category Collection
	   * @param {Object} obj - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - A function to apply to each value and key
	   * in `coll`.
	   * The iteratee should complete with the transformed value as its result.
	   * Invoked with (value, key, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. `result` is a new object consisting
	   * of each key from `obj`, with each transformed value on the right-hand side.
	   * Invoked with (err, result).
	   */
	  function mapValuesLimit(obj, limit, iteratee, callback) {
	    callback = once(callback || noop);
	    var newObj = {};
	    var _iteratee = wrapAsync(iteratee);
	    eachOfLimit(obj, limit, function (val, key, next) {
	      _iteratee(val, key, function (err, result) {
	        if (err) return next(err);
	        newObj[key] = result;
	        next();
	      });
	    }, function (err) {
	      callback(err, newObj);
	    });
	  }

	  /**
	   * A relative of [`map`]{@link module:Collections.map}, designed for use with objects.
	   *
	   * Produces a new Object by mapping each value of `obj` through the `iteratee`
	   * function. The `iteratee` is called each `value` and `key` from `obj` and a
	   * callback for when it has finished processing. Each of these callbacks takes
	   * two arguments: an `error`, and the transformed item from `obj`. If `iteratee`
	   * passes an error to its callback, the main `callback` (for the `mapValues`
	   * function) is immediately called with the error.
	   *
	   * Note, the order of the keys in the result is not guaranteed.  The keys will
	   * be roughly in the order they complete, (but this is very engine-specific)
	   *
	   * @name mapValues
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @category Collection
	   * @param {Object} obj - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - A function to apply to each value and key
	   * in `coll`.
	   * The iteratee should complete with the transformed value as its result.
	   * Invoked with (value, key, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. `result` is a new object consisting
	   * of each key from `obj`, with each transformed value on the right-hand side.
	   * Invoked with (err, result).
	   * @example
	   *
	   * async.mapValues({
	   *     f1: 'file1',
	   *     f2: 'file2',
	   *     f3: 'file3'
	   * }, function (file, key, callback) {
	   *   fs.stat(file, callback);
	   * }, function(err, result) {
	   *     // result is now a map of stats for each file, e.g.
	   *     // {
	   *     //     f1: [stats for file1],
	   *     //     f2: [stats for file2],
	   *     //     f3: [stats for file3]
	   *     // }
	   * });
	   */

	  var mapValues = doLimit(mapValuesLimit, Infinity);

	  /**
	   * The same as [`mapValues`]{@link module:Collections.mapValues} but runs only a single async operation at a time.
	   *
	   * @name mapValuesSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.mapValues]{@link module:Collections.mapValues}
	   * @category Collection
	   * @param {Object} obj - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - A function to apply to each value and key
	   * in `coll`.
	   * The iteratee should complete with the transformed value as its result.
	   * Invoked with (value, key, callback).
	   * @param {Function} [callback] - A callback which is called when all `iteratee`
	   * functions have finished, or an error occurs. `result` is a new object consisting
	   * of each key from `obj`, with each transformed value on the right-hand side.
	   * Invoked with (err, result).
	   */
	  var mapValuesSeries = doLimit(mapValuesLimit, 1);
	  function has(obj, key) {
	    return key in obj;
	  }

	  /**
	   * Caches the results of an async function. When creating a hash to store
	   * function results against, the callback is omitted from the hash and an
	   * optional hash function can be used.
	   *
	   * If no hash function is specified, the first argument is used as a hash key,
	   * which may work reasonably if it is a string or a data type that converts to a
	   * distinct string. Note that objects and arrays will not behave reasonably.
	   * Neither will cases where the other arguments are significant. In such cases,
	   * specify your own hash function.
	   *
	   * The cache of results is exposed as the `memo` property of the function
	   * returned by `memoize`.
	   *
	   * @name memoize
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @category Util
	   * @param {AsyncFunction} fn - The async function to proxy and cache results from.
	   * @param {Function} hasher - An optional function for generating a custom hash
	   * for storing results. It has all the arguments applied to it apart from the
	   * callback, and must be synchronous.
	   * @returns {AsyncFunction} a memoized version of `fn`
	   * @example
	   *
	   * var slow_fn = function(name, callback) {
	   *     // do something
	   *     callback(null, result);
	   * };
	   * var fn = async.memoize(slow_fn);
	   *
	   * // fn can now be used as if it were slow_fn
	   * fn('some name', function() {
	   *     // callback
	   * });
	   */
	  function memoize(fn, hasher) {
	    var memo = Object.create(null);
	    var queues = Object.create(null);
	    hasher = hasher || identity;
	    var _fn = wrapAsync(fn);
	    var memoized = initialParams(function memoized(args, callback) {
	      var key = hasher.apply(null, args);
	      if (has(memo, key)) {
	        setImmediate$1(function () {
	          callback.apply(null, memo[key]);
	        });
	      } else if (has(queues, key)) {
	        queues[key].push(callback);
	      } else {
	        queues[key] = [callback];
	        _fn.apply(null, args.concat(function /*args*/
	        () {
	          var args = slice(arguments);
	          memo[key] = args;
	          var q = queues[key];
	          delete queues[key];
	          for (var i = 0, l = q.length; i < l; i++) {
	            q[i].apply(null, args);
	          }
	        }));
	      }
	    });
	    memoized.memo = memo;
	    memoized.unmemoized = fn;
	    return memoized;
	  }

	  /**
	   * Calls `callback` on a later loop around the event loop. In Node.js this just
	   * calls `process.nextTick`.  In the browser it will use `setImmediate` if
	   * available, otherwise `setTimeout(callback, 0)`, which means other higher
	   * priority events may precede the execution of `callback`.
	   *
	   * This is used internally for browser-compatibility purposes.
	   *
	   * @name nextTick
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @see [async.setImmediate]{@link module:Utils.setImmediate}
	   * @category Util
	   * @param {Function} callback - The function to call on a later loop around
	   * the event loop. Invoked with (args...).
	   * @param {...*} args... - any number of additional arguments to pass to the
	   * callback on the next tick.
	   * @example
	   *
	   * var call_order = [];
	   * async.nextTick(function() {
	   *     call_order.push('two');
	   *     // call_order now equals ['one','two']
	   * });
	   * call_order.push('one');
	   *
	   * async.setImmediate(function (a, b, c) {
	   *     // a, b, and c equal 1, 2, and 3
	   * }, 1, 2, 3);
	   */
	  var _defer$1;
	  if (hasNextTick) {
	    _defer$1 = process.nextTick;
	  } else if (hasSetImmediate) {
	    _defer$1 = setImmediate;
	  } else {
	    _defer$1 = fallback;
	  }
	  var nextTick = wrap(_defer$1);
	  function _parallel(eachfn, tasks, callback) {
	    callback = callback || noop;
	    var results = isArrayLike(tasks) ? [] : {};
	    eachfn(tasks, function (task, key, callback) {
	      wrapAsync(task)(function (err, result) {
	        if (arguments.length > 2) {
	          result = slice(arguments, 1);
	        }
	        results[key] = result;
	        callback(err);
	      });
	    }, function (err) {
	      callback(err, results);
	    });
	  }

	  /**
	   * Run the `tasks` collection of functions in parallel, without waiting until
	   * the previous function has completed. If any of the functions pass an error to
	   * its callback, the main `callback` is immediately called with the value of the
	   * error. Once the `tasks` have completed, the results are passed to the final
	   * `callback` as an array.
	   *
	   * **Note:** `parallel` is about kicking-off I/O tasks in parallel, not about
	   * parallel execution of code.  If your tasks do not use any timers or perform
	   * any I/O, they will actually be executed in series.  Any synchronous setup
	   * sections for each task will happen one after the other.  JavaScript remains
	   * single-threaded.
	   *
	   * **Hint:** Use [`reflect`]{@link module:Utils.reflect} to continue the
	   * execution of other tasks when a task fails.
	   *
	   * It is also possible to use an object instead of an array. Each property will
	   * be run as a function and the results will be passed to the final `callback`
	   * as an object instead of an array. This can be a more readable way of handling
	   * results from {@link async.parallel}.
	   *
	   * @name parallel
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {Array|Iterable|Object} tasks - A collection of
	   * [async functions]{@link AsyncFunction} to run.
	   * Each async function can complete with any number of optional `result` values.
	   * @param {Function} [callback] - An optional callback to run once all the
	   * functions have completed successfully. This function gets a results array
	   * (or object) containing all the result arguments passed to the task callbacks.
	   * Invoked with (err, results).
	   *
	   * @example
	   * async.parallel([
	   *     function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 'one');
	   *         }, 200);
	   *     },
	   *     function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 'two');
	   *         }, 100);
	   *     }
	   * ],
	   * // optional callback
	   * function(err, results) {
	   *     // the results array will equal ['one','two'] even though
	   *     // the second function had a shorter timeout.
	   * });
	   *
	   * // an example using an object instead of an array
	   * async.parallel({
	   *     one: function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 1);
	   *         }, 200);
	   *     },
	   *     two: function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 2);
	   *         }, 100);
	   *     }
	   * }, function(err, results) {
	   *     // results is now equals to: {one: 1, two: 2}
	   * });
	   */
	  function parallelLimit(tasks, callback) {
	    _parallel(eachOf, tasks, callback);
	  }

	  /**
	   * The same as [`parallel`]{@link module:ControlFlow.parallel} but runs a maximum of `limit` async operations at a
	   * time.
	   *
	   * @name parallelLimit
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.parallel]{@link module:ControlFlow.parallel}
	   * @category Control Flow
	   * @param {Array|Iterable|Object} tasks - A collection of
	   * [async functions]{@link AsyncFunction} to run.
	   * Each async function can complete with any number of optional `result` values.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {Function} [callback] - An optional callback to run once all the
	   * functions have completed successfully. This function gets a results array
	   * (or object) containing all the result arguments passed to the task callbacks.
	   * Invoked with (err, results).
	   */
	  function parallelLimit$1(tasks, limit, callback) {
	    _parallel(_eachOfLimit(limit), tasks, callback);
	  }

	  /**
	   * A queue of tasks for the worker function to complete.
	   * @typedef {Object} QueueObject
	   * @memberOf module:ControlFlow
	   * @property {Function} length - a function returning the number of items
	   * waiting to be processed. Invoke with `queue.length()`.
	   * @property {boolean} started - a boolean indicating whether or not any
	   * items have been pushed and processed by the queue.
	   * @property {Function} running - a function returning the number of items
	   * currently being processed. Invoke with `queue.running()`.
	   * @property {Function} workersList - a function returning the array of items
	   * currently being processed. Invoke with `queue.workersList()`.
	   * @property {Function} idle - a function returning false if there are items
	   * waiting or being processed, or true if not. Invoke with `queue.idle()`.
	   * @property {number} concurrency - an integer for determining how many `worker`
	   * functions should be run in parallel. This property can be changed after a
	   * `queue` is created to alter the concurrency on-the-fly.
	   * @property {Function} push - add a new task to the `queue`. Calls `callback`
	   * once the `worker` has finished processing the task. Instead of a single task,
	   * a `tasks` array can be submitted. The respective callback is used for every
	   * task in the list. Invoke with `queue.push(task, [callback])`,
	   * @property {Function} unshift - add a new task to the front of the `queue`.
	   * Invoke with `queue.unshift(task, [callback])`.
	   * @property {Function} remove - remove items from the queue that match a test
	   * function.  The test function will be passed an object with a `data` property,
	   * and a `priority` property, if this is a
	   * [priorityQueue]{@link module:ControlFlow.priorityQueue} object.
	   * Invoked with `queue.remove(testFn)`, where `testFn` is of the form
	   * `function ({data, priority}) {}` and returns a Boolean.
	   * @property {Function} saturated - a callback that is called when the number of
	   * running workers hits the `concurrency` limit, and further tasks will be
	   * queued.
	   * @property {Function} unsaturated - a callback that is called when the number
	   * of running workers is less than the `concurrency` & `buffer` limits, and
	   * further tasks will not be queued.
	   * @property {number} buffer - A minimum threshold buffer in order to say that
	   * the `queue` is `unsaturated`.
	   * @property {Function} empty - a callback that is called when the last item
	   * from the `queue` is given to a `worker`.
	   * @property {Function} drain - a callback that is called when the last item
	   * from the `queue` has returned from the `worker`.
	   * @property {Function} error - a callback that is called when a task errors.
	   * Has the signature `function(error, task)`.
	   * @property {boolean} paused - a boolean for determining whether the queue is
	   * in a paused state.
	   * @property {Function} pause - a function that pauses the processing of tasks
	   * until `resume()` is called. Invoke with `queue.pause()`.
	   * @property {Function} resume - a function that resumes the processing of
	   * queued tasks when the queue is paused. Invoke with `queue.resume()`.
	   * @property {Function} kill - a function that removes the `drain` callback and
	   * empties remaining tasks from the queue forcing it to go idle. No more tasks
	   * should be pushed to the queue after calling this function. Invoke with `queue.kill()`.
	   */

	  /**
	   * Creates a `queue` object with the specified `concurrency`. Tasks added to the
	   * `queue` are processed in parallel (up to the `concurrency` limit). If all
	   * `worker`s are in progress, the task is queued until one becomes available.
	   * Once a `worker` completes a `task`, that `task`'s callback is called.
	   *
	   * @name queue
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {AsyncFunction} worker - An async function for processing a queued task.
	   * If you want to handle errors from an individual task, pass a callback to
	   * `q.push()`. Invoked with (task, callback).
	   * @param {number} [concurrency=1] - An `integer` for determining how many
	   * `worker` functions should be run in parallel.  If omitted, the concurrency
	   * defaults to `1`.  If the concurrency is `0`, an error is thrown.
	   * @returns {module:ControlFlow.QueueObject} A queue object to manage the tasks. Callbacks can
	   * attached as certain properties to listen for specific events during the
	   * lifecycle of the queue.
	   * @example
	   *
	   * // create a queue object with concurrency 2
	   * var q = async.queue(function(task, callback) {
	   *     console.log('hello ' + task.name);
	   *     callback();
	   * }, 2);
	   *
	   * // assign a callback
	   * q.drain = function() {
	   *     console.log('all items have been processed');
	   * };
	   *
	   * // add some items to the queue
	   * q.push({name: 'foo'}, function(err) {
	   *     console.log('finished processing foo');
	   * });
	   * q.push({name: 'bar'}, function (err) {
	   *     console.log('finished processing bar');
	   * });
	   *
	   * // add some items to the queue (batch-wise)
	   * q.push([{name: 'baz'},{name: 'bay'},{name: 'bax'}], function(err) {
	   *     console.log('finished processing item');
	   * });
	   *
	   * // add some items to the front of the queue
	   * q.unshift({name: 'bar'}, function (err) {
	   *     console.log('finished processing bar');
	   * });
	   */
	  var queue$1 = function (worker, concurrency) {
	    var _worker = wrapAsync(worker);
	    return queue(function (items, cb) {
	      _worker(items[0], cb);
	    }, concurrency, 1);
	  };

	  /**
	   * The same as [async.queue]{@link module:ControlFlow.queue} only tasks are assigned a priority and
	   * completed in ascending priority order.
	   *
	   * @name priorityQueue
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.queue]{@link module:ControlFlow.queue}
	   * @category Control Flow
	   * @param {AsyncFunction} worker - An async function for processing a queued task.
	   * If you want to handle errors from an individual task, pass a callback to
	   * `q.push()`.
	   * Invoked with (task, callback).
	   * @param {number} concurrency - An `integer` for determining how many `worker`
	   * functions should be run in parallel.  If omitted, the concurrency defaults to
	   * `1`.  If the concurrency is `0`, an error is thrown.
	   * @returns {module:ControlFlow.QueueObject} A priorityQueue object to manage the tasks. There are two
	   * differences between `queue` and `priorityQueue` objects:
	   * * `push(task, priority, [callback])` - `priority` should be a number. If an
	   *   array of `tasks` is given, all tasks will be assigned the same priority.
	   * * The `unshift` method was removed.
	   */
	  var priorityQueue = function (worker, concurrency) {
	    // Start with a normal queue
	    var q = queue$1(worker, concurrency);

	    // Override push to accept second parameter representing priority
	    q.push = function (data, priority, callback) {
	      if (callback == null) callback = noop;
	      if (typeof callback !== 'function') {
	        throw new Error('task callback must be a function');
	      }
	      q.started = true;
	      if (!isArray(data)) {
	        data = [data];
	      }
	      if (data.length === 0) {
	        // call drain immediately if there are no tasks
	        return setImmediate$1(function () {
	          q.drain();
	        });
	      }
	      priority = priority || 0;
	      var nextNode = q._tasks.head;
	      while (nextNode && priority >= nextNode.priority) {
	        nextNode = nextNode.next;
	      }
	      for (var i = 0, l = data.length; i < l; i++) {
	        var item = {
	          data: data[i],
	          priority: priority,
	          callback: callback
	        };
	        if (nextNode) {
	          q._tasks.insertBefore(nextNode, item);
	        } else {
	          q._tasks.push(item);
	        }
	      }
	      setImmediate$1(q.process);
	    };

	    // Remove unshift function
	    delete q.unshift;
	    return q;
	  };

	  /**
	   * Runs the `tasks` array of functions in parallel, without waiting until the
	   * previous function has completed. Once any of the `tasks` complete or pass an
	   * error to its callback, the main `callback` is immediately called. It's
	   * equivalent to `Promise.race()`.
	   *
	   * @name race
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {Array} tasks - An array containing [async functions]{@link AsyncFunction}
	   * to run. Each function can complete with an optional `result` value.
	   * @param {Function} callback - A callback to run once any of the functions have
	   * completed. This function gets an error or result from the first function that
	   * completed. Invoked with (err, result).
	   * @returns undefined
	   * @example
	   *
	   * async.race([
	   *     function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 'one');
	   *         }, 200);
	   *     },
	   *     function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 'two');
	   *         }, 100);
	   *     }
	   * ],
	   * // main callback
	   * function(err, result) {
	   *     // the result will be equal to 'two' as it finishes earlier
	   * });
	   */
	  function race(tasks, callback) {
	    callback = once(callback || noop);
	    if (!isArray(tasks)) return callback(new TypeError('First argument to race must be an array of functions'));
	    if (!tasks.length) return callback();
	    for (var i = 0, l = tasks.length; i < l; i++) {
	      wrapAsync(tasks[i])(callback);
	    }
	  }

	  /**
	   * Same as [`reduce`]{@link module:Collections.reduce}, only operates on `array` in reverse order.
	   *
	   * @name reduceRight
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.reduce]{@link module:Collections.reduce}
	   * @alias foldr
	   * @category Collection
	   * @param {Array} array - A collection to iterate over.
	   * @param {*} memo - The initial state of the reduction.
	   * @param {AsyncFunction} iteratee - A function applied to each item in the
	   * array to produce the next step in the reduction.
	   * The `iteratee` should complete with the next state of the reduction.
	   * If the iteratee complete with an error, the reduction is stopped and the
	   * main `callback` is immediately called with the error.
	   * Invoked with (memo, item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Result is the reduced value. Invoked with
	   * (err, result).
	   */
	  function reduceRight(array, memo, iteratee, callback) {
	    var reversed = slice(array).reverse();
	    reduce(reversed, memo, iteratee, callback);
	  }

	  /**
	   * Wraps the async function in another function that always completes with a
	   * result object, even when it errors.
	   *
	   * The result object has either the property `error` or `value`.
	   *
	   * @name reflect
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @category Util
	   * @param {AsyncFunction} fn - The async function you want to wrap
	   * @returns {Function} - A function that always passes null to it's callback as
	   * the error. The second argument to the callback will be an `object` with
	   * either an `error` or a `value` property.
	   * @example
	   *
	   * async.parallel([
	   *     async.reflect(function(callback) {
	   *         // do some stuff ...
	   *         callback(null, 'one');
	   *     }),
	   *     async.reflect(function(callback) {
	   *         // do some more stuff but error ...
	   *         callback('bad stuff happened');
	   *     }),
	   *     async.reflect(function(callback) {
	   *         // do some more stuff ...
	   *         callback(null, 'two');
	   *     })
	   * ],
	   * // optional callback
	   * function(err, results) {
	   *     // values
	   *     // results[0].value = 'one'
	   *     // results[1].error = 'bad stuff happened'
	   *     // results[2].value = 'two'
	   * });
	   */
	  function reflect(fn) {
	    var _fn = wrapAsync(fn);
	    return initialParams(function reflectOn(args, reflectCallback) {
	      args.push(function callback(error, cbArg) {
	        if (error) {
	          reflectCallback(null, {
	            error: error
	          });
	        } else {
	          var value;
	          if (arguments.length <= 2) {
	            value = cbArg;
	          } else {
	            value = slice(arguments, 1);
	          }
	          reflectCallback(null, {
	            value: value
	          });
	        }
	      });
	      return _fn.apply(this, args);
	    });
	  }

	  /**
	   * A helper function that wraps an array or an object of functions with `reflect`.
	   *
	   * @name reflectAll
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @see [async.reflect]{@link module:Utils.reflect}
	   * @category Util
	   * @param {Array|Object|Iterable} tasks - The collection of
	   * [async functions]{@link AsyncFunction} to wrap in `async.reflect`.
	   * @returns {Array} Returns an array of async functions, each wrapped in
	   * `async.reflect`
	   * @example
	   *
	   * let tasks = [
	   *     function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 'one');
	   *         }, 200);
	   *     },
	   *     function(callback) {
	   *         // do some more stuff but error ...
	   *         callback(new Error('bad stuff happened'));
	   *     },
	   *     function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 'two');
	   *         }, 100);
	   *     }
	   * ];
	   *
	   * async.parallel(async.reflectAll(tasks),
	   * // optional callback
	   * function(err, results) {
	   *     // values
	   *     // results[0].value = 'one'
	   *     // results[1].error = Error('bad stuff happened')
	   *     // results[2].value = 'two'
	   * });
	   *
	   * // an example using an object instead of an array
	   * let tasks = {
	   *     one: function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 'one');
	   *         }, 200);
	   *     },
	   *     two: function(callback) {
	   *         callback('two');
	   *     },
	   *     three: function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 'three');
	   *         }, 100);
	   *     }
	   * };
	   *
	   * async.parallel(async.reflectAll(tasks),
	   * // optional callback
	   * function(err, results) {
	   *     // values
	   *     // results.one.value = 'one'
	   *     // results.two.error = 'two'
	   *     // results.three.value = 'three'
	   * });
	   */
	  function reflectAll(tasks) {
	    var results;
	    if (isArray(tasks)) {
	      results = arrayMap(tasks, reflect);
	    } else {
	      results = {};
	      baseForOwn(tasks, function (task, key) {
	        results[key] = reflect.call(this, task);
	      });
	    }
	    return results;
	  }
	  function reject$1(eachfn, arr, iteratee, callback) {
	    _filter(eachfn, arr, function (value, cb) {
	      iteratee(value, function (err, v) {
	        cb(err, !v);
	      });
	    }, callback);
	  }

	  /**
	   * The opposite of [`filter`]{@link module:Collections.filter}. Removes values that pass an `async` truth test.
	   *
	   * @name reject
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.filter]{@link module:Collections.filter}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {Function} iteratee - An async truth test to apply to each item in
	   * `coll`.
	   * The should complete with a boolean value as its `result`.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Invoked with (err, results).
	   * @example
	   *
	   * async.reject(['file1','file2','file3'], function(filePath, callback) {
	   *     fs.access(filePath, function(err) {
	   *         callback(null, !err)
	   *     });
	   * }, function(err, results) {
	   *     // results now equals an array of missing files
	   *     createFiles(results);
	   * });
	   */
	  var reject = doParallel(reject$1);

	  /**
	   * The same as [`reject`]{@link module:Collections.reject} but runs a maximum of `limit` async operations at a
	   * time.
	   *
	   * @name rejectLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.reject]{@link module:Collections.reject}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {Function} iteratee - An async truth test to apply to each item in
	   * `coll`.
	   * The should complete with a boolean value as its `result`.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Invoked with (err, results).
	   */
	  var rejectLimit = doParallelLimit(reject$1);

	  /**
	   * The same as [`reject`]{@link module:Collections.reject} but runs only a single async operation at a time.
	   *
	   * @name rejectSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.reject]{@link module:Collections.reject}
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {Function} iteratee - An async truth test to apply to each item in
	   * `coll`.
	   * The should complete with a boolean value as its `result`.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Invoked with (err, results).
	   */
	  var rejectSeries = doLimit(rejectLimit, 1);

	  /**
	   * Creates a function that returns `value`.
	   *
	   * @static
	   * @memberOf _
	   * @since 2.4.0
	   * @category Util
	   * @param {*} value The value to return from the new function.
	   * @returns {Function} Returns the new constant function.
	   * @example
	   *
	   * var objects = _.times(2, _.constant({ 'a': 1 }));
	   *
	   * console.log(objects);
	   * // => [{ 'a': 1 }, { 'a': 1 }]
	   *
	   * console.log(objects[0] === objects[1]);
	   * // => true
	   */
	  function constant$1(value) {
	    return function () {
	      return value;
	    };
	  }

	  /**
	   * Attempts to get a successful response from `task` no more than `times` times
	   * before returning an error. If the task is successful, the `callback` will be
	   * passed the result of the successful task. If all attempts fail, the callback
	   * will be passed the error and result (if any) of the final attempt.
	   *
	   * @name retry
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @see [async.retryable]{@link module:ControlFlow.retryable}
	   * @param {Object|number} [opts = {times: 5, interval: 0}| 5] - Can be either an
	   * object with `times` and `interval` or a number.
	   * * `times` - The number of attempts to make before giving up.  The default
	   *   is `5`.
	   * * `interval` - The time to wait between retries, in milliseconds.  The
	   *   default is `0`. The interval may also be specified as a function of the
	   *   retry count (see example).
	   * * `errorFilter` - An optional synchronous function that is invoked on
	   *   erroneous result. If it returns `true` the retry attempts will continue;
	   *   if the function returns `false` the retry flow is aborted with the current
	   *   attempt's error and result being returned to the final callback.
	   *   Invoked with (err).
	   * * If `opts` is a number, the number specifies the number of times to retry,
	   *   with the default interval of `0`.
	   * @param {AsyncFunction} task - An async function to retry.
	   * Invoked with (callback).
	   * @param {Function} [callback] - An optional callback which is called when the
	   * task has succeeded, or after the final failed attempt. It receives the `err`
	   * and `result` arguments of the last attempt at completing the `task`. Invoked
	   * with (err, results).
	   *
	   * @example
	   *
	   * // The `retry` function can be used as a stand-alone control flow by passing
	   * // a callback, as shown below:
	   *
	   * // try calling apiMethod 3 times
	   * async.retry(3, apiMethod, function(err, result) {
	   *     // do something with the result
	   * });
	   *
	   * // try calling apiMethod 3 times, waiting 200 ms between each retry
	   * async.retry({times: 3, interval: 200}, apiMethod, function(err, result) {
	   *     // do something with the result
	   * });
	   *
	   * // try calling apiMethod 10 times with exponential backoff
	   * // (i.e. intervals of 100, 200, 400, 800, 1600, ... milliseconds)
	   * async.retry({
	   *   times: 10,
	   *   interval: function(retryCount) {
	   *     return 50 * Math.pow(2, retryCount);
	   *   }
	   * }, apiMethod, function(err, result) {
	   *     // do something with the result
	   * });
	   *
	   * // try calling apiMethod the default 5 times no delay between each retry
	   * async.retry(apiMethod, function(err, result) {
	   *     // do something with the result
	   * });
	   *
	   * // try calling apiMethod only when error condition satisfies, all other
	   * // errors will abort the retry control flow and return to final callback
	   * async.retry({
	   *   errorFilter: function(err) {
	   *     return err.message === 'Temporary error'; // only retry on a specific error
	   *   }
	   * }, apiMethod, function(err, result) {
	   *     // do something with the result
	   * });
	   *
	   * // to retry individual methods that are not as reliable within other
	   * // control flow functions, use the `retryable` wrapper:
	   * async.auto({
	   *     users: api.getUsers.bind(api),
	   *     payments: async.retryable(3, api.getPayments.bind(api))
	   * }, function(err, results) {
	   *     // do something with the results
	   * });
	   *
	   */
	  function retry(opts, task, callback) {
	    var DEFAULT_TIMES = 5;
	    var DEFAULT_INTERVAL = 0;
	    var options = {
	      times: DEFAULT_TIMES,
	      intervalFunc: constant$1(DEFAULT_INTERVAL)
	    };
	    function parseTimes(acc, t) {
	      if (typeof t === 'object') {
	        acc.times = +t.times || DEFAULT_TIMES;
	        acc.intervalFunc = typeof t.interval === 'function' ? t.interval : constant$1(+t.interval || DEFAULT_INTERVAL);
	        acc.errorFilter = t.errorFilter;
	      } else if (typeof t === 'number' || typeof t === 'string') {
	        acc.times = +t || DEFAULT_TIMES;
	      } else {
	        throw new Error("Invalid arguments for async.retry");
	      }
	    }
	    if (arguments.length < 3 && typeof opts === 'function') {
	      callback = task || noop;
	      task = opts;
	    } else {
	      parseTimes(options, opts);
	      callback = callback || noop;
	    }
	    if (typeof task !== 'function') {
	      throw new Error("Invalid arguments for async.retry");
	    }
	    var _task = wrapAsync(task);
	    var attempt = 1;
	    function retryAttempt() {
	      _task(function (err) {
	        if (err && attempt++ < options.times && (typeof options.errorFilter != 'function' || options.errorFilter(err))) {
	          setTimeout(retryAttempt, options.intervalFunc(attempt));
	        } else {
	          callback.apply(null, arguments);
	        }
	      });
	    }
	    retryAttempt();
	  }

	  /**
	   * A close relative of [`retry`]{@link module:ControlFlow.retry}.  This method
	   * wraps a task and makes it retryable, rather than immediately calling it
	   * with retries.
	   *
	   * @name retryable
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.retry]{@link module:ControlFlow.retry}
	   * @category Control Flow
	   * @param {Object|number} [opts = {times: 5, interval: 0}| 5] - optional
	   * options, exactly the same as from `retry`
	   * @param {AsyncFunction} task - the asynchronous function to wrap.
	   * This function will be passed any arguments passed to the returned wrapper.
	   * Invoked with (...args, callback).
	   * @returns {AsyncFunction} The wrapped function, which when invoked, will
	   * retry on an error, based on the parameters specified in `opts`.
	   * This function will accept the same parameters as `task`.
	   * @example
	   *
	   * async.auto({
	   *     dep1: async.retryable(3, getFromFlakyService),
	   *     process: ["dep1", async.retryable(3, function (results, cb) {
	   *         maybeProcessData(results.dep1, cb);
	   *     })]
	   * }, callback);
	   */
	  var retryable = function (opts, task) {
	    if (!task) {
	      task = opts;
	      opts = null;
	    }
	    var _task = wrapAsync(task);
	    return initialParams(function (args, callback) {
	      function taskFn(cb) {
	        _task.apply(null, args.concat(cb));
	      }
	      if (opts) retry(opts, taskFn, callback);else retry(taskFn, callback);
	    });
	  };

	  /**
	   * Run the functions in the `tasks` collection in series, each one running once
	   * the previous function has completed. If any functions in the series pass an
	   * error to its callback, no more functions are run, and `callback` is
	   * immediately called with the value of the error. Otherwise, `callback`
	   * receives an array of results when `tasks` have completed.
	   *
	   * It is also possible to use an object instead of an array. Each property will
	   * be run as a function, and the results will be passed to the final `callback`
	   * as an object instead of an array. This can be a more readable way of handling
	   *  results from {@link async.series}.
	   *
	   * **Note** that while many implementations preserve the order of object
	   * properties, the [ECMAScript Language Specification](http://www.ecma-international.org/ecma-262/5.1/#sec-8.6)
	   * explicitly states that
	   *
	   * > The mechanics and order of enumerating the properties is not specified.
	   *
	   * So if you rely on the order in which your series of functions are executed,
	   * and want this to work on all platforms, consider using an array.
	   *
	   * @name series
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {Array|Iterable|Object} tasks - A collection containing
	   * [async functions]{@link AsyncFunction} to run in series.
	   * Each function can complete with any number of optional `result` values.
	   * @param {Function} [callback] - An optional callback to run once all the
	   * functions have completed. This function gets a results array (or object)
	   * containing all the result arguments passed to the `task` callbacks. Invoked
	   * with (err, result).
	   * @example
	   * async.series([
	   *     function(callback) {
	   *         // do some stuff ...
	   *         callback(null, 'one');
	   *     },
	   *     function(callback) {
	   *         // do some more stuff ...
	   *         callback(null, 'two');
	   *     }
	   * ],
	   * // optional callback
	   * function(err, results) {
	   *     // results is now equal to ['one', 'two']
	   * });
	   *
	   * async.series({
	   *     one: function(callback) {
	   *         setTimeout(function() {
	   *             callback(null, 1);
	   *         }, 200);
	   *     },
	   *     two: function(callback){
	   *         setTimeout(function() {
	   *             callback(null, 2);
	   *         }, 100);
	   *     }
	   * }, function(err, results) {
	   *     // results is now equal to: {one: 1, two: 2}
	   * });
	   */
	  function series(tasks, callback) {
	    _parallel(eachOfSeries, tasks, callback);
	  }

	  /**
	   * Returns `true` if at least one element in the `coll` satisfies an async test.
	   * If any iteratee call returns `true`, the main `callback` is immediately
	   * called.
	   *
	   * @name some
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @alias any
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async truth test to apply to each item
	   * in the collections in parallel.
	   * The iteratee should complete with a boolean `result` value.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called as soon as any
	   * iteratee returns `true`, or after all the iteratee functions have finished.
	   * Result will be either `true` or `false` depending on the values of the async
	   * tests. Invoked with (err, result).
	   * @example
	   *
	   * async.some(['file1','file2','file3'], function(filePath, callback) {
	   *     fs.access(filePath, function(err) {
	   *         callback(null, !err)
	   *     });
	   * }, function(err, result) {
	   *     // if result is true then at least one of the files exists
	   * });
	   */
	  var some = doParallel(_createTester(Boolean, identity));

	  /**
	   * The same as [`some`]{@link module:Collections.some} but runs a maximum of `limit` async operations at a time.
	   *
	   * @name someLimit
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.some]{@link module:Collections.some}
	   * @alias anyLimit
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - An async truth test to apply to each item
	   * in the collections in parallel.
	   * The iteratee should complete with a boolean `result` value.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called as soon as any
	   * iteratee returns `true`, or after all the iteratee functions have finished.
	   * Result will be either `true` or `false` depending on the values of the async
	   * tests. Invoked with (err, result).
	   */
	  var someLimit = doParallelLimit(_createTester(Boolean, identity));

	  /**
	   * The same as [`some`]{@link module:Collections.some} but runs only a single async operation at a time.
	   *
	   * @name someSeries
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @see [async.some]{@link module:Collections.some}
	   * @alias anySeries
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async truth test to apply to each item
	   * in the collections in series.
	   * The iteratee should complete with a boolean `result` value.
	   * Invoked with (item, callback).
	   * @param {Function} [callback] - A callback which is called as soon as any
	   * iteratee returns `true`, or after all the iteratee functions have finished.
	   * Result will be either `true` or `false` depending on the values of the async
	   * tests. Invoked with (err, result).
	   */
	  var someSeries = doLimit(someLimit, 1);

	  /**
	   * Sorts a list by the results of running each `coll` value through an async
	   * `iteratee`.
	   *
	   * @name sortBy
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {AsyncFunction} iteratee - An async function to apply to each item in
	   * `coll`.
	   * The iteratee should complete with a value to use as the sort criteria as
	   * its `result`.
	   * Invoked with (item, callback).
	   * @param {Function} callback - A callback which is called after all the
	   * `iteratee` functions have finished, or an error occurs. Results is the items
	   * from the original `coll` sorted by the values returned by the `iteratee`
	   * calls. Invoked with (err, results).
	   * @example
	   *
	   * async.sortBy(['file1','file2','file3'], function(file, callback) {
	   *     fs.stat(file, function(err, stats) {
	   *         callback(err, stats.mtime);
	   *     });
	   * }, function(err, results) {
	   *     // results is now the original array of files sorted by
	   *     // modified date
	   * });
	   *
	   * // By modifying the callback parameter the
	   * // sorting order can be influenced:
	   *
	   * // ascending order
	   * async.sortBy([1,9,3,5], function(x, callback) {
	   *     callback(null, x);
	   * }, function(err,result) {
	   *     // result callback
	   * });
	   *
	   * // descending order
	   * async.sortBy([1,9,3,5], function(x, callback) {
	   *     callback(null, x*-1);    //<- x*-1 instead of x, turns the order around
	   * }, function(err,result) {
	   *     // result callback
	   * });
	   */
	  function sortBy(coll, iteratee, callback) {
	    var _iteratee = wrapAsync(iteratee);
	    map(coll, function (x, callback) {
	      _iteratee(x, function (err, criteria) {
	        if (err) return callback(err);
	        callback(null, {
	          value: x,
	          criteria: criteria
	        });
	      });
	    }, function (err, results) {
	      if (err) return callback(err);
	      callback(null, arrayMap(results.sort(comparator), baseProperty('value')));
	    });
	    function comparator(left, right) {
	      var a = left.criteria,
	        b = right.criteria;
	      return a < b ? -1 : a > b ? 1 : 0;
	    }
	  }

	  /**
	   * Sets a time limit on an asynchronous function. If the function does not call
	   * its callback within the specified milliseconds, it will be called with a
	   * timeout error. The code property for the error object will be `'ETIMEDOUT'`.
	   *
	   * @name timeout
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @category Util
	   * @param {AsyncFunction} asyncFn - The async function to limit in time.
	   * @param {number} milliseconds - The specified time limit.
	   * @param {*} [info] - Any variable you want attached (`string`, `object`, etc)
	   * to timeout Error for more information..
	   * @returns {AsyncFunction} Returns a wrapped function that can be used with any
	   * of the control flow functions.
	   * Invoke this function with the same parameters as you would `asyncFunc`.
	   * @example
	   *
	   * function myFunction(foo, callback) {
	   *     doAsyncTask(foo, function(err, data) {
	   *         // handle errors
	   *         if (err) return callback(err);
	   *
	   *         // do some stuff ...
	   *
	   *         // return processed data
	   *         return callback(null, data);
	   *     });
	   * }
	   *
	   * var wrapped = async.timeout(myFunction, 1000);
	   *
	   * // call `wrapped` as you would `myFunction`
	   * wrapped({ bar: 'bar' }, function(err, data) {
	   *     // if `myFunction` takes < 1000 ms to execute, `err`
	   *     // and `data` will have their expected values
	   *
	   *     // else `err` will be an Error with the code 'ETIMEDOUT'
	   * });
	   */
	  function timeout(asyncFn, milliseconds, info) {
	    var fn = wrapAsync(asyncFn);
	    return initialParams(function (args, callback) {
	      var timedOut = false;
	      var timer;
	      function timeoutCallback() {
	        var name = asyncFn.name || 'anonymous';
	        var error = new Error('Callback function "' + name + '" timed out.');
	        error.code = 'ETIMEDOUT';
	        if (info) {
	          error.info = info;
	        }
	        timedOut = true;
	        callback(error);
	      }
	      args.push(function () {
	        if (!timedOut) {
	          callback.apply(null, arguments);
	          clearTimeout(timer);
	        }
	      });

	      // setup timer and call original function
	      timer = setTimeout(timeoutCallback, milliseconds);
	      fn.apply(null, args);
	    });
	  }

	  /* Built-in method references for those with the same name as other `lodash` methods. */
	  var nativeCeil = Math.ceil;
	  var nativeMax = Math.max;

	  /**
	   * The base implementation of `_.range` and `_.rangeRight` which doesn't
	   * coerce arguments.
	   *
	   * @private
	   * @param {number} start The start of the range.
	   * @param {number} end The end of the range.
	   * @param {number} step The value to increment or decrement by.
	   * @param {boolean} [fromRight] Specify iterating from right to left.
	   * @returns {Array} Returns the range of numbers.
	   */
	  function baseRange(start, end, step, fromRight) {
	    var index = -1,
	      length = nativeMax(nativeCeil((end - start) / (step || 1)), 0),
	      result = Array(length);
	    while (length--) {
	      result[fromRight ? length : ++index] = start;
	      start += step;
	    }
	    return result;
	  }

	  /**
	   * The same as [times]{@link module:ControlFlow.times} but runs a maximum of `limit` async operations at a
	   * time.
	   *
	   * @name timesLimit
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.times]{@link module:ControlFlow.times}
	   * @category Control Flow
	   * @param {number} count - The number of times to run the function.
	   * @param {number} limit - The maximum number of async operations at a time.
	   * @param {AsyncFunction} iteratee - The async function to call `n` times.
	   * Invoked with the iteration index and a callback: (n, next).
	   * @param {Function} callback - see [async.map]{@link module:Collections.map}.
	   */
	  function timeLimit(count, limit, iteratee, callback) {
	    var _iteratee = wrapAsync(iteratee);
	    mapLimit(baseRange(0, count, 1), limit, _iteratee, callback);
	  }

	  /**
	   * Calls the `iteratee` function `n` times, and accumulates results in the same
	   * manner you would use with [map]{@link module:Collections.map}.
	   *
	   * @name times
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.map]{@link module:Collections.map}
	   * @category Control Flow
	   * @param {number} n - The number of times to run the function.
	   * @param {AsyncFunction} iteratee - The async function to call `n` times.
	   * Invoked with the iteration index and a callback: (n, next).
	   * @param {Function} callback - see {@link module:Collections.map}.
	   * @example
	   *
	   * // Pretend this is some complicated async factory
	   * var createUser = function(id, callback) {
	   *     callback(null, {
	   *         id: 'user' + id
	   *     });
	   * };
	   *
	   * // generate 5 users
	   * async.times(5, function(n, next) {
	   *     createUser(n, function(err, user) {
	   *         next(err, user);
	   *     });
	   * }, function(err, users) {
	   *     // we should now have 5 users
	   * });
	   */
	  var times = doLimit(timeLimit, Infinity);

	  /**
	   * The same as [times]{@link module:ControlFlow.times} but runs only a single async operation at a time.
	   *
	   * @name timesSeries
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.times]{@link module:ControlFlow.times}
	   * @category Control Flow
	   * @param {number} n - The number of times to run the function.
	   * @param {AsyncFunction} iteratee - The async function to call `n` times.
	   * Invoked with the iteration index and a callback: (n, next).
	   * @param {Function} callback - see {@link module:Collections.map}.
	   */
	  var timesSeries = doLimit(timeLimit, 1);

	  /**
	   * A relative of `reduce`.  Takes an Object or Array, and iterates over each
	   * element in series, each step potentially mutating an `accumulator` value.
	   * The type of the accumulator defaults to the type of collection passed in.
	   *
	   * @name transform
	   * @static
	   * @memberOf module:Collections
	   * @method
	   * @category Collection
	   * @param {Array|Iterable|Object} coll - A collection to iterate over.
	   * @param {*} [accumulator] - The initial state of the transform.  If omitted,
	   * it will default to an empty Object or Array, depending on the type of `coll`
	   * @param {AsyncFunction} iteratee - A function applied to each item in the
	   * collection that potentially modifies the accumulator.
	   * Invoked with (accumulator, item, key, callback).
	   * @param {Function} [callback] - A callback which is called after all the
	   * `iteratee` functions have finished. Result is the transformed accumulator.
	   * Invoked with (err, result).
	   * @example
	   *
	   * async.transform([1,2,3], function(acc, item, index, callback) {
	   *     // pointless async:
	   *     process.nextTick(function() {
	   *         acc.push(item * 2)
	   *         callback(null)
	   *     });
	   * }, function(err, result) {
	   *     // result is now equal to [2, 4, 6]
	   * });
	   *
	   * @example
	   *
	   * async.transform({a: 1, b: 2, c: 3}, function (obj, val, key, callback) {
	   *     setImmediate(function () {
	   *         obj[key] = val * 2;
	   *         callback();
	   *     })
	   * }, function (err, result) {
	   *     // result is equal to {a: 2, b: 4, c: 6}
	   * })
	   */
	  function transform(coll, accumulator, iteratee, callback) {
	    if (arguments.length <= 3) {
	      callback = iteratee;
	      iteratee = accumulator;
	      accumulator = isArray(coll) ? [] : {};
	    }
	    callback = once(callback || noop);
	    var _iteratee = wrapAsync(iteratee);
	    eachOf(coll, function (v, k, cb) {
	      _iteratee(accumulator, v, k, cb);
	    }, function (err) {
	      callback(err, accumulator);
	    });
	  }

	  /**
	   * It runs each task in series but stops whenever any of the functions were
	   * successful. If one of the tasks were successful, the `callback` will be
	   * passed the result of the successful task. If all tasks fail, the callback
	   * will be passed the error and result (if any) of the final attempt.
	   *
	   * @name tryEach
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {Array|Iterable|Object} tasks - A collection containing functions to
	   * run, each function is passed a `callback(err, result)` it must call on
	   * completion with an error `err` (which can be `null`) and an optional `result`
	   * value.
	   * @param {Function} [callback] - An optional callback which is called when one
	   * of the tasks has succeeded, or all have failed. It receives the `err` and
	   * `result` arguments of the last attempt at completing the `task`. Invoked with
	   * (err, results).
	   * @example
	   * async.tryEach([
	   *     function getDataFromFirstWebsite(callback) {
	   *         // Try getting the data from the first website
	   *         callback(err, data);
	   *     },
	   *     function getDataFromSecondWebsite(callback) {
	   *         // First website failed,
	   *         // Try getting the data from the backup website
	   *         callback(err, data);
	   *     }
	   * ],
	   * // optional callback
	   * function(err, results) {
	   *     Now do something with the data.
	   * });
	   *
	   */
	  function tryEach(tasks, callback) {
	    var error = null;
	    var result;
	    callback = callback || noop;
	    eachSeries(tasks, function (task, callback) {
	      wrapAsync(task)(function (err, res /*, ...args*/) {
	        if (arguments.length > 2) {
	          result = slice(arguments, 1);
	        } else {
	          result = res;
	        }
	        error = err;
	        callback(!err);
	      });
	    }, function () {
	      callback(error, result);
	    });
	  }

	  /**
	   * Undoes a [memoize]{@link module:Utils.memoize}d function, reverting it to the original,
	   * unmemoized form. Handy for testing.
	   *
	   * @name unmemoize
	   * @static
	   * @memberOf module:Utils
	   * @method
	   * @see [async.memoize]{@link module:Utils.memoize}
	   * @category Util
	   * @param {AsyncFunction} fn - the memoized function
	   * @returns {AsyncFunction} a function that calls the original unmemoized function
	   */
	  function unmemoize(fn) {
	    return function () {
	      return (fn.unmemoized || fn).apply(null, arguments);
	    };
	  }

	  /**
	   * Repeatedly call `iteratee`, while `test` returns `true`. Calls `callback` when
	   * stopped, or an error occurs.
	   *
	   * @name whilst
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {Function} test - synchronous truth test to perform before each
	   * execution of `iteratee`. Invoked with ().
	   * @param {AsyncFunction} iteratee - An async function which is called each time
	   * `test` passes. Invoked with (callback).
	   * @param {Function} [callback] - A callback which is called after the test
	   * function has failed and repeated execution of `iteratee` has stopped. `callback`
	   * will be passed an error and any arguments passed to the final `iteratee`'s
	   * callback. Invoked with (err, [results]);
	   * @returns undefined
	   * @example
	   *
	   * var count = 0;
	   * async.whilst(
	   *     function() { return count < 5; },
	   *     function(callback) {
	   *         count++;
	   *         setTimeout(function() {
	   *             callback(null, count);
	   *         }, 1000);
	   *     },
	   *     function (err, n) {
	   *         // 5 seconds have passed, n = 5
	   *     }
	   * );
	   */
	  function whilst(test, iteratee, callback) {
	    callback = onlyOnce(callback || noop);
	    var _iteratee = wrapAsync(iteratee);
	    if (!test()) return callback(null);
	    var next = function (err /*, ...args*/) {
	      if (err) return callback(err);
	      if (test()) return _iteratee(next);
	      var args = slice(arguments, 1);
	      callback.apply(null, [null].concat(args));
	    };
	    _iteratee(next);
	  }

	  /**
	   * Repeatedly call `iteratee` until `test` returns `true`. Calls `callback` when
	   * stopped, or an error occurs. `callback` will be passed an error and any
	   * arguments passed to the final `iteratee`'s callback.
	   *
	   * The inverse of [whilst]{@link module:ControlFlow.whilst}.
	   *
	   * @name until
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @see [async.whilst]{@link module:ControlFlow.whilst}
	   * @category Control Flow
	   * @param {Function} test - synchronous truth test to perform before each
	   * execution of `iteratee`. Invoked with ().
	   * @param {AsyncFunction} iteratee - An async function which is called each time
	   * `test` fails. Invoked with (callback).
	   * @param {Function} [callback] - A callback which is called after the test
	   * function has passed and repeated execution of `iteratee` has stopped. `callback`
	   * will be passed an error and any arguments passed to the final `iteratee`'s
	   * callback. Invoked with (err, [results]);
	   */
	  function until(test, iteratee, callback) {
	    whilst(function () {
	      return !test.apply(this, arguments);
	    }, iteratee, callback);
	  }

	  /**
	   * Runs the `tasks` array of functions in series, each passing their results to
	   * the next in the array. However, if any of the `tasks` pass an error to their
	   * own callback, the next function is not executed, and the main `callback` is
	   * immediately called with the error.
	   *
	   * @name waterfall
	   * @static
	   * @memberOf module:ControlFlow
	   * @method
	   * @category Control Flow
	   * @param {Array} tasks - An array of [async functions]{@link AsyncFunction}
	   * to run.
	   * Each function should complete with any number of `result` values.
	   * The `result` values will be passed as arguments, in order, to the next task.
	   * @param {Function} [callback] - An optional callback to run once all the
	   * functions have completed. This will be passed the results of the last task's
	   * callback. Invoked with (err, [results]).
	   * @returns undefined
	   * @example
	   *
	   * async.waterfall([
	   *     function(callback) {
	   *         callback(null, 'one', 'two');
	   *     },
	   *     function(arg1, arg2, callback) {
	   *         // arg1 now equals 'one' and arg2 now equals 'two'
	   *         callback(null, 'three');
	   *     },
	   *     function(arg1, callback) {
	   *         // arg1 now equals 'three'
	   *         callback(null, 'done');
	   *     }
	   * ], function (err, result) {
	   *     // result now equals 'done'
	   * });
	   *
	   * // Or, with named functions:
	   * async.waterfall([
	   *     myFirstFunction,
	   *     mySecondFunction,
	   *     myLastFunction,
	   * ], function (err, result) {
	   *     // result now equals 'done'
	   * });
	   * function myFirstFunction(callback) {
	   *     callback(null, 'one', 'two');
	   * }
	   * function mySecondFunction(arg1, arg2, callback) {
	   *     // arg1 now equals 'one' and arg2 now equals 'two'
	   *     callback(null, 'three');
	   * }
	   * function myLastFunction(arg1, callback) {
	   *     // arg1 now equals 'three'
	   *     callback(null, 'done');
	   * }
	   */
	  var waterfall = function (tasks, callback) {
	    callback = once(callback || noop);
	    if (!isArray(tasks)) return callback(new Error('First argument to waterfall must be an array of functions'));
	    if (!tasks.length) return callback();
	    var taskIndex = 0;
	    function nextTask(args) {
	      var task = wrapAsync(tasks[taskIndex++]);
	      args.push(onlyOnce(next));
	      task.apply(null, args);
	    }
	    function next(err /*, ...args*/) {
	      if (err || taskIndex === tasks.length) {
	        return callback.apply(null, arguments);
	      }
	      nextTask(slice(arguments, 1));
	    }
	    nextTask([]);
	  };

	  /**
	   * An "async function" in the context of Async is an asynchronous function with
	   * a variable number of parameters, with the final parameter being a callback.
	   * (`function (arg1, arg2, ..., callback) {}`)
	   * The final callback is of the form `callback(err, results...)`, which must be
	   * called once the function is completed.  The callback should be called with a
	   * Error as its first argument to signal that an error occurred.
	   * Otherwise, if no error occurred, it should be called with `null` as the first
	   * argument, and any additional `result` arguments that may apply, to signal
	   * successful completion.
	   * The callback must be called exactly once, ideally on a later tick of the
	   * JavaScript event loop.
	   *
	   * This type of function is also referred to as a "Node-style async function",
	   * or a "continuation passing-style function" (CPS). Most of the methods of this
	   * library are themselves CPS/Node-style async functions, or functions that
	   * return CPS/Node-style async functions.
	   *
	   * Wherever we accept a Node-style async function, we also directly accept an
	   * [ES2017 `async` function]{@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/async_function}.
	   * In this case, the `async` function will not be passed a final callback
	   * argument, and any thrown error will be used as the `err` argument of the
	   * implicit callback, and the return value will be used as the `result` value.
	   * (i.e. a `rejected` of the returned Promise becomes the `err` callback
	   * argument, and a `resolved` value becomes the `result`.)
	   *
	   * Note, due to JavaScript limitations, we can only detect native `async`
	   * functions and not transpilied implementations.
	   * Your environment must have `async`/`await` support for this to work.
	   * (e.g. Node > v7.6, or a recent version of a modern browser).
	   * If you are using `async` functions through a transpiler (e.g. Babel), you
	   * must still wrap the function with [asyncify]{@link module:Utils.asyncify},
	   * because the `async function` will be compiled to an ordinary function that
	   * returns a promise.
	   *
	   * @typedef {Function} AsyncFunction
	   * @static
	   */

	  /**
	   * Async is a utility module which provides straight-forward, powerful functions
	   * for working with asynchronous JavaScript. Although originally designed for
	   * use with [Node.js](http://nodejs.org) and installable via
	   * `npm install --save async`, it can also be used directly in the browser.
	   * @module async
	   * @see AsyncFunction
	   */

	  /**
	   * A collection of `async` functions for manipulating collections, such as
	   * arrays and objects.
	   * @module Collections
	   */

	  /**
	   * A collection of `async` functions for controlling the flow through a script.
	   * @module ControlFlow
	   */

	  /**
	   * A collection of `async` utility functions.
	   * @module Utils
	   */

	  var index = {
	    apply: apply,
	    applyEach: applyEach,
	    applyEachSeries: applyEachSeries,
	    asyncify: asyncify,
	    auto: auto,
	    autoInject: autoInject,
	    cargo: cargo,
	    compose: compose,
	    concat: concat,
	    concatLimit: concatLimit,
	    concatSeries: concatSeries,
	    constant: constant,
	    detect: detect,
	    detectLimit: detectLimit,
	    detectSeries: detectSeries,
	    dir: dir,
	    doDuring: doDuring,
	    doUntil: doUntil,
	    doWhilst: doWhilst,
	    during: during,
	    each: eachLimit,
	    eachLimit: eachLimit$1,
	    eachOf: eachOf,
	    eachOfLimit: eachOfLimit,
	    eachOfSeries: eachOfSeries,
	    eachSeries: eachSeries,
	    ensureAsync: ensureAsync,
	    every: every,
	    everyLimit: everyLimit,
	    everySeries: everySeries,
	    filter: filter,
	    filterLimit: filterLimit,
	    filterSeries: filterSeries,
	    forever: forever,
	    groupBy: groupBy,
	    groupByLimit: groupByLimit,
	    groupBySeries: groupBySeries,
	    log: log,
	    map: map,
	    mapLimit: mapLimit,
	    mapSeries: mapSeries,
	    mapValues: mapValues,
	    mapValuesLimit: mapValuesLimit,
	    mapValuesSeries: mapValuesSeries,
	    memoize: memoize,
	    nextTick: nextTick,
	    parallel: parallelLimit,
	    parallelLimit: parallelLimit$1,
	    priorityQueue: priorityQueue,
	    queue: queue$1,
	    race: race,
	    reduce: reduce,
	    reduceRight: reduceRight,
	    reflect: reflect,
	    reflectAll: reflectAll,
	    reject: reject,
	    rejectLimit: rejectLimit,
	    rejectSeries: rejectSeries,
	    retry: retry,
	    retryable: retryable,
	    seq: seq,
	    series: series,
	    setImmediate: setImmediate$1,
	    some: some,
	    someLimit: someLimit,
	    someSeries: someSeries,
	    sortBy: sortBy,
	    timeout: timeout,
	    times: times,
	    timesLimit: timeLimit,
	    timesSeries: timesSeries,
	    transform: transform,
	    tryEach: tryEach,
	    unmemoize: unmemoize,
	    until: until,
	    waterfall: waterfall,
	    whilst: whilst,
	    // aliases
	    all: every,
	    allLimit: everyLimit,
	    allSeries: everySeries,
	    any: some,
	    anyLimit: someLimit,
	    anySeries: someSeries,
	    find: detect,
	    findLimit: detectLimit,
	    findSeries: detectSeries,
	    forEach: eachLimit,
	    forEachSeries: eachSeries,
	    forEachLimit: eachLimit$1,
	    forEachOf: eachOf,
	    forEachOfSeries: eachOfSeries,
	    forEachOfLimit: eachOfLimit,
	    inject: reduce,
	    foldl: reduce,
	    foldr: reduceRight,
	    select: filter,
	    selectLimit: filterLimit,
	    selectSeries: filterSeries,
	    wrapSync: asyncify
	  };
	  exports['default'] = index;
	  exports.apply = apply;
	  exports.applyEach = applyEach;
	  exports.applyEachSeries = applyEachSeries;
	  exports.asyncify = asyncify;
	  exports.auto = auto;
	  exports.autoInject = autoInject;
	  exports.cargo = cargo;
	  exports.compose = compose;
	  exports.concat = concat;
	  exports.concatLimit = concatLimit;
	  exports.concatSeries = concatSeries;
	  exports.constant = constant;
	  exports.detect = detect;
	  exports.detectLimit = detectLimit;
	  exports.detectSeries = detectSeries;
	  exports.dir = dir;
	  exports.doDuring = doDuring;
	  exports.doUntil = doUntil;
	  exports.doWhilst = doWhilst;
	  exports.during = during;
	  exports.each = eachLimit;
	  exports.eachLimit = eachLimit$1;
	  exports.eachOf = eachOf;
	  exports.eachOfLimit = eachOfLimit;
	  exports.eachOfSeries = eachOfSeries;
	  exports.eachSeries = eachSeries;
	  exports.ensureAsync = ensureAsync;
	  exports.every = every;
	  exports.everyLimit = everyLimit;
	  exports.everySeries = everySeries;
	  exports.filter = filter;
	  exports.filterLimit = filterLimit;
	  exports.filterSeries = filterSeries;
	  exports.forever = forever;
	  exports.groupBy = groupBy;
	  exports.groupByLimit = groupByLimit;
	  exports.groupBySeries = groupBySeries;
	  exports.log = log;
	  exports.map = map;
	  exports.mapLimit = mapLimit;
	  exports.mapSeries = mapSeries;
	  exports.mapValues = mapValues;
	  exports.mapValuesLimit = mapValuesLimit;
	  exports.mapValuesSeries = mapValuesSeries;
	  exports.memoize = memoize;
	  exports.nextTick = nextTick;
	  exports.parallel = parallelLimit;
	  exports.parallelLimit = parallelLimit$1;
	  exports.priorityQueue = priorityQueue;
	  exports.queue = queue$1;
	  exports.race = race;
	  exports.reduce = reduce;
	  exports.reduceRight = reduceRight;
	  exports.reflect = reflect;
	  exports.reflectAll = reflectAll;
	  exports.reject = reject;
	  exports.rejectLimit = rejectLimit;
	  exports.rejectSeries = rejectSeries;
	  exports.retry = retry;
	  exports.retryable = retryable;
	  exports.seq = seq;
	  exports.series = series;
	  exports.setImmediate = setImmediate$1;
	  exports.some = some;
	  exports.someLimit = someLimit;
	  exports.someSeries = someSeries;
	  exports.sortBy = sortBy;
	  exports.timeout = timeout;
	  exports.times = times;
	  exports.timesLimit = timeLimit;
	  exports.timesSeries = timesSeries;
	  exports.transform = transform;
	  exports.tryEach = tryEach;
	  exports.unmemoize = unmemoize;
	  exports.until = until;
	  exports.waterfall = waterfall;
	  exports.whilst = whilst;
	  exports.all = every;
	  exports.allLimit = everyLimit;
	  exports.allSeries = everySeries;
	  exports.any = some;
	  exports.anyLimit = someLimit;
	  exports.anySeries = someSeries;
	  exports.find = detect;
	  exports.findLimit = detectLimit;
	  exports.findSeries = detectSeries;
	  exports.forEach = eachLimit;
	  exports.forEachSeries = eachSeries;
	  exports.forEachLimit = eachLimit$1;
	  exports.forEachOf = eachOf;
	  exports.forEachOfSeries = eachOfSeries;
	  exports.forEachOfLimit = eachOfLimit;
	  exports.inject = reduce;
	  exports.foldl = reduce;
	  exports.foldr = reduceRight;
	  exports.select = filter;
	  exports.selectLimit = filterLimit;
	  exports.selectSeries = filterSeries;
	  exports.wrapSync = asyncify;
	  Object.defineProperty(exports, '__esModule', {
	    value: true
	  });
	}); 
} (async, async.exports));

var asyncExports = async.exports;

var src = {exports: {}};

var node = {exports: {}};

var hasFlag;
var hasRequiredHasFlag;

function requireHasFlag () {
	if (hasRequiredHasFlag) return hasFlag;
	hasRequiredHasFlag = 1;

	hasFlag = (flag, argv = process.argv) => {
	  const prefix = flag.startsWith('-') ? '' : flag.length === 1 ? '-' : '--';
	  const position = argv.indexOf(prefix + flag);
	  const terminatorPosition = argv.indexOf('--');
	  return position !== -1 && (terminatorPosition === -1 || position < terminatorPosition);
	};
	return hasFlag;
}

var supportsColor_1;
var hasRequiredSupportsColor;

function requireSupportsColor () {
	if (hasRequiredSupportsColor) return supportsColor_1;
	hasRequiredSupportsColor = 1;

	const os = require$$0$1;
	const tty = require$$1;
	const hasFlag = requireHasFlag();
	const {
	  env
	} = process;
	let forceColor;
	if (hasFlag('no-color') || hasFlag('no-colors') || hasFlag('color=false') || hasFlag('color=never')) {
	  forceColor = 0;
	} else if (hasFlag('color') || hasFlag('colors') || hasFlag('color=true') || hasFlag('color=always')) {
	  forceColor = 1;
	}
	if ('FORCE_COLOR' in env) {
	  if (env.FORCE_COLOR === 'true') {
	    forceColor = 1;
	  } else if (env.FORCE_COLOR === 'false') {
	    forceColor = 0;
	  } else {
	    forceColor = env.FORCE_COLOR.length === 0 ? 1 : Math.min(parseInt(env.FORCE_COLOR, 10), 3);
	  }
	}
	function translateLevel(level) {
	  if (level === 0) {
	    return false;
	  }
	  return {
	    level,
	    hasBasic: true,
	    has256: level >= 2,
	    has16m: level >= 3
	  };
	}
	function supportsColor(haveStream, streamIsTTY) {
	  if (forceColor === 0) {
	    return 0;
	  }
	  if (hasFlag('color=16m') || hasFlag('color=full') || hasFlag('color=truecolor')) {
	    return 3;
	  }
	  if (hasFlag('color=256')) {
	    return 2;
	  }
	  if (haveStream && !streamIsTTY && forceColor === undefined) {
	    return 0;
	  }
	  const min = forceColor || 0;
	  if (env.TERM === 'dumb') {
	    return min;
	  }
	  if (process.platform === 'win32') {
	    // Windows 10 build 10586 is the first Windows release that supports 256 colors.
	    // Windows 10 build 14931 is the first release that supports 16m/TrueColor.
	    const osRelease = os.release().split('.');
	    if (Number(osRelease[0]) >= 10 && Number(osRelease[2]) >= 10586) {
	      return Number(osRelease[2]) >= 14931 ? 3 : 2;
	    }
	    return 1;
	  }
	  if ('CI' in env) {
	    if (['TRAVIS', 'CIRCLECI', 'APPVEYOR', 'GITLAB_CI', 'GITHUB_ACTIONS', 'BUILDKITE'].some(sign => sign in env) || env.CI_NAME === 'codeship') {
	      return 1;
	    }
	    return min;
	  }
	  if ('TEAMCITY_VERSION' in env) {
	    return /^(9\.(0*[1-9]\d*)\.|\d{2,}\.)/.test(env.TEAMCITY_VERSION) ? 1 : 0;
	  }
	  if (env.COLORTERM === 'truecolor') {
	    return 3;
	  }
	  if ('TERM_PROGRAM' in env) {
	    const version = parseInt((env.TERM_PROGRAM_VERSION || '').split('.')[0], 10);
	    switch (env.TERM_PROGRAM) {
	      case 'iTerm.app':
	        return version >= 3 ? 3 : 2;
	      case 'Apple_Terminal':
	        return 2;
	      // No default
	    }
	  }

	  if (/-256(color)?$/i.test(env.TERM)) {
	    return 2;
	  }
	  if (/^screen|^xterm|^vt100|^vt220|^rxvt|color|ansi|cygwin|linux/i.test(env.TERM)) {
	    return 1;
	  }
	  if ('COLORTERM' in env) {
	    return 1;
	  }
	  return min;
	}
	function getSupportLevel(stream) {
	  const level = supportsColor(stream, stream && stream.isTTY);
	  return translateLevel(level);
	}
	supportsColor_1 = {
	  supportsColor: getSupportLevel,
	  stdout: translateLevel(supportsColor(true, tty.isatty(1))),
	  stderr: translateLevel(supportsColor(true, tty.isatty(2)))
	};
	return supportsColor_1;
}

/**
 * Helpers.
 */

var ms;
var hasRequiredMs;

function requireMs () {
	if (hasRequiredMs) return ms;
	hasRequiredMs = 1;
	var s = 1000;
	var m = s * 60;
	var h = m * 60;
	var d = h * 24;
	var w = d * 7;
	var y = d * 365.25;

	/**
	 * Parse or format the given `val`.
	 *
	 * Options:
	 *
	 *  - `long` verbose formatting [false]
	 *
	 * @param {String|Number} val
	 * @param {Object} [options]
	 * @throws {Error} throw an error if val is not a non-empty string or a number
	 * @return {String|Number}
	 * @api public
	 */

	ms = function (val, options) {
	  options = options || {};
	  var type = typeof val;
	  if (type === 'string' && val.length > 0) {
	    return parse(val);
	  } else if (type === 'number' && isFinite(val)) {
	    return options.long ? fmtLong(val) : fmtShort(val);
	  }
	  throw new Error('val is not a non-empty string or a valid number. val=' + JSON.stringify(val));
	};

	/**
	 * Parse the given `str` and return milliseconds.
	 *
	 * @param {String} str
	 * @return {Number}
	 * @api private
	 */

	function parse(str) {
	  str = String(str);
	  if (str.length > 100) {
	    return;
	  }
	  var match = /^(-?(?:\d+)?\.?\d+) *(milliseconds?|msecs?|ms|seconds?|secs?|s|minutes?|mins?|m|hours?|hrs?|h|days?|d|weeks?|w|years?|yrs?|y)?$/i.exec(str);
	  if (!match) {
	    return;
	  }
	  var n = parseFloat(match[1]);
	  var type = (match[2] || 'ms').toLowerCase();
	  switch (type) {
	    case 'years':
	    case 'year':
	    case 'yrs':
	    case 'yr':
	    case 'y':
	      return n * y;
	    case 'weeks':
	    case 'week':
	    case 'w':
	      return n * w;
	    case 'days':
	    case 'day':
	    case 'd':
	      return n * d;
	    case 'hours':
	    case 'hour':
	    case 'hrs':
	    case 'hr':
	    case 'h':
	      return n * h;
	    case 'minutes':
	    case 'minute':
	    case 'mins':
	    case 'min':
	    case 'm':
	      return n * m;
	    case 'seconds':
	    case 'second':
	    case 'secs':
	    case 'sec':
	    case 's':
	      return n * s;
	    case 'milliseconds':
	    case 'millisecond':
	    case 'msecs':
	    case 'msec':
	    case 'ms':
	      return n;
	    default:
	      return undefined;
	  }
	}

	/**
	 * Short format for `ms`.
	 *
	 * @param {Number} ms
	 * @return {String}
	 * @api private
	 */

	function fmtShort(ms) {
	  var msAbs = Math.abs(ms);
	  if (msAbs >= d) {
	    return Math.round(ms / d) + 'd';
	  }
	  if (msAbs >= h) {
	    return Math.round(ms / h) + 'h';
	  }
	  if (msAbs >= m) {
	    return Math.round(ms / m) + 'm';
	  }
	  if (msAbs >= s) {
	    return Math.round(ms / s) + 's';
	  }
	  return ms + 'ms';
	}

	/**
	 * Long format for `ms`.
	 *
	 * @param {Number} ms
	 * @return {String}
	 * @api private
	 */

	function fmtLong(ms) {
	  var msAbs = Math.abs(ms);
	  if (msAbs >= d) {
	    return plural(ms, msAbs, d, 'day');
	  }
	  if (msAbs >= h) {
	    return plural(ms, msAbs, h, 'hour');
	  }
	  if (msAbs >= m) {
	    return plural(ms, msAbs, m, 'minute');
	  }
	  if (msAbs >= s) {
	    return plural(ms, msAbs, s, 'second');
	  }
	  return ms + ' ms';
	}

	/**
	 * Pluralization helper.
	 */

	function plural(ms, msAbs, n, name) {
	  var isPlural = msAbs >= n * 1.5;
	  return Math.round(ms / n) + ' ' + name + (isPlural ? 's' : '');
	}
	return ms;
}

var common;
var hasRequiredCommon;

function requireCommon () {
	if (hasRequiredCommon) return common;
	hasRequiredCommon = 1;

	/**
	 * This is the common logic for both the Node.js and web browser
	 * implementations of `debug()`.
	 */
	function setup(env) {
	  createDebug.debug = createDebug;
	  createDebug.default = createDebug;
	  createDebug.coerce = coerce;
	  createDebug.disable = disable;
	  createDebug.enable = enable;
	  createDebug.enabled = enabled;
	  createDebug.humanize = requireMs();
	  Object.keys(env).forEach(function (key) {
	    createDebug[key] = env[key];
	  });
	  /**
	  * Active `debug` instances.
	  */

	  createDebug.instances = [];
	  /**
	  * The currently active debug mode names, and names to skip.
	  */

	  createDebug.names = [];
	  createDebug.skips = [];
	  /**
	  * Map of special "%n" handling functions, for the debug "format" argument.
	  *
	  * Valid key names are a single, lower or upper-case letter, i.e. "n" and "N".
	  */

	  createDebug.formatters = {};
	  /**
	  * Selects a color for a debug namespace
	  * @param {String} namespace The namespace string for the for the debug instance to be colored
	  * @return {Number|String} An ANSI color code for the given namespace
	  * @api private
	  */

	  function selectColor(namespace) {
	    var hash = 0;
	    for (var i = 0; i < namespace.length; i++) {
	      hash = (hash << 5) - hash + namespace.charCodeAt(i);
	      hash |= 0; // Convert to 32bit integer
	    }

	    return createDebug.colors[Math.abs(hash) % createDebug.colors.length];
	  }
	  createDebug.selectColor = selectColor;
	  /**
	  * Create a debugger with the given `namespace`.
	  *
	  * @param {String} namespace
	  * @return {Function}
	  * @api public
	  */

	  function createDebug(namespace) {
	    var prevTime;
	    function debug() {
	      // Disabled?
	      if (!debug.enabled) {
	        return;
	      }
	      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
	        args[_key] = arguments[_key];
	      }
	      var self = debug; // Set `diff` timestamp

	      var curr = Number(new Date());
	      var ms = curr - (prevTime || curr);
	      self.diff = ms;
	      self.prev = prevTime;
	      self.curr = curr;
	      prevTime = curr;
	      args[0] = createDebug.coerce(args[0]);
	      if (typeof args[0] !== 'string') {
	        // Anything else let's inspect with %O
	        args.unshift('%O');
	      } // Apply any `formatters` transformations

	      var index = 0;
	      args[0] = args[0].replace(/%([a-zA-Z%])/g, function (match, format) {
	        // If we encounter an escaped % then don't increase the array index
	        if (match === '%%') {
	          return match;
	        }
	        index++;
	        var formatter = createDebug.formatters[format];
	        if (typeof formatter === 'function') {
	          var val = args[index];
	          match = formatter.call(self, val); // Now we need to remove `args[index]` since it's inlined in the `format`

	          args.splice(index, 1);
	          index--;
	        }
	        return match;
	      }); // Apply env-specific formatting (colors, etc.)

	      createDebug.formatArgs.call(self, args);
	      var logFn = self.log || createDebug.log;
	      logFn.apply(self, args);
	    }
	    debug.namespace = namespace;
	    debug.enabled = createDebug.enabled(namespace);
	    debug.useColors = createDebug.useColors();
	    debug.color = selectColor(namespace);
	    debug.destroy = destroy;
	    debug.extend = extend; // Debug.formatArgs = formatArgs;
	    // debug.rawLog = rawLog;
	    // env-specific initialization logic for debug instances

	    if (typeof createDebug.init === 'function') {
	      createDebug.init(debug);
	    }
	    createDebug.instances.push(debug);
	    return debug;
	  }
	  function destroy() {
	    var index = createDebug.instances.indexOf(this);
	    if (index !== -1) {
	      createDebug.instances.splice(index, 1);
	      return true;
	    }
	    return false;
	  }
	  function extend(namespace, delimiter) {
	    return createDebug(this.namespace + (typeof delimiter === 'undefined' ? ':' : delimiter) + namespace);
	  }
	  /**
	  * Enables a debug mode by namespaces. This can include modes
	  * separated by a colon and wildcards.
	  *
	  * @param {String} namespaces
	  * @api public
	  */

	  function enable(namespaces) {
	    createDebug.save(namespaces);
	    createDebug.names = [];
	    createDebug.skips = [];
	    var i;
	    var split = (typeof namespaces === 'string' ? namespaces : '').split(/[\s,]+/);
	    var len = split.length;
	    for (i = 0; i < len; i++) {
	      if (!split[i]) {
	        // ignore empty strings
	        continue;
	      }
	      namespaces = split[i].replace(/\*/g, '.*?');
	      if (namespaces[0] === '-') {
	        createDebug.skips.push(new RegExp('^' + namespaces.substr(1) + '$'));
	      } else {
	        createDebug.names.push(new RegExp('^' + namespaces + '$'));
	      }
	    }
	    for (i = 0; i < createDebug.instances.length; i++) {
	      var instance = createDebug.instances[i];
	      instance.enabled = createDebug.enabled(instance.namespace);
	    }
	  }
	  /**
	  * Disable debug output.
	  *
	  * @api public
	  */

	  function disable() {
	    createDebug.enable('');
	  }
	  /**
	  * Returns true if the given mode name is enabled, false otherwise.
	  *
	  * @param {String} name
	  * @return {Boolean}
	  * @api public
	  */

	  function enabled(name) {
	    if (name[name.length - 1] === '*') {
	      return true;
	    }
	    var i;
	    var len;
	    for (i = 0, len = createDebug.skips.length; i < len; i++) {
	      if (createDebug.skips[i].test(name)) {
	        return false;
	      }
	    }
	    for (i = 0, len = createDebug.names.length; i < len; i++) {
	      if (createDebug.names[i].test(name)) {
	        return true;
	      }
	    }
	    return false;
	  }
	  /**
	  * Coerce `val`.
	  *
	  * @param {Mixed} val
	  * @return {Mixed}
	  * @api private
	  */

	  function coerce(val) {
	    if (val instanceof Error) {
	      return val.stack || val.message;
	    }
	    return val;
	  }
	  createDebug.enable(createDebug.load());
	  return createDebug;
	}
	common = setup;
	return common;
}

var hasRequiredNode;

function requireNode () {
	if (hasRequiredNode) return node.exports;
	hasRequiredNode = 1;
	(function (module, exports) {

		/**
		 * Module dependencies.
		 */
		var tty = require$$1;
		var util = require$$1$1;
		/**
		 * This is the Node.js implementation of `debug()`.
		 */

		exports.init = init;
		exports.log = log;
		exports.formatArgs = formatArgs;
		exports.save = save;
		exports.load = load;
		exports.useColors = useColors;
		/**
		 * Colors.
		 */

		exports.colors = [6, 2, 3, 4, 5, 1];
		try {
		  // Optional dependency (as in, doesn't need to be installed, NOT like optionalDependencies in package.json)
		  // eslint-disable-next-line import/no-extraneous-dependencies
		  var supportsColor = requireSupportsColor();
		  if (supportsColor && (supportsColor.stderr || supportsColor).level >= 2) {
		    exports.colors = [20, 21, 26, 27, 32, 33, 38, 39, 40, 41, 42, 43, 44, 45, 56, 57, 62, 63, 68, 69, 74, 75, 76, 77, 78, 79, 80, 81, 92, 93, 98, 99, 112, 113, 128, 129, 134, 135, 148, 149, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 178, 179, 184, 185, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 214, 215, 220, 221];
		  }
		} catch (error) {} // Swallow - we only care if `supports-color` is available; it doesn't have to be.

		/**
		 * Build up the default `inspectOpts` object from the environment variables.
		 *
		 *   $ DEBUG_COLORS=no DEBUG_DEPTH=10 DEBUG_SHOW_HIDDEN=enabled node script.js
		 */

		exports.inspectOpts = Object.keys(process.env).filter(function (key) {
		  return /^debug_/i.test(key);
		}).reduce(function (obj, key) {
		  // Camel-case
		  var prop = key.substring(6).toLowerCase().replace(/_([a-z])/g, function (_, k) {
		    return k.toUpperCase();
		  }); // Coerce string value into JS value

		  var val = process.env[key];
		  if (/^(yes|on|true|enabled)$/i.test(val)) {
		    val = true;
		  } else if (/^(no|off|false|disabled)$/i.test(val)) {
		    val = false;
		  } else if (val === 'null') {
		    val = null;
		  } else {
		    val = Number(val);
		  }
		  obj[prop] = val;
		  return obj;
		}, {});
		/**
		 * Is stdout a TTY? Colored output is enabled when `true`.
		 */

		function useColors() {
		  return 'colors' in exports.inspectOpts ? Boolean(exports.inspectOpts.colors) : tty.isatty(process.stderr.fd);
		}
		/**
		 * Adds ANSI color escape codes if enabled.
		 *
		 * @api public
		 */

		function formatArgs(args) {
		  var name = this.namespace,
		    useColors = this.useColors;
		  if (useColors) {
		    var c = this.color;
		    var colorCode = "\x1B[3" + (c < 8 ? c : '8;5;' + c);
		    var prefix = "  ".concat(colorCode, ";1m").concat(name, " \x1B[0m");
		    args[0] = prefix + args[0].split('\n').join('\n' + prefix);
		    args.push(colorCode + 'm+' + module.exports.humanize(this.diff) + "\x1B[0m");
		  } else {
		    args[0] = getDate() + name + ' ' + args[0];
		  }
		}
		function getDate() {
		  if (exports.inspectOpts.hideDate) {
		    return '';
		  }
		  return new Date().toISOString() + ' ';
		}
		/**
		 * Invokes `util.format()` with the specified arguments and writes to stderr.
		 */

		function log() {
		  return process.stderr.write(util.format.apply(util, arguments) + '\n');
		}
		/**
		 * Save `namespaces`.
		 *
		 * @param {String} namespaces
		 * @api private
		 */

		function save(namespaces) {
		  if (namespaces) {
		    process.env.DEBUG = namespaces;
		  } else {
		    // If you set a process.env field to null or undefined, it gets cast to the
		    // string 'null' or 'undefined'. Just delete instead.
		    delete process.env.DEBUG;
		  }
		}
		/**
		 * Load `namespaces`.
		 *
		 * @return {String} returns the previously persisted debug modes
		 * @api private
		 */

		function load() {
		  return process.env.DEBUG;
		}
		/**
		 * Init logic for `debug` instances.
		 *
		 * Create a new `inspectOpts` object in case `useColors` is set
		 * differently for a particular `debug` instance.
		 */

		function init(debug) {
		  debug.inspectOpts = {};
		  var keys = Object.keys(exports.inspectOpts);
		  for (var i = 0; i < keys.length; i++) {
		    debug.inspectOpts[keys[i]] = exports.inspectOpts[keys[i]];
		  }
		}
		module.exports = requireCommon()(exports);
		var formatters = module.exports.formatters;
		/**
		 * Map %o to `util.inspect()`, all on a single line.
		 */

		formatters.o = function (v) {
		  this.inspectOpts.colors = this.useColors;
		  return util.inspect(v, this.inspectOpts).split('\n').map(function (str) {
		    return str.trim();
		  }).join(' ');
		};
		/**
		 * Map %O to `util.inspect()`, allowing multiple lines if needed.
		 */

		formatters.O = function (v) {
		  this.inspectOpts.colors = this.useColors;
		  return util.inspect(v, this.inspectOpts);
		}; 
	} (node, node.exports));
	return node.exports;
}

var browser = {exports: {}};

var hasRequiredBrowser;

function requireBrowser () {
	if (hasRequiredBrowser) return browser.exports;
	hasRequiredBrowser = 1;
	(function (module, exports) {

		function _typeof(obj) {
		  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
		    _typeof = function _typeof(obj) {
		      return typeof obj;
		    };
		  } else {
		    _typeof = function _typeof(obj) {
		      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
		    };
		  }
		  return _typeof(obj);
		}

		/* eslint-env browser */

		/**
		 * This is the web browser implementation of `debug()`.
		 */
		exports.log = log;
		exports.formatArgs = formatArgs;
		exports.save = save;
		exports.load = load;
		exports.useColors = useColors;
		exports.storage = localstorage();
		/**
		 * Colors.
		 */

		exports.colors = ['#0000CC', '#0000FF', '#0033CC', '#0033FF', '#0066CC', '#0066FF', '#0099CC', '#0099FF', '#00CC00', '#00CC33', '#00CC66', '#00CC99', '#00CCCC', '#00CCFF', '#3300CC', '#3300FF', '#3333CC', '#3333FF', '#3366CC', '#3366FF', '#3399CC', '#3399FF', '#33CC00', '#33CC33', '#33CC66', '#33CC99', '#33CCCC', '#33CCFF', '#6600CC', '#6600FF', '#6633CC', '#6633FF', '#66CC00', '#66CC33', '#9900CC', '#9900FF', '#9933CC', '#9933FF', '#99CC00', '#99CC33', '#CC0000', '#CC0033', '#CC0066', '#CC0099', '#CC00CC', '#CC00FF', '#CC3300', '#CC3333', '#CC3366', '#CC3399', '#CC33CC', '#CC33FF', '#CC6600', '#CC6633', '#CC9900', '#CC9933', '#CCCC00', '#CCCC33', '#FF0000', '#FF0033', '#FF0066', '#FF0099', '#FF00CC', '#FF00FF', '#FF3300', '#FF3333', '#FF3366', '#FF3399', '#FF33CC', '#FF33FF', '#FF6600', '#FF6633', '#FF9900', '#FF9933', '#FFCC00', '#FFCC33'];
		/**
		 * Currently only WebKit-based Web Inspectors, Firefox >= v31,
		 * and the Firebug extension (any Firefox version) are known
		 * to support "%c" CSS customizations.
		 *
		 * TODO: add a `localStorage` variable to explicitly enable/disable colors
		 */
		// eslint-disable-next-line complexity

		function useColors() {
		  // NB: In an Electron preload script, document will be defined but not fully
		  // initialized. Since we know we're in Chrome, we'll just detect this case
		  // explicitly
		  if (typeof window !== 'undefined' && window.process && (window.process.type === 'renderer' || window.process.__nwjs)) {
		    return true;
		  } // Internet Explorer and Edge do not support colors.

		  if (typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/(edge|trident)\/(\d+)/)) {
		    return false;
		  } // Is webkit? http://stackoverflow.com/a/16459606/376773
		  // document is undefined in react-native: https://github.com/facebook/react-native/pull/1632

		  return typeof document !== 'undefined' && document.documentElement && document.documentElement.style && document.documentElement.style.WebkitAppearance ||
		  // Is firebug? http://stackoverflow.com/a/398120/376773
		  typeof window !== 'undefined' && window.console && (window.console.firebug || window.console.exception && window.console.table) ||
		  // Is firefox >= v31?
		  // https://developer.mozilla.org/en-US/docs/Tools/Web_Console#Styling_messages
		  typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/firefox\/(\d+)/) && parseInt(RegExp.$1, 10) >= 31 ||
		  // Double check webkit in userAgent just in case we are in a worker
		  typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/applewebkit\/(\d+)/);
		}
		/**
		 * Colorize log arguments if enabled.
		 *
		 * @api public
		 */

		function formatArgs(args) {
		  args[0] = (this.useColors ? '%c' : '') + this.namespace + (this.useColors ? ' %c' : ' ') + args[0] + (this.useColors ? '%c ' : ' ') + '+' + module.exports.humanize(this.diff);
		  if (!this.useColors) {
		    return;
		  }
		  var c = 'color: ' + this.color;
		  args.splice(1, 0, c, 'color: inherit'); // The final "%c" is somewhat tricky, because there could be other
		  // arguments passed either before or after the %c, so we need to
		  // figure out the correct index to insert the CSS into

		  var index = 0;
		  var lastC = 0;
		  args[0].replace(/%[a-zA-Z%]/g, function (match) {
		    if (match === '%%') {
		      return;
		    }
		    index++;
		    if (match === '%c') {
		      // We only are interested in the *last* %c
		      // (the user may have provided their own)
		      lastC = index;
		    }
		  });
		  args.splice(lastC, 0, c);
		}
		/**
		 * Invokes `console.log()` when available.
		 * No-op when `console.log` is not a "function".
		 *
		 * @api public
		 */

		function log() {
		  var _console;

		  // This hackery is required for IE8/9, where
		  // the `console.log` function doesn't have 'apply'
		  return (typeof console === "undefined" ? "undefined" : _typeof(console)) === 'object' && console.log && (_console = console).log.apply(_console, arguments);
		}
		/**
		 * Save `namespaces`.
		 *
		 * @param {String} namespaces
		 * @api private
		 */

		function save(namespaces) {
		  try {
		    if (namespaces) {
		      exports.storage.setItem('debug', namespaces);
		    } else {
		      exports.storage.removeItem('debug');
		    }
		  } catch (error) {// Swallow
		    // XXX (@Qix-) should we be logging these?
		  }
		}
		/**
		 * Load `namespaces`.
		 *
		 * @return {String} returns the previously persisted debug modes
		 * @api private
		 */

		function load() {
		  var r;
		  try {
		    r = exports.storage.getItem('debug');
		  } catch (error) {} // Swallow
		  // XXX (@Qix-) should we be logging these?
		  // If debug isn't set in LS, and we're in Electron, try to load $DEBUG

		  if (!r && typeof process !== 'undefined' && 'env' in process) {
		    r = process.env.DEBUG;
		  }
		  return r;
		}
		/**
		 * Localstorage attempts to return the localstorage.
		 *
		 * This is necessary because safari throws
		 * when a user disables cookies/localstorage
		 * and you attempt to access it.
		 *
		 * @return {LocalStorage}
		 * @api private
		 */

		function localstorage() {
		  try {
		    // TVMLKit (Apple TV JS Runtime) does not have a window object, just localStorage in the global context
		    // The Browser also has localStorage in the global context.
		    return localStorage;
		  } catch (error) {// Swallow
		    // XXX (@Qix-) should we be logging these?
		  }
		}
		module.exports = requireCommon()(exports);
		var formatters = module.exports.formatters;
		/**
		 * Map %j to `JSON.stringify()`, since no Web Inspectors do that by default.
		 */

		formatters.j = function (v) {
		  try {
		    return JSON.stringify(v);
		  } catch (error) {
		    return '[UnexpectedJSONParseError]: ' + error.message;
		  }
		}; 
	} (browser, browser.exports));
	return browser.exports;
}

/**
 * Detect Electron renderer / nwjs process, which is node, but we should
 * treat as a browser.
 */
if (typeof process === 'undefined' || process.type === 'renderer' || process.browser === true || process.__nwjs) {
  src.exports = requireBrowser();
} else {
  src.exports = requireNode();
}

var srcExports = src.exports;

var path$5 = path$6;
var fs$4 = require$$0;
var _0777 = parseInt('0777', 8);
var mkdirp = mkdirP.mkdirp = mkdirP.mkdirP = mkdirP;
function mkdirP(p, opts, f, made) {
  if (typeof opts === 'function') {
    f = opts;
    opts = {};
  } else if (!opts || typeof opts !== 'object') {
    opts = {
      mode: opts
    };
  }
  var mode = opts.mode;
  var xfs = opts.fs || fs$4;
  if (mode === undefined) {
    mode = _0777;
  }
  if (!made) made = null;
  var cb = f || /* istanbul ignore next */function () {};
  p = path$5.resolve(p);
  xfs.mkdir(p, mode, function (er) {
    if (!er) {
      made = made || p;
      return cb(null, made);
    }
    switch (er.code) {
      case 'ENOENT':
        /* istanbul ignore if */
        if (path$5.dirname(p) === p) return cb(er);
        mkdirP(path$5.dirname(p), opts, function (er, made) {
          /* istanbul ignore if */
          if (er) cb(er, made);else mkdirP(p, opts, cb, made);
        });
        break;

      // In the case of any other error, just see if there's a dir
      // there already.  If so, then hooray!  If not, then something
      // is borked.
      default:
        xfs.stat(p, function (er2, stat) {
          // if the stat fails, then that's super weird.
          // let the original error be the failure reason.
          if (er2 || !stat.isDirectory()) cb(er, made);else cb(null, made);
        });
        break;
    }
  });
}
mkdirP.sync = function sync(p, opts, made) {
  if (!opts || typeof opts !== 'object') {
    opts = {
      mode: opts
    };
  }
  var mode = opts.mode;
  var xfs = opts.fs || fs$4;
  if (mode === undefined) {
    mode = _0777;
  }
  if (!made) made = null;
  p = path$5.resolve(p);
  try {
    xfs.mkdirSync(p, mode);
    made = made || p;
  } catch (err0) {
    switch (err0.code) {
      case 'ENOENT':
        made = sync(path$5.dirname(p), opts, made);
        sync(p, opts, made);
        break;

      // In the case of any other error, just see if there's a dir
      // there already.  If so, then hooray!  If not, then something
      // is borked.
      default:
        var stat;
        try {
          stat = xfs.statSync(p);
        } catch (err1) /* istanbul ignore next */{
          throw err0;
        }
        /* istanbul ignore if */
        if (!stat.isDirectory()) throw err0;
        break;
    }
  }
  return made;
};

/*
 * portfinder.js: A simple tool to find an open port on the current machine.
 *
 * (C) 2011, Charlie Robbins
 *
 */

(function (exports) {

	var fs = require$$0,
	  os = require$$0$1,
	  net = require$$2,
	  path = path$6,
	  _async = asyncExports,
	  debug = srcExports,
	  mkdirp$1 = mkdirp.mkdirp;
	var debugTestPort = debug('portfinder:testPort'),
	  debugGetPort = debug('portfinder:getPort'),
	  debugDefaultHosts = debug('portfinder:defaultHosts');
	var internals = {};
	internals.testPort = function (options, callback) {
	  if (!callback) {
	    callback = options;
	    options = {};
	  }
	  options.server = options.server || net.createServer(function () {
	    //
	    // Create an empty listener for the port testing server.
	    //
	  });
	  debugTestPort("entered testPort(): trying", options.host, "port", options.port);
	  function onListen() {
	    debugTestPort("done w/ testPort(): OK", options.host, "port", options.port);
	    options.server.removeListener('error', onError);
	    options.server.close();
	    callback(null, options.port);
	  }
	  function onError(err) {
	    debugTestPort("done w/ testPort(): failed", options.host, "w/ port", options.port, "with error", err.code);
	    options.server.removeListener('listening', onListen);
	    if (!(err.code == 'EADDRINUSE' || err.code == 'EACCES')) {
	      return callback(err);
	    }
	    var nextPort = exports.nextPort(options.port);
	    if (nextPort > exports.highestPort) {
	      return callback(new Error('No open ports available'));
	    }
	    internals.testPort({
	      port: nextPort,
	      host: options.host,
	      server: options.server
	    }, callback);
	  }
	  options.server.once('error', onError);
	  options.server.once('listening', onListen);
	  if (options.host) {
	    options.server.listen(options.port, options.host);
	  } else {
	    /*
	      Judgement of service without host
	      example:
	        express().listen(options.port)
	    */
	    options.server.listen(options.port);
	  }
	};

	//
	// ### @basePort {Number}
	// The lowest port to begin any port search from
	//
	exports.basePort = 8000;

	//
	// ### function setBasePort (port)
	// #### @port {Number} The new base port
	//
	exports.setBasePort = function (port) {
	  exports.basePort = port;
	};

	//
	// ### @highestPort {Number}
	// Largest port number is an unsigned short 2**16 -1=65335
	//
	exports.highestPort = 65535;

	//
	// ### function setHighestPort (port)
	// #### @port {Number} The new highest port
	//
	exports.setHighestPort = function (port) {
	  exports.highestPort = port;
	};

	//
	// ### @basePath {string}
	// Default path to begin any socket search from
	//
	exports.basePath = '/tmp/portfinder';

	//
	// ### function getPort (options, callback)
	// #### @options {Object} Settings to use when finding the necessary port
	// #### @callback {function} Continuation to respond to when complete.
	// Responds with a unbound port on the current machine.
	//
	exports.getPort = function (options, callback) {
	  if (!callback) {
	    callback = options;
	    options = {};
	  }
	  options.port = Number(options.port) || Number(exports.basePort);
	  options.host = options.host || null;
	  options.stopPort = Number(options.stopPort) || Number(exports.highestPort);
	  if (!options.startPort) {
	    options.startPort = Number(options.port);
	    if (options.startPort < 0) {
	      throw Error('Provided options.startPort(' + options.startPort + ') is less than 0, which are cannot be bound.');
	    }
	    if (options.stopPort < options.startPort) {
	      throw Error('Provided options.stopPort(' + options.stopPort + 'is less than options.startPort (' + options.startPort + ')');
	    }
	  }
	  if (options.host) {
	    if (exports._defaultHosts.indexOf(options.host) !== -1) {
	      exports._defaultHosts.push(options.host);
	    }
	  }
	  var openPorts = [],
	    currentHost;
	  return _async.eachSeries(exports._defaultHosts, function (host, next) {
	    debugGetPort("in eachSeries() iteration callback: host is", host);
	    return internals.testPort({
	      host: host,
	      port: options.port
	    }, function (err, port) {
	      if (err) {
	        debugGetPort("in eachSeries() iteration callback testPort() callback", "with an err:", err.code);
	        currentHost = host;
	        return next(err);
	      } else {
	        debugGetPort("in eachSeries() iteration callback testPort() callback", "with a success for port", port);
	        openPorts.push(port);
	        return next();
	      }
	    });
	  }, function (err) {
	    if (err) {
	      debugGetPort("in eachSeries() result callback: err is", err);
	      // If we get EADDRNOTAVAIL it means the host is not bindable, so remove it
	      // from exports._defaultHosts and start over. For ubuntu, we use EINVAL for the same
	      if (err.code === 'EADDRNOTAVAIL' || err.code === 'EINVAL') {
	        if (options.host === currentHost) {
	          // if bad address matches host given by user, tell them
	          //
	          // NOTE: We may need to one day handle `my-non-existent-host.local` if users
	          // report frustration with passing in hostnames that DONT map to bindable
	          // hosts, without showing them a good error.
	          var msg = 'Provided host ' + options.host + ' could NOT be bound. Please provide a different host address or hostname';
	          return callback(Error(msg));
	        } else {
	          var idx = exports._defaultHosts.indexOf(currentHost);
	          exports._defaultHosts.splice(idx, 1);
	          return exports.getPort(options, callback);
	        }
	      } else {
	        // error is not accounted for, file ticket, handle special case
	        return callback(err);
	      }
	    }

	    // sort so we can compare first host to last host
	    openPorts.sort(function (a, b) {
	      return a - b;
	    });
	    debugGetPort("in eachSeries() result callback: openPorts is", openPorts);
	    if (openPorts[0] === openPorts[openPorts.length - 1]) {
	      // if first === last, we found an open port
	      if (openPorts[0] <= options.stopPort) {
	        return callback(null, openPorts[0]);
	      } else {
	        var msg = 'No open ports found in between ' + options.startPort + ' and ' + options.stopPort;
	        return callback(Error(msg));
	      }
	    } else {
	      // otherwise, try again, using sorted port, aka, highest open for >= 1 host
	      return exports.getPort({
	        port: openPorts.pop(),
	        host: options.host,
	        startPort: options.startPort,
	        stopPort: options.stopPort
	      }, callback);
	    }
	  });
	};

	//
	// ### function getPortPromise (options)
	// #### @options {Object} Settings to use when finding the necessary port
	// Responds a promise to an unbound port on the current machine.
	//
	exports.getPortPromise = function (options) {
	  if (typeof Promise !== 'function') {
	    throw Error('Native promise support is not available in this version of node.' + 'Please install a polyfill and assign Promise to global.Promise before calling this method');
	  }
	  if (!options) {
	    options = {};
	  }
	  return new Promise(function (resolve, reject) {
	    exports.getPort(options, function (err, port) {
	      if (err) {
	        return reject(err);
	      }
	      resolve(port);
	    });
	  });
	};

	//
	// ### function getPorts (count, options, callback)
	// #### @count {Number} The number of ports to find
	// #### @options {Object} Settings to use when finding the necessary port
	// #### @callback {function} Continuation to respond to when complete.
	// Responds with an array of unbound ports on the current machine.
	//
	exports.getPorts = function (count, options, callback) {
	  if (!callback) {
	    callback = options;
	    options = {};
	  }
	  var lastPort = null;
	  _async.timesSeries(count, function (index, asyncCallback) {
	    if (lastPort) {
	      options.port = exports.nextPort(lastPort);
	    }
	    exports.getPort(options, function (err, port) {
	      if (err) {
	        asyncCallback(err);
	      } else {
	        lastPort = port;
	        asyncCallback(null, port);
	      }
	    });
	  }, callback);
	};

	//
	// ### function getSocket (options, callback)
	// #### @options {Object} Settings to use when finding the necessary port
	// #### @callback {function} Continuation to respond to when complete.
	// Responds with a unbound socket using the specified directory and base
	// name on the current machine.
	//
	exports.getSocket = function (options, callback) {
	  if (!callback) {
	    callback = options;
	    options = {};
	  }
	  options.mod = options.mod || parseInt(755, 8);
	  options.path = options.path || exports.basePath + '.sock';

	  //
	  // Tests the specified socket
	  //
	  function testSocket() {
	    fs.stat(options.path, function (err) {
	      //
	      // If file we're checking doesn't exist (thus, stating it emits ENOENT),
	      // we should be OK with listening on this socket.
	      //
	      if (err) {
	        if (err.code == 'ENOENT') {
	          callback(null, options.path);
	        } else {
	          callback(err);
	        }
	      } else {
	        //
	        // This file exists, so it isn't possible to listen on it. Lets try
	        // next socket.
	        //
	        options.path = exports.nextSocket(options.path);
	        exports.getSocket(options, callback);
	      }
	    });
	  }

	  //
	  // Create the target `dir` then test connection
	  // against the socket.
	  //
	  function createAndTestSocket(dir) {
	    mkdirp$1(dir, options.mod, function (err) {
	      if (err) {
	        return callback(err);
	      }
	      options.exists = true;
	      testSocket();
	    });
	  }

	  //
	  // Check if the parent directory of the target
	  // socket path exists. If it does, test connection
	  // against the socket. Otherwise, create the directory
	  // then test connection.
	  //
	  function checkAndTestSocket() {
	    var dir = path.dirname(options.path);
	    fs.stat(dir, function (err, stats) {
	      if (err || !stats.isDirectory()) {
	        return createAndTestSocket(dir);
	      }
	      options.exists = true;
	      testSocket();
	    });
	  }

	  //
	  // If it has been explicitly stated that the
	  // target `options.path` already exists, then
	  // simply test the socket.
	  //
	  return options.exists ? testSocket() : checkAndTestSocket();
	};

	//
	// ### function nextPort (port)
	// #### @port {Number} Port to increment from.
	// Gets the next port in sequence from the
	// specified `port`.
	//
	exports.nextPort = function (port) {
	  return port + 1;
	};

	//
	// ### function nextSocket (socketPath)
	// #### @socketPath {string} Path to increment from
	// Gets the next socket path in sequence from the
	// specified `socketPath`.
	//
	exports.nextSocket = function (socketPath) {
	  var dir = path.dirname(socketPath),
	    name = path.basename(socketPath, '.sock'),
	    match = name.match(/^([a-zA-z]+)(\d*)$/i),
	    index = parseInt(match[2]),
	    base = match[1];
	  if (isNaN(index)) {
	    index = 0;
	  }
	  index += 1;
	  return path.join(dir, base + index + '.sock');
	};

	/**
	 * @desc List of internal hostnames provided by your machine. A user
	 *       provided hostname may also be provided when calling portfinder.getPort,
	 *       which would then be added to the default hosts we lookup and return here.
	 *
	 * @return {array}
	 *
	 * Long Form Explantion:
	 *
	 *    - Input: (os.networkInterfaces() w/ MacOS 10.11.5+ and running a VM)
	 *
	 *        { lo0:
	 *         [ { address: '::1',
	 *             netmask: 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff',
	 *             family: 'IPv6',
	 *             mac: '00:00:00:00:00:00',
	 *             scopeid: 0,
	 *             internal: true },
	 *           { address: '127.0.0.1',
	 *             netmask: '255.0.0.0',
	 *             family: 'IPv4',
	 *             mac: '00:00:00:00:00:00',
	 *             internal: true },
	 *           { address: 'fe80::1',
	 *             netmask: 'ffff:ffff:ffff:ffff::',
	 *             family: 'IPv6',
	 *             mac: '00:00:00:00:00:00',
	 *             scopeid: 1,
	 *             internal: true } ],
	 *        en0:
	 *         [ { address: 'fe80::a299:9bff:fe17:766d',
	 *             netmask: 'ffff:ffff:ffff:ffff::',
	 *             family: 'IPv6',
	 *             mac: 'a0:99:9b:17:76:6d',
	 *             scopeid: 4,
	 *             internal: false },
	 *           { address: '10.0.1.22',
	 *             netmask: '255.255.255.0',
	 *             family: 'IPv4',
	 *             mac: 'a0:99:9b:17:76:6d',
	 *             internal: false } ],
	 *        awdl0:
	 *         [ { address: 'fe80::48a8:37ff:fe34:aaef',
	 *             netmask: 'ffff:ffff:ffff:ffff::',
	 *             family: 'IPv6',
	 *             mac: '4a:a8:37:34:aa:ef',
	 *             scopeid: 8,
	 *             internal: false } ],
	 *        vnic0:
	 *         [ { address: '10.211.55.2',
	 *             netmask: '255.255.255.0',
	 *             family: 'IPv4',
	 *             mac: '00:1c:42:00:00:08',
	 *             internal: false } ],
	 *        vnic1:
	 *         [ { address: '10.37.129.2',
	 *             netmask: '255.255.255.0',
	 *             family: 'IPv4',
	 *             mac: '00:1c:42:00:00:09',
	 *             internal: false } ] }
	 *
	 *    - Output:
	 *
	 *         [
	 *          '0.0.0.0',
	 *          '::1',
	 *          '127.0.0.1',
	 *          'fe80::1',
	 *          '10.0.1.22',
	 *          'fe80::48a8:37ff:fe34:aaef',
	 *          '10.211.55.2',
	 *          '10.37.129.2'
	 *         ]
	 *
	 *     Note we export this so we can use it in our tests, otherwise this API is private
	 */
	exports._defaultHosts = function () {
	  var interfaces = {};
	  try {
	    interfaces = os.networkInterfaces();
	  } catch (e) {
	    // As of October 2016, Windows Subsystem for Linux (WSL) does not support
	    // the os.networkInterfaces() call and throws instead. For this platform,
	    // assume 0.0.0.0 as the only address
	    //
	    // - https://github.com/Microsoft/BashOnWindows/issues/468
	    //
	    // - Workaround is a mix of good work from the community:
	    //   - https://github.com/http-party/node-portfinder/commit/8d7e30a648ff5034186551fa8a6652669dec2f2f
	    //   - https://github.com/yarnpkg/yarn/pull/772/files
	    if (e.syscall === 'uv_interface_addresses') ; else {
	      throw e;
	    }
	  }
	  var interfaceNames = Object.keys(interfaces),
	    hiddenButImportantHost = '0.0.0.0',
	    // !important - dont remove, hence the naming :)
	    results = [hiddenButImportantHost];
	  for (var i = 0; i < interfaceNames.length; i++) {
	    var _interface = interfaces[interfaceNames[i]];
	    for (var j = 0; j < _interface.length; j++) {
	      var curr = _interface[j];
	      results.push(curr.address);
	    }
	  }

	  // add null value, For createServer function, do not use host.
	  results.push(null);
	  debugDefaultHosts("exports._defaultHosts is: %o", results);
	  return results;
	}(); 
} (portfinder$1));

var portfinder = /*@__PURE__*/getDefaultExportFromCjs(portfinder$1);

var isWsl$2 = {exports: {}};

const fs$3 = require$$0;
let isDocker$2;
function hasDockerEnv$1() {
  try {
    fs$3.statSync('/.dockerenv');
    return true;
  } catch (_) {
    return false;
  }
}
function hasDockerCGroup$1() {
  try {
    return fs$3.readFileSync('/proc/self/cgroup', 'utf8').includes('docker');
  } catch (_) {
    return false;
  }
}
var isDocker_1 = () => {
  if (isDocker$2 === undefined) {
    isDocker$2 = hasDockerEnv$1() || hasDockerCGroup$1();
  }
  return isDocker$2;
};

const os$2 = require$$0$1;
const fs$2 = require$$0;
const isDocker$1 = isDocker_1;
const isWsl = () => {
  if (process.platform !== 'linux') {
    return false;
  }
  if (os$2.release().toLowerCase().includes('microsoft')) {
    if (isDocker$1()) {
      return false;
    }
    return true;
  }
  try {
    return fs$2.readFileSync('/proc/version', 'utf8').toLowerCase().includes('microsoft') ? !isDocker$1() : false;
  } catch (_) {
    return false;
  }
};
if (process.env.__IS_WSL_TEST__) {
  isWsl$2.exports = isWsl;
} else {
  isWsl$2.exports = isWsl();
}

var isWslExports = isWsl$2.exports;
var isWsl$1 = /*@__PURE__*/getDefaultExportFromCjs(isWslExports);

function defineLazyProperty(object, propertyName, valueGetter) {
  const define = value => Object.defineProperty(object, propertyName, {
    value,
    enumerable: true,
    writable: true
  });
  Object.defineProperty(object, propertyName, {
    configurable: true,
    enumerable: true,
    get() {
      const result = valueGetter();
      define(result);
      return result;
    },
    set(value) {
      define(value);
    }
  });
  return object;
}

var bplistParser = {};

var BigInteger = {exports: {}};

(function (module) {
	var bigInt = function (undefined$1) {

	  var BASE = 1e7,
	    LOG_BASE = 7,
	    MAX_INT = 9007199254740992,
	    MAX_INT_ARR = smallToArray(MAX_INT),
	    DEFAULT_ALPHABET = "0123456789abcdefghijklmnopqrstuvwxyz";
	  var supportsNativeBigInt = typeof BigInt === "function";
	  function Integer(v, radix, alphabet, caseSensitive) {
	    if (typeof v === "undefined") return Integer[0];
	    if (typeof radix !== "undefined") return +radix === 10 && !alphabet ? parseValue(v) : parseBase(v, radix, alphabet, caseSensitive);
	    return parseValue(v);
	  }
	  function BigInteger(value, sign) {
	    this.value = value;
	    this.sign = sign;
	    this.isSmall = false;
	  }
	  BigInteger.prototype = Object.create(Integer.prototype);
	  function SmallInteger(value) {
	    this.value = value;
	    this.sign = value < 0;
	    this.isSmall = true;
	  }
	  SmallInteger.prototype = Object.create(Integer.prototype);
	  function NativeBigInt(value) {
	    this.value = value;
	  }
	  NativeBigInt.prototype = Object.create(Integer.prototype);
	  function isPrecise(n) {
	    return -MAX_INT < n && n < MAX_INT;
	  }
	  function smallToArray(n) {
	    // For performance reasons doesn't reference BASE, need to change this function if BASE changes
	    if (n < 1e7) return [n];
	    if (n < 1e14) return [n % 1e7, Math.floor(n / 1e7)];
	    return [n % 1e7, Math.floor(n / 1e7) % 1e7, Math.floor(n / 1e14)];
	  }
	  function arrayToSmall(arr) {
	    // If BASE changes this function may need to change
	    trim(arr);
	    var length = arr.length;
	    if (length < 4 && compareAbs(arr, MAX_INT_ARR) < 0) {
	      switch (length) {
	        case 0:
	          return 0;
	        case 1:
	          return arr[0];
	        case 2:
	          return arr[0] + arr[1] * BASE;
	        default:
	          return arr[0] + (arr[1] + arr[2] * BASE) * BASE;
	      }
	    }
	    return arr;
	  }
	  function trim(v) {
	    var i = v.length;
	    while (v[--i] === 0);
	    v.length = i + 1;
	  }
	  function createArray(length) {
	    // function shamelessly stolen from Yaffle's library https://github.com/Yaffle/BigInteger
	    var x = new Array(length);
	    var i = -1;
	    while (++i < length) {
	      x[i] = 0;
	    }
	    return x;
	  }
	  function truncate(n) {
	    if (n > 0) return Math.floor(n);
	    return Math.ceil(n);
	  }
	  function add(a, b) {
	    // assumes a and b are arrays with a.length >= b.length
	    var l_a = a.length,
	      l_b = b.length,
	      r = new Array(l_a),
	      carry = 0,
	      base = BASE,
	      sum,
	      i;
	    for (i = 0; i < l_b; i++) {
	      sum = a[i] + b[i] + carry;
	      carry = sum >= base ? 1 : 0;
	      r[i] = sum - carry * base;
	    }
	    while (i < l_a) {
	      sum = a[i] + carry;
	      carry = sum === base ? 1 : 0;
	      r[i++] = sum - carry * base;
	    }
	    if (carry > 0) r.push(carry);
	    return r;
	  }
	  function addAny(a, b) {
	    if (a.length >= b.length) return add(a, b);
	    return add(b, a);
	  }
	  function addSmall(a, carry) {
	    // assumes a is array, carry is number with 0 <= carry < MAX_INT
	    var l = a.length,
	      r = new Array(l),
	      base = BASE,
	      sum,
	      i;
	    for (i = 0; i < l; i++) {
	      sum = a[i] - base + carry;
	      carry = Math.floor(sum / base);
	      r[i] = sum - carry * base;
	      carry += 1;
	    }
	    while (carry > 0) {
	      r[i++] = carry % base;
	      carry = Math.floor(carry / base);
	    }
	    return r;
	  }
	  BigInteger.prototype.add = function (v) {
	    var n = parseValue(v);
	    if (this.sign !== n.sign) {
	      return this.subtract(n.negate());
	    }
	    var a = this.value,
	      b = n.value;
	    if (n.isSmall) {
	      return new BigInteger(addSmall(a, Math.abs(b)), this.sign);
	    }
	    return new BigInteger(addAny(a, b), this.sign);
	  };
	  BigInteger.prototype.plus = BigInteger.prototype.add;
	  SmallInteger.prototype.add = function (v) {
	    var n = parseValue(v);
	    var a = this.value;
	    if (a < 0 !== n.sign) {
	      return this.subtract(n.negate());
	    }
	    var b = n.value;
	    if (n.isSmall) {
	      if (isPrecise(a + b)) return new SmallInteger(a + b);
	      b = smallToArray(Math.abs(b));
	    }
	    return new BigInteger(addSmall(b, Math.abs(a)), a < 0);
	  };
	  SmallInteger.prototype.plus = SmallInteger.prototype.add;
	  NativeBigInt.prototype.add = function (v) {
	    return new NativeBigInt(this.value + parseValue(v).value);
	  };
	  NativeBigInt.prototype.plus = NativeBigInt.prototype.add;
	  function subtract(a, b) {
	    // assumes a and b are arrays with a >= b
	    var a_l = a.length,
	      b_l = b.length,
	      r = new Array(a_l),
	      borrow = 0,
	      base = BASE,
	      i,
	      difference;
	    for (i = 0; i < b_l; i++) {
	      difference = a[i] - borrow - b[i];
	      if (difference < 0) {
	        difference += base;
	        borrow = 1;
	      } else borrow = 0;
	      r[i] = difference;
	    }
	    for (i = b_l; i < a_l; i++) {
	      difference = a[i] - borrow;
	      if (difference < 0) difference += base;else {
	        r[i++] = difference;
	        break;
	      }
	      r[i] = difference;
	    }
	    for (; i < a_l; i++) {
	      r[i] = a[i];
	    }
	    trim(r);
	    return r;
	  }
	  function subtractAny(a, b, sign) {
	    var value;
	    if (compareAbs(a, b) >= 0) {
	      value = subtract(a, b);
	    } else {
	      value = subtract(b, a);
	      sign = !sign;
	    }
	    value = arrayToSmall(value);
	    if (typeof value === "number") {
	      if (sign) value = -value;
	      return new SmallInteger(value);
	    }
	    return new BigInteger(value, sign);
	  }
	  function subtractSmall(a, b, sign) {
	    // assumes a is array, b is number with 0 <= b < MAX_INT
	    var l = a.length,
	      r = new Array(l),
	      carry = -b,
	      base = BASE,
	      i,
	      difference;
	    for (i = 0; i < l; i++) {
	      difference = a[i] + carry;
	      carry = Math.floor(difference / base);
	      difference %= base;
	      r[i] = difference < 0 ? difference + base : difference;
	    }
	    r = arrayToSmall(r);
	    if (typeof r === "number") {
	      if (sign) r = -r;
	      return new SmallInteger(r);
	    }
	    return new BigInteger(r, sign);
	  }
	  BigInteger.prototype.subtract = function (v) {
	    var n = parseValue(v);
	    if (this.sign !== n.sign) {
	      return this.add(n.negate());
	    }
	    var a = this.value,
	      b = n.value;
	    if (n.isSmall) return subtractSmall(a, Math.abs(b), this.sign);
	    return subtractAny(a, b, this.sign);
	  };
	  BigInteger.prototype.minus = BigInteger.prototype.subtract;
	  SmallInteger.prototype.subtract = function (v) {
	    var n = parseValue(v);
	    var a = this.value;
	    if (a < 0 !== n.sign) {
	      return this.add(n.negate());
	    }
	    var b = n.value;
	    if (n.isSmall) {
	      return new SmallInteger(a - b);
	    }
	    return subtractSmall(b, Math.abs(a), a >= 0);
	  };
	  SmallInteger.prototype.minus = SmallInteger.prototype.subtract;
	  NativeBigInt.prototype.subtract = function (v) {
	    return new NativeBigInt(this.value - parseValue(v).value);
	  };
	  NativeBigInt.prototype.minus = NativeBigInt.prototype.subtract;
	  BigInteger.prototype.negate = function () {
	    return new BigInteger(this.value, !this.sign);
	  };
	  SmallInteger.prototype.negate = function () {
	    var sign = this.sign;
	    var small = new SmallInteger(-this.value);
	    small.sign = !sign;
	    return small;
	  };
	  NativeBigInt.prototype.negate = function () {
	    return new NativeBigInt(-this.value);
	  };
	  BigInteger.prototype.abs = function () {
	    return new BigInteger(this.value, false);
	  };
	  SmallInteger.prototype.abs = function () {
	    return new SmallInteger(Math.abs(this.value));
	  };
	  NativeBigInt.prototype.abs = function () {
	    return new NativeBigInt(this.value >= 0 ? this.value : -this.value);
	  };
	  function multiplyLong(a, b) {
	    var a_l = a.length,
	      b_l = b.length,
	      l = a_l + b_l,
	      r = createArray(l),
	      base = BASE,
	      product,
	      carry,
	      i,
	      a_i,
	      b_j;
	    for (i = 0; i < a_l; ++i) {
	      a_i = a[i];
	      for (var j = 0; j < b_l; ++j) {
	        b_j = b[j];
	        product = a_i * b_j + r[i + j];
	        carry = Math.floor(product / base);
	        r[i + j] = product - carry * base;
	        r[i + j + 1] += carry;
	      }
	    }
	    trim(r);
	    return r;
	  }
	  function multiplySmall(a, b) {
	    // assumes a is array, b is number with |b| < BASE
	    var l = a.length,
	      r = new Array(l),
	      base = BASE,
	      carry = 0,
	      product,
	      i;
	    for (i = 0; i < l; i++) {
	      product = a[i] * b + carry;
	      carry = Math.floor(product / base);
	      r[i] = product - carry * base;
	    }
	    while (carry > 0) {
	      r[i++] = carry % base;
	      carry = Math.floor(carry / base);
	    }
	    return r;
	  }
	  function shiftLeft(x, n) {
	    var r = [];
	    while (n-- > 0) r.push(0);
	    return r.concat(x);
	  }
	  function multiplyKaratsuba(x, y) {
	    var n = Math.max(x.length, y.length);
	    if (n <= 30) return multiplyLong(x, y);
	    n = Math.ceil(n / 2);
	    var b = x.slice(n),
	      a = x.slice(0, n),
	      d = y.slice(n),
	      c = y.slice(0, n);
	    var ac = multiplyKaratsuba(a, c),
	      bd = multiplyKaratsuba(b, d),
	      abcd = multiplyKaratsuba(addAny(a, b), addAny(c, d));
	    var product = addAny(addAny(ac, shiftLeft(subtract(subtract(abcd, ac), bd), n)), shiftLeft(bd, 2 * n));
	    trim(product);
	    return product;
	  }

	  // The following function is derived from a surface fit of a graph plotting the performance difference
	  // between long multiplication and karatsuba multiplication versus the lengths of the two arrays.
	  function useKaratsuba(l1, l2) {
	    return -0.012 * l1 - 0.012 * l2 + 0.000015 * l1 * l2 > 0;
	  }
	  BigInteger.prototype.multiply = function (v) {
	    var n = parseValue(v),
	      a = this.value,
	      b = n.value,
	      sign = this.sign !== n.sign,
	      abs;
	    if (n.isSmall) {
	      if (b === 0) return Integer[0];
	      if (b === 1) return this;
	      if (b === -1) return this.negate();
	      abs = Math.abs(b);
	      if (abs < BASE) {
	        return new BigInteger(multiplySmall(a, abs), sign);
	      }
	      b = smallToArray(abs);
	    }
	    if (useKaratsuba(a.length, b.length))
	      // Karatsuba is only faster for certain array sizes
	      return new BigInteger(multiplyKaratsuba(a, b), sign);
	    return new BigInteger(multiplyLong(a, b), sign);
	  };
	  BigInteger.prototype.times = BigInteger.prototype.multiply;
	  function multiplySmallAndArray(a, b, sign) {
	    // a >= 0
	    if (a < BASE) {
	      return new BigInteger(multiplySmall(b, a), sign);
	    }
	    return new BigInteger(multiplyLong(b, smallToArray(a)), sign);
	  }
	  SmallInteger.prototype._multiplyBySmall = function (a) {
	    if (isPrecise(a.value * this.value)) {
	      return new SmallInteger(a.value * this.value);
	    }
	    return multiplySmallAndArray(Math.abs(a.value), smallToArray(Math.abs(this.value)), this.sign !== a.sign);
	  };
	  BigInteger.prototype._multiplyBySmall = function (a) {
	    if (a.value === 0) return Integer[0];
	    if (a.value === 1) return this;
	    if (a.value === -1) return this.negate();
	    return multiplySmallAndArray(Math.abs(a.value), this.value, this.sign !== a.sign);
	  };
	  SmallInteger.prototype.multiply = function (v) {
	    return parseValue(v)._multiplyBySmall(this);
	  };
	  SmallInteger.prototype.times = SmallInteger.prototype.multiply;
	  NativeBigInt.prototype.multiply = function (v) {
	    return new NativeBigInt(this.value * parseValue(v).value);
	  };
	  NativeBigInt.prototype.times = NativeBigInt.prototype.multiply;
	  function square(a) {
	    //console.assert(2 * BASE * BASE < MAX_INT);
	    var l = a.length,
	      r = createArray(l + l),
	      base = BASE,
	      product,
	      carry,
	      i,
	      a_i,
	      a_j;
	    for (i = 0; i < l; i++) {
	      a_i = a[i];
	      carry = 0 - a_i * a_i;
	      for (var j = i; j < l; j++) {
	        a_j = a[j];
	        product = 2 * (a_i * a_j) + r[i + j] + carry;
	        carry = Math.floor(product / base);
	        r[i + j] = product - carry * base;
	      }
	      r[i + l] = carry;
	    }
	    trim(r);
	    return r;
	  }
	  BigInteger.prototype.square = function () {
	    return new BigInteger(square(this.value), false);
	  };
	  SmallInteger.prototype.square = function () {
	    var value = this.value * this.value;
	    if (isPrecise(value)) return new SmallInteger(value);
	    return new BigInteger(square(smallToArray(Math.abs(this.value))), false);
	  };
	  NativeBigInt.prototype.square = function (v) {
	    return new NativeBigInt(this.value * this.value);
	  };
	  function divMod1(a, b) {
	    // Left over from previous version. Performs faster than divMod2 on smaller input sizes.
	    var a_l = a.length,
	      b_l = b.length,
	      base = BASE,
	      result = createArray(b.length),
	      divisorMostSignificantDigit = b[b_l - 1],
	      // normalization
	      lambda = Math.ceil(base / (2 * divisorMostSignificantDigit)),
	      remainder = multiplySmall(a, lambda),
	      divisor = multiplySmall(b, lambda),
	      quotientDigit,
	      shift,
	      carry,
	      borrow,
	      i,
	      l,
	      q;
	    if (remainder.length <= a_l) remainder.push(0);
	    divisor.push(0);
	    divisorMostSignificantDigit = divisor[b_l - 1];
	    for (shift = a_l - b_l; shift >= 0; shift--) {
	      quotientDigit = base - 1;
	      if (remainder[shift + b_l] !== divisorMostSignificantDigit) {
	        quotientDigit = Math.floor((remainder[shift + b_l] * base + remainder[shift + b_l - 1]) / divisorMostSignificantDigit);
	      }
	      // quotientDigit <= base - 1
	      carry = 0;
	      borrow = 0;
	      l = divisor.length;
	      for (i = 0; i < l; i++) {
	        carry += quotientDigit * divisor[i];
	        q = Math.floor(carry / base);
	        borrow += remainder[shift + i] - (carry - q * base);
	        carry = q;
	        if (borrow < 0) {
	          remainder[shift + i] = borrow + base;
	          borrow = -1;
	        } else {
	          remainder[shift + i] = borrow;
	          borrow = 0;
	        }
	      }
	      while (borrow !== 0) {
	        quotientDigit -= 1;
	        carry = 0;
	        for (i = 0; i < l; i++) {
	          carry += remainder[shift + i] - base + divisor[i];
	          if (carry < 0) {
	            remainder[shift + i] = carry + base;
	            carry = 0;
	          } else {
	            remainder[shift + i] = carry;
	            carry = 1;
	          }
	        }
	        borrow += carry;
	      }
	      result[shift] = quotientDigit;
	    }
	    // denormalization
	    remainder = divModSmall(remainder, lambda)[0];
	    return [arrayToSmall(result), arrayToSmall(remainder)];
	  }
	  function divMod2(a, b) {
	    // Implementation idea shamelessly stolen from Silent Matt's library http://silentmatt.com/biginteger/
	    // Performs faster than divMod1 on larger input sizes.
	    var a_l = a.length,
	      b_l = b.length,
	      result = [],
	      part = [],
	      base = BASE,
	      guess,
	      xlen,
	      highx,
	      highy,
	      check;
	    while (a_l) {
	      part.unshift(a[--a_l]);
	      trim(part);
	      if (compareAbs(part, b) < 0) {
	        result.push(0);
	        continue;
	      }
	      xlen = part.length;
	      highx = part[xlen - 1] * base + part[xlen - 2];
	      highy = b[b_l - 1] * base + b[b_l - 2];
	      if (xlen > b_l) {
	        highx = (highx + 1) * base;
	      }
	      guess = Math.ceil(highx / highy);
	      do {
	        check = multiplySmall(b, guess);
	        if (compareAbs(check, part) <= 0) break;
	        guess--;
	      } while (guess);
	      result.push(guess);
	      part = subtract(part, check);
	    }
	    result.reverse();
	    return [arrayToSmall(result), arrayToSmall(part)];
	  }
	  function divModSmall(value, lambda) {
	    var length = value.length,
	      quotient = createArray(length),
	      base = BASE,
	      i,
	      q,
	      remainder,
	      divisor;
	    remainder = 0;
	    for (i = length - 1; i >= 0; --i) {
	      divisor = remainder * base + value[i];
	      q = truncate(divisor / lambda);
	      remainder = divisor - q * lambda;
	      quotient[i] = q | 0;
	    }
	    return [quotient, remainder | 0];
	  }
	  function divModAny(self, v) {
	    var value,
	      n = parseValue(v);
	    if (supportsNativeBigInt) {
	      return [new NativeBigInt(self.value / n.value), new NativeBigInt(self.value % n.value)];
	    }
	    var a = self.value,
	      b = n.value;
	    var quotient;
	    if (b === 0) throw new Error("Cannot divide by zero");
	    if (self.isSmall) {
	      if (n.isSmall) {
	        return [new SmallInteger(truncate(a / b)), new SmallInteger(a % b)];
	      }
	      return [Integer[0], self];
	    }
	    if (n.isSmall) {
	      if (b === 1) return [self, Integer[0]];
	      if (b == -1) return [self.negate(), Integer[0]];
	      var abs = Math.abs(b);
	      if (abs < BASE) {
	        value = divModSmall(a, abs);
	        quotient = arrayToSmall(value[0]);
	        var remainder = value[1];
	        if (self.sign) remainder = -remainder;
	        if (typeof quotient === "number") {
	          if (self.sign !== n.sign) quotient = -quotient;
	          return [new SmallInteger(quotient), new SmallInteger(remainder)];
	        }
	        return [new BigInteger(quotient, self.sign !== n.sign), new SmallInteger(remainder)];
	      }
	      b = smallToArray(abs);
	    }
	    var comparison = compareAbs(a, b);
	    if (comparison === -1) return [Integer[0], self];
	    if (comparison === 0) return [Integer[self.sign === n.sign ? 1 : -1], Integer[0]];

	    // divMod1 is faster on smaller input sizes
	    if (a.length + b.length <= 200) value = divMod1(a, b);else value = divMod2(a, b);
	    quotient = value[0];
	    var qSign = self.sign !== n.sign,
	      mod = value[1],
	      mSign = self.sign;
	    if (typeof quotient === "number") {
	      if (qSign) quotient = -quotient;
	      quotient = new SmallInteger(quotient);
	    } else quotient = new BigInteger(quotient, qSign);
	    if (typeof mod === "number") {
	      if (mSign) mod = -mod;
	      mod = new SmallInteger(mod);
	    } else mod = new BigInteger(mod, mSign);
	    return [quotient, mod];
	  }
	  BigInteger.prototype.divmod = function (v) {
	    var result = divModAny(this, v);
	    return {
	      quotient: result[0],
	      remainder: result[1]
	    };
	  };
	  NativeBigInt.prototype.divmod = SmallInteger.prototype.divmod = BigInteger.prototype.divmod;
	  BigInteger.prototype.divide = function (v) {
	    return divModAny(this, v)[0];
	  };
	  NativeBigInt.prototype.over = NativeBigInt.prototype.divide = function (v) {
	    return new NativeBigInt(this.value / parseValue(v).value);
	  };
	  SmallInteger.prototype.over = SmallInteger.prototype.divide = BigInteger.prototype.over = BigInteger.prototype.divide;
	  BigInteger.prototype.mod = function (v) {
	    return divModAny(this, v)[1];
	  };
	  NativeBigInt.prototype.mod = NativeBigInt.prototype.remainder = function (v) {
	    return new NativeBigInt(this.value % parseValue(v).value);
	  };
	  SmallInteger.prototype.remainder = SmallInteger.prototype.mod = BigInteger.prototype.remainder = BigInteger.prototype.mod;
	  BigInteger.prototype.pow = function (v) {
	    var n = parseValue(v),
	      a = this.value,
	      b = n.value,
	      value,
	      x,
	      y;
	    if (b === 0) return Integer[1];
	    if (a === 0) return Integer[0];
	    if (a === 1) return Integer[1];
	    if (a === -1) return n.isEven() ? Integer[1] : Integer[-1];
	    if (n.sign) {
	      return Integer[0];
	    }
	    if (!n.isSmall) throw new Error("The exponent " + n.toString() + " is too large.");
	    if (this.isSmall) {
	      if (isPrecise(value = Math.pow(a, b))) return new SmallInteger(truncate(value));
	    }
	    x = this;
	    y = Integer[1];
	    while (true) {
	      if (b & 1 === 1) {
	        y = y.times(x);
	        --b;
	      }
	      if (b === 0) break;
	      b /= 2;
	      x = x.square();
	    }
	    return y;
	  };
	  SmallInteger.prototype.pow = BigInteger.prototype.pow;
	  NativeBigInt.prototype.pow = function (v) {
	    var n = parseValue(v);
	    var a = this.value,
	      b = n.value;
	    var _0 = BigInt(0),
	      _1 = BigInt(1),
	      _2 = BigInt(2);
	    if (b === _0) return Integer[1];
	    if (a === _0) return Integer[0];
	    if (a === _1) return Integer[1];
	    if (a === BigInt(-1)) return n.isEven() ? Integer[1] : Integer[-1];
	    if (n.isNegative()) return new NativeBigInt(_0);
	    var x = this;
	    var y = Integer[1];
	    while (true) {
	      if ((b & _1) === _1) {
	        y = y.times(x);
	        --b;
	      }
	      if (b === _0) break;
	      b /= _2;
	      x = x.square();
	    }
	    return y;
	  };
	  BigInteger.prototype.modPow = function (exp, mod) {
	    exp = parseValue(exp);
	    mod = parseValue(mod);
	    if (mod.isZero()) throw new Error("Cannot take modPow with modulus 0");
	    var r = Integer[1],
	      base = this.mod(mod);
	    if (exp.isNegative()) {
	      exp = exp.multiply(Integer[-1]);
	      base = base.modInv(mod);
	    }
	    while (exp.isPositive()) {
	      if (base.isZero()) return Integer[0];
	      if (exp.isOdd()) r = r.multiply(base).mod(mod);
	      exp = exp.divide(2);
	      base = base.square().mod(mod);
	    }
	    return r;
	  };
	  NativeBigInt.prototype.modPow = SmallInteger.prototype.modPow = BigInteger.prototype.modPow;
	  function compareAbs(a, b) {
	    if (a.length !== b.length) {
	      return a.length > b.length ? 1 : -1;
	    }
	    for (var i = a.length - 1; i >= 0; i--) {
	      if (a[i] !== b[i]) return a[i] > b[i] ? 1 : -1;
	    }
	    return 0;
	  }
	  BigInteger.prototype.compareAbs = function (v) {
	    var n = parseValue(v),
	      a = this.value,
	      b = n.value;
	    if (n.isSmall) return 1;
	    return compareAbs(a, b);
	  };
	  SmallInteger.prototype.compareAbs = function (v) {
	    var n = parseValue(v),
	      a = Math.abs(this.value),
	      b = n.value;
	    if (n.isSmall) {
	      b = Math.abs(b);
	      return a === b ? 0 : a > b ? 1 : -1;
	    }
	    return -1;
	  };
	  NativeBigInt.prototype.compareAbs = function (v) {
	    var a = this.value;
	    var b = parseValue(v).value;
	    a = a >= 0 ? a : -a;
	    b = b >= 0 ? b : -b;
	    return a === b ? 0 : a > b ? 1 : -1;
	  };
	  BigInteger.prototype.compare = function (v) {
	    // See discussion about comparison with Infinity:
	    // https://github.com/peterolson/BigInteger.js/issues/61
	    if (v === Infinity) {
	      return -1;
	    }
	    if (v === -Infinity) {
	      return 1;
	    }
	    var n = parseValue(v),
	      a = this.value,
	      b = n.value;
	    if (this.sign !== n.sign) {
	      return n.sign ? 1 : -1;
	    }
	    if (n.isSmall) {
	      return this.sign ? -1 : 1;
	    }
	    return compareAbs(a, b) * (this.sign ? -1 : 1);
	  };
	  BigInteger.prototype.compareTo = BigInteger.prototype.compare;
	  SmallInteger.prototype.compare = function (v) {
	    if (v === Infinity) {
	      return -1;
	    }
	    if (v === -Infinity) {
	      return 1;
	    }
	    var n = parseValue(v),
	      a = this.value,
	      b = n.value;
	    if (n.isSmall) {
	      return a == b ? 0 : a > b ? 1 : -1;
	    }
	    if (a < 0 !== n.sign) {
	      return a < 0 ? -1 : 1;
	    }
	    return a < 0 ? 1 : -1;
	  };
	  SmallInteger.prototype.compareTo = SmallInteger.prototype.compare;
	  NativeBigInt.prototype.compare = function (v) {
	    if (v === Infinity) {
	      return -1;
	    }
	    if (v === -Infinity) {
	      return 1;
	    }
	    var a = this.value;
	    var b = parseValue(v).value;
	    return a === b ? 0 : a > b ? 1 : -1;
	  };
	  NativeBigInt.prototype.compareTo = NativeBigInt.prototype.compare;
	  BigInteger.prototype.equals = function (v) {
	    return this.compare(v) === 0;
	  };
	  NativeBigInt.prototype.eq = NativeBigInt.prototype.equals = SmallInteger.prototype.eq = SmallInteger.prototype.equals = BigInteger.prototype.eq = BigInteger.prototype.equals;
	  BigInteger.prototype.notEquals = function (v) {
	    return this.compare(v) !== 0;
	  };
	  NativeBigInt.prototype.neq = NativeBigInt.prototype.notEquals = SmallInteger.prototype.neq = SmallInteger.prototype.notEquals = BigInteger.prototype.neq = BigInteger.prototype.notEquals;
	  BigInteger.prototype.greater = function (v) {
	    return this.compare(v) > 0;
	  };
	  NativeBigInt.prototype.gt = NativeBigInt.prototype.greater = SmallInteger.prototype.gt = SmallInteger.prototype.greater = BigInteger.prototype.gt = BigInteger.prototype.greater;
	  BigInteger.prototype.lesser = function (v) {
	    return this.compare(v) < 0;
	  };
	  NativeBigInt.prototype.lt = NativeBigInt.prototype.lesser = SmallInteger.prototype.lt = SmallInteger.prototype.lesser = BigInteger.prototype.lt = BigInteger.prototype.lesser;
	  BigInteger.prototype.greaterOrEquals = function (v) {
	    return this.compare(v) >= 0;
	  };
	  NativeBigInt.prototype.geq = NativeBigInt.prototype.greaterOrEquals = SmallInteger.prototype.geq = SmallInteger.prototype.greaterOrEquals = BigInteger.prototype.geq = BigInteger.prototype.greaterOrEquals;
	  BigInteger.prototype.lesserOrEquals = function (v) {
	    return this.compare(v) <= 0;
	  };
	  NativeBigInt.prototype.leq = NativeBigInt.prototype.lesserOrEquals = SmallInteger.prototype.leq = SmallInteger.prototype.lesserOrEquals = BigInteger.prototype.leq = BigInteger.prototype.lesserOrEquals;
	  BigInteger.prototype.isEven = function () {
	    return (this.value[0] & 1) === 0;
	  };
	  SmallInteger.prototype.isEven = function () {
	    return (this.value & 1) === 0;
	  };
	  NativeBigInt.prototype.isEven = function () {
	    return (this.value & BigInt(1)) === BigInt(0);
	  };
	  BigInteger.prototype.isOdd = function () {
	    return (this.value[0] & 1) === 1;
	  };
	  SmallInteger.prototype.isOdd = function () {
	    return (this.value & 1) === 1;
	  };
	  NativeBigInt.prototype.isOdd = function () {
	    return (this.value & BigInt(1)) === BigInt(1);
	  };
	  BigInteger.prototype.isPositive = function () {
	    return !this.sign;
	  };
	  SmallInteger.prototype.isPositive = function () {
	    return this.value > 0;
	  };
	  NativeBigInt.prototype.isPositive = SmallInteger.prototype.isPositive;
	  BigInteger.prototype.isNegative = function () {
	    return this.sign;
	  };
	  SmallInteger.prototype.isNegative = function () {
	    return this.value < 0;
	  };
	  NativeBigInt.prototype.isNegative = SmallInteger.prototype.isNegative;
	  BigInteger.prototype.isUnit = function () {
	    return false;
	  };
	  SmallInteger.prototype.isUnit = function () {
	    return Math.abs(this.value) === 1;
	  };
	  NativeBigInt.prototype.isUnit = function () {
	    return this.abs().value === BigInt(1);
	  };
	  BigInteger.prototype.isZero = function () {
	    return false;
	  };
	  SmallInteger.prototype.isZero = function () {
	    return this.value === 0;
	  };
	  NativeBigInt.prototype.isZero = function () {
	    return this.value === BigInt(0);
	  };
	  BigInteger.prototype.isDivisibleBy = function (v) {
	    var n = parseValue(v);
	    if (n.isZero()) return false;
	    if (n.isUnit()) return true;
	    if (n.compareAbs(2) === 0) return this.isEven();
	    return this.mod(n).isZero();
	  };
	  NativeBigInt.prototype.isDivisibleBy = SmallInteger.prototype.isDivisibleBy = BigInteger.prototype.isDivisibleBy;
	  function isBasicPrime(v) {
	    var n = v.abs();
	    if (n.isUnit()) return false;
	    if (n.equals(2) || n.equals(3) || n.equals(5)) return true;
	    if (n.isEven() || n.isDivisibleBy(3) || n.isDivisibleBy(5)) return false;
	    if (n.lesser(49)) return true;
	    // we don't know if it's prime: let the other functions figure it out
	  }

	  function millerRabinTest(n, a) {
	    var nPrev = n.prev(),
	      b = nPrev,
	      r = 0,
	      d,
	      i,
	      x;
	    while (b.isEven()) b = b.divide(2), r++;
	    next: for (i = 0; i < a.length; i++) {
	      if (n.lesser(a[i])) continue;
	      x = bigInt(a[i]).modPow(b, n);
	      if (x.isUnit() || x.equals(nPrev)) continue;
	      for (d = r - 1; d != 0; d--) {
	        x = x.square().mod(n);
	        if (x.isUnit()) return false;
	        if (x.equals(nPrev)) continue next;
	      }
	      return false;
	    }
	    return true;
	  }

	  // Set "strict" to true to force GRH-supported lower bound of 2*log(N)^2
	  BigInteger.prototype.isPrime = function (strict) {
	    var isPrime = isBasicPrime(this);
	    if (isPrime !== undefined$1) return isPrime;
	    var n = this.abs();
	    var bits = n.bitLength();
	    if (bits <= 64) return millerRabinTest(n, [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37]);
	    var logN = Math.log(2) * bits.toJSNumber();
	    var t = Math.ceil(strict === true ? 2 * Math.pow(logN, 2) : logN);
	    for (var a = [], i = 0; i < t; i++) {
	      a.push(bigInt(i + 2));
	    }
	    return millerRabinTest(n, a);
	  };
	  NativeBigInt.prototype.isPrime = SmallInteger.prototype.isPrime = BigInteger.prototype.isPrime;
	  BigInteger.prototype.isProbablePrime = function (iterations, rng) {
	    var isPrime = isBasicPrime(this);
	    if (isPrime !== undefined$1) return isPrime;
	    var n = this.abs();
	    var t = iterations === undefined$1 ? 5 : iterations;
	    for (var a = [], i = 0; i < t; i++) {
	      a.push(bigInt.randBetween(2, n.minus(2), rng));
	    }
	    return millerRabinTest(n, a);
	  };
	  NativeBigInt.prototype.isProbablePrime = SmallInteger.prototype.isProbablePrime = BigInteger.prototype.isProbablePrime;
	  BigInteger.prototype.modInv = function (n) {
	    var t = bigInt.zero,
	      newT = bigInt.one,
	      r = parseValue(n),
	      newR = this.abs(),
	      q,
	      lastT,
	      lastR;
	    while (!newR.isZero()) {
	      q = r.divide(newR);
	      lastT = t;
	      lastR = r;
	      t = newT;
	      r = newR;
	      newT = lastT.subtract(q.multiply(newT));
	      newR = lastR.subtract(q.multiply(newR));
	    }
	    if (!r.isUnit()) throw new Error(this.toString() + " and " + n.toString() + " are not co-prime");
	    if (t.compare(0) === -1) {
	      t = t.add(n);
	    }
	    if (this.isNegative()) {
	      return t.negate();
	    }
	    return t;
	  };
	  NativeBigInt.prototype.modInv = SmallInteger.prototype.modInv = BigInteger.prototype.modInv;
	  BigInteger.prototype.next = function () {
	    var value = this.value;
	    if (this.sign) {
	      return subtractSmall(value, 1, this.sign);
	    }
	    return new BigInteger(addSmall(value, 1), this.sign);
	  };
	  SmallInteger.prototype.next = function () {
	    var value = this.value;
	    if (value + 1 < MAX_INT) return new SmallInteger(value + 1);
	    return new BigInteger(MAX_INT_ARR, false);
	  };
	  NativeBigInt.prototype.next = function () {
	    return new NativeBigInt(this.value + BigInt(1));
	  };
	  BigInteger.prototype.prev = function () {
	    var value = this.value;
	    if (this.sign) {
	      return new BigInteger(addSmall(value, 1), true);
	    }
	    return subtractSmall(value, 1, this.sign);
	  };
	  SmallInteger.prototype.prev = function () {
	    var value = this.value;
	    if (value - 1 > -MAX_INT) return new SmallInteger(value - 1);
	    return new BigInteger(MAX_INT_ARR, true);
	  };
	  NativeBigInt.prototype.prev = function () {
	    return new NativeBigInt(this.value - BigInt(1));
	  };
	  var powersOfTwo = [1];
	  while (2 * powersOfTwo[powersOfTwo.length - 1] <= BASE) powersOfTwo.push(2 * powersOfTwo[powersOfTwo.length - 1]);
	  var powers2Length = powersOfTwo.length,
	    highestPower2 = powersOfTwo[powers2Length - 1];
	  function shift_isSmall(n) {
	    return Math.abs(n) <= BASE;
	  }
	  BigInteger.prototype.shiftLeft = function (v) {
	    var n = parseValue(v).toJSNumber();
	    if (!shift_isSmall(n)) {
	      throw new Error(String(n) + " is too large for shifting.");
	    }
	    if (n < 0) return this.shiftRight(-n);
	    var result = this;
	    if (result.isZero()) return result;
	    while (n >= powers2Length) {
	      result = result.multiply(highestPower2);
	      n -= powers2Length - 1;
	    }
	    return result.multiply(powersOfTwo[n]);
	  };
	  NativeBigInt.prototype.shiftLeft = SmallInteger.prototype.shiftLeft = BigInteger.prototype.shiftLeft;
	  BigInteger.prototype.shiftRight = function (v) {
	    var remQuo;
	    var n = parseValue(v).toJSNumber();
	    if (!shift_isSmall(n)) {
	      throw new Error(String(n) + " is too large for shifting.");
	    }
	    if (n < 0) return this.shiftLeft(-n);
	    var result = this;
	    while (n >= powers2Length) {
	      if (result.isZero() || result.isNegative() && result.isUnit()) return result;
	      remQuo = divModAny(result, highestPower2);
	      result = remQuo[1].isNegative() ? remQuo[0].prev() : remQuo[0];
	      n -= powers2Length - 1;
	    }
	    remQuo = divModAny(result, powersOfTwo[n]);
	    return remQuo[1].isNegative() ? remQuo[0].prev() : remQuo[0];
	  };
	  NativeBigInt.prototype.shiftRight = SmallInteger.prototype.shiftRight = BigInteger.prototype.shiftRight;
	  function bitwise(x, y, fn) {
	    y = parseValue(y);
	    var xSign = x.isNegative(),
	      ySign = y.isNegative();
	    var xRem = xSign ? x.not() : x,
	      yRem = ySign ? y.not() : y;
	    var xDigit = 0,
	      yDigit = 0;
	    var xDivMod = null,
	      yDivMod = null;
	    var result = [];
	    while (!xRem.isZero() || !yRem.isZero()) {
	      xDivMod = divModAny(xRem, highestPower2);
	      xDigit = xDivMod[1].toJSNumber();
	      if (xSign) {
	        xDigit = highestPower2 - 1 - xDigit; // two's complement for negative numbers
	      }

	      yDivMod = divModAny(yRem, highestPower2);
	      yDigit = yDivMod[1].toJSNumber();
	      if (ySign) {
	        yDigit = highestPower2 - 1 - yDigit; // two's complement for negative numbers
	      }

	      xRem = xDivMod[0];
	      yRem = yDivMod[0];
	      result.push(fn(xDigit, yDigit));
	    }
	    var sum = fn(xSign ? 1 : 0, ySign ? 1 : 0) !== 0 ? bigInt(-1) : bigInt(0);
	    for (var i = result.length - 1; i >= 0; i -= 1) {
	      sum = sum.multiply(highestPower2).add(bigInt(result[i]));
	    }
	    return sum;
	  }
	  BigInteger.prototype.not = function () {
	    return this.negate().prev();
	  };
	  NativeBigInt.prototype.not = SmallInteger.prototype.not = BigInteger.prototype.not;
	  BigInteger.prototype.and = function (n) {
	    return bitwise(this, n, function (a, b) {
	      return a & b;
	    });
	  };
	  NativeBigInt.prototype.and = SmallInteger.prototype.and = BigInteger.prototype.and;
	  BigInteger.prototype.or = function (n) {
	    return bitwise(this, n, function (a, b) {
	      return a | b;
	    });
	  };
	  NativeBigInt.prototype.or = SmallInteger.prototype.or = BigInteger.prototype.or;
	  BigInteger.prototype.xor = function (n) {
	    return bitwise(this, n, function (a, b) {
	      return a ^ b;
	    });
	  };
	  NativeBigInt.prototype.xor = SmallInteger.prototype.xor = BigInteger.prototype.xor;
	  var LOBMASK_I = 1 << 30,
	    LOBMASK_BI = (BASE & -BASE) * (BASE & -BASE) | LOBMASK_I;
	  function roughLOB(n) {
	    // get lowestOneBit (rough)
	    // SmallInteger: return Min(lowestOneBit(n), 1 << 30)
	    // BigInteger: return Min(lowestOneBit(n), 1 << 14) [BASE=1e7]
	    var v = n.value,
	      x = typeof v === "number" ? v | LOBMASK_I : typeof v === "bigint" ? v | BigInt(LOBMASK_I) : v[0] + v[1] * BASE | LOBMASK_BI;
	    return x & -x;
	  }
	  function integerLogarithm(value, base) {
	    if (base.compareTo(value) <= 0) {
	      var tmp = integerLogarithm(value, base.square(base));
	      var p = tmp.p;
	      var e = tmp.e;
	      var t = p.multiply(base);
	      return t.compareTo(value) <= 0 ? {
	        p: t,
	        e: e * 2 + 1
	      } : {
	        p: p,
	        e: e * 2
	      };
	    }
	    return {
	      p: bigInt(1),
	      e: 0
	    };
	  }
	  BigInteger.prototype.bitLength = function () {
	    var n = this;
	    if (n.compareTo(bigInt(0)) < 0) {
	      n = n.negate().subtract(bigInt(1));
	    }
	    if (n.compareTo(bigInt(0)) === 0) {
	      return bigInt(0);
	    }
	    return bigInt(integerLogarithm(n, bigInt(2)).e).add(bigInt(1));
	  };
	  NativeBigInt.prototype.bitLength = SmallInteger.prototype.bitLength = BigInteger.prototype.bitLength;
	  function max(a, b) {
	    a = parseValue(a);
	    b = parseValue(b);
	    return a.greater(b) ? a : b;
	  }
	  function min(a, b) {
	    a = parseValue(a);
	    b = parseValue(b);
	    return a.lesser(b) ? a : b;
	  }
	  function gcd(a, b) {
	    a = parseValue(a).abs();
	    b = parseValue(b).abs();
	    if (a.equals(b)) return a;
	    if (a.isZero()) return b;
	    if (b.isZero()) return a;
	    var c = Integer[1],
	      d,
	      t;
	    while (a.isEven() && b.isEven()) {
	      d = min(roughLOB(a), roughLOB(b));
	      a = a.divide(d);
	      b = b.divide(d);
	      c = c.multiply(d);
	    }
	    while (a.isEven()) {
	      a = a.divide(roughLOB(a));
	    }
	    do {
	      while (b.isEven()) {
	        b = b.divide(roughLOB(b));
	      }
	      if (a.greater(b)) {
	        t = b;
	        b = a;
	        a = t;
	      }
	      b = b.subtract(a);
	    } while (!b.isZero());
	    return c.isUnit() ? a : a.multiply(c);
	  }
	  function lcm(a, b) {
	    a = parseValue(a).abs();
	    b = parseValue(b).abs();
	    return a.divide(gcd(a, b)).multiply(b);
	  }
	  function randBetween(a, b, rng) {
	    a = parseValue(a);
	    b = parseValue(b);
	    var usedRNG = rng || Math.random;
	    var low = min(a, b),
	      high = max(a, b);
	    var range = high.subtract(low).add(1);
	    if (range.isSmall) return low.add(Math.floor(usedRNG() * range));
	    var digits = toBase(range, BASE).value;
	    var result = [],
	      restricted = true;
	    for (var i = 0; i < digits.length; i++) {
	      var top = restricted ? digits[i] + (i + 1 < digits.length ? digits[i + 1] / BASE : 0) : BASE;
	      var digit = truncate(usedRNG() * top);
	      result.push(digit);
	      if (digit < digits[i]) restricted = false;
	    }
	    return low.add(Integer.fromArray(result, BASE, false));
	  }
	  var parseBase = function (text, base, alphabet, caseSensitive) {
	    alphabet = alphabet || DEFAULT_ALPHABET;
	    text = String(text);
	    if (!caseSensitive) {
	      text = text.toLowerCase();
	      alphabet = alphabet.toLowerCase();
	    }
	    var length = text.length;
	    var i;
	    var absBase = Math.abs(base);
	    var alphabetValues = {};
	    for (i = 0; i < alphabet.length; i++) {
	      alphabetValues[alphabet[i]] = i;
	    }
	    for (i = 0; i < length; i++) {
	      var c = text[i];
	      if (c === "-") continue;
	      if (c in alphabetValues) {
	        if (alphabetValues[c] >= absBase) {
	          if (c === "1" && absBase === 1) continue;
	          throw new Error(c + " is not a valid digit in base " + base + ".");
	        }
	      }
	    }
	    base = parseValue(base);
	    var digits = [];
	    var isNegative = text[0] === "-";
	    for (i = isNegative ? 1 : 0; i < text.length; i++) {
	      var c = text[i];
	      if (c in alphabetValues) digits.push(parseValue(alphabetValues[c]));else if (c === "<") {
	        var start = i;
	        do {
	          i++;
	        } while (text[i] !== ">" && i < text.length);
	        digits.push(parseValue(text.slice(start + 1, i)));
	      } else throw new Error(c + " is not a valid character");
	    }
	    return parseBaseFromArray(digits, base, isNegative);
	  };
	  function parseBaseFromArray(digits, base, isNegative) {
	    var val = Integer[0],
	      pow = Integer[1],
	      i;
	    for (i = digits.length - 1; i >= 0; i--) {
	      val = val.add(digits[i].times(pow));
	      pow = pow.times(base);
	    }
	    return isNegative ? val.negate() : val;
	  }
	  function stringify(digit, alphabet) {
	    alphabet = alphabet || DEFAULT_ALPHABET;
	    if (digit < alphabet.length) {
	      return alphabet[digit];
	    }
	    return "<" + digit + ">";
	  }
	  function toBase(n, base) {
	    base = bigInt(base);
	    if (base.isZero()) {
	      if (n.isZero()) return {
	        value: [0],
	        isNegative: false
	      };
	      throw new Error("Cannot convert nonzero numbers to base 0.");
	    }
	    if (base.equals(-1)) {
	      if (n.isZero()) return {
	        value: [0],
	        isNegative: false
	      };
	      if (n.isNegative()) return {
	        value: [].concat.apply([], Array.apply(null, Array(-n.toJSNumber())).map(Array.prototype.valueOf, [1, 0])),
	        isNegative: false
	      };
	      var arr = Array.apply(null, Array(n.toJSNumber() - 1)).map(Array.prototype.valueOf, [0, 1]);
	      arr.unshift([1]);
	      return {
	        value: [].concat.apply([], arr),
	        isNegative: false
	      };
	    }
	    var neg = false;
	    if (n.isNegative() && base.isPositive()) {
	      neg = true;
	      n = n.abs();
	    }
	    if (base.isUnit()) {
	      if (n.isZero()) return {
	        value: [0],
	        isNegative: false
	      };
	      return {
	        value: Array.apply(null, Array(n.toJSNumber())).map(Number.prototype.valueOf, 1),
	        isNegative: neg
	      };
	    }
	    var out = [];
	    var left = n,
	      divmod;
	    while (left.isNegative() || left.compareAbs(base) >= 0) {
	      divmod = left.divmod(base);
	      left = divmod.quotient;
	      var digit = divmod.remainder;
	      if (digit.isNegative()) {
	        digit = base.minus(digit).abs();
	        left = left.next();
	      }
	      out.push(digit.toJSNumber());
	    }
	    out.push(left.toJSNumber());
	    return {
	      value: out.reverse(),
	      isNegative: neg
	    };
	  }
	  function toBaseString(n, base, alphabet) {
	    var arr = toBase(n, base);
	    return (arr.isNegative ? "-" : "") + arr.value.map(function (x) {
	      return stringify(x, alphabet);
	    }).join('');
	  }
	  BigInteger.prototype.toArray = function (radix) {
	    return toBase(this, radix);
	  };
	  SmallInteger.prototype.toArray = function (radix) {
	    return toBase(this, radix);
	  };
	  NativeBigInt.prototype.toArray = function (radix) {
	    return toBase(this, radix);
	  };
	  BigInteger.prototype.toString = function (radix, alphabet) {
	    if (radix === undefined$1) radix = 10;
	    if (radix !== 10 || alphabet) return toBaseString(this, radix, alphabet);
	    var v = this.value,
	      l = v.length,
	      str = String(v[--l]),
	      zeros = "0000000",
	      digit;
	    while (--l >= 0) {
	      digit = String(v[l]);
	      str += zeros.slice(digit.length) + digit;
	    }
	    var sign = this.sign ? "-" : "";
	    return sign + str;
	  };
	  SmallInteger.prototype.toString = function (radix, alphabet) {
	    if (radix === undefined$1) radix = 10;
	    if (radix != 10 || alphabet) return toBaseString(this, radix, alphabet);
	    return String(this.value);
	  };
	  NativeBigInt.prototype.toString = SmallInteger.prototype.toString;
	  NativeBigInt.prototype.toJSON = BigInteger.prototype.toJSON = SmallInteger.prototype.toJSON = function () {
	    return this.toString();
	  };
	  BigInteger.prototype.valueOf = function () {
	    return parseInt(this.toString(), 10);
	  };
	  BigInteger.prototype.toJSNumber = BigInteger.prototype.valueOf;
	  SmallInteger.prototype.valueOf = function () {
	    return this.value;
	  };
	  SmallInteger.prototype.toJSNumber = SmallInteger.prototype.valueOf;
	  NativeBigInt.prototype.valueOf = NativeBigInt.prototype.toJSNumber = function () {
	    return parseInt(this.toString(), 10);
	  };
	  function parseStringValue(v) {
	    if (isPrecise(+v)) {
	      var x = +v;
	      if (x === truncate(x)) return supportsNativeBigInt ? new NativeBigInt(BigInt(x)) : new SmallInteger(x);
	      throw new Error("Invalid integer: " + v);
	    }
	    var sign = v[0] === "-";
	    if (sign) v = v.slice(1);
	    var split = v.split(/e/i);
	    if (split.length > 2) throw new Error("Invalid integer: " + split.join("e"));
	    if (split.length === 2) {
	      var exp = split[1];
	      if (exp[0] === "+") exp = exp.slice(1);
	      exp = +exp;
	      if (exp !== truncate(exp) || !isPrecise(exp)) throw new Error("Invalid integer: " + exp + " is not a valid exponent.");
	      var text = split[0];
	      var decimalPlace = text.indexOf(".");
	      if (decimalPlace >= 0) {
	        exp -= text.length - decimalPlace - 1;
	        text = text.slice(0, decimalPlace) + text.slice(decimalPlace + 1);
	      }
	      if (exp < 0) throw new Error("Cannot include negative exponent part for integers");
	      text += new Array(exp + 1).join("0");
	      v = text;
	    }
	    var isValid = /^([0-9][0-9]*)$/.test(v);
	    if (!isValid) throw new Error("Invalid integer: " + v);
	    if (supportsNativeBigInt) {
	      return new NativeBigInt(BigInt(sign ? "-" + v : v));
	    }
	    var r = [],
	      max = v.length,
	      l = LOG_BASE,
	      min = max - l;
	    while (max > 0) {
	      r.push(+v.slice(min, max));
	      min -= l;
	      if (min < 0) min = 0;
	      max -= l;
	    }
	    trim(r);
	    return new BigInteger(r, sign);
	  }
	  function parseNumberValue(v) {
	    if (supportsNativeBigInt) {
	      return new NativeBigInt(BigInt(v));
	    }
	    if (isPrecise(v)) {
	      if (v !== truncate(v)) throw new Error(v + " is not an integer.");
	      return new SmallInteger(v);
	    }
	    return parseStringValue(v.toString());
	  }
	  function parseValue(v) {
	    if (typeof v === "number") {
	      return parseNumberValue(v);
	    }
	    if (typeof v === "string") {
	      return parseStringValue(v);
	    }
	    if (typeof v === "bigint") {
	      return new NativeBigInt(v);
	    }
	    return v;
	  }
	  // Pre-define numbers in range [-999,999]
	  for (var i = 0; i < 1000; i++) {
	    Integer[i] = parseValue(i);
	    if (i > 0) Integer[-i] = parseValue(-i);
	  }
	  // Backwards compatibility
	  Integer.one = Integer[1];
	  Integer.zero = Integer[0];
	  Integer.minusOne = Integer[-1];
	  Integer.max = max;
	  Integer.min = min;
	  Integer.gcd = gcd;
	  Integer.lcm = lcm;
	  Integer.isInstance = function (x) {
	    return x instanceof BigInteger || x instanceof SmallInteger || x instanceof NativeBigInt;
	  };
	  Integer.randBetween = randBetween;
	  Integer.fromArray = function (digits, base, isNegative) {
	    return parseBaseFromArray(digits.map(parseValue), parseValue(base || 10), isNegative);
	  };
	  return Integer;
	}();

	// Node.js check
	if (module.hasOwnProperty("exports")) {
	  module.exports = bigInt;
	}
} (BigInteger));

var BigIntegerExports = BigInteger.exports;

(function (exports) {

	// adapted from http://code.google.com/p/plist/source/browse/trunk/src/com/dd/plist/BinaryPropertyListParser.java
	const fs = require$$0;
	const bigInt = BigIntegerExports;
	exports.maxObjectSize = 100 * 1000 * 1000; // 100Meg
	exports.maxObjectCount = 32768;

	// EPOCH = new SimpleDateFormat("yyyy MM dd zzz").parse("2001 01 01 GMT").getTime();
	// ...but that's annoying in a static initializer because it can throw exceptions, ick.
	// So we just hardcode the correct value.
	const EPOCH = 978307200000;

	// UID object definition
	const UID = exports.UID = function (id) {
	  this.UID = id;
	};
	exports.parseFile = function (fileNameOrBuffer, callback) {
	  return new Promise(function (resolve, reject) {
	    function tryParseBuffer(buffer) {
	      let err = null;
	      let result;
	      try {
	        result = parseBuffer(buffer);
	        resolve(result);
	      } catch (ex) {
	        err = ex;
	        reject(err);
	      } finally {
	        if (callback) callback(err, result);
	      }
	    }
	    if (Buffer.isBuffer(fileNameOrBuffer)) {
	      return tryParseBuffer(fileNameOrBuffer);
	    }
	    fs.readFile(fileNameOrBuffer, function (err, data) {
	      if (err) {
	        reject(err);
	        return callback(err);
	      }
	      tryParseBuffer(data);
	    });
	  });
	};
	const parseBuffer = exports.parseBuffer = function (buffer) {
	  // check header
	  const header = buffer.slice(0, 'bplist'.length).toString('utf8');
	  if (header !== 'bplist') {
	    throw new Error("Invalid binary plist. Expected 'bplist' at offset 0.");
	  }

	  // Handle trailer, last 32 bytes of the file
	  const trailer = buffer.slice(buffer.length - 32, buffer.length);
	  // 6 null bytes (index 0 to 5)
	  const offsetSize = trailer.readUInt8(6);
	  const objectRefSize = trailer.readUInt8(7);
	  const numObjects = readUInt64BE(trailer, 8);
	  const topObject = readUInt64BE(trailer, 16);
	  const offsetTableOffset = readUInt64BE(trailer, 24);
	  if (numObjects > exports.maxObjectCount) {
	    throw new Error("maxObjectCount exceeded");
	  }

	  // Handle offset table
	  const offsetTable = [];
	  for (let i = 0; i < numObjects; i++) {
	    const offsetBytes = buffer.slice(offsetTableOffset + i * offsetSize, offsetTableOffset + (i + 1) * offsetSize);
	    offsetTable[i] = readUInt(offsetBytes, 0);
	  }

	  // Parses an object inside the currently parsed binary property list.
	  // For the format specification check
	  // <a href="http://www.opensource.apple.com/source/CF/CF-635/CFBinaryPList.c">
	  // Apple's binary property list parser implementation</a>.
	  function parseObject(tableOffset) {
	    const offset = offsetTable[tableOffset];
	    const type = buffer[offset];
	    const objType = (type & 0xF0) >> 4; //First  4 bits
	    const objInfo = type & 0x0F; //Second 4 bits
	    switch (objType) {
	      case 0x0:
	        return parseSimple();
	      case 0x1:
	        return parseInteger();
	      case 0x8:
	        return parseUID();
	      case 0x2:
	        return parseReal();
	      case 0x3:
	        return parseDate();
	      case 0x4:
	        return parseData();
	      case 0x5:
	        // ASCII
	        return parsePlistString();
	      case 0x6:
	        // UTF-16
	        return parsePlistString(true);
	      case 0xA:
	        return parseArray();
	      case 0xD:
	        return parseDictionary();
	      default:
	        throw new Error("Unhandled type 0x" + objType.toString(16));
	    }
	    function parseSimple() {
	      //Simple
	      switch (objInfo) {
	        case 0x0:
	          // null
	          return null;
	        case 0x8:
	          // false
	          return false;
	        case 0x9:
	          // true
	          return true;
	        case 0xF:
	          // filler byte
	          return null;
	        default:
	          throw new Error("Unhandled simple type 0x" + objType.toString(16));
	      }
	    }
	    function bufferToHexString(buffer) {
	      let str = '';
	      let i;
	      for (i = 0; i < buffer.length; i++) {
	        if (buffer[i] != 0x00) {
	          break;
	        }
	      }
	      for (; i < buffer.length; i++) {
	        const part = '00' + buffer[i].toString(16);
	        str += part.substr(part.length - 2);
	      }
	      return str;
	    }
	    function parseInteger() {
	      const length = Math.pow(2, objInfo);
	      if (objInfo == 0x4) {
	        const data = buffer.slice(offset + 1, offset + 1 + length);
	        const str = bufferToHexString(data);
	        return bigInt(str, 16);
	      }
	      if (objInfo == 0x3) {
	        return buffer.readInt32BE(offset + 1);
	      }
	      if (length < exports.maxObjectSize) {
	        return readUInt(buffer.slice(offset + 1, offset + 1 + length));
	      }
	      throw new Error("To little heap space available! Wanted to read " + length + " bytes, but only " + exports.maxObjectSize + " are available.");
	    }
	    function parseUID() {
	      const length = objInfo + 1;
	      if (length < exports.maxObjectSize) {
	        return new UID(readUInt(buffer.slice(offset + 1, offset + 1 + length)));
	      }
	      throw new Error("To little heap space available! Wanted to read " + length + " bytes, but only " + exports.maxObjectSize + " are available.");
	    }
	    function parseReal() {
	      const length = Math.pow(2, objInfo);
	      if (length < exports.maxObjectSize) {
	        const realBuffer = buffer.slice(offset + 1, offset + 1 + length);
	        if (length === 4) {
	          return realBuffer.readFloatBE(0);
	        }
	        if (length === 8) {
	          return realBuffer.readDoubleBE(0);
	        }
	      } else {
	        throw new Error("To little heap space available! Wanted to read " + length + " bytes, but only " + exports.maxObjectSize + " are available.");
	      }
	    }
	    function parseDate() {
	      if (objInfo != 0x3) {
	        console.error("Unknown date type :" + objInfo + ". Parsing anyway...");
	      }
	      const dateBuffer = buffer.slice(offset + 1, offset + 9);
	      return new Date(EPOCH + 1000 * dateBuffer.readDoubleBE(0));
	    }
	    function parseData() {
	      let dataoffset = 1;
	      let length = objInfo;
	      if (objInfo == 0xF) {
	        const int_type = buffer[offset + 1];
	        const intType = (int_type & 0xF0) / 0x10;
	        if (intType != 0x1) {
	          console.error("0x4: UNEXPECTED LENGTH-INT TYPE! " + intType);
	        }
	        const intInfo = int_type & 0x0F;
	        const intLength = Math.pow(2, intInfo);
	        dataoffset = 2 + intLength;
	        if (intLength < 3) {
	          length = readUInt(buffer.slice(offset + 2, offset + 2 + intLength));
	        } else {
	          length = readUInt(buffer.slice(offset + 2, offset + 2 + intLength));
	        }
	      }
	      if (length < exports.maxObjectSize) {
	        return buffer.slice(offset + dataoffset, offset + dataoffset + length);
	      }
	      throw new Error("To little heap space available! Wanted to read " + length + " bytes, but only " + exports.maxObjectSize + " are available.");
	    }
	    function parsePlistString(isUtf16) {
	      isUtf16 = isUtf16 || 0;
	      let enc = "utf8";
	      let length = objInfo;
	      let stroffset = 1;
	      if (objInfo == 0xF) {
	        const int_type = buffer[offset + 1];
	        const intType = (int_type & 0xF0) / 0x10;
	        if (intType != 0x1) {
	          console.err("UNEXPECTED LENGTH-INT TYPE! " + intType);
	        }
	        const intInfo = int_type & 0x0F;
	        const intLength = Math.pow(2, intInfo);
	        stroffset = 2 + intLength;
	        if (intLength < 3) {
	          length = readUInt(buffer.slice(offset + 2, offset + 2 + intLength));
	        } else {
	          length = readUInt(buffer.slice(offset + 2, offset + 2 + intLength));
	        }
	      }
	      // length is String length -> to get byte length multiply by 2, as 1 character takes 2 bytes in UTF-16
	      length *= isUtf16 + 1;
	      if (length < exports.maxObjectSize) {
	        let plistString = Buffer.from(buffer.slice(offset + stroffset, offset + stroffset + length));
	        if (isUtf16) {
	          plistString = swapBytes(plistString);
	          enc = "ucs2";
	        }
	        return plistString.toString(enc);
	      }
	      throw new Error("To little heap space available! Wanted to read " + length + " bytes, but only " + exports.maxObjectSize + " are available.");
	    }
	    function parseArray() {
	      let length = objInfo;
	      let arrayoffset = 1;
	      if (objInfo == 0xF) {
	        const int_type = buffer[offset + 1];
	        const intType = (int_type & 0xF0) / 0x10;
	        if (intType != 0x1) {
	          console.error("0xa: UNEXPECTED LENGTH-INT TYPE! " + intType);
	        }
	        const intInfo = int_type & 0x0F;
	        const intLength = Math.pow(2, intInfo);
	        arrayoffset = 2 + intLength;
	        if (intLength < 3) {
	          length = readUInt(buffer.slice(offset + 2, offset + 2 + intLength));
	        } else {
	          length = readUInt(buffer.slice(offset + 2, offset + 2 + intLength));
	        }
	      }
	      if (length * objectRefSize > exports.maxObjectSize) {
	        throw new Error("To little heap space available!");
	      }
	      const array = [];
	      for (let i = 0; i < length; i++) {
	        const objRef = readUInt(buffer.slice(offset + arrayoffset + i * objectRefSize, offset + arrayoffset + (i + 1) * objectRefSize));
	        array[i] = parseObject(objRef);
	      }
	      return array;
	    }
	    function parseDictionary() {
	      let length = objInfo;
	      let dictoffset = 1;
	      if (objInfo == 0xF) {
	        const int_type = buffer[offset + 1];
	        const intType = (int_type & 0xF0) / 0x10;
	        if (intType != 0x1) {
	          console.error("0xD: UNEXPECTED LENGTH-INT TYPE! " + intType);
	        }
	        const intInfo = int_type & 0x0F;
	        const intLength = Math.pow(2, intInfo);
	        dictoffset = 2 + intLength;
	        if (intLength < 3) {
	          length = readUInt(buffer.slice(offset + 2, offset + 2 + intLength));
	        } else {
	          length = readUInt(buffer.slice(offset + 2, offset + 2 + intLength));
	        }
	      }
	      if (length * 2 * objectRefSize > exports.maxObjectSize) {
	        throw new Error("To little heap space available!");
	      }
	      const dict = {};
	      for (let i = 0; i < length; i++) {
	        const keyRef = readUInt(buffer.slice(offset + dictoffset + i * objectRefSize, offset + dictoffset + (i + 1) * objectRefSize));
	        const valRef = readUInt(buffer.slice(offset + dictoffset + length * objectRefSize + i * objectRefSize, offset + dictoffset + length * objectRefSize + (i + 1) * objectRefSize));
	        const key = parseObject(keyRef);
	        const val = parseObject(valRef);
	        dict[key] = val;
	      }
	      return dict;
	    }
	  }
	  return [parseObject(topObject)];
	};
	function readUInt(buffer, start) {
	  start = start || 0;
	  let l = 0;
	  for (let i = start; i < buffer.length; i++) {
	    l <<= 8;
	    l |= buffer[i] & 0xFF;
	  }
	  return l;
	}

	// we're just going to toss the high order bits because javascript doesn't have 64-bit ints
	function readUInt64BE(buffer, start) {
	  const data = buffer.slice(start, start + 8);
	  return data.readUInt32BE(4, 8);
	}
	function swapBytes(buffer) {
	  const len = buffer.length;
	  for (let i = 0; i < len; i += 2) {
	    const a = buffer[i];
	    buffer[i] = buffer[i + 1];
	    buffer[i + 1] = a;
	  }
	  return buffer;
	} 
} (bplistParser));

var bplist = /*@__PURE__*/getDefaultExportFromCjs(bplistParser);

const os$1 = require$$0$1;
const homeDirectory = os$1.homedir();
var untildify = pathWithTilde => {
  if (typeof pathWithTilde !== 'string') {
    throw new TypeError(`Expected a string, got ${typeof pathWithTilde}`);
  }
  return homeDirectory ? pathWithTilde.replace(/^~(?=$|\/|\\)/, homeDirectory) : pathWithTilde;
};

var untildify$1 = /*@__PURE__*/getDefaultExportFromCjs(untildify);

const macOsVersion = Number(require$$0$1.release().split('.')[0]);
const filePath = untildify$1(macOsVersion >= 14 ? '~/Library/Preferences/com.apple.LaunchServices/com.apple.launchservices.secure.plist' : '~/Library/Preferences/com.apple.LaunchServices.plist');
async function defaultBrowserId() {
  if (process.platform !== 'darwin') {
    throw new Error('macOS only');
  }
  let bundleId = 'com.apple.Safari';
  let buffer;
  try {
    buffer = await promises.readFile(filePath);
  } catch (error) {
    if (error.code === 'ENOENT') {
      return bundleId;
    }
    throw error;
  }
  const data = bplist.parseBuffer(buffer);
  const handlers = data && data[0].LSHandlers;
  if (!handlers || handlers.length === 0) {
    return bundleId;
  }
  for (const handler of handlers) {
    if (handler.LSHandlerURLScheme === 'http' && handler.LSHandlerRoleAll) {
      bundleId = handler.LSHandlerRoleAll;
      break;
    }
  }
  return bundleId;
}

var execa$3 = {exports: {}};

var crossSpawn$2 = {exports: {}};

var mode;
var hasRequiredMode;

function requireMode () {
	if (hasRequiredMode) return mode;
	hasRequiredMode = 1;
	mode = isexe;
	isexe.sync = sync;
	var fs = require$$0;
	function isexe(path, options, cb) {
	  fs.stat(path, function (er, stat) {
	    cb(er, er ? false : checkStat(stat, options));
	  });
	}
	function sync(path, options) {
	  return checkStat(fs.statSync(path), options);
	}
	function checkStat(stat, options) {
	  return stat.isFile() && checkMode(stat, options);
	}
	function checkMode(stat, options) {
	  var mod = stat.mode;
	  var uid = stat.uid;
	  var gid = stat.gid;
	  var myUid = options.uid !== undefined ? options.uid : process.getuid && process.getuid();
	  var myGid = options.gid !== undefined ? options.gid : process.getgid && process.getgid();
	  var u = parseInt('100', 8);
	  var g = parseInt('010', 8);
	  var o = parseInt('001', 8);
	  var ug = u | g;
	  var ret = mod & o || mod & g && gid === myGid || mod & u && uid === myUid || mod & ug && myUid === 0;
	  return ret;
	}
	return mode;
}

var windows;
var hasRequiredWindows;

function requireWindows () {
	if (hasRequiredWindows) return windows;
	hasRequiredWindows = 1;
	windows = isexe;
	isexe.sync = sync;
	var fs = require$$0;
	function checkPathExt(path, options) {
	  var pathext = options.pathExt !== undefined ? options.pathExt : process.env.PATHEXT;
	  if (!pathext) {
	    return true;
	  }
	  pathext = pathext.split(';');
	  if (pathext.indexOf('') !== -1) {
	    return true;
	  }
	  for (var i = 0; i < pathext.length; i++) {
	    var p = pathext[i].toLowerCase();
	    if (p && path.substr(-p.length).toLowerCase() === p) {
	      return true;
	    }
	  }
	  return false;
	}
	function checkStat(stat, path, options) {
	  if (!stat.isSymbolicLink() && !stat.isFile()) {
	    return false;
	  }
	  return checkPathExt(path, options);
	}
	function isexe(path, options, cb) {
	  fs.stat(path, function (er, stat) {
	    cb(er, er ? false : checkStat(stat, path, options));
	  });
	}
	function sync(path, options) {
	  return checkStat(fs.statSync(path), path, options);
	}
	return windows;
}

var core$1;
if (process.platform === 'win32' || commonjsGlobal.TESTING_WINDOWS) {
  core$1 = requireWindows();
} else {
  core$1 = requireMode();
}
var isexe_1 = isexe$1;
isexe$1.sync = sync;
function isexe$1(path, options, cb) {
  if (typeof options === 'function') {
    cb = options;
    options = {};
  }
  if (!cb) {
    if (typeof Promise !== 'function') {
      throw new TypeError('callback not provided');
    }
    return new Promise(function (resolve, reject) {
      isexe$1(path, options || {}, function (er, is) {
        if (er) {
          reject(er);
        } else {
          resolve(is);
        }
      });
    });
  }
  core$1(path, options || {}, function (er, is) {
    // ignore EACCES because that just means we aren't allowed to run it
    if (er) {
      if (er.code === 'EACCES' || options && options.ignoreErrors) {
        er = null;
        is = false;
      }
    }
    cb(er, is);
  });
}
function sync(path, options) {
  // my kingdom for a filtered catch
  try {
    return core$1.sync(path, options || {});
  } catch (er) {
    if (options && options.ignoreErrors || er.code === 'EACCES') {
      return false;
    } else {
      throw er;
    }
  }
}

const isWindows = process.platform === 'win32' || process.env.OSTYPE === 'cygwin' || process.env.OSTYPE === 'msys';
const path$4 = path$6;
const COLON = isWindows ? ';' : ':';
const isexe = isexe_1;
const getNotFoundError = cmd => Object.assign(new Error(`not found: ${cmd}`), {
  code: 'ENOENT'
});
const getPathInfo = (cmd, opt) => {
  const colon = opt.colon || COLON;

  // If it has a slash, then we don't bother searching the pathenv.
  // just check the file itself, and that's it.
  const pathEnv = cmd.match(/\//) || isWindows && cmd.match(/\\/) ? [''] : [
  // windows always checks the cwd first
  ...(isWindows ? [process.cwd()] : []), ...(opt.path || process.env.PATH || /* istanbul ignore next: very unusual */'').split(colon)];
  const pathExtExe = isWindows ? opt.pathExt || process.env.PATHEXT || '.EXE;.CMD;.BAT;.COM' : '';
  const pathExt = isWindows ? pathExtExe.split(colon) : [''];
  if (isWindows) {
    if (cmd.indexOf('.') !== -1 && pathExt[0] !== '') pathExt.unshift('');
  }
  return {
    pathEnv,
    pathExt,
    pathExtExe
  };
};
const which$1 = (cmd, opt, cb) => {
  if (typeof opt === 'function') {
    cb = opt;
    opt = {};
  }
  if (!opt) opt = {};
  const {
    pathEnv,
    pathExt,
    pathExtExe
  } = getPathInfo(cmd, opt);
  const found = [];
  const step = i => new Promise((resolve, reject) => {
    if (i === pathEnv.length) return opt.all && found.length ? resolve(found) : reject(getNotFoundError(cmd));
    const ppRaw = pathEnv[i];
    const pathPart = /^".*"$/.test(ppRaw) ? ppRaw.slice(1, -1) : ppRaw;
    const pCmd = path$4.join(pathPart, cmd);
    const p = !pathPart && /^\.[\\\/]/.test(cmd) ? cmd.slice(0, 2) + pCmd : pCmd;
    resolve(subStep(p, i, 0));
  });
  const subStep = (p, i, ii) => new Promise((resolve, reject) => {
    if (ii === pathExt.length) return resolve(step(i + 1));
    const ext = pathExt[ii];
    isexe(p + ext, {
      pathExt: pathExtExe
    }, (er, is) => {
      if (!er && is) {
        if (opt.all) found.push(p + ext);else return resolve(p + ext);
      }
      return resolve(subStep(p, i, ii + 1));
    });
  });
  return cb ? step(0).then(res => cb(null, res), cb) : step(0);
};
const whichSync = (cmd, opt) => {
  opt = opt || {};
  const {
    pathEnv,
    pathExt,
    pathExtExe
  } = getPathInfo(cmd, opt);
  const found = [];
  for (let i = 0; i < pathEnv.length; i++) {
    const ppRaw = pathEnv[i];
    const pathPart = /^".*"$/.test(ppRaw) ? ppRaw.slice(1, -1) : ppRaw;
    const pCmd = path$4.join(pathPart, cmd);
    const p = !pathPart && /^\.[\\\/]/.test(cmd) ? cmd.slice(0, 2) + pCmd : pCmd;
    for (let j = 0; j < pathExt.length; j++) {
      const cur = p + pathExt[j];
      try {
        const is = isexe.sync(cur, {
          pathExt: pathExtExe
        });
        if (is) {
          if (opt.all) found.push(cur);else return cur;
        }
      } catch (ex) {}
    }
  }
  if (opt.all && found.length) return found;
  if (opt.nothrow) return null;
  throw getNotFoundError(cmd);
};
var which_1 = which$1;
which$1.sync = whichSync;

var pathKey$2 = {exports: {}};

const pathKey$1 = (options = {}) => {
  const environment = options.env || process.env;
  const platform = options.platform || process.platform;
  if (platform !== 'win32') {
    return 'PATH';
  }
  return Object.keys(environment).reverse().find(key => key.toUpperCase() === 'PATH') || 'Path';
};
pathKey$2.exports = pathKey$1;
// TODO: Remove this for the next major release
pathKey$2.exports.default = pathKey$1;

var pathKeyExports = pathKey$2.exports;

const path$3 = path$6;
const which = which_1;
const getPathKey = pathKeyExports;
function resolveCommandAttempt(parsed, withoutPathExt) {
  const env = parsed.options.env || process.env;
  const cwd = process.cwd();
  const hasCustomCwd = parsed.options.cwd != null;
  // Worker threads do not have process.chdir()
  const shouldSwitchCwd = hasCustomCwd && process.chdir !== undefined && !process.chdir.disabled;

  // If a custom `cwd` was specified, we need to change the process cwd
  // because `which` will do stat calls but does not support a custom cwd
  if (shouldSwitchCwd) {
    try {
      process.chdir(parsed.options.cwd);
    } catch (err) {
      /* Empty */
    }
  }
  let resolved;
  try {
    resolved = which.sync(parsed.command, {
      path: env[getPathKey({
        env
      })],
      pathExt: withoutPathExt ? path$3.delimiter : undefined
    });
  } catch (e) {
    /* Empty */
  } finally {
    if (shouldSwitchCwd) {
      process.chdir(cwd);
    }
  }

  // If we successfully resolved, ensure that an absolute path is returned
  // Note that when a custom `cwd` was used, we need to resolve to an absolute path based on it
  if (resolved) {
    resolved = path$3.resolve(hasCustomCwd ? parsed.options.cwd : '', resolved);
  }
  return resolved;
}
function resolveCommand$1(parsed) {
  return resolveCommandAttempt(parsed) || resolveCommandAttempt(parsed, true);
}
var resolveCommand_1 = resolveCommand$1;

var _escape = {};

// See http://www.robvanderwoude.com/escapechars.php
const metaCharsRegExp = /([()\][%!^"`<>&|;, *?])/g;
function escapeCommand(arg) {
  // Escape meta chars
  arg = arg.replace(metaCharsRegExp, '^$1');
  return arg;
}
function escapeArgument(arg, doubleEscapeMetaChars) {
  // Convert to string
  arg = `${arg}`;

  // Algorithm below is based on https://qntm.org/cmd

  // Sequence of backslashes followed by a double quote:
  // double up all the backslashes and escape the double quote
  arg = arg.replace(/(\\*)"/g, '$1$1\\"');

  // Sequence of backslashes followed by the end of the string
  // (which will become a double quote later):
  // double up all the backslashes
  arg = arg.replace(/(\\*)$/, '$1$1');

  // All other backslashes occur literally

  // Quote the whole thing:
  arg = `"${arg}"`;

  // Escape meta chars
  arg = arg.replace(metaCharsRegExp, '^$1');

  // Double escape meta chars if necessary
  if (doubleEscapeMetaChars) {
    arg = arg.replace(metaCharsRegExp, '^$1');
  }
  return arg;
}
_escape.command = escapeCommand;
_escape.argument = escapeArgument;

var shebangRegex$1 = /^#!(.*)/;

const shebangRegex = shebangRegex$1;
var shebangCommand$1 = (string = '') => {
  const match = string.match(shebangRegex);
  if (!match) {
    return null;
  }
  const [path, argument] = match[0].replace(/#! ?/, '').split(' ');
  const binary = path.split('/').pop();
  if (binary === 'env') {
    return argument;
  }
  return argument ? `${binary} ${argument}` : binary;
};

const fs$1 = require$$0;
const shebangCommand = shebangCommand$1;
function readShebang$1(command) {
  // Read the first 150 bytes from the file
  const size = 150;
  const buffer = Buffer.alloc(size);
  let fd;
  try {
    fd = fs$1.openSync(command, 'r');
    fs$1.readSync(fd, buffer, 0, size, 0);
    fs$1.closeSync(fd);
  } catch (e) {/* Empty */}

  // Attempt to extract shebang (null is returned if not a shebang)
  return shebangCommand(buffer.toString());
}
var readShebang_1 = readShebang$1;

const path$2 = path$6;
const resolveCommand = resolveCommand_1;
const escape = _escape;
const readShebang = readShebang_1;
const isWin$2 = process.platform === 'win32';
const isExecutableRegExp = /\.(?:com|exe)$/i;
const isCmdShimRegExp = /node_modules[\\/].bin[\\/][^\\/]+\.cmd$/i;
function detectShebang(parsed) {
  parsed.file = resolveCommand(parsed);
  const shebang = parsed.file && readShebang(parsed.file);
  if (shebang) {
    parsed.args.unshift(parsed.file);
    parsed.command = shebang;
    return resolveCommand(parsed);
  }
  return parsed.file;
}
function parseNonShell(parsed) {
  if (!isWin$2) {
    return parsed;
  }

  // Detect & add support for shebangs
  const commandFile = detectShebang(parsed);

  // We don't need a shell if the command filename is an executable
  const needsShell = !isExecutableRegExp.test(commandFile);

  // If a shell is required, use cmd.exe and take care of escaping everything correctly
  // Note that `forceShell` is an hidden option used only in tests
  if (parsed.options.forceShell || needsShell) {
    // Need to double escape meta chars if the command is a cmd-shim located in `node_modules/.bin/`
    // The cmd-shim simply calls execute the package bin file with NodeJS, proxying any argument
    // Because the escape of metachars with ^ gets interpreted when the cmd.exe is first called,
    // we need to double escape them
    const needsDoubleEscapeMetaChars = isCmdShimRegExp.test(commandFile);

    // Normalize posix paths into OS compatible paths (e.g.: foo/bar -> foo\bar)
    // This is necessary otherwise it will always fail with ENOENT in those cases
    parsed.command = path$2.normalize(parsed.command);

    // Escape command & arguments
    parsed.command = escape.command(parsed.command);
    parsed.args = parsed.args.map(arg => escape.argument(arg, needsDoubleEscapeMetaChars));
    const shellCommand = [parsed.command].concat(parsed.args).join(' ');
    parsed.args = ['/d', '/s', '/c', `"${shellCommand}"`];
    parsed.command = process.env.comspec || 'cmd.exe';
    parsed.options.windowsVerbatimArguments = true; // Tell node's spawn that the arguments are already escaped
  }

  return parsed;
}
function parse$1(command, args, options) {
  // Normalize arguments, similar to nodejs
  if (args && !Array.isArray(args)) {
    options = args;
    args = null;
  }
  args = args ? args.slice(0) : []; // Clone array to avoid changing the original
  options = Object.assign({}, options); // Clone object to avoid changing the original

  // Build our parsed object
  const parsed = {
    command,
    args,
    options,
    file: undefined,
    original: {
      command,
      args
    }
  };

  // Delegate further parsing to shell or non-shell
  return options.shell ? parsed : parseNonShell(parsed);
}
var parse_1 = parse$1;

const isWin$1 = process.platform === 'win32';
function notFoundError(original, syscall) {
  return Object.assign(new Error(`${syscall} ${original.command} ENOENT`), {
    code: 'ENOENT',
    errno: 'ENOENT',
    syscall: `${syscall} ${original.command}`,
    path: original.command,
    spawnargs: original.args
  });
}
function hookChildProcess(cp, parsed) {
  if (!isWin$1) {
    return;
  }
  const originalEmit = cp.emit;
  cp.emit = function (name, arg1) {
    // If emitting "exit" event and exit code is 1, we need to check if
    // the command exists and emit an "error" instead
    // See https://github.com/IndigoUnited/node-cross-spawn/issues/16
    if (name === 'exit') {
      const err = verifyENOENT(arg1, parsed);
      if (err) {
        return originalEmit.call(cp, 'error', err);
      }
    }
    return originalEmit.apply(cp, arguments); // eslint-disable-line prefer-rest-params
  };
}

function verifyENOENT(status, parsed) {
  if (isWin$1 && status === 1 && !parsed.file) {
    return notFoundError(parsed.original, 'spawn');
  }
  return null;
}
function verifyENOENTSync(status, parsed) {
  if (isWin$1 && status === 1 && !parsed.file) {
    return notFoundError(parsed.original, 'spawnSync');
  }
  return null;
}
var enoent$1 = {
  hookChildProcess,
  verifyENOENT,
  verifyENOENTSync,
  notFoundError
};

const cp = require$$1$2;
const parse = parse_1;
const enoent = enoent$1;
function spawn(command, args, options) {
  // Parse the arguments
  const parsed = parse(command, args, options);

  // Spawn the child process
  const spawned = cp.spawn(parsed.command, parsed.args, parsed.options);

  // Hook into child process "exit" event to emit an error if the command
  // does not exists, see: https://github.com/IndigoUnited/node-cross-spawn/issues/16
  enoent.hookChildProcess(spawned, parsed);
  return spawned;
}
function spawnSync(command, args, options) {
  // Parse the arguments
  const parsed = parse(command, args, options);

  // Spawn the child process
  const result = cp.spawnSync(parsed.command, parsed.args, parsed.options);

  // Analyze if the command does not exist, see: https://github.com/IndigoUnited/node-cross-spawn/issues/16
  result.error = result.error || enoent.verifyENOENTSync(result.status, parsed);
  return result;
}
crossSpawn$2.exports = spawn;
crossSpawn$2.exports.spawn = spawn;
crossSpawn$2.exports.sync = spawnSync;
crossSpawn$2.exports._parse = parse;
crossSpawn$2.exports._enoent = enoent;

var crossSpawnExports = crossSpawn$2.exports;
var crossSpawn$1 = /*@__PURE__*/getDefaultExportFromCjs(crossSpawnExports);

var stripFinalNewline$2 = input => {
  const LF = typeof input === 'string' ? '\n' : '\n'.charCodeAt();
  const CR = typeof input === 'string' ? '\r' : '\r'.charCodeAt();
  if (input[input.length - 1] === LF) {
    input = input.slice(0, input.length - 1);
  }
  if (input[input.length - 1] === CR) {
    input = input.slice(0, input.length - 1);
  }
  return input;
};

var npmRunPath$2 = {exports: {}};

npmRunPath$2.exports;

(function (module) {

	const path = path$6;
	const pathKey = pathKeyExports;
	const npmRunPath = options => {
	  options = {
	    cwd: process.cwd(),
	    path: process.env[pathKey()],
	    execPath: process.execPath,
	    ...options
	  };
	  let previous;
	  let cwdPath = path.resolve(options.cwd);
	  const result = [];
	  while (previous !== cwdPath) {
	    result.push(path.join(cwdPath, 'node_modules/.bin'));
	    previous = cwdPath;
	    cwdPath = path.resolve(cwdPath, '..');
	  }

	  // Ensure the running `node` binary is used
	  const execPathDir = path.resolve(options.cwd, options.execPath, '..');
	  result.push(execPathDir);
	  return result.concat(options.path).join(path.delimiter);
	};
	module.exports = npmRunPath;
	// TODO: Remove this for the next major release
	module.exports.default = npmRunPath;
	module.exports.env = options => {
	  options = {
	    env: process.env,
	    ...options
	  };
	  const env = {
	    ...options.env
	  };
	  const path = pathKey({
	    env
	  });
	  options.path = env[path];
	  env[path] = module.exports(options);
	  return env;
	}; 
} (npmRunPath$2));

var npmRunPathExports = npmRunPath$2.exports;

var onetime$3 = {exports: {}};

var mimicFn$2 = {exports: {}};

const mimicFn$1 = (to, from) => {
  for (const prop of Reflect.ownKeys(from)) {
    Object.defineProperty(to, prop, Object.getOwnPropertyDescriptor(from, prop));
  }
  return to;
};
mimicFn$2.exports = mimicFn$1;
// TODO: Remove this for the next major release
mimicFn$2.exports.default = mimicFn$1;

var mimicFnExports = mimicFn$2.exports;

const mimicFn = mimicFnExports;
const calledFunctions$1 = new WeakMap();
const onetime$2 = (function_, options = {}) => {
  if (typeof function_ !== 'function') {
    throw new TypeError('Expected a function');
  }
  let returnValue;
  let callCount = 0;
  const functionName = function_.displayName || function_.name || '<anonymous>';
  const onetime = function (...arguments_) {
    calledFunctions$1.set(onetime, ++callCount);
    if (callCount === 1) {
      returnValue = function_.apply(this, arguments_);
      function_ = null;
    } else if (options.throw === true) {
      throw new Error(`Function \`${functionName}\` can only be called once`);
    }
    return returnValue;
  };
  mimicFn(onetime, function_);
  calledFunctions$1.set(onetime, callCount);
  return onetime;
};
onetime$3.exports = onetime$2;
// TODO: Remove this for the next major release
onetime$3.exports.default = onetime$2;
onetime$3.exports.callCount = function_ => {
  if (!calledFunctions$1.has(function_)) {
    throw new Error(`The given function \`${function_.name}\` is not wrapped by the \`onetime\` package`);
  }
  return calledFunctions$1.get(function_);
};

var onetimeExports = onetime$3.exports;

var main$1 = {};

var signals$2 = {};

var core = {};

Object.defineProperty(core, "__esModule", {
  value: true
});
core.SIGNALS = void 0;
const SIGNALS$1 = [{
  name: "SIGHUP",
  number: 1,
  action: "terminate",
  description: "Terminal closed",
  standard: "posix"
}, {
  name: "SIGINT",
  number: 2,
  action: "terminate",
  description: "User interruption with CTRL-C",
  standard: "ansi"
}, {
  name: "SIGQUIT",
  number: 3,
  action: "core",
  description: "User interruption with CTRL-\\",
  standard: "posix"
}, {
  name: "SIGILL",
  number: 4,
  action: "core",
  description: "Invalid machine instruction",
  standard: "ansi"
}, {
  name: "SIGTRAP",
  number: 5,
  action: "core",
  description: "Debugger breakpoint",
  standard: "posix"
}, {
  name: "SIGABRT",
  number: 6,
  action: "core",
  description: "Aborted",
  standard: "ansi"
}, {
  name: "SIGIOT",
  number: 6,
  action: "core",
  description: "Aborted",
  standard: "bsd"
}, {
  name: "SIGBUS",
  number: 7,
  action: "core",
  description: "Bus error due to misaligned, non-existing address or paging error",
  standard: "bsd"
}, {
  name: "SIGEMT",
  number: 7,
  action: "terminate",
  description: "Command should be emulated but is not implemented",
  standard: "other"
}, {
  name: "SIGFPE",
  number: 8,
  action: "core",
  description: "Floating point arithmetic error",
  standard: "ansi"
}, {
  name: "SIGKILL",
  number: 9,
  action: "terminate",
  description: "Forced termination",
  standard: "posix",
  forced: true
}, {
  name: "SIGUSR1",
  number: 10,
  action: "terminate",
  description: "Application-specific signal",
  standard: "posix"
}, {
  name: "SIGSEGV",
  number: 11,
  action: "core",
  description: "Segmentation fault",
  standard: "ansi"
}, {
  name: "SIGUSR2",
  number: 12,
  action: "terminate",
  description: "Application-specific signal",
  standard: "posix"
}, {
  name: "SIGPIPE",
  number: 13,
  action: "terminate",
  description: "Broken pipe or socket",
  standard: "posix"
}, {
  name: "SIGALRM",
  number: 14,
  action: "terminate",
  description: "Timeout or timer",
  standard: "posix"
}, {
  name: "SIGTERM",
  number: 15,
  action: "terminate",
  description: "Termination",
  standard: "ansi"
}, {
  name: "SIGSTKFLT",
  number: 16,
  action: "terminate",
  description: "Stack is empty or overflowed",
  standard: "other"
}, {
  name: "SIGCHLD",
  number: 17,
  action: "ignore",
  description: "Child process terminated, paused or unpaused",
  standard: "posix"
}, {
  name: "SIGCLD",
  number: 17,
  action: "ignore",
  description: "Child process terminated, paused or unpaused",
  standard: "other"
}, {
  name: "SIGCONT",
  number: 18,
  action: "unpause",
  description: "Unpaused",
  standard: "posix",
  forced: true
}, {
  name: "SIGSTOP",
  number: 19,
  action: "pause",
  description: "Paused",
  standard: "posix",
  forced: true
}, {
  name: "SIGTSTP",
  number: 20,
  action: "pause",
  description: "Paused using CTRL-Z or \"suspend\"",
  standard: "posix"
}, {
  name: "SIGTTIN",
  number: 21,
  action: "pause",
  description: "Background process cannot read terminal input",
  standard: "posix"
}, {
  name: "SIGBREAK",
  number: 21,
  action: "terminate",
  description: "User interruption with CTRL-BREAK",
  standard: "other"
}, {
  name: "SIGTTOU",
  number: 22,
  action: "pause",
  description: "Background process cannot write to terminal output",
  standard: "posix"
}, {
  name: "SIGURG",
  number: 23,
  action: "ignore",
  description: "Socket received out-of-band data",
  standard: "bsd"
}, {
  name: "SIGXCPU",
  number: 24,
  action: "core",
  description: "Process timed out",
  standard: "bsd"
}, {
  name: "SIGXFSZ",
  number: 25,
  action: "core",
  description: "File too big",
  standard: "bsd"
}, {
  name: "SIGVTALRM",
  number: 26,
  action: "terminate",
  description: "Timeout or timer",
  standard: "bsd"
}, {
  name: "SIGPROF",
  number: 27,
  action: "terminate",
  description: "Timeout or timer",
  standard: "bsd"
}, {
  name: "SIGWINCH",
  number: 28,
  action: "ignore",
  description: "Terminal window size changed",
  standard: "bsd"
}, {
  name: "SIGIO",
  number: 29,
  action: "terminate",
  description: "I/O is available",
  standard: "other"
}, {
  name: "SIGPOLL",
  number: 29,
  action: "terminate",
  description: "Watched event",
  standard: "other"
}, {
  name: "SIGINFO",
  number: 29,
  action: "ignore",
  description: "Request for process information",
  standard: "other"
}, {
  name: "SIGPWR",
  number: 30,
  action: "terminate",
  description: "Device running out of power",
  standard: "systemv"
}, {
  name: "SIGSYS",
  number: 31,
  action: "core",
  description: "Invalid system call",
  standard: "other"
}, {
  name: "SIGUNUSED",
  number: 31,
  action: "terminate",
  description: "Invalid system call",
  standard: "other"
}];
core.SIGNALS = SIGNALS$1;

var realtime = {};

Object.defineProperty(realtime, "__esModule", {
  value: true
});
realtime.SIGRTMAX = realtime.getRealtimeSignals = void 0;
const getRealtimeSignals$1 = function () {
  const length = SIGRTMAX$1 - SIGRTMIN$1 + 1;
  return Array.from({
    length
  }, getRealtimeSignal$1);
};
realtime.getRealtimeSignals = getRealtimeSignals$1;
const getRealtimeSignal$1 = function (value, index) {
  return {
    name: `SIGRT${index + 1}`,
    number: SIGRTMIN$1 + index,
    action: "terminate",
    description: "Application-specific signal (realtime)",
    standard: "posix"
  };
};
const SIGRTMIN$1 = 34;
const SIGRTMAX$1 = 64;
realtime.SIGRTMAX = SIGRTMAX$1;

Object.defineProperty(signals$2, "__esModule", {
  value: true
});
signals$2.getSignals = void 0;
var _os$1 = require$$0$1;
var _core = core;
var _realtime$1 = realtime;
const getSignals$1 = function () {
  const realtimeSignals = (0, _realtime$1.getRealtimeSignals)();
  const signals = [..._core.SIGNALS, ...realtimeSignals].map(normalizeSignal$1);
  return signals;
};
signals$2.getSignals = getSignals$1;
const normalizeSignal$1 = function ({
  name,
  number: defaultNumber,
  description,
  action,
  forced = false,
  standard
}) {
  const {
    signals: {
      [name]: constantSignal
    }
  } = _os$1.constants;
  const supported = constantSignal !== undefined;
  const number = supported ? constantSignal : defaultNumber;
  return {
    name,
    number,
    description,
    supported,
    action,
    forced,
    standard
  };
};

Object.defineProperty(main$1, "__esModule", {
  value: true
});
main$1.signalsByNumber = main$1.signalsByName = void 0;
var _os = require$$0$1;
var _signals = signals$2;
var _realtime = realtime;
const getSignalsByName$1 = function () {
  const signals = (0, _signals.getSignals)();
  return signals.reduce(getSignalByName$1, {});
};
const getSignalByName$1 = function (signalByNameMemo, {
  name,
  number,
  description,
  supported,
  action,
  forced,
  standard
}) {
  return {
    ...signalByNameMemo,
    [name]: {
      name,
      number,
      description,
      supported,
      action,
      forced,
      standard
    }
  };
};
const signalsByName$2 = getSignalsByName$1();
main$1.signalsByName = signalsByName$2;
const getSignalsByNumber$1 = function () {
  const signals = (0, _signals.getSignals)();
  const length = _realtime.SIGRTMAX + 1;
  const signalsA = Array.from({
    length
  }, (value, number) => getSignalByNumber$1(number, signals));
  return Object.assign({}, ...signalsA);
};
const getSignalByNumber$1 = function (number, signals) {
  const signal = findSignalByNumber$1(number, signals);
  if (signal === undefined) {
    return {};
  }
  const {
    name,
    description,
    supported,
    action,
    forced,
    standard
  } = signal;
  return {
    [number]: {
      name,
      number,
      description,
      supported,
      action,
      forced,
      standard
    }
  };
};
const findSignalByNumber$1 = function (number, signals) {
  const signal = signals.find(({
    name
  }) => _os.constants.signals[name] === number);
  if (signal !== undefined) {
    return signal;
  }
  return signals.find(signalA => signalA.number === number);
};
const signalsByNumber = getSignalsByNumber$1();
main$1.signalsByNumber = signalsByNumber;

const {
  signalsByName: signalsByName$1
} = main$1;
const getErrorPrefix$1 = ({
  timedOut,
  timeout,
  errorCode,
  signal,
  signalDescription,
  exitCode,
  isCanceled
}) => {
  if (timedOut) {
    return `timed out after ${timeout} milliseconds`;
  }
  if (isCanceled) {
    return 'was canceled';
  }
  if (errorCode !== undefined) {
    return `failed with ${errorCode}`;
  }
  if (signal !== undefined) {
    return `was killed with ${signal} (${signalDescription})`;
  }
  if (exitCode !== undefined) {
    return `failed with exit code ${exitCode}`;
  }
  return 'failed';
};
const makeError$2 = ({
  stdout,
  stderr,
  all,
  error,
  signal,
  exitCode,
  command,
  escapedCommand,
  timedOut,
  isCanceled,
  killed,
  parsed: {
    options: {
      timeout
    }
  }
}) => {
  // `signal` and `exitCode` emitted on `spawned.on('exit')` event can be `null`.
  // We normalize them to `undefined`
  exitCode = exitCode === null ? undefined : exitCode;
  signal = signal === null ? undefined : signal;
  const signalDescription = signal === undefined ? undefined : signalsByName$1[signal].description;
  const errorCode = error && error.code;
  const prefix = getErrorPrefix$1({
    timedOut,
    timeout,
    errorCode,
    signal,
    signalDescription,
    exitCode,
    isCanceled
  });
  const execaMessage = `Command ${prefix}: ${command}`;
  const isError = Object.prototype.toString.call(error) === '[object Error]';
  const shortMessage = isError ? `${execaMessage}\n${error.message}` : execaMessage;
  const message = [shortMessage, stderr, stdout].filter(Boolean).join('\n');
  if (isError) {
    error.originalMessage = error.message;
    error.message = message;
  } else {
    error = new Error(message);
  }
  error.shortMessage = shortMessage;
  error.command = command;
  error.escapedCommand = escapedCommand;
  error.exitCode = exitCode;
  error.signal = signal;
  error.signalDescription = signalDescription;
  error.stdout = stdout;
  error.stderr = stderr;
  if (all !== undefined) {
    error.all = all;
  }
  if ('bufferedData' in error) {
    delete error.bufferedData;
  }
  error.failed = true;
  error.timedOut = Boolean(timedOut);
  error.isCanceled = isCanceled;
  error.killed = killed && !timedOut;
  return error;
};
var error$1 = makeError$2;

var stdio = {exports: {}};

const aliases$1 = ['stdin', 'stdout', 'stderr'];
const hasAlias$1 = options => aliases$1.some(alias => options[alias] !== undefined);
const normalizeStdio$2 = options => {
  if (!options) {
    return;
  }
  const {
    stdio
  } = options;
  if (stdio === undefined) {
    return aliases$1.map(alias => options[alias]);
  }
  if (hasAlias$1(options)) {
    throw new Error(`It's not possible to provide \`stdio\` in combination with one of ${aliases$1.map(alias => `\`${alias}\``).join(', ')}`);
  }
  if (typeof stdio === 'string') {
    return stdio;
  }
  if (!Array.isArray(stdio)) {
    throw new TypeError(`Expected \`stdio\` to be of type \`string\` or \`Array\`, got \`${typeof stdio}\``);
  }
  const length = Math.max(stdio.length, aliases$1.length);
  return Array.from({
    length
  }, (value, index) => stdio[index]);
};
stdio.exports = normalizeStdio$2;

// `ipc` is pushed unless it is already present
stdio.exports.node = options => {
  const stdio = normalizeStdio$2(options);
  if (stdio === 'ipc') {
    return 'ipc';
  }
  if (stdio === undefined || typeof stdio === 'string') {
    return [stdio, stdio, stdio, 'ipc'];
  }
  if (stdio.includes('ipc')) {
    return stdio;
  }
  return [...stdio, 'ipc'];
};

var stdioExports = stdio.exports;

var signalExit = {exports: {}};

var signals$1 = {exports: {}};

var hasRequiredSignals;

function requireSignals () {
	if (hasRequiredSignals) return signals$1.exports;
	hasRequiredSignals = 1;
	(function (module) {
		// This is not the set of all possible signals.
		//
		// It IS, however, the set of all signals that trigger
		// an exit on either Linux or BSD systems.  Linux is a
		// superset of the signal names supported on BSD, and
		// the unknown signals just fail to register, so we can
		// catch that easily enough.
		//
		// Don't bother with SIGKILL.  It's uncatchable, which
		// means that we can't fire any callbacks anyway.
		//
		// If a user does happen to register a handler on a non-
		// fatal signal like SIGWINCH or something, and then
		// exit, it'll end up firing `process.emit('exit')`, so
		// the handler will be fired anyway.
		//
		// SIGBUS, SIGFPE, SIGSEGV and SIGILL, when not raised
		// artificially, inherently leave the process in a
		// state from which it is not safe to try and enter JS
		// listeners.
		module.exports = ['SIGABRT', 'SIGALRM', 'SIGHUP', 'SIGINT', 'SIGTERM'];
		if (process.platform !== 'win32') {
		  module.exports.push('SIGVTALRM', 'SIGXCPU', 'SIGXFSZ', 'SIGUSR2', 'SIGTRAP', 'SIGSYS', 'SIGQUIT', 'SIGIOT'
		  // should detect profiler and enable/disable accordingly.
		  // see #21
		  // 'SIGPROF'
		  );
		}

		if (process.platform === 'linux') {
		  module.exports.push('SIGIO', 'SIGPOLL', 'SIGPWR', 'SIGSTKFLT', 'SIGUNUSED');
		} 
	} (signals$1));
	return signals$1.exports;
}

// Note: since nyc uses this module to output coverage, any lines
// that are in the direct sync flow of nyc's outputCoverage are
// ignored, since we can never get coverage for them.
// grab a reference to node's real process object right away
var process$2 = commonjsGlobal.process;
const processOk = function (process) {
  return process && typeof process === 'object' && typeof process.removeListener === 'function' && typeof process.emit === 'function' && typeof process.reallyExit === 'function' && typeof process.listeners === 'function' && typeof process.kill === 'function' && typeof process.pid === 'number' && typeof process.on === 'function';
};

// some kind of non-node environment, just no-op
/* istanbul ignore if */
if (!processOk(process$2)) {
  signalExit.exports = function () {
    return function () {};
  };
} else {
  var assert = require$$0$2;
  var signals = requireSignals();
  var isWin = /^win/i.test(process$2.platform);
  var EE = require$$2$1;
  /* istanbul ignore if */
  if (typeof EE !== 'function') {
    EE = EE.EventEmitter;
  }
  var emitter;
  if (process$2.__signal_exit_emitter__) {
    emitter = process$2.__signal_exit_emitter__;
  } else {
    emitter = process$2.__signal_exit_emitter__ = new EE();
    emitter.count = 0;
    emitter.emitted = {};
  }

  // Because this emitter is a global, we have to check to see if a
  // previous version of this library failed to enable infinite listeners.
  // I know what you're about to say.  But literally everything about
  // signal-exit is a compromise with evil.  Get used to it.
  if (!emitter.infinite) {
    emitter.setMaxListeners(Infinity);
    emitter.infinite = true;
  }
  signalExit.exports = function (cb, opts) {
    /* istanbul ignore if */
    if (!processOk(commonjsGlobal.process)) {
      return function () {};
    }
    assert.equal(typeof cb, 'function', 'a callback must be provided for exit handler');
    if (loaded === false) {
      load();
    }
    var ev = 'exit';
    if (opts && opts.alwaysLast) {
      ev = 'afterexit';
    }
    var remove = function () {
      emitter.removeListener(ev, cb);
      if (emitter.listeners('exit').length === 0 && emitter.listeners('afterexit').length === 0) {
        unload();
      }
    };
    emitter.on(ev, cb);
    return remove;
  };
  var unload = function unload() {
    if (!loaded || !processOk(commonjsGlobal.process)) {
      return;
    }
    loaded = false;
    signals.forEach(function (sig) {
      try {
        process$2.removeListener(sig, sigListeners[sig]);
      } catch (er) {}
    });
    process$2.emit = originalProcessEmit;
    process$2.reallyExit = originalProcessReallyExit;
    emitter.count -= 1;
  };
  signalExit.exports.unload = unload;
  var emit = function emit(event, code, signal) {
    /* istanbul ignore if */
    if (emitter.emitted[event]) {
      return;
    }
    emitter.emitted[event] = true;
    emitter.emit(event, code, signal);
  };

  // { <signal>: <listener fn>, ... }
  var sigListeners = {};
  signals.forEach(function (sig) {
    sigListeners[sig] = function listener() {
      /* istanbul ignore if */
      if (!processOk(commonjsGlobal.process)) {
        return;
      }
      // If there are no other listeners, an exit is coming!
      // Simplest way: remove us and then re-send the signal.
      // We know that this will kill the process, so we can
      // safely emit now.
      var listeners = process$2.listeners(sig);
      if (listeners.length === emitter.count) {
        unload();
        emit('exit', null, sig);
        /* istanbul ignore next */
        emit('afterexit', null, sig);
        /* istanbul ignore next */
        if (isWin && sig === 'SIGHUP') {
          // "SIGHUP" throws an `ENOSYS` error on Windows,
          // so use a supported signal instead
          sig = 'SIGINT';
        }
        /* istanbul ignore next */
        process$2.kill(process$2.pid, sig);
      }
    };
  });
  signalExit.exports.signals = function () {
    return signals;
  };
  var loaded = false;
  var load = function load() {
    if (loaded || !processOk(commonjsGlobal.process)) {
      return;
    }
    loaded = true;

    // This is the number of onSignalExit's that are in play.
    // It's important so that we can count the correct number of
    // listeners on signals, and don't wait for the other one to
    // handle it instead of us.
    emitter.count += 1;
    signals = signals.filter(function (sig) {
      try {
        process$2.on(sig, sigListeners[sig]);
        return true;
      } catch (er) {
        return false;
      }
    });
    process$2.emit = processEmit;
    process$2.reallyExit = processReallyExit;
  };
  signalExit.exports.load = load;
  var originalProcessReallyExit = process$2.reallyExit;
  var processReallyExit = function processReallyExit(code) {
    /* istanbul ignore if */
    if (!processOk(commonjsGlobal.process)) {
      return;
    }
    process$2.exitCode = code || /* istanbul ignore next */0;
    emit('exit', process$2.exitCode, null);
    /* istanbul ignore next */
    emit('afterexit', process$2.exitCode, null);
    /* istanbul ignore next */
    originalProcessReallyExit.call(process$2, process$2.exitCode);
  };
  var originalProcessEmit = process$2.emit;
  var processEmit = function processEmit(ev, arg) {
    if (ev === 'exit' && processOk(commonjsGlobal.process)) {
      /* istanbul ignore else */
      if (arg !== undefined) {
        process$2.exitCode = arg;
      }
      var ret = originalProcessEmit.apply(this, arguments);
      /* istanbul ignore next */
      emit('exit', process$2.exitCode, null);
      /* istanbul ignore next */
      emit('afterexit', process$2.exitCode, null);
      /* istanbul ignore next */
      return ret;
    } else {
      return originalProcessEmit.apply(this, arguments);
    }
  };
}

var signalExitExports = signalExit.exports;
var onExit$1 = /*@__PURE__*/getDefaultExportFromCjs(signalExitExports);

const os = require$$0$1;
const onExit = signalExitExports;
const DEFAULT_FORCE_KILL_TIMEOUT$1 = 1000 * 5;

// Monkey-patches `childProcess.kill()` to add `forceKillAfterTimeout` behavior
const spawnedKill$2 = (kill, signal = 'SIGTERM', options = {}) => {
  const killResult = kill(signal);
  setKillTimeout$1(kill, signal, options, killResult);
  return killResult;
};
const setKillTimeout$1 = (kill, signal, options, killResult) => {
  if (!shouldForceKill$1(signal, options, killResult)) {
    return;
  }
  const timeout = getForceKillAfterTimeout$1(options);
  const t = setTimeout(() => {
    kill('SIGKILL');
  }, timeout);

  // Guarded because there's no `.unref()` when `execa` is used in the renderer
  // process in Electron. This cannot be tested since we don't run tests in
  // Electron.
  // istanbul ignore else
  if (t.unref) {
    t.unref();
  }
};
const shouldForceKill$1 = (signal, {
  forceKillAfterTimeout
}, killResult) => {
  return isSigterm$1(signal) && forceKillAfterTimeout !== false && killResult;
};
const isSigterm$1 = signal => {
  return signal === os.constants.signals.SIGTERM || typeof signal === 'string' && signal.toUpperCase() === 'SIGTERM';
};
const getForceKillAfterTimeout$1 = ({
  forceKillAfterTimeout = true
}) => {
  if (forceKillAfterTimeout === true) {
    return DEFAULT_FORCE_KILL_TIMEOUT$1;
  }
  if (!Number.isFinite(forceKillAfterTimeout) || forceKillAfterTimeout < 0) {
    throw new TypeError(`Expected the \`forceKillAfterTimeout\` option to be a non-negative integer, got \`${forceKillAfterTimeout}\` (${typeof forceKillAfterTimeout})`);
  }
  return forceKillAfterTimeout;
};

// `childProcess.cancel()`
const spawnedCancel$2 = (spawned, context) => {
  const killResult = spawned.kill();
  if (killResult) {
    context.isCanceled = true;
  }
};
const timeoutKill$1 = (spawned, signal, reject) => {
  spawned.kill(signal);
  reject(Object.assign(new Error('Timed out'), {
    timedOut: true,
    signal
  }));
};

// `timeout` option handling
const setupTimeout$2 = (spawned, {
  timeout,
  killSignal = 'SIGTERM'
}, spawnedPromise) => {
  if (timeout === 0 || timeout === undefined) {
    return spawnedPromise;
  }
  let timeoutId;
  const timeoutPromise = new Promise((resolve, reject) => {
    timeoutId = setTimeout(() => {
      timeoutKill$1(spawned, killSignal, reject);
    }, timeout);
  });
  const safeSpawnedPromise = spawnedPromise.finally(() => {
    clearTimeout(timeoutId);
  });
  return Promise.race([timeoutPromise, safeSpawnedPromise]);
};
const validateTimeout$2 = ({
  timeout
}) => {
  if (timeout !== undefined && (!Number.isFinite(timeout) || timeout < 0)) {
    throw new TypeError(`Expected the \`timeout\` option to be a non-negative integer, got \`${timeout}\` (${typeof timeout})`);
  }
};

// `cleanup` option handling
const setExitHandler$2 = async (spawned, {
  cleanup,
  detached
}, timedPromise) => {
  if (!cleanup || detached) {
    return timedPromise;
  }
  const removeExitHandler = onExit(() => {
    spawned.kill();
  });
  return timedPromise.finally(() => {
    removeExitHandler();
  });
};
var kill = {
  spawnedKill: spawnedKill$2,
  spawnedCancel: spawnedCancel$2,
  setupTimeout: setupTimeout$2,
  validateTimeout: validateTimeout$2,
  setExitHandler: setExitHandler$2
};

const isStream$2 = stream => stream !== null && typeof stream === 'object' && typeof stream.pipe === 'function';
isStream$2.writable = stream => isStream$2(stream) && stream.writable !== false && typeof stream._write === 'function' && typeof stream._writableState === 'object';
isStream$2.readable = stream => isStream$2(stream) && stream.readable !== false && typeof stream._read === 'function' && typeof stream._readableState === 'object';
isStream$2.duplex = stream => isStream$2.writable(stream) && isStream$2.readable(stream);
isStream$2.transform = stream => isStream$2.duplex(stream) && typeof stream._transform === 'function';
var isStream_1 = isStream$2;

var getStream$3 = {exports: {}};

const {
  PassThrough: PassThroughStream
} = require$$0$3;
var bufferStream$1 = options => {
  options = {
    ...options
  };
  const {
    array
  } = options;
  let {
    encoding
  } = options;
  const isBuffer = encoding === 'buffer';
  let objectMode = false;
  if (array) {
    objectMode = !(encoding || isBuffer);
  } else {
    encoding = encoding || 'utf8';
  }
  if (isBuffer) {
    encoding = null;
  }
  const stream = new PassThroughStream({
    objectMode
  });
  if (encoding) {
    stream.setEncoding(encoding);
  }
  let length = 0;
  const chunks = [];
  stream.on('data', chunk => {
    chunks.push(chunk);
    if (objectMode) {
      length = chunks.length;
    } else {
      length += chunk.length;
    }
  });
  stream.getBufferedValue = () => {
    if (array) {
      return chunks;
    }
    return isBuffer ? Buffer.concat(chunks, length) : chunks.join('');
  };
  stream.getBufferedLength = () => length;
  return stream;
};

const {
  constants: BufferConstants
} = require$$0$4;
const stream$1 = require$$0$3;
const {
  promisify
} = require$$1$1;
const bufferStream = bufferStream$1;
const streamPipelinePromisified = promisify(stream$1.pipeline);
class MaxBufferError extends Error {
  constructor() {
    super('maxBuffer exceeded');
    this.name = 'MaxBufferError';
  }
}
async function getStream$1(inputStream, options) {
  if (!inputStream) {
    throw new Error('Expected a stream');
  }
  options = {
    maxBuffer: Infinity,
    ...options
  };
  const {
    maxBuffer
  } = options;
  const stream = bufferStream(options);
  await new Promise((resolve, reject) => {
    const rejectPromise = error => {
      // Don't retrieve an oversized buffer.
      if (error && stream.getBufferedLength() <= BufferConstants.MAX_LENGTH) {
        error.bufferedData = stream.getBufferedValue();
      }
      reject(error);
    };
    (async () => {
      try {
        await streamPipelinePromisified(inputStream, stream);
        resolve();
      } catch (error) {
        rejectPromise(error);
      }
    })();
    stream.on('data', () => {
      if (stream.getBufferedLength() > maxBuffer) {
        rejectPromise(new MaxBufferError());
      }
    });
  });
  return stream.getBufferedValue();
}
getStream$3.exports = getStream$1;
getStream$3.exports.buffer = (stream, options) => getStream$1(stream, {
  ...options,
  encoding: 'buffer'
});
getStream$3.exports.array = (stream, options) => getStream$1(stream, {
  ...options,
  array: true
});
getStream$3.exports.MaxBufferError = MaxBufferError;

var getStreamExports = getStream$3.exports;
var getStream$2 = /*@__PURE__*/getDefaultExportFromCjs(getStreamExports);

const {
  PassThrough
} = require$$0$3;
var mergeStream$1 = function /*streams...*/
() {
  var sources = [];
  var output = new PassThrough({
    objectMode: true
  });
  output.setMaxListeners(0);
  output.add = add;
  output.isEmpty = isEmpty;
  output.on('unpipe', remove);
  Array.prototype.slice.call(arguments).forEach(add);
  return output;
  function add(source) {
    if (Array.isArray(source)) {
      source.forEach(add);
      return this;
    }
    sources.push(source);
    source.once('end', remove.bind(null, source));
    source.once('error', output.emit.bind(output, 'error'));
    source.pipe(output, {
      end: false
    });
    return this;
  }
  function isEmpty() {
    return sources.length == 0;
  }
  function remove(source) {
    sources = sources.filter(function (it) {
      return it !== source;
    });
    if (!sources.length && output.readable) {
      output.end();
    }
  }
};

var mergeStream$2 = /*@__PURE__*/getDefaultExportFromCjs(mergeStream$1);

const isStream$1 = isStream_1;
const getStream = getStreamExports;
const mergeStream = mergeStream$1;

// `input` option
const handleInput$2 = (spawned, input) => {
  // Checking for stdin is workaround for https://github.com/nodejs/node/issues/26852
  // @todo remove `|| spawned.stdin === undefined` once we drop support for Node.js <=12.2.0
  if (input === undefined || spawned.stdin === undefined) {
    return;
  }
  if (isStream$1(input)) {
    input.pipe(spawned.stdin);
  } else {
    spawned.stdin.end(input);
  }
};

// `all` interleaves `stdout` and `stderr`
const makeAllStream$2 = (spawned, {
  all
}) => {
  if (!all || !spawned.stdout && !spawned.stderr) {
    return;
  }
  const mixed = mergeStream();
  if (spawned.stdout) {
    mixed.add(spawned.stdout);
  }
  if (spawned.stderr) {
    mixed.add(spawned.stderr);
  }
  return mixed;
};

// On failure, `result.stdout|stderr|all` should contain the currently buffered stream
const getBufferedData$1 = async (stream, streamPromise) => {
  if (!stream) {
    return;
  }
  stream.destroy();
  try {
    return await streamPromise;
  } catch (error) {
    return error.bufferedData;
  }
};
const getStreamPromise$1 = (stream, {
  encoding,
  buffer,
  maxBuffer
}) => {
  if (!stream || !buffer) {
    return;
  }
  if (encoding) {
    return getStream(stream, {
      encoding,
      maxBuffer
    });
  }
  return getStream.buffer(stream, {
    maxBuffer
  });
};

// Retrieve result of child process: exit code, signal, error, streams (stdout/stderr/all)
const getSpawnedResult$2 = async ({
  stdout,
  stderr,
  all
}, {
  encoding,
  buffer,
  maxBuffer
}, processDone) => {
  const stdoutPromise = getStreamPromise$1(stdout, {
    encoding,
    buffer,
    maxBuffer
  });
  const stderrPromise = getStreamPromise$1(stderr, {
    encoding,
    buffer,
    maxBuffer
  });
  const allPromise = getStreamPromise$1(all, {
    encoding,
    buffer,
    maxBuffer: maxBuffer * 2
  });
  try {
    return await Promise.all([processDone, stdoutPromise, stderrPromise, allPromise]);
  } catch (error) {
    return Promise.all([{
      error,
      signal: error.signal,
      timedOut: error.timedOut
    }, getBufferedData$1(stdout, stdoutPromise), getBufferedData$1(stderr, stderrPromise), getBufferedData$1(all, allPromise)]);
  }
};
const validateInputSync$1 = ({
  input
}) => {
  if (isStream$1(input)) {
    throw new TypeError('The `input` option cannot be a stream in sync mode');
  }
};
var stream = {
  handleInput: handleInput$2,
  makeAllStream: makeAllStream$2,
  getSpawnedResult: getSpawnedResult$2,
  validateInputSync: validateInputSync$1
};

const nativePromisePrototype$1 = (async () => {})().constructor.prototype;
const descriptors$1 = ['then', 'catch', 'finally'].map(property => [property, Reflect.getOwnPropertyDescriptor(nativePromisePrototype$1, property)]);

// The return value is a mixin of `childProcess` and `Promise`
const mergePromise$2 = (spawned, promise) => {
  for (const [property, descriptor] of descriptors$1) {
    // Starting the main `promise` is deferred to avoid consuming streams
    const value = typeof promise === 'function' ? (...args) => Reflect.apply(descriptor.value, promise(), args) : descriptor.value.bind(promise);
    Reflect.defineProperty(spawned, property, {
      ...descriptor,
      value
    });
  }
  return spawned;
};

// Use promises instead of `child_process` events
const getSpawnedPromise$2 = spawned => {
  return new Promise((resolve, reject) => {
    spawned.on('exit', (exitCode, signal) => {
      resolve({
        exitCode,
        signal
      });
    });
    spawned.on('error', error => {
      reject(error);
    });
    if (spawned.stdin) {
      spawned.stdin.on('error', error => {
        reject(error);
      });
    }
  });
};
var promise = {
  mergePromise: mergePromise$2,
  getSpawnedPromise: getSpawnedPromise$2
};

const normalizeArgs$1 = (file, args = []) => {
  if (!Array.isArray(args)) {
    return [file];
  }
  return [file, ...args];
};
const NO_ESCAPE_REGEXP$1 = /^[\w.-]+$/;
const DOUBLE_QUOTES_REGEXP$1 = /"/g;
const escapeArg$1 = arg => {
  if (typeof arg !== 'string' || NO_ESCAPE_REGEXP$1.test(arg)) {
    return arg;
  }
  return `"${arg.replace(DOUBLE_QUOTES_REGEXP$1, '\\"')}"`;
};
const joinCommand$2 = (file, args) => {
  return normalizeArgs$1(file, args).join(' ');
};
const getEscapedCommand$2 = (file, args) => {
  return normalizeArgs$1(file, args).map(arg => escapeArg$1(arg)).join(' ');
};
const SPACES_REGEXP = / +/g;

// Handle `execa.command()`
const parseCommand$1 = command => {
  const tokens = [];
  for (const token of command.trim().split(SPACES_REGEXP)) {
    // Allow spaces to be escaped by a backslash if not meant as a delimiter
    const previousToken = tokens[tokens.length - 1];
    if (previousToken && previousToken.endsWith('\\')) {
      // Merge previous token with current one
      tokens[tokens.length - 1] = `${previousToken.slice(0, -1)} ${token}`;
    } else {
      tokens.push(token);
    }
  }
  return tokens;
};
var command$1 = {
  joinCommand: joinCommand$2,
  getEscapedCommand: getEscapedCommand$2,
  parseCommand: parseCommand$1
};

const path$1 = path$6;
const childProcess$1 = require$$1$2;
const crossSpawn = crossSpawnExports;
const stripFinalNewline$1 = stripFinalNewline$2;
const npmRunPath$1 = npmRunPathExports;
const onetime$1 = onetimeExports;
const makeError$1 = error$1;
const normalizeStdio$1 = stdioExports;
const {
  spawnedKill: spawnedKill$1,
  spawnedCancel: spawnedCancel$1,
  setupTimeout: setupTimeout$1,
  validateTimeout: validateTimeout$1,
  setExitHandler: setExitHandler$1
} = kill;
const {
  handleInput: handleInput$1,
  getSpawnedResult: getSpawnedResult$1,
  makeAllStream: makeAllStream$1,
  validateInputSync
} = stream;
const {
  mergePromise: mergePromise$1,
  getSpawnedPromise: getSpawnedPromise$1
} = promise;
const {
  joinCommand: joinCommand$1,
  parseCommand,
  getEscapedCommand: getEscapedCommand$1
} = command$1;
const DEFAULT_MAX_BUFFER$1 = 1000 * 1000 * 100;
const getEnv$1 = ({
  env: envOption,
  extendEnv,
  preferLocal,
  localDir,
  execPath
}) => {
  const env = extendEnv ? {
    ...process.env,
    ...envOption
  } : envOption;
  if (preferLocal) {
    return npmRunPath$1.env({
      env,
      cwd: localDir,
      execPath
    });
  }
  return env;
};
const handleArguments$1 = (file, args, options = {}) => {
  const parsed = crossSpawn._parse(file, args, options);
  file = parsed.command;
  args = parsed.args;
  options = parsed.options;
  options = {
    maxBuffer: DEFAULT_MAX_BUFFER$1,
    buffer: true,
    stripFinalNewline: true,
    extendEnv: true,
    preferLocal: false,
    localDir: options.cwd || process.cwd(),
    execPath: process.execPath,
    encoding: 'utf8',
    reject: true,
    cleanup: true,
    all: false,
    windowsHide: true,
    ...options
  };
  options.env = getEnv$1(options);
  options.stdio = normalizeStdio$1(options);
  if (process.platform === 'win32' && path$1.basename(file, '.exe') === 'cmd') {
    // #116
    args.unshift('/q');
  }
  return {
    file,
    args,
    options,
    parsed
  };
};
const handleOutput$1 = (options, value, error) => {
  if (typeof value !== 'string' && !Buffer.isBuffer(value)) {
    // When `execa.sync()` errors, we normalize it to '' to mimic `execa()`
    return error === undefined ? undefined : '';
  }
  if (options.stripFinalNewline) {
    return stripFinalNewline$1(value);
  }
  return value;
};
const execa$1 = (file, args, options) => {
  const parsed = handleArguments$1(file, args, options);
  const command = joinCommand$1(file, args);
  const escapedCommand = getEscapedCommand$1(file, args);
  validateTimeout$1(parsed.options);
  let spawned;
  try {
    spawned = childProcess$1.spawn(parsed.file, parsed.args, parsed.options);
  } catch (error) {
    // Ensure the returned error is always both a promise and a child process
    const dummySpawned = new childProcess$1.ChildProcess();
    const errorPromise = Promise.reject(makeError$1({
      error,
      stdout: '',
      stderr: '',
      all: '',
      command,
      escapedCommand,
      parsed,
      timedOut: false,
      isCanceled: false,
      killed: false
    }));
    return mergePromise$1(dummySpawned, errorPromise);
  }
  const spawnedPromise = getSpawnedPromise$1(spawned);
  const timedPromise = setupTimeout$1(spawned, parsed.options, spawnedPromise);
  const processDone = setExitHandler$1(spawned, parsed.options, timedPromise);
  const context = {
    isCanceled: false
  };
  spawned.kill = spawnedKill$1.bind(null, spawned.kill.bind(spawned));
  spawned.cancel = spawnedCancel$1.bind(null, spawned, context);
  const handlePromise = async () => {
    const [{
      error,
      exitCode,
      signal,
      timedOut
    }, stdoutResult, stderrResult, allResult] = await getSpawnedResult$1(spawned, parsed.options, processDone);
    const stdout = handleOutput$1(parsed.options, stdoutResult);
    const stderr = handleOutput$1(parsed.options, stderrResult);
    const all = handleOutput$1(parsed.options, allResult);
    if (error || exitCode !== 0 || signal !== null) {
      const returnedError = makeError$1({
        error,
        exitCode,
        signal,
        stdout,
        stderr,
        all,
        command,
        escapedCommand,
        parsed,
        timedOut,
        isCanceled: context.isCanceled,
        killed: spawned.killed
      });
      if (!parsed.options.reject) {
        return returnedError;
      }
      throw returnedError;
    }
    return {
      command,
      escapedCommand,
      exitCode: 0,
      stdout,
      stderr,
      all,
      failed: false,
      timedOut: false,
      isCanceled: false,
      killed: false
    };
  };
  const handlePromiseOnce = onetime$1(handlePromise);
  handleInput$1(spawned, parsed.options.input);
  spawned.all = makeAllStream$1(spawned, parsed.options);
  return mergePromise$1(spawned, handlePromiseOnce);
};
execa$3.exports = execa$1;
execa$3.exports.sync = (file, args, options) => {
  const parsed = handleArguments$1(file, args, options);
  const command = joinCommand$1(file, args);
  const escapedCommand = getEscapedCommand$1(file, args);
  validateInputSync(parsed.options);
  let result;
  try {
    result = childProcess$1.spawnSync(parsed.file, parsed.args, parsed.options);
  } catch (error) {
    throw makeError$1({
      error,
      stdout: '',
      stderr: '',
      all: '',
      command,
      escapedCommand,
      parsed,
      timedOut: false,
      isCanceled: false,
      killed: false
    });
  }
  const stdout = handleOutput$1(parsed.options, result.stdout, result.error);
  const stderr = handleOutput$1(parsed.options, result.stderr, result.error);
  if (result.error || result.status !== 0 || result.signal !== null) {
    const error = makeError$1({
      stdout,
      stderr,
      error: result.error,
      signal: result.signal,
      exitCode: result.status,
      command,
      escapedCommand,
      parsed,
      timedOut: result.error && result.error.code === 'ETIMEDOUT',
      isCanceled: false,
      killed: result.signal !== null
    });
    if (!parsed.options.reject) {
      return error;
    }
    throw error;
  }
  return {
    command,
    escapedCommand,
    exitCode: 0,
    stdout,
    stderr,
    failed: false,
    timedOut: false,
    isCanceled: false,
    killed: false
  };
};
execa$3.exports.command = (command, options) => {
  const [file, ...args] = parseCommand(command);
  return execa$1(file, args, options);
};
execa$3.exports.commandSync = (command, options) => {
  const [file, ...args] = parseCommand(command);
  return execa$1.sync(file, args, options);
};
execa$3.exports.node = (scriptPath, args, options = {}) => {
  if (args && !Array.isArray(args) && typeof args === 'object') {
    options = args;
    args = [];
  }
  const stdio = normalizeStdio$1.node(options);
  const defaultExecArgv = process.execArgv.filter(arg => !arg.startsWith('--inspect'));
  const {
    nodePath = process.execPath,
    nodeOptions = defaultExecArgv
  } = options;
  return execa$1(nodePath, [...nodeOptions, scriptPath, ...(Array.isArray(args) ? args : [])], {
    ...options,
    stdin: undefined,
    stdout: undefined,
    stderr: undefined,
    stdio,
    shell: false
  });
};

var execaExports = execa$3.exports;
var execa$2 = /*@__PURE__*/getDefaultExportFromCjs(execaExports);

async function runAppleScriptAsync(script) {
  if (process.platform !== 'darwin') {
    throw new Error('macOS only');
  }
  const {
    stdout
  } = await execa$2('osascript', ['-e', script]);
  return stdout;
}

async function bundleName(bundleId) {
  return runAppleScriptAsync(`tell application "Finder" to set app_path to application file id "${bundleId}" as string\ntell application "System Events" to get value of property list item "CFBundleName" of property list file (app_path & ":Contents:Info.plist")`);
}

function titleize(string) {
  if (typeof string !== 'string') {
    throw new TypeError('Expected a string');
  }
  return string.toLowerCase().replace(/(?:^|\s|-)\S/g, x => x.toUpperCase());
}

function stripFinalNewline(input) {
  const LF = typeof input === 'string' ? '\n' : '\n'.charCodeAt();
  const CR = typeof input === 'string' ? '\r' : '\r'.charCodeAt();
  if (input[input.length - 1] === LF) {
    input = input.slice(0, -1);
  }
  if (input[input.length - 1] === CR) {
    input = input.slice(0, -1);
  }
  return input;
}

function pathKey(options = {}) {
  const {
    env = process.env,
    platform = process.platform
  } = options;
  if (platform !== 'win32') {
    return 'PATH';
  }
  return Object.keys(env).reverse().find(key => key.toUpperCase() === 'PATH') || 'Path';
}

function npmRunPath(options = {}) {
  const {
    cwd = process$4.cwd(),
    path: path_ = process$4.env[pathKey()],
    execPath = process$4.execPath
  } = options;
  let previous;
  const cwdString = cwd instanceof URL ? url.fileURLToPath(cwd) : cwd;
  let cwdPath = path$7.resolve(cwdString);
  const result = [];
  while (previous !== cwdPath) {
    result.push(path$7.join(cwdPath, 'node_modules/.bin'));
    previous = cwdPath;
    cwdPath = path$7.resolve(cwdPath, '..');
  }

  // Ensure the running `node` binary is used.
  result.push(path$7.resolve(cwdString, execPath, '..'));
  return [...result, path_].join(path$7.delimiter);
}
function npmRunPathEnv({
  env = process$4.env,
  ...options
} = {}) {
  env = {
    ...env
  };
  const path = pathKey({
    env
  });
  options.path = env[path];
  env[path] = npmRunPath(options);
  return env;
}

const copyProperty = (to, from, property, ignoreNonConfigurable) => {
  // `Function#length` should reflect the parameters of `to` not `from` since we keep its body.
  // `Function#prototype` is non-writable and non-configurable so can never be modified.
  if (property === 'length' || property === 'prototype') {
    return;
  }

  // `Function#arguments` and `Function#caller` should not be copied. They were reported to be present in `Reflect.ownKeys` for some devices in React Native (#41), so we explicitly ignore them here.
  if (property === 'arguments' || property === 'caller') {
    return;
  }
  const toDescriptor = Object.getOwnPropertyDescriptor(to, property);
  const fromDescriptor = Object.getOwnPropertyDescriptor(from, property);
  if (!canCopyProperty(toDescriptor, fromDescriptor) && ignoreNonConfigurable) {
    return;
  }
  Object.defineProperty(to, property, fromDescriptor);
};

// `Object.defineProperty()` throws if the property exists, is not configurable and either:
// - one its descriptors is changed
// - it is non-writable and its value is changed
const canCopyProperty = function (toDescriptor, fromDescriptor) {
  return toDescriptor === undefined || toDescriptor.configurable || toDescriptor.writable === fromDescriptor.writable && toDescriptor.enumerable === fromDescriptor.enumerable && toDescriptor.configurable === fromDescriptor.configurable && (toDescriptor.writable || toDescriptor.value === fromDescriptor.value);
};
const changePrototype = (to, from) => {
  const fromPrototype = Object.getPrototypeOf(from);
  if (fromPrototype === Object.getPrototypeOf(to)) {
    return;
  }
  Object.setPrototypeOf(to, fromPrototype);
};
const wrappedToString = (withName, fromBody) => `/* Wrapped ${withName}*/\n${fromBody}`;
const toStringDescriptor = Object.getOwnPropertyDescriptor(Function.prototype, 'toString');
const toStringName = Object.getOwnPropertyDescriptor(Function.prototype.toString, 'name');

// We call `from.toString()` early (not lazily) to ensure `from` can be garbage collected.
// We use `bind()` instead of a closure for the same reason.
// Calling `from.toString()` early also allows caching it in case `to.toString()` is called several times.
const changeToString = (to, from, name) => {
  const withName = name === '' ? '' : `with ${name.trim()}() `;
  const newToString = wrappedToString.bind(null, withName, from.toString());
  // Ensure `to.toString.toString` is non-enumerable and has the same `same`
  Object.defineProperty(newToString, 'name', toStringName);
  Object.defineProperty(to, 'toString', {
    ...toStringDescriptor,
    value: newToString
  });
};
function mimicFunction(to, from, {
  ignoreNonConfigurable = false
} = {}) {
  const {
    name
  } = to;
  for (const property of Reflect.ownKeys(from)) {
    copyProperty(to, from, property, ignoreNonConfigurable);
  }
  changePrototype(to, from);
  changeToString(to, from, name);
  return to;
}

const calledFunctions = new WeakMap();
const onetime = (function_, options = {}) => {
  if (typeof function_ !== 'function') {
    throw new TypeError('Expected a function');
  }
  let returnValue;
  let callCount = 0;
  const functionName = function_.displayName || function_.name || '<anonymous>';
  const onetime = function (...arguments_) {
    calledFunctions.set(onetime, ++callCount);
    if (callCount === 1) {
      returnValue = function_.apply(this, arguments_);
      function_ = null;
    } else if (options.throw === true) {
      throw new Error(`Function \`${functionName}\` can only be called once`);
    }
    return returnValue;
  };
  mimicFunction(onetime, function_);
  calledFunctions.set(onetime, callCount);
  return onetime;
};
onetime.callCount = function_ => {
  if (!calledFunctions.has(function_)) {
    throw new Error(`The given function \`${function_.name}\` is not wrapped by the \`onetime\` package`);
  }
  return calledFunctions.get(function_);
};

const getRealtimeSignals = () => {
  const length = SIGRTMAX - SIGRTMIN + 1;
  return Array.from({
    length
  }, getRealtimeSignal);
};
const getRealtimeSignal = (value, index) => ({
  name: `SIGRT${index + 1}`,
  number: SIGRTMIN + index,
  action: "terminate",
  description: "Application-specific signal (realtime)",
  standard: "posix"
});
const SIGRTMIN = 34;
const SIGRTMAX = 64;

const SIGNALS = [{
  name: "SIGHUP",
  number: 1,
  action: "terminate",
  description: "Terminal closed",
  standard: "posix"
}, {
  name: "SIGINT",
  number: 2,
  action: "terminate",
  description: "User interruption with CTRL-C",
  standard: "ansi"
}, {
  name: "SIGQUIT",
  number: 3,
  action: "core",
  description: "User interruption with CTRL-\\",
  standard: "posix"
}, {
  name: "SIGILL",
  number: 4,
  action: "core",
  description: "Invalid machine instruction",
  standard: "ansi"
}, {
  name: "SIGTRAP",
  number: 5,
  action: "core",
  description: "Debugger breakpoint",
  standard: "posix"
}, {
  name: "SIGABRT",
  number: 6,
  action: "core",
  description: "Aborted",
  standard: "ansi"
}, {
  name: "SIGIOT",
  number: 6,
  action: "core",
  description: "Aborted",
  standard: "bsd"
}, {
  name: "SIGBUS",
  number: 7,
  action: "core",
  description: "Bus error due to misaligned, non-existing address or paging error",
  standard: "bsd"
}, {
  name: "SIGEMT",
  number: 7,
  action: "terminate",
  description: "Command should be emulated but is not implemented",
  standard: "other"
}, {
  name: "SIGFPE",
  number: 8,
  action: "core",
  description: "Floating point arithmetic error",
  standard: "ansi"
}, {
  name: "SIGKILL",
  number: 9,
  action: "terminate",
  description: "Forced termination",
  standard: "posix",
  forced: true
}, {
  name: "SIGUSR1",
  number: 10,
  action: "terminate",
  description: "Application-specific signal",
  standard: "posix"
}, {
  name: "SIGSEGV",
  number: 11,
  action: "core",
  description: "Segmentation fault",
  standard: "ansi"
}, {
  name: "SIGUSR2",
  number: 12,
  action: "terminate",
  description: "Application-specific signal",
  standard: "posix"
}, {
  name: "SIGPIPE",
  number: 13,
  action: "terminate",
  description: "Broken pipe or socket",
  standard: "posix"
}, {
  name: "SIGALRM",
  number: 14,
  action: "terminate",
  description: "Timeout or timer",
  standard: "posix"
}, {
  name: "SIGTERM",
  number: 15,
  action: "terminate",
  description: "Termination",
  standard: "ansi"
}, {
  name: "SIGSTKFLT",
  number: 16,
  action: "terminate",
  description: "Stack is empty or overflowed",
  standard: "other"
}, {
  name: "SIGCHLD",
  number: 17,
  action: "ignore",
  description: "Child process terminated, paused or unpaused",
  standard: "posix"
}, {
  name: "SIGCLD",
  number: 17,
  action: "ignore",
  description: "Child process terminated, paused or unpaused",
  standard: "other"
}, {
  name: "SIGCONT",
  number: 18,
  action: "unpause",
  description: "Unpaused",
  standard: "posix",
  forced: true
}, {
  name: "SIGSTOP",
  number: 19,
  action: "pause",
  description: "Paused",
  standard: "posix",
  forced: true
}, {
  name: "SIGTSTP",
  number: 20,
  action: "pause",
  description: "Paused using CTRL-Z or \"suspend\"",
  standard: "posix"
}, {
  name: "SIGTTIN",
  number: 21,
  action: "pause",
  description: "Background process cannot read terminal input",
  standard: "posix"
}, {
  name: "SIGBREAK",
  number: 21,
  action: "terminate",
  description: "User interruption with CTRL-BREAK",
  standard: "other"
}, {
  name: "SIGTTOU",
  number: 22,
  action: "pause",
  description: "Background process cannot write to terminal output",
  standard: "posix"
}, {
  name: "SIGURG",
  number: 23,
  action: "ignore",
  description: "Socket received out-of-band data",
  standard: "bsd"
}, {
  name: "SIGXCPU",
  number: 24,
  action: "core",
  description: "Process timed out",
  standard: "bsd"
}, {
  name: "SIGXFSZ",
  number: 25,
  action: "core",
  description: "File too big",
  standard: "bsd"
}, {
  name: "SIGVTALRM",
  number: 26,
  action: "terminate",
  description: "Timeout or timer",
  standard: "bsd"
}, {
  name: "SIGPROF",
  number: 27,
  action: "terminate",
  description: "Timeout or timer",
  standard: "bsd"
}, {
  name: "SIGWINCH",
  number: 28,
  action: "ignore",
  description: "Terminal window size changed",
  standard: "bsd"
}, {
  name: "SIGIO",
  number: 29,
  action: "terminate",
  description: "I/O is available",
  standard: "other"
}, {
  name: "SIGPOLL",
  number: 29,
  action: "terminate",
  description: "Watched event",
  standard: "other"
}, {
  name: "SIGINFO",
  number: 29,
  action: "ignore",
  description: "Request for process information",
  standard: "other"
}, {
  name: "SIGPWR",
  number: 30,
  action: "terminate",
  description: "Device running out of power",
  standard: "systemv"
}, {
  name: "SIGSYS",
  number: 31,
  action: "core",
  description: "Invalid system call",
  standard: "other"
}, {
  name: "SIGUNUSED",
  number: 31,
  action: "terminate",
  description: "Invalid system call",
  standard: "other"
}];

const getSignals = () => {
  const realtimeSignals = getRealtimeSignals();
  const signals = [...SIGNALS, ...realtimeSignals].map(normalizeSignal);
  return signals;
};
const normalizeSignal = ({
  name,
  number: defaultNumber,
  description,
  action,
  forced = false,
  standard
}) => {
  const {
    signals: {
      [name]: constantSignal
    }
  } = constants;
  const supported = constantSignal !== undefined;
  const number = supported ? constantSignal : defaultNumber;
  return {
    name,
    number,
    description,
    supported,
    action,
    forced,
    standard
  };
};

const getSignalsByName = () => {
  const signals = getSignals();
  return Object.fromEntries(signals.map(getSignalByName));
};
const getSignalByName = ({
  name,
  number,
  description,
  supported,
  action,
  forced,
  standard
}) => [name, {
  name,
  number,
  description,
  supported,
  action,
  forced,
  standard
}];
const signalsByName = getSignalsByName();
const getSignalsByNumber = () => {
  const signals = getSignals();
  const length = SIGRTMAX + 1;
  const signalsA = Array.from({
    length
  }, (value, number) => getSignalByNumber(number, signals));
  return Object.assign({}, ...signalsA);
};
const getSignalByNumber = (number, signals) => {
  const signal = findSignalByNumber(number, signals);
  if (signal === undefined) {
    return {};
  }
  const {
    name,
    description,
    supported,
    action,
    forced,
    standard
  } = signal;
  return {
    [number]: {
      name,
      number,
      description,
      supported,
      action,
      forced,
      standard
    }
  };
};
const findSignalByNumber = (number, signals) => {
  const signal = signals.find(({
    name
  }) => constants.signals[name] === number);
  if (signal !== undefined) {
    return signal;
  }
  return signals.find(signalA => signalA.number === number);
};
getSignalsByNumber();

const getErrorPrefix = ({
  timedOut,
  timeout,
  errorCode,
  signal,
  signalDescription,
  exitCode,
  isCanceled
}) => {
  if (timedOut) {
    return `timed out after ${timeout} milliseconds`;
  }
  if (isCanceled) {
    return 'was canceled';
  }
  if (errorCode !== undefined) {
    return `failed with ${errorCode}`;
  }
  if (signal !== undefined) {
    return `was killed with ${signal} (${signalDescription})`;
  }
  if (exitCode !== undefined) {
    return `failed with exit code ${exitCode}`;
  }
  return 'failed';
};
const makeError = ({
  stdout,
  stderr,
  all,
  error,
  signal,
  exitCode,
  command,
  escapedCommand,
  timedOut,
  isCanceled,
  killed,
  parsed: {
    options: {
      timeout,
      cwd = process$4.cwd()
    }
  }
}) => {
  // `signal` and `exitCode` emitted on `spawned.on('exit')` event can be `null`.
  // We normalize them to `undefined`
  exitCode = exitCode === null ? undefined : exitCode;
  signal = signal === null ? undefined : signal;
  const signalDescription = signal === undefined ? undefined : signalsByName[signal].description;
  const errorCode = error && error.code;
  const prefix = getErrorPrefix({
    timedOut,
    timeout,
    errorCode,
    signal,
    signalDescription,
    exitCode,
    isCanceled
  });
  const execaMessage = `Command ${prefix}: ${command}`;
  const isError = Object.prototype.toString.call(error) === '[object Error]';
  const shortMessage = isError ? `${execaMessage}\n${error.message}` : execaMessage;
  const message = [shortMessage, stderr, stdout].filter(Boolean).join('\n');
  if (isError) {
    error.originalMessage = error.message;
    error.message = message;
  } else {
    error = new Error(message);
  }
  error.shortMessage = shortMessage;
  error.command = command;
  error.escapedCommand = escapedCommand;
  error.exitCode = exitCode;
  error.signal = signal;
  error.signalDescription = signalDescription;
  error.stdout = stdout;
  error.stderr = stderr;
  error.cwd = cwd;
  if (all !== undefined) {
    error.all = all;
  }
  if ('bufferedData' in error) {
    delete error.bufferedData;
  }
  error.failed = true;
  error.timedOut = Boolean(timedOut);
  error.isCanceled = isCanceled;
  error.killed = killed && !timedOut;
  return error;
};

const aliases = ['stdin', 'stdout', 'stderr'];
const hasAlias = options => aliases.some(alias => options[alias] !== undefined);
const normalizeStdio = options => {
  if (!options) {
    return;
  }
  const {
    stdio
  } = options;
  if (stdio === undefined) {
    return aliases.map(alias => options[alias]);
  }
  if (hasAlias(options)) {
    throw new Error(`It's not possible to provide \`stdio\` in combination with one of ${aliases.map(alias => `\`${alias}\``).join(', ')}`);
  }
  if (typeof stdio === 'string') {
    return stdio;
  }
  if (!Array.isArray(stdio)) {
    throw new TypeError(`Expected \`stdio\` to be of type \`string\` or \`Array\`, got \`${typeof stdio}\``);
  }
  const length = Math.max(stdio.length, aliases.length);
  return Array.from({
    length
  }, (value, index) => stdio[index]);
};

const DEFAULT_FORCE_KILL_TIMEOUT = 1000 * 5;

// Monkey-patches `childProcess.kill()` to add `forceKillAfterTimeout` behavior
const spawnedKill = (kill, signal = 'SIGTERM', options = {}) => {
  const killResult = kill(signal);
  setKillTimeout(kill, signal, options, killResult);
  return killResult;
};
const setKillTimeout = (kill, signal, options, killResult) => {
  if (!shouldForceKill(signal, options, killResult)) {
    return;
  }
  const timeout = getForceKillAfterTimeout(options);
  const t = setTimeout(() => {
    kill('SIGKILL');
  }, timeout);

  // Guarded because there's no `.unref()` when `execa` is used in the renderer
  // process in Electron. This cannot be tested since we don't run tests in
  // Electron.
  // istanbul ignore else
  if (t.unref) {
    t.unref();
  }
};
const shouldForceKill = (signal, {
  forceKillAfterTimeout
}, killResult) => isSigterm(signal) && forceKillAfterTimeout !== false && killResult;
const isSigterm = signal => signal === os$3.constants.signals.SIGTERM || typeof signal === 'string' && signal.toUpperCase() === 'SIGTERM';
const getForceKillAfterTimeout = ({
  forceKillAfterTimeout = true
}) => {
  if (forceKillAfterTimeout === true) {
    return DEFAULT_FORCE_KILL_TIMEOUT;
  }
  if (!Number.isFinite(forceKillAfterTimeout) || forceKillAfterTimeout < 0) {
    throw new TypeError(`Expected the \`forceKillAfterTimeout\` option to be a non-negative integer, got \`${forceKillAfterTimeout}\` (${typeof forceKillAfterTimeout})`);
  }
  return forceKillAfterTimeout;
};

// `childProcess.cancel()`
const spawnedCancel = (spawned, context) => {
  const killResult = spawned.kill();
  if (killResult) {
    context.isCanceled = true;
  }
};
const timeoutKill = (spawned, signal, reject) => {
  spawned.kill(signal);
  reject(Object.assign(new Error('Timed out'), {
    timedOut: true,
    signal
  }));
};

// `timeout` option handling
const setupTimeout = (spawned, {
  timeout,
  killSignal = 'SIGTERM'
}, spawnedPromise) => {
  if (timeout === 0 || timeout === undefined) {
    return spawnedPromise;
  }
  let timeoutId;
  const timeoutPromise = new Promise((resolve, reject) => {
    timeoutId = setTimeout(() => {
      timeoutKill(spawned, killSignal, reject);
    }, timeout);
  });
  const safeSpawnedPromise = spawnedPromise.finally(() => {
    clearTimeout(timeoutId);
  });
  return Promise.race([timeoutPromise, safeSpawnedPromise]);
};
const validateTimeout = ({
  timeout
}) => {
  if (timeout !== undefined && (!Number.isFinite(timeout) || timeout < 0)) {
    throw new TypeError(`Expected the \`timeout\` option to be a non-negative integer, got \`${timeout}\` (${typeof timeout})`);
  }
};

// `cleanup` option handling
const setExitHandler = async (spawned, {
  cleanup,
  detached
}, timedPromise) => {
  if (!cleanup || detached) {
    return timedPromise;
  }
  const removeExitHandler = onExit$1(() => {
    spawned.kill();
  });
  return timedPromise.finally(() => {
    removeExitHandler();
  });
};

function isStream(stream) {
  return stream !== null && typeof stream === 'object' && typeof stream.pipe === 'function';
}
function isWritableStream(stream) {
  return isStream(stream) && stream.writable !== false && typeof stream._write === 'function' && typeof stream._writableState === 'object';
}

const isExecaChildProcess = target => target instanceof ChildProcess && typeof target.then === 'function';
const pipeToTarget = (spawned, streamName, target) => {
  if (typeof target === 'string') {
    spawned[streamName].pipe(createWriteStream(target));
    return spawned;
  }
  if (isWritableStream(target)) {
    spawned[streamName].pipe(target);
    return spawned;
  }
  if (!isExecaChildProcess(target)) {
    throw new TypeError('The second argument must be a string, a stream or an Execa child process.');
  }
  if (!isWritableStream(target.stdin)) {
    throw new TypeError('The target child process\'s stdin must be available.');
  }
  spawned[streamName].pipe(target.stdin);
  return target;
};
const addPipeMethods = spawned => {
  if (spawned.stdout !== null) {
    spawned.pipeStdout = pipeToTarget.bind(undefined, spawned, 'stdout');
  }
  if (spawned.stderr !== null) {
    spawned.pipeStderr = pipeToTarget.bind(undefined, spawned, 'stderr');
  }
  if (spawned.all !== undefined) {
    spawned.pipeAll = pipeToTarget.bind(undefined, spawned, 'all');
  }
};

const validateInputOptions = input => {
  if (input !== undefined) {
    throw new TypeError('The `input` and `inputFile` options cannot be both set.');
  }
};
const getInput = ({
  input,
  inputFile
}) => {
  if (typeof inputFile !== 'string') {
    return input;
  }
  validateInputOptions(input);
  return createReadStream(inputFile);
};

// `input` and `inputFile` option in async mode
const handleInput = (spawned, options) => {
  const input = getInput(options);
  if (input === undefined) {
    return;
  }
  if (isStream(input)) {
    input.pipe(spawned.stdin);
  } else {
    spawned.stdin.end(input);
  }
};

// `all` interleaves `stdout` and `stderr`
const makeAllStream = (spawned, {
  all
}) => {
  if (!all || !spawned.stdout && !spawned.stderr) {
    return;
  }
  const mixed = mergeStream$2();
  if (spawned.stdout) {
    mixed.add(spawned.stdout);
  }
  if (spawned.stderr) {
    mixed.add(spawned.stderr);
  }
  return mixed;
};

// On failure, `result.stdout|stderr|all` should contain the currently buffered stream
const getBufferedData = async (stream, streamPromise) => {
  // When `buffer` is `false`, `streamPromise` is `undefined` and there is no buffered data to retrieve
  if (!stream || streamPromise === undefined) {
    return;
  }
  stream.destroy();
  try {
    return await streamPromise;
  } catch (error) {
    return error.bufferedData;
  }
};
const getStreamPromise = (stream, {
  encoding,
  buffer,
  maxBuffer
}) => {
  if (!stream || !buffer) {
    return;
  }
  if (encoding) {
    return getStream$2(stream, {
      encoding,
      maxBuffer
    });
  }
  return getStream$2.buffer(stream, {
    maxBuffer
  });
};

// Retrieve result of child process: exit code, signal, error, streams (stdout/stderr/all)
const getSpawnedResult = async ({
  stdout,
  stderr,
  all
}, {
  encoding,
  buffer,
  maxBuffer
}, processDone) => {
  const stdoutPromise = getStreamPromise(stdout, {
    encoding,
    buffer,
    maxBuffer
  });
  const stderrPromise = getStreamPromise(stderr, {
    encoding,
    buffer,
    maxBuffer
  });
  const allPromise = getStreamPromise(all, {
    encoding,
    buffer,
    maxBuffer: maxBuffer * 2
  });
  try {
    return await Promise.all([processDone, stdoutPromise, stderrPromise, allPromise]);
  } catch (error) {
    return Promise.all([{
      error,
      signal: error.signal,
      timedOut: error.timedOut
    }, getBufferedData(stdout, stdoutPromise), getBufferedData(stderr, stderrPromise), getBufferedData(all, allPromise)]);
  }
};

// eslint-disable-next-line unicorn/prefer-top-level-await
const nativePromisePrototype = (async () => {})().constructor.prototype;
const descriptors = ['then', 'catch', 'finally'].map(property => [property, Reflect.getOwnPropertyDescriptor(nativePromisePrototype, property)]);

// The return value is a mixin of `childProcess` and `Promise`
const mergePromise = (spawned, promise) => {
  for (const [property, descriptor] of descriptors) {
    // Starting the main `promise` is deferred to avoid consuming streams
    const value = typeof promise === 'function' ? (...args) => Reflect.apply(descriptor.value, promise(), args) : descriptor.value.bind(promise);
    Reflect.defineProperty(spawned, property, {
      ...descriptor,
      value
    });
  }
};

// Use promises instead of `child_process` events
const getSpawnedPromise = spawned => new Promise((resolve, reject) => {
  spawned.on('exit', (exitCode, signal) => {
    resolve({
      exitCode,
      signal
    });
  });
  spawned.on('error', error => {
    reject(error);
  });
  if (spawned.stdin) {
    spawned.stdin.on('error', error => {
      reject(error);
    });
  }
});

const normalizeArgs = (file, args = []) => {
  if (!Array.isArray(args)) {
    return [file];
  }
  return [file, ...args];
};
const NO_ESCAPE_REGEXP = /^[\w.-]+$/;
const DOUBLE_QUOTES_REGEXP = /"/g;
const escapeArg = arg => {
  if (typeof arg !== 'string' || NO_ESCAPE_REGEXP.test(arg)) {
    return arg;
  }
  return `"${arg.replace(DOUBLE_QUOTES_REGEXP, '\\"')}"`;
};
const joinCommand = (file, args) => normalizeArgs(file, args).join(' ');
const getEscapedCommand = (file, args) => normalizeArgs(file, args).map(arg => escapeArg(arg)).join(' ');

const verboseDefault = debuglog('execa').enabled;
const padField = (field, padding) => String(field).padStart(padding, '0');
const getTimestamp = () => {
  const date = new Date();
  return `${padField(date.getHours(), 2)}:${padField(date.getMinutes(), 2)}:${padField(date.getSeconds(), 2)}.${padField(date.getMilliseconds(), 3)}`;
};
const logCommand = (escapedCommand, {
  verbose
}) => {
  if (!verbose) {
    return;
  }
  process$4.stderr.write(`[${getTimestamp()}] ${escapedCommand}\n`);
};

const DEFAULT_MAX_BUFFER = 1000 * 1000 * 100;
const getEnv = ({
  env: envOption,
  extendEnv,
  preferLocal,
  localDir,
  execPath
}) => {
  const env = extendEnv ? {
    ...process$4.env,
    ...envOption
  } : envOption;
  if (preferLocal) {
    return npmRunPathEnv({
      env,
      cwd: localDir,
      execPath
    });
  }
  return env;
};
const handleArguments = (file, args, options = {}) => {
  const parsed = crossSpawn$1._parse(file, args, options);
  file = parsed.command;
  args = parsed.args;
  options = parsed.options;
  options = {
    maxBuffer: DEFAULT_MAX_BUFFER,
    buffer: true,
    stripFinalNewline: true,
    extendEnv: true,
    preferLocal: false,
    localDir: options.cwd || process$4.cwd(),
    execPath: process$4.execPath,
    encoding: 'utf8',
    reject: true,
    cleanup: true,
    all: false,
    windowsHide: true,
    verbose: verboseDefault,
    ...options
  };
  options.env = getEnv(options);
  options.stdio = normalizeStdio(options);
  if (process$4.platform === 'win32' && path$7.basename(file, '.exe') === 'cmd') {
    // #116
    args.unshift('/q');
  }
  return {
    file,
    args,
    options,
    parsed
  };
};
const handleOutput = (options, value, error) => {
  if (typeof value !== 'string' && !Buffer$1.isBuffer(value)) {
    // When `execaSync()` errors, we normalize it to '' to mimic `execa()`
    return error === undefined ? undefined : '';
  }
  if (options.stripFinalNewline) {
    return stripFinalNewline(value);
  }
  return value;
};
function execa(file, args, options) {
  const parsed = handleArguments(file, args, options);
  const command = joinCommand(file, args);
  const escapedCommand = getEscapedCommand(file, args);
  logCommand(escapedCommand, parsed.options);
  validateTimeout(parsed.options);
  let spawned;
  try {
    spawned = childProcess$2.spawn(parsed.file, parsed.args, parsed.options);
  } catch (error) {
    // Ensure the returned error is always both a promise and a child process
    const dummySpawned = new childProcess$2.ChildProcess();
    const errorPromise = Promise.reject(makeError({
      error,
      stdout: '',
      stderr: '',
      all: '',
      command,
      escapedCommand,
      parsed,
      timedOut: false,
      isCanceled: false,
      killed: false
    }));
    mergePromise(dummySpawned, errorPromise);
    return dummySpawned;
  }
  const spawnedPromise = getSpawnedPromise(spawned);
  const timedPromise = setupTimeout(spawned, parsed.options, spawnedPromise);
  const processDone = setExitHandler(spawned, parsed.options, timedPromise);
  const context = {
    isCanceled: false
  };
  spawned.kill = spawnedKill.bind(null, spawned.kill.bind(spawned));
  spawned.cancel = spawnedCancel.bind(null, spawned, context);
  const handlePromise = async () => {
    const [{
      error,
      exitCode,
      signal,
      timedOut
    }, stdoutResult, stderrResult, allResult] = await getSpawnedResult(spawned, parsed.options, processDone);
    const stdout = handleOutput(parsed.options, stdoutResult);
    const stderr = handleOutput(parsed.options, stderrResult);
    const all = handleOutput(parsed.options, allResult);
    if (error || exitCode !== 0 || signal !== null) {
      const returnedError = makeError({
        error,
        exitCode,
        signal,
        stdout,
        stderr,
        all,
        command,
        escapedCommand,
        parsed,
        timedOut,
        isCanceled: (parsed.options.signal ? parsed.options.signal.aborted : false),
        killed: spawned.killed
      });
      if (!parsed.options.reject) {
        return returnedError;
      }
      throw returnedError;
    }
    return {
      command,
      escapedCommand,
      exitCode: 0,
      stdout,
      stderr,
      all,
      failed: false,
      timedOut: false,
      isCanceled: false,
      killed: false
    };
  };
  const handlePromiseOnce = onetime(handlePromise);
  handleInput(spawned, parsed.options);
  spawned.all = makeAllStream(spawned, parsed.options);
  addPipeMethods(spawned);
  mergePromise(spawned, handlePromiseOnce);
  return spawned;
}

// Windows doesn't have browser IDs in the same way macOS/Linux does so we give fake
// ones that look real and match the macOS/Linux versions for cross-platform apps.
const windowsBrowserProgIds = {
  AppXq0fevzme2pys62n3e0fbqa7peapykr8v: {
    name: 'Edge',
    id: 'com.microsoft.edge.old'
  },
  MSEdgeDHTML: {
    name: 'Edge',
    id: 'com.microsoft.edge'
  },
  // On macOS, it's "com.microsoft.edgemac"
  MSEdgeHTM: {
    name: 'Edge',
    id: 'com.microsoft.edge'
  },
  // Newer Edge/Win10 releases
  'IE.HTTP': {
    name: 'Internet Explorer',
    id: 'com.microsoft.ie'
  },
  FirefoxURL: {
    name: 'Firefox',
    id: 'org.mozilla.firefox'
  },
  ChromeHTML: {
    name: 'Chrome',
    id: 'com.google.chrome'
  }
};
class UnknownBrowserError extends Error {}
async function defaultBrowser$1(_execa = execa) {
  const result = await _execa('reg', ['QUERY', ' HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\Shell\\Associations\\UrlAssociations\\http\\UserChoice', '/v', 'ProgId']);
  const match = /ProgId\s*REG_SZ\s*(?<id>\S+)/.exec(result.stdout);
  if (!match) {
    throw new UnknownBrowserError(`Cannot find Windows browser in stdout: ${JSON.stringify(result.stdout)}`);
  }
  const {
    id
  } = match.groups;
  const browser = windowsBrowserProgIds[id];
  if (!browser) {
    throw new UnknownBrowserError(`Unknown browser ID: ${id}`);
  }
  return browser;
}

async function defaultBrowser() {
  if (process$4.platform === 'linux') {
    const {
      stdout
    } = await execa('xdg-mime', ['query', 'default', 'x-scheme-handler/http']);
    const name = titleize(stdout.trim().replace(/.desktop$/, '').replace('-', ' '));
    return {
      name,
      id: stdout
    };
  }
  if (process$4.platform === 'darwin') {
    const id = await defaultBrowserId();
    const name = await bundleName(id);
    return {
      name,
      id
    };
  }
  if (process$4.platform === 'win32') {
    return defaultBrowser$1();
  }
  throw new Error('Only macOS, Linux, and Windows are supported');
}

let isDockerCached;
function hasDockerEnv() {
  try {
    fs$5.statSync('/.dockerenv');
    return true;
  } catch {
    return false;
  }
}
function hasDockerCGroup() {
  try {
    return fs$5.readFileSync('/proc/self/cgroup', 'utf8').includes('docker');
  } catch {
    return false;
  }
}
function isDocker() {
  // TODO: Use `??=` when targeting Node.js 16.
  if (isDockerCached === undefined) {
    isDockerCached = hasDockerEnv() || hasDockerCGroup();
  }
  return isDockerCached;
}

let cachedResult;

// Podman detection
const hasContainerEnv = () => {
  try {
    fs$5.statSync('/run/.containerenv');
    return true;
  } catch {
    return false;
  }
};
function isInsideContainer() {
  // TODO: Use `??=` when targeting Node.js 16.
  if (cachedResult === undefined) {
    cachedResult = hasContainerEnv() || isDocker();
  }
  return cachedResult;
}

// Path to included `xdg-open`.
const __dirname = path$7.dirname(fileURLToPath(import.meta.url));
const localXdgOpenPath = path$7.join(__dirname, 'xdg-open');
const {
  platform,
  arch
} = process$4;

/**
Get the mount point for fixed drives in WSL.

@inner
@returns {string} The mount point.
*/
const getWslDrivesMountPoint = (() => {
  // Default value for "root" param
  // according to https://docs.microsoft.com/en-us/windows/wsl/wsl-config
  const defaultMountPoint = '/mnt/';
  let mountPoint;
  return async function () {
    if (mountPoint) {
      // Return memoized mount point value
      return mountPoint;
    }
    const configFilePath = '/etc/wsl.conf';
    let isConfigFileExists = false;
    try {
      await fs$6.access(configFilePath, constants$1.F_OK);
      isConfigFileExists = true;
    } catch {}
    if (!isConfigFileExists) {
      return defaultMountPoint;
    }
    const configContent = await fs$6.readFile(configFilePath, {
      encoding: 'utf8'
    });
    const configMountPoint = /(?<!#.*)root\s*=\s*(?<mountPoint>.*)/g.exec(configContent);
    if (!configMountPoint) {
      return defaultMountPoint;
    }
    mountPoint = configMountPoint.groups.mountPoint.trim();
    mountPoint = mountPoint.endsWith('/') ? mountPoint : `${mountPoint}/`;
    return mountPoint;
  };
})();
const pTryEach = async (array, mapper) => {
  let latestError;
  for (const item of array) {
    try {
      return await mapper(item); // eslint-disable-line no-await-in-loop
    } catch (error) {
      latestError = error;
    }
  }
  throw latestError;
};
const baseOpen = async options => {
  options = {
    wait: false,
    background: false,
    newInstance: false,
    allowNonzeroExitCode: false,
    ...options
  };
  if (Array.isArray(options.app)) {
    return pTryEach(options.app, singleApp => baseOpen({
      ...options,
      app: singleApp
    }));
  }
  let {
    name: app,
    arguments: appArguments = []
  } = options.app ?? {};
  appArguments = [...appArguments];
  if (Array.isArray(app)) {
    return pTryEach(app, appName => baseOpen({
      ...options,
      app: {
        name: appName,
        arguments: appArguments
      }
    }));
  }
  if (app === 'browser' || app === 'browserPrivate') {
    // IDs from default-browser for macOS and windows are the same
    const ids = {
      'com.google.chrome': 'chrome',
      'google-chrome.desktop': 'chrome',
      'org.mozilla.firefox': 'firefox',
      'firefox.desktop': 'firefox',
      'com.microsoft.msedge': 'edge',
      'com.microsoft.edge': 'edge',
      'microsoft-edge.desktop': 'edge'
    };

    // Incognito flags for each browser in `apps`.
    const flags = {
      chrome: '--incognito',
      firefox: '--private-window',
      edge: '--inPrivate'
    };
    const browser = await defaultBrowser();
    if (browser.id in ids) {
      const browserName = ids[browser.id];
      if (app === 'browserPrivate') {
        appArguments.push(flags[browserName]);
      }
      return baseOpen({
        ...options,
        app: {
          name: apps[browserName],
          arguments: appArguments
        }
      });
    }
    throw new Error(`${browser.name} is not supported as a default browser`);
  }
  let command;
  const cliArguments = [];
  const childProcessOptions = {};
  if (platform === 'darwin') {
    command = 'open';
    if (options.wait) {
      cliArguments.push('--wait-apps');
    }
    if (options.background) {
      cliArguments.push('--background');
    }
    if (options.newInstance) {
      cliArguments.push('--new');
    }
    if (app) {
      cliArguments.push('-a', app);
    }
  } else if (platform === 'win32' || isWsl$1 && !isInsideContainer() && !app) {
    const mountPoint = await getWslDrivesMountPoint();
    command = isWsl$1 ? `${mountPoint}c/Windows/System32/WindowsPowerShell/v1.0/powershell.exe` : `${process$4.env.SYSTEMROOT}\\System32\\WindowsPowerShell\\v1.0\\powershell`;
    cliArguments.push('-NoProfile', '-NonInteractive', '-ExecutionPolicy', 'Bypass', '-EncodedCommand');
    if (!isWsl$1) {
      childProcessOptions.windowsVerbatimArguments = true;
    }
    const encodedArguments = ['Start'];
    if (options.wait) {
      encodedArguments.push('-Wait');
    }
    if (app) {
      // Double quote with double quotes to ensure the inner quotes are passed through.
      // Inner quotes are delimited for PowerShell interpretation with backticks.
      encodedArguments.push(`"\`"${app}\`""`);
      if (options.target) {
        appArguments.push(options.target);
      }
    } else if (options.target) {
      encodedArguments.push(`"${options.target}"`);
    }
    if (appArguments.length > 0) {
      appArguments = appArguments.map(arg => `"\`"${arg}\`""`);
      encodedArguments.push('-ArgumentList', appArguments.join(','));
    }

    // Using Base64-encoded command, accepted by PowerShell, to allow special characters.
    options.target = Buffer$1.from(encodedArguments.join(' '), 'utf16le').toString('base64');
  } else {
    if (app) {
      command = app;
    } else {
      // When bundled by Webpack, there's no actual package file path and no local `xdg-open`.
      const isBundled = !__dirname || __dirname === '/';

      // Check if local `xdg-open` exists and is executable.
      let exeLocalXdgOpen = false;
      try {
        await fs$6.access(localXdgOpenPath, constants$1.X_OK);
        exeLocalXdgOpen = true;
      } catch {}
      const useSystemXdgOpen = process$4.versions.electron ?? (platform === 'android' || isBundled || !exeLocalXdgOpen);
      command = useSystemXdgOpen ? 'xdg-open' : localXdgOpenPath;
    }
    if (appArguments.length > 0) {
      cliArguments.push(...appArguments);
    }
    if (!options.wait) {
      // `xdg-open` will block the process unless stdio is ignored
      // and it's detached from the parent even if it's unref'd.
      childProcessOptions.stdio = 'ignore';
      childProcessOptions.detached = true;
    }
  }
  if (options.target) {
    cliArguments.push(options.target);
  }
  if (platform === 'darwin' && appArguments.length > 0) {
    cliArguments.push('--args', ...appArguments);
  }
  const subprocess = childProcess$2.spawn(command, cliArguments, childProcessOptions);
  if (options.wait) {
    return new Promise((resolve, reject) => {
      subprocess.once('error', reject);
      subprocess.once('close', exitCode => {
        if (!options.allowNonzeroExitCode && exitCode > 0) {
          reject(new Error(`Exited with code ${exitCode}`));
          return;
        }
        resolve(subprocess);
      });
    });
  }
  subprocess.unref();
  return subprocess;
};
const open = (target, options) => {
  if (typeof target !== 'string') {
    throw new TypeError('Expected a `target`');
  }
  return baseOpen({
    ...options,
    target
  });
};
function detectArchBinary(binary) {
  if (typeof binary === 'string' || Array.isArray(binary)) {
    return binary;
  }
  const {
    [arch]: archBinary
  } = binary;
  if (!archBinary) {
    throw new Error(`${arch} is not supported`);
  }
  return archBinary;
}
function detectPlatformBinary({
  [platform]: platformBinary
}, {
  wsl
}) {
  if (wsl && isWsl$1) {
    return detectArchBinary(wsl);
  }
  if (!platformBinary) {
    throw new Error(`${platform} is not supported`);
  }
  return detectArchBinary(platformBinary);
}
const apps = {};
defineLazyProperty(apps, 'chrome', () => detectPlatformBinary({
  darwin: 'google chrome',
  win32: 'chrome',
  linux: ['google-chrome', 'google-chrome-stable', 'chromium']
}, {
  wsl: {
    ia32: '/mnt/c/Program Files (x86)/Google/Chrome/Application/chrome.exe',
    x64: ['/mnt/c/Program Files/Google/Chrome/Application/chrome.exe', '/mnt/c/Program Files (x86)/Google/Chrome/Application/chrome.exe']
  }
}));
defineLazyProperty(apps, 'firefox', () => detectPlatformBinary({
  darwin: 'firefox',
  win32: 'C:\\Program Files\\Mozilla Firefox\\firefox.exe',
  linux: 'firefox'
}, {
  wsl: '/mnt/c/Program Files/Mozilla Firefox/firefox.exe'
}));
defineLazyProperty(apps, 'edge', () => detectPlatformBinary({
  darwin: 'microsoft edge',
  win32: 'msedge',
  linux: ['microsoft-edge', 'microsoft-edge-dev']
}, {
  wsl: '/mnt/c/Program Files (x86)/Microsoft/Edge/Application/msedge.exe'
}));
defineLazyProperty(apps, 'browser', () => 'browser');
defineLazyProperty(apps, 'browserPrivate', () => 'browserPrivate');

process$3.env.NPA_ENV = "development" /* Development */;
function createDataServer(analyzer, ui) {
  const app = express();
  const port = 3e3;
  {
    app.use(express.static(ui));
  }
  app.get("/api/nodes", (req, res) => {
    res.json(analyzer.dependencyGraph.exportPackages());
  });
  app.get("/api/edges", (req, res) => {
    res.json(analyzer.dependencyGraph.exportEdges());
  });
  app.get("/api/data", (req, res) => {
    res.json(analyzer.getGraphData());
  });
  app.get("/api/packageData", (req, res) => {
    if (req.query.id === void 0) {
      res.status(400).send("Invalid parameters");
    }
    const id = parseInt(req.query.id);
    try {
      const pth = analyzer.dependencyGraph.packages[id].path;
      const packageJson = analyzer.packages[pth];
      const packageData = {
        path: pth,
        packageJson
      };
      res.json(packageData);
    } catch (e) {
      console.log(e);
      res.status(500).send("Error");
    }
  });
  app.get("/api/dependencies", (req, res) => {
    const packageId = parseInt(req.query.id);
    const depth = parseInt(req.query.depth);
    if (isNaN(packageId) || isNaN(depth)) {
      res.status(400).send("Invalid parameters");
      return;
    }
    try {
      res.json(analyzer.getPackageDependencies(packageId, depth));
    } catch (e) {
      console.log(e);
      res.status(500).send("Error");
    }
  });
  app.get("/api/directDependencyList", (req, res) => {
    const packageId = parseInt(req.query.id);
    if (isNaN(packageId)) {
      res.status(400).send("Invalid parameters");
      return;
    }
    try {
      res.json(analyzer.dependencyGraph.getDirectDependency(packageId));
    } catch (e) {
      console.log(e);
      res.status(500).send("Error");
    }
  });
  app.get("/api/whyInstalled", (req, res) => {
    const packageId = parseInt(req.query.id);
    if (isNaN(packageId)) {
      res.status(400).send("Invalid parameters");
      return;
    }
    try {
      res.json(analyzer.whyInstalledIt(packageId));
    } catch (e) {
      console.log(e);
      res.status(500).send("Error");
    }
  });
  app.get("/api/searchPackage", (req, res) => {
    const str = req.query.pattern;
    if (typeof str !== "string") {
      res.status(400).send("Invalid parameters");
      return;
    }
    try {
      res.json(analyzer.dependencyGraph.queryPackage(str));
    } catch (e) {
      console.log(e);
      res.status(500).send("Error");
    }
  });
  portfinder.setBasePort(port);
  portfinder.getPortPromise().then((port2) => {
    app.listen(port2, () => {
      console.log(chalk.cyan(figlet.textSync("NPA-TOOL")));
      console.log(`  ${chalk.green("->")}  ${chalk.gray("Local: ")}   ${chalk.blue.underline(`http://localhost:${port2}`)}`);
      console.log(`  ${chalk.red("->")}  press Ctrl+C to terminate the process`);
      {
        open(`http://localhost:${port2}`);
      }
    });
  }).catch((err) => {
    console.log(err);
  });
}

var commander$1 = {exports: {}};

var argument = {};

var error = {};

// @ts-check

/**
 * CommanderError class
 * @class
 */
let CommanderError$2 = class CommanderError extends Error {
  /**
   * Constructs the CommanderError class
   * @param {number} exitCode suggested exit code which could be used with process.exit
   * @param {string} code an id string representing the error
   * @param {string} message human-readable description of the error
   * @constructor
   */
  constructor(exitCode, code, message) {
    super(message);
    // properly capture stack trace in Node.js
    Error.captureStackTrace(this, this.constructor);
    this.name = this.constructor.name;
    this.code = code;
    this.exitCode = exitCode;
    this.nestedError = undefined;
  }
};

/**
 * InvalidArgumentError class
 * @class
 */
let InvalidArgumentError$3 = class InvalidArgumentError extends CommanderError$2 {
  /**
   * Constructs the InvalidArgumentError class
   * @param {string} [message] explanation of why argument is invalid
   * @constructor
   */
  constructor(message) {
    super(1, 'commander.invalidArgument', message);
    // properly capture stack trace in Node.js
    Error.captureStackTrace(this, this.constructor);
    this.name = this.constructor.name;
  }
};
error.CommanderError = CommanderError$2;
error.InvalidArgumentError = InvalidArgumentError$3;

const {
  InvalidArgumentError: InvalidArgumentError$2
} = error;

// @ts-check

let Argument$2 = class Argument {
  /**
   * Initialize a new command argument with the given name and description.
   * The default is that the argument is required, and you can explicitly
   * indicate this with <> around the name. Put [] around the name for an optional argument.
   *
   * @param {string} name
   * @param {string} [description]
   */

  constructor(name, description) {
    this.description = description || '';
    this.variadic = false;
    this.parseArg = undefined;
    this.defaultValue = undefined;
    this.defaultValueDescription = undefined;
    this.argChoices = undefined;
    switch (name[0]) {
      case '<':
        // e.g. <required>
        this.required = true;
        this._name = name.slice(1, -1);
        break;
      case '[':
        // e.g. [optional]
        this.required = false;
        this._name = name.slice(1, -1);
        break;
      default:
        this.required = true;
        this._name = name;
        break;
    }
    if (this._name.length > 3 && this._name.slice(-3) === '...') {
      this.variadic = true;
      this._name = this._name.slice(0, -3);
    }
  }

  /**
   * Return argument name.
   *
   * @return {string}
   */

  name() {
    return this._name;
  }

  /**
   * @api private
   */

  _concatValue(value, previous) {
    if (previous === this.defaultValue || !Array.isArray(previous)) {
      return [value];
    }
    return previous.concat(value);
  }

  /**
   * Set the default value, and optionally supply the description to be displayed in the help.
   *
   * @param {any} value
   * @param {string} [description]
   * @return {Argument}
   */

  default(value, description) {
    this.defaultValue = value;
    this.defaultValueDescription = description;
    return this;
  }

  /**
   * Set the custom handler for processing CLI command arguments into argument values.
   *
   * @param {Function} [fn]
   * @return {Argument}
   */

  argParser(fn) {
    this.parseArg = fn;
    return this;
  }

  /**
   * Only allow argument value to be one of choices.
   *
   * @param {string[]} values
   * @return {Argument}
   */

  choices(values) {
    this.argChoices = values.slice();
    this.parseArg = (arg, previous) => {
      if (!this.argChoices.includes(arg)) {
        throw new InvalidArgumentError$2(`Allowed choices are ${this.argChoices.join(', ')}.`);
      }
      if (this.variadic) {
        return this._concatValue(arg, previous);
      }
      return arg;
    };
    return this;
  }

  /**
   * Make argument required.
   */
  argRequired() {
    this.required = true;
    return this;
  }

  /**
   * Make argument optional.
   */
  argOptional() {
    this.required = false;
    return this;
  }
};

/**
 * Takes an argument and returns its human readable equivalent for help usage.
 *
 * @param {Argument} arg
 * @return {string}
 * @api private
 */

function humanReadableArgName$2(arg) {
  const nameOutput = arg.name() + (arg.variadic === true ? '...' : '');
  return arg.required ? '<' + nameOutput + '>' : '[' + nameOutput + ']';
}
argument.Argument = Argument$2;
argument.humanReadableArgName = humanReadableArgName$2;

var command = {};

var help = {};

const {
  humanReadableArgName: humanReadableArgName$1
} = argument;

/**
 * TypeScript import types for JSDoc, used by Visual Studio Code IntelliSense and `npm run typescript-checkJS`
 * https://www.typescriptlang.org/docs/handbook/jsdoc-supported-types.html#import-types
 * @typedef { import("./argument.js").Argument } Argument
 * @typedef { import("./command.js").Command } Command
 * @typedef { import("./option.js").Option } Option
 */

// @ts-check

// Although this is a class, methods are static in style to allow override using subclass or just functions.
let Help$2 = class Help {
  constructor() {
    this.helpWidth = undefined;
    this.sortSubcommands = false;
    this.sortOptions = false;
    this.showGlobalOptions = false;
  }

  /**
   * Get an array of the visible subcommands. Includes a placeholder for the implicit help command, if there is one.
   *
   * @param {Command} cmd
   * @returns {Command[]}
   */

  visibleCommands(cmd) {
    const visibleCommands = cmd.commands.filter(cmd => !cmd._hidden);
    if (cmd._hasImplicitHelpCommand()) {
      // Create a command matching the implicit help command.
      const [, helpName, helpArgs] = cmd._helpCommandnameAndArgs.match(/([^ ]+) *(.*)/);
      const helpCommand = cmd.createCommand(helpName).helpOption(false);
      helpCommand.description(cmd._helpCommandDescription);
      if (helpArgs) helpCommand.arguments(helpArgs);
      visibleCommands.push(helpCommand);
    }
    if (this.sortSubcommands) {
      visibleCommands.sort((a, b) => {
        // @ts-ignore: overloaded return type
        return a.name().localeCompare(b.name());
      });
    }
    return visibleCommands;
  }

  /**
   * Compare options for sort.
   *
   * @param {Option} a
   * @param {Option} b
   * @returns number
   */
  compareOptions(a, b) {
    const getSortKey = option => {
      // WYSIWYG for order displayed in help. Short used for comparison if present. No special handling for negated.
      return option.short ? option.short.replace(/^-/, '') : option.long.replace(/^--/, '');
    };
    return getSortKey(a).localeCompare(getSortKey(b));
  }

  /**
   * Get an array of the visible options. Includes a placeholder for the implicit help option, if there is one.
   *
   * @param {Command} cmd
   * @returns {Option[]}
   */

  visibleOptions(cmd) {
    const visibleOptions = cmd.options.filter(option => !option.hidden);
    // Implicit help
    const showShortHelpFlag = cmd._hasHelpOption && cmd._helpShortFlag && !cmd._findOption(cmd._helpShortFlag);
    const showLongHelpFlag = cmd._hasHelpOption && !cmd._findOption(cmd._helpLongFlag);
    if (showShortHelpFlag || showLongHelpFlag) {
      let helpOption;
      if (!showShortHelpFlag) {
        helpOption = cmd.createOption(cmd._helpLongFlag, cmd._helpDescription);
      } else if (!showLongHelpFlag) {
        helpOption = cmd.createOption(cmd._helpShortFlag, cmd._helpDescription);
      } else {
        helpOption = cmd.createOption(cmd._helpFlags, cmd._helpDescription);
      }
      visibleOptions.push(helpOption);
    }
    if (this.sortOptions) {
      visibleOptions.sort(this.compareOptions);
    }
    return visibleOptions;
  }

  /**
   * Get an array of the visible global options. (Not including help.)
   *
   * @param {Command} cmd
   * @returns {Option[]}
   */

  visibleGlobalOptions(cmd) {
    if (!this.showGlobalOptions) return [];
    const globalOptions = [];
    for (let parentCmd = cmd.parent; parentCmd; parentCmd = parentCmd.parent) {
      const visibleOptions = parentCmd.options.filter(option => !option.hidden);
      globalOptions.push(...visibleOptions);
    }
    if (this.sortOptions) {
      globalOptions.sort(this.compareOptions);
    }
    return globalOptions;
  }

  /**
   * Get an array of the arguments if any have a description.
   *
   * @param {Command} cmd
   * @returns {Argument[]}
   */

  visibleArguments(cmd) {
    // Side effect! Apply the legacy descriptions before the arguments are displayed.
    if (cmd._argsDescription) {
      cmd._args.forEach(argument => {
        argument.description = argument.description || cmd._argsDescription[argument.name()] || '';
      });
    }

    // If there are any arguments with a description then return all the arguments.
    if (cmd._args.find(argument => argument.description)) {
      return cmd._args;
    }
    return [];
  }

  /**
   * Get the command term to show in the list of subcommands.
   *
   * @param {Command} cmd
   * @returns {string}
   */

  subcommandTerm(cmd) {
    // Legacy. Ignores custom usage string, and nested commands.
    const args = cmd._args.map(arg => humanReadableArgName$1(arg)).join(' ');
    return cmd._name + (cmd._aliases[0] ? '|' + cmd._aliases[0] : '') + (cmd.options.length ? ' [options]' : '') + (
    // simplistic check for non-help option
    args ? ' ' + args : '');
  }

  /**
   * Get the option term to show in the list of options.
   *
   * @param {Option} option
   * @returns {string}
   */

  optionTerm(option) {
    return option.flags;
  }

  /**
   * Get the argument term to show in the list of arguments.
   *
   * @param {Argument} argument
   * @returns {string}
   */

  argumentTerm(argument) {
    return argument.name();
  }

  /**
   * Get the longest command term length.
   *
   * @param {Command} cmd
   * @param {Help} helper
   * @returns {number}
   */

  longestSubcommandTermLength(cmd, helper) {
    return helper.visibleCommands(cmd).reduce((max, command) => {
      return Math.max(max, helper.subcommandTerm(command).length);
    }, 0);
  }

  /**
   * Get the longest option term length.
   *
   * @param {Command} cmd
   * @param {Help} helper
   * @returns {number}
   */

  longestOptionTermLength(cmd, helper) {
    return helper.visibleOptions(cmd).reduce((max, option) => {
      return Math.max(max, helper.optionTerm(option).length);
    }, 0);
  }

  /**
   * Get the longest global option term length.
   *
   * @param {Command} cmd
   * @param {Help} helper
   * @returns {number}
   */

  longestGlobalOptionTermLength(cmd, helper) {
    return helper.visibleGlobalOptions(cmd).reduce((max, option) => {
      return Math.max(max, helper.optionTerm(option).length);
    }, 0);
  }

  /**
   * Get the longest argument term length.
   *
   * @param {Command} cmd
   * @param {Help} helper
   * @returns {number}
   */

  longestArgumentTermLength(cmd, helper) {
    return helper.visibleArguments(cmd).reduce((max, argument) => {
      return Math.max(max, helper.argumentTerm(argument).length);
    }, 0);
  }

  /**
   * Get the command usage to be displayed at the top of the built-in help.
   *
   * @param {Command} cmd
   * @returns {string}
   */

  commandUsage(cmd) {
    // Usage
    let cmdName = cmd._name;
    if (cmd._aliases[0]) {
      cmdName = cmdName + '|' + cmd._aliases[0];
    }
    let parentCmdNames = '';
    for (let parentCmd = cmd.parent; parentCmd; parentCmd = parentCmd.parent) {
      parentCmdNames = parentCmd.name() + ' ' + parentCmdNames;
    }
    return parentCmdNames + cmdName + ' ' + cmd.usage();
  }

  /**
   * Get the description for the command.
   *
   * @param {Command} cmd
   * @returns {string}
   */

  commandDescription(cmd) {
    // @ts-ignore: overloaded return type
    return cmd.description();
  }

  /**
   * Get the subcommand summary to show in the list of subcommands.
   * (Fallback to description for backwards compatibility.)
   *
   * @param {Command} cmd
   * @returns {string}
   */

  subcommandDescription(cmd) {
    // @ts-ignore: overloaded return type
    return cmd.summary() || cmd.description();
  }

  /**
   * Get the option description to show in the list of options.
   *
   * @param {Option} option
   * @return {string}
   */

  optionDescription(option) {
    const extraInfo = [];
    if (option.argChoices) {
      extraInfo.push(
      // use stringify to match the display of the default value
      `choices: ${option.argChoices.map(choice => JSON.stringify(choice)).join(', ')}`);
    }
    if (option.defaultValue !== undefined) {
      // default for boolean and negated more for programmer than end user,
      // but show true/false for boolean option as may be for hand-rolled env or config processing.
      const showDefault = option.required || option.optional || option.isBoolean() && typeof option.defaultValue === 'boolean';
      if (showDefault) {
        extraInfo.push(`default: ${option.defaultValueDescription || JSON.stringify(option.defaultValue)}`);
      }
    }
    // preset for boolean and negated are more for programmer than end user
    if (option.presetArg !== undefined && option.optional) {
      extraInfo.push(`preset: ${JSON.stringify(option.presetArg)}`);
    }
    if (option.envVar !== undefined) {
      extraInfo.push(`env: ${option.envVar}`);
    }
    if (extraInfo.length > 0) {
      return `${option.description} (${extraInfo.join(', ')})`;
    }
    return option.description;
  }

  /**
   * Get the argument description to show in the list of arguments.
   *
   * @param {Argument} argument
   * @return {string}
   */

  argumentDescription(argument) {
    const extraInfo = [];
    if (argument.argChoices) {
      extraInfo.push(
      // use stringify to match the display of the default value
      `choices: ${argument.argChoices.map(choice => JSON.stringify(choice)).join(', ')}`);
    }
    if (argument.defaultValue !== undefined) {
      extraInfo.push(`default: ${argument.defaultValueDescription || JSON.stringify(argument.defaultValue)}`);
    }
    if (extraInfo.length > 0) {
      const extraDescripton = `(${extraInfo.join(', ')})`;
      if (argument.description) {
        return `${argument.description} ${extraDescripton}`;
      }
      return extraDescripton;
    }
    return argument.description;
  }

  /**
   * Generate the built-in help text.
   *
   * @param {Command} cmd
   * @param {Help} helper
   * @returns {string}
   */

  formatHelp(cmd, helper) {
    const termWidth = helper.padWidth(cmd, helper);
    const helpWidth = helper.helpWidth || 80;
    const itemIndentWidth = 2;
    const itemSeparatorWidth = 2; // between term and description
    function formatItem(term, description) {
      if (description) {
        const fullText = `${term.padEnd(termWidth + itemSeparatorWidth)}${description}`;
        return helper.wrap(fullText, helpWidth - itemIndentWidth, termWidth + itemSeparatorWidth);
      }
      return term;
    }
    function formatList(textArray) {
      return textArray.join('\n').replace(/^/gm, ' '.repeat(itemIndentWidth));
    }

    // Usage
    let output = [`Usage: ${helper.commandUsage(cmd)}`, ''];

    // Description
    const commandDescription = helper.commandDescription(cmd);
    if (commandDescription.length > 0) {
      output = output.concat([helper.wrap(commandDescription, helpWidth, 0), '']);
    }

    // Arguments
    const argumentList = helper.visibleArguments(cmd).map(argument => {
      return formatItem(helper.argumentTerm(argument), helper.argumentDescription(argument));
    });
    if (argumentList.length > 0) {
      output = output.concat(['Arguments:', formatList(argumentList), '']);
    }

    // Options
    const optionList = helper.visibleOptions(cmd).map(option => {
      return formatItem(helper.optionTerm(option), helper.optionDescription(option));
    });
    if (optionList.length > 0) {
      output = output.concat(['Options:', formatList(optionList), '']);
    }
    if (this.showGlobalOptions) {
      const globalOptionList = helper.visibleGlobalOptions(cmd).map(option => {
        return formatItem(helper.optionTerm(option), helper.optionDescription(option));
      });
      if (globalOptionList.length > 0) {
        output = output.concat(['Global Options:', formatList(globalOptionList), '']);
      }
    }

    // Commands
    const commandList = helper.visibleCommands(cmd).map(cmd => {
      return formatItem(helper.subcommandTerm(cmd), helper.subcommandDescription(cmd));
    });
    if (commandList.length > 0) {
      output = output.concat(['Commands:', formatList(commandList), '']);
    }
    return output.join('\n');
  }

  /**
   * Calculate the pad width from the maximum term length.
   *
   * @param {Command} cmd
   * @param {Help} helper
   * @returns {number}
   */

  padWidth(cmd, helper) {
    return Math.max(helper.longestOptionTermLength(cmd, helper), helper.longestGlobalOptionTermLength(cmd, helper), helper.longestSubcommandTermLength(cmd, helper), helper.longestArgumentTermLength(cmd, helper));
  }

  /**
   * Wrap the given string to width characters per line, with lines after the first indented.
   * Do not wrap if insufficient room for wrapping (minColumnWidth), or string is manually formatted.
   *
   * @param {string} str
   * @param {number} width
   * @param {number} indent
   * @param {number} [minColumnWidth=40]
   * @return {string}
   *
   */

  wrap(str, width, indent, minColumnWidth = 40) {
    // Full \s characters, minus the linefeeds.
    const indents = ' \\f\\t\\v\u00a0\u1680\u2000-\u200a\u202f\u205f\u3000\ufeff';
    // Detect manually wrapped and indented strings by searching for line break followed by spaces.
    const manualIndent = new RegExp(`[\\n][${indents}]+`);
    if (str.match(manualIndent)) return str;
    // Do not wrap if not enough room for a wrapped column of text (as could end up with a word per line).
    const columnWidth = width - indent;
    if (columnWidth < minColumnWidth) return str;
    const leadingStr = str.slice(0, indent);
    const columnText = str.slice(indent).replace('\r\n', '\n');
    const indentString = ' '.repeat(indent);
    const zeroWidthSpace = '\u200B';
    const breaks = `\\s${zeroWidthSpace}`;
    // Match line end (so empty lines don't collapse),
    // or as much text as will fit in column, or excess text up to first break.
    const regex = new RegExp(`\n|.{1,${columnWidth - 1}}([${breaks}]|$)|[^${breaks}]+?([${breaks}]|$)`, 'g');
    const lines = columnText.match(regex) || [];
    return leadingStr + lines.map((line, i) => {
      if (line === '\n') return ''; // preserve empty lines
      return (i > 0 ? indentString : '') + line.trimEnd();
    }).join('\n');
  }
};
help.Help = Help$2;

var option = {};

const {
  InvalidArgumentError: InvalidArgumentError$1
} = error;

// @ts-check

let Option$2 = class Option {
  /**
   * Initialize a new `Option` with the given `flags` and `description`.
   *
   * @param {string} flags
   * @param {string} [description]
   */

  constructor(flags, description) {
    this.flags = flags;
    this.description = description || '';
    this.required = flags.includes('<'); // A value must be supplied when the option is specified.
    this.optional = flags.includes('['); // A value is optional when the option is specified.
    // variadic test ignores <value,...> et al which might be used to describe custom splitting of single argument
    this.variadic = /\w\.\.\.[>\]]$/.test(flags); // The option can take multiple values.
    this.mandatory = false; // The option must have a value after parsing, which usually means it must be specified on command line.
    const optionFlags = splitOptionFlags$1(flags);
    this.short = optionFlags.shortFlag;
    this.long = optionFlags.longFlag;
    this.negate = false;
    if (this.long) {
      this.negate = this.long.startsWith('--no-');
    }
    this.defaultValue = undefined;
    this.defaultValueDescription = undefined;
    this.presetArg = undefined;
    this.envVar = undefined;
    this.parseArg = undefined;
    this.hidden = false;
    this.argChoices = undefined;
    this.conflictsWith = [];
    this.implied = undefined;
  }

  /**
   * Set the default value, and optionally supply the description to be displayed in the help.
   *
   * @param {any} value
   * @param {string} [description]
   * @return {Option}
   */

  default(value, description) {
    this.defaultValue = value;
    this.defaultValueDescription = description;
    return this;
  }

  /**
   * Preset to use when option used without option-argument, especially optional but also boolean and negated.
   * The custom processing (parseArg) is called.
   *
   * @example
   * new Option('--color').default('GREYSCALE').preset('RGB');
   * new Option('--donate [amount]').preset('20').argParser(parseFloat);
   *
   * @param {any} arg
   * @return {Option}
   */

  preset(arg) {
    this.presetArg = arg;
    return this;
  }

  /**
   * Add option name(s) that conflict with this option.
   * An error will be displayed if conflicting options are found during parsing.
   *
   * @example
   * new Option('--rgb').conflicts('cmyk');
   * new Option('--js').conflicts(['ts', 'jsx']);
   *
   * @param {string | string[]} names
   * @return {Option}
   */

  conflicts(names) {
    this.conflictsWith = this.conflictsWith.concat(names);
    return this;
  }

  /**
   * Specify implied option values for when this option is set and the implied options are not.
   *
   * The custom processing (parseArg) is not called on the implied values.
   *
   * @example
   * program
   *   .addOption(new Option('--log', 'write logging information to file'))
   *   .addOption(new Option('--trace', 'log extra details').implies({ log: 'trace.txt' }));
   *
   * @param {Object} impliedOptionValues
   * @return {Option}
   */
  implies(impliedOptionValues) {
    let newImplied = impliedOptionValues;
    if (typeof impliedOptionValues === 'string') {
      // string is not documented, but easy mistake and we can do what user probably intended.
      newImplied = {
        [impliedOptionValues]: true
      };
    }
    this.implied = Object.assign(this.implied || {}, newImplied);
    return this;
  }

  /**
   * Set environment variable to check for option value.
   *
   * An environment variable is only used if when processed the current option value is
   * undefined, or the source of the current value is 'default' or 'config' or 'env'.
   *
   * @param {string} name
   * @return {Option}
   */

  env(name) {
    this.envVar = name;
    return this;
  }

  /**
   * Set the custom handler for processing CLI option arguments into option values.
   *
   * @param {Function} [fn]
   * @return {Option}
   */

  argParser(fn) {
    this.parseArg = fn;
    return this;
  }

  /**
   * Whether the option is mandatory and must have a value after parsing.
   *
   * @param {boolean} [mandatory=true]
   * @return {Option}
   */

  makeOptionMandatory(mandatory = true) {
    this.mandatory = !!mandatory;
    return this;
  }

  /**
   * Hide option in help.
   *
   * @param {boolean} [hide=true]
   * @return {Option}
   */

  hideHelp(hide = true) {
    this.hidden = !!hide;
    return this;
  }

  /**
   * @api private
   */

  _concatValue(value, previous) {
    if (previous === this.defaultValue || !Array.isArray(previous)) {
      return [value];
    }
    return previous.concat(value);
  }

  /**
   * Only allow option value to be one of choices.
   *
   * @param {string[]} values
   * @return {Option}
   */

  choices(values) {
    this.argChoices = values.slice();
    this.parseArg = (arg, previous) => {
      if (!this.argChoices.includes(arg)) {
        throw new InvalidArgumentError$1(`Allowed choices are ${this.argChoices.join(', ')}.`);
      }
      if (this.variadic) {
        return this._concatValue(arg, previous);
      }
      return arg;
    };
    return this;
  }

  /**
   * Return option name.
   *
   * @return {string}
   */

  name() {
    if (this.long) {
      return this.long.replace(/^--/, '');
    }
    return this.short.replace(/^-/, '');
  }

  /**
   * Return option name, in a camelcase format that can be used
   * as a object attribute key.
   *
   * @return {string}
   * @api private
   */

  attributeName() {
    return camelcase(this.name().replace(/^no-/, ''));
  }

  /**
   * Check if `arg` matches the short or long flag.
   *
   * @param {string} arg
   * @return {boolean}
   * @api private
   */

  is(arg) {
    return this.short === arg || this.long === arg;
  }

  /**
   * Return whether a boolean option.
   *
   * Options are one of boolean, negated, required argument, or optional argument.
   *
   * @return {boolean}
   * @api private
   */

  isBoolean() {
    return !this.required && !this.optional && !this.negate;
  }
};

/**
 * This class is to make it easier to work with dual options, without changing the existing
 * implementation. We support separate dual options for separate positive and negative options,
 * like `--build` and `--no-build`, which share a single option value. This works nicely for some
 * use cases, but is tricky for others where we want separate behaviours despite
 * the single shared option value.
 */
let DualOptions$1 = class DualOptions {
  /**
   * @param {Option[]} options
   */
  constructor(options) {
    this.positiveOptions = new Map();
    this.negativeOptions = new Map();
    this.dualOptions = new Set();
    options.forEach(option => {
      if (option.negate) {
        this.negativeOptions.set(option.attributeName(), option);
      } else {
        this.positiveOptions.set(option.attributeName(), option);
      }
    });
    this.negativeOptions.forEach((value, key) => {
      if (this.positiveOptions.has(key)) {
        this.dualOptions.add(key);
      }
    });
  }

  /**
   * Did the value come from the option, and not from possible matching dual option?
   *
   * @param {any} value
   * @param {Option} option
   * @returns {boolean}
   */
  valueFromOption(value, option) {
    const optionKey = option.attributeName();
    if (!this.dualOptions.has(optionKey)) return true;

    // Use the value to deduce if (probably) came from the option.
    const preset = this.negativeOptions.get(optionKey).presetArg;
    const negativeValue = preset !== undefined ? preset : false;
    return option.negate === (negativeValue === value);
  }
};

/**
 * Convert string from kebab-case to camelCase.
 *
 * @param {string} str
 * @return {string}
 * @api private
 */

function camelcase(str) {
  return str.split('-').reduce((str, word) => {
    return str + word[0].toUpperCase() + word.slice(1);
  });
}

/**
 * Split the short and long flag out of something like '-m,--mixed <value>'
 *
 * @api private
 */

function splitOptionFlags$1(flags) {
  let shortFlag;
  let longFlag;
  // Use original very loose parsing to maintain backwards compatibility for now,
  // which allowed for example unintended `-sw, --short-word` [sic].
  const flagParts = flags.split(/[ |,]+/);
  if (flagParts.length > 1 && !/^[[<]/.test(flagParts[1])) shortFlag = flagParts.shift();
  longFlag = flagParts.shift();
  // Add support for lone short flag without significantly changing parsing!
  if (!shortFlag && /^-[^-]$/.test(longFlag)) {
    shortFlag = longFlag;
    longFlag = undefined;
  }
  return {
    shortFlag,
    longFlag
  };
}
option.Option = Option$2;
option.splitOptionFlags = splitOptionFlags$1;
option.DualOptions = DualOptions$1;

var suggestSimilar$2 = {};

const maxDistance = 3;
function editDistance(a, b) {
  // https://en.wikipedia.org/wiki/Damerau–Levenshtein_distance
  // Calculating optimal string alignment distance, no substring is edited more than once.
  // (Simple implementation.)

  // Quick early exit, return worst case.
  if (Math.abs(a.length - b.length) > maxDistance) return Math.max(a.length, b.length);

  // distance between prefix substrings of a and b
  const d = [];

  // pure deletions turn a into empty string
  for (let i = 0; i <= a.length; i++) {
    d[i] = [i];
  }
  // pure insertions turn empty string into b
  for (let j = 0; j <= b.length; j++) {
    d[0][j] = j;
  }

  // fill matrix
  for (let j = 1; j <= b.length; j++) {
    for (let i = 1; i <= a.length; i++) {
      let cost = 1;
      if (a[i - 1] === b[j - 1]) {
        cost = 0;
      } else {
        cost = 1;
      }
      d[i][j] = Math.min(d[i - 1][j] + 1,
      // deletion
      d[i][j - 1] + 1,
      // insertion
      d[i - 1][j - 1] + cost // substitution
      );
      // transposition
      if (i > 1 && j > 1 && a[i - 1] === b[j - 2] && a[i - 2] === b[j - 1]) {
        d[i][j] = Math.min(d[i][j], d[i - 2][j - 2] + 1);
      }
    }
  }
  return d[a.length][b.length];
}

/**
 * Find close matches, restricted to same number of edits.
 *
 * @param {string} word
 * @param {string[]} candidates
 * @returns {string}
 */

function suggestSimilar$1(word, candidates) {
  if (!candidates || candidates.length === 0) return '';
  // remove possible duplicates
  candidates = Array.from(new Set(candidates));
  const searchingOptions = word.startsWith('--');
  if (searchingOptions) {
    word = word.slice(2);
    candidates = candidates.map(candidate => candidate.slice(2));
  }
  let similar = [];
  let bestDistance = maxDistance;
  const minSimilarity = 0.4;
  candidates.forEach(candidate => {
    if (candidate.length <= 1) return; // no one character guesses

    const distance = editDistance(word, candidate);
    const length = Math.max(word.length, candidate.length);
    const similarity = (length - distance) / length;
    if (similarity > minSimilarity) {
      if (distance < bestDistance) {
        // better edit distance, throw away previous worse matches
        bestDistance = distance;
        similar = [candidate];
      } else if (distance === bestDistance) {
        similar.push(candidate);
      }
    }
  });
  similar.sort((a, b) => a.localeCompare(b));
  if (searchingOptions) {
    similar = similar.map(candidate => `--${candidate}`);
  }
  if (similar.length > 1) {
    return `\n(Did you mean one of ${similar.join(', ')}?)`;
  }
  if (similar.length === 1) {
    return `\n(Did you mean ${similar[0]}?)`;
  }
  return '';
}
suggestSimilar$2.suggestSimilar = suggestSimilar$1;

const EventEmitter = require$$2$1.EventEmitter;
const childProcess = require$$1$2;
const path = path$6;
const fs = require$$0;
const process$1 = process__default;
const {
  Argument: Argument$1,
  humanReadableArgName
} = argument;
const {
  CommanderError: CommanderError$1
} = error;
const {
  Help: Help$1
} = help;
const {
  Option: Option$1,
  splitOptionFlags,
  DualOptions
} = option;
const {
  suggestSimilar
} = suggestSimilar$2;

// @ts-check

let Command$1 = class Command extends EventEmitter {
  /**
   * Initialize a new `Command`.
   *
   * @param {string} [name]
   */

  constructor(name) {
    super();
    /** @type {Command[]} */
    this.commands = [];
    /** @type {Option[]} */
    this.options = [];
    this.parent = null;
    this._allowUnknownOption = false;
    this._allowExcessArguments = true;
    /** @type {Argument[]} */
    this._args = [];
    /** @type {string[]} */
    this.args = []; // cli args with options removed
    this.rawArgs = [];
    this.processedArgs = []; // like .args but after custom processing and collecting variadic
    this._scriptPath = null;
    this._name = name || '';
    this._optionValues = {};
    this._optionValueSources = {}; // default, env, cli etc
    this._storeOptionsAsProperties = false;
    this._actionHandler = null;
    this._executableHandler = false;
    this._executableFile = null; // custom name for executable
    this._executableDir = null; // custom search directory for subcommands
    this._defaultCommandName = null;
    this._exitCallback = null;
    this._aliases = [];
    this._combineFlagAndOptionalValue = true;
    this._description = '';
    this._summary = '';
    this._argsDescription = undefined; // legacy
    this._enablePositionalOptions = false;
    this._passThroughOptions = false;
    this._lifeCycleHooks = {}; // a hash of arrays
    /** @type {boolean | string} */
    this._showHelpAfterError = false;
    this._showSuggestionAfterError = true;

    // see .configureOutput() for docs
    this._outputConfiguration = {
      writeOut: str => process$1.stdout.write(str),
      writeErr: str => process$1.stderr.write(str),
      getOutHelpWidth: () => process$1.stdout.isTTY ? process$1.stdout.columns : undefined,
      getErrHelpWidth: () => process$1.stderr.isTTY ? process$1.stderr.columns : undefined,
      outputError: (str, write) => write(str)
    };
    this._hidden = false;
    this._hasHelpOption = true;
    this._helpFlags = '-h, --help';
    this._helpDescription = 'display help for command';
    this._helpShortFlag = '-h';
    this._helpLongFlag = '--help';
    this._addImplicitHelpCommand = undefined; // Deliberately undefined, not decided whether true or false
    this._helpCommandName = 'help';
    this._helpCommandnameAndArgs = 'help [command]';
    this._helpCommandDescription = 'display help for command';
    this._helpConfiguration = {};
  }

  /**
   * Copy settings that are useful to have in common across root command and subcommands.
   *
   * (Used internally when adding a command using `.command()` so subcommands inherit parent settings.)
   *
   * @param {Command} sourceCommand
   * @return {Command} `this` command for chaining
   */
  copyInheritedSettings(sourceCommand) {
    this._outputConfiguration = sourceCommand._outputConfiguration;
    this._hasHelpOption = sourceCommand._hasHelpOption;
    this._helpFlags = sourceCommand._helpFlags;
    this._helpDescription = sourceCommand._helpDescription;
    this._helpShortFlag = sourceCommand._helpShortFlag;
    this._helpLongFlag = sourceCommand._helpLongFlag;
    this._helpCommandName = sourceCommand._helpCommandName;
    this._helpCommandnameAndArgs = sourceCommand._helpCommandnameAndArgs;
    this._helpCommandDescription = sourceCommand._helpCommandDescription;
    this._helpConfiguration = sourceCommand._helpConfiguration;
    this._exitCallback = sourceCommand._exitCallback;
    this._storeOptionsAsProperties = sourceCommand._storeOptionsAsProperties;
    this._combineFlagAndOptionalValue = sourceCommand._combineFlagAndOptionalValue;
    this._allowExcessArguments = sourceCommand._allowExcessArguments;
    this._enablePositionalOptions = sourceCommand._enablePositionalOptions;
    this._showHelpAfterError = sourceCommand._showHelpAfterError;
    this._showSuggestionAfterError = sourceCommand._showSuggestionAfterError;
    return this;
  }

  /**
   * Define a command.
   *
   * There are two styles of command: pay attention to where to put the description.
   *
   * @example
   * // Command implemented using action handler (description is supplied separately to `.command`)
   * program
   *   .command('clone <source> [destination]')
   *   .description('clone a repository into a newly created directory')
   *   .action((source, destination) => {
   *     console.log('clone command called');
   *   });
   *
   * // Command implemented using separate executable file (description is second parameter to `.command`)
   * program
   *   .command('start <service>', 'start named service')
   *   .command('stop [service]', 'stop named service, or all if no name supplied');
   *
   * @param {string} nameAndArgs - command name and arguments, args are `<required>` or `[optional]` and last may also be `variadic...`
   * @param {Object|string} [actionOptsOrExecDesc] - configuration options (for action), or description (for executable)
   * @param {Object} [execOpts] - configuration options (for executable)
   * @return {Command} returns new command for action handler, or `this` for executable command
   */

  command(nameAndArgs, actionOptsOrExecDesc, execOpts) {
    let desc = actionOptsOrExecDesc;
    let opts = execOpts;
    if (typeof desc === 'object' && desc !== null) {
      opts = desc;
      desc = null;
    }
    opts = opts || {};
    const [, name, args] = nameAndArgs.match(/([^ ]+) *(.*)/);
    const cmd = this.createCommand(name);
    if (desc) {
      cmd.description(desc);
      cmd._executableHandler = true;
    }
    if (opts.isDefault) this._defaultCommandName = cmd._name;
    cmd._hidden = !!(opts.noHelp || opts.hidden); // noHelp is deprecated old name for hidden
    cmd._executableFile = opts.executableFile || null; // Custom name for executable file, set missing to null to match constructor
    if (args) cmd.arguments(args);
    this.commands.push(cmd);
    cmd.parent = this;
    cmd.copyInheritedSettings(this);
    if (desc) return this;
    return cmd;
  }

  /**
   * Factory routine to create a new unattached command.
   *
   * See .command() for creating an attached subcommand, which uses this routine to
   * create the command. You can override createCommand to customise subcommands.
   *
   * @param {string} [name]
   * @return {Command} new command
   */

  createCommand(name) {
    return new Command(name);
  }

  /**
   * You can customise the help with a subclass of Help by overriding createHelp,
   * or by overriding Help properties using configureHelp().
   *
   * @return {Help}
   */

  createHelp() {
    return Object.assign(new Help$1(), this.configureHelp());
  }

  /**
   * You can customise the help by overriding Help properties using configureHelp(),
   * or with a subclass of Help by overriding createHelp().
   *
   * @param {Object} [configuration] - configuration options
   * @return {Command|Object} `this` command for chaining, or stored configuration
   */

  configureHelp(configuration) {
    if (configuration === undefined) return this._helpConfiguration;
    this._helpConfiguration = configuration;
    return this;
  }

  /**
   * The default output goes to stdout and stderr. You can customise this for special
   * applications. You can also customise the display of errors by overriding outputError.
   *
   * The configuration properties are all functions:
   *
   *     // functions to change where being written, stdout and stderr
   *     writeOut(str)
   *     writeErr(str)
   *     // matching functions to specify width for wrapping help
   *     getOutHelpWidth()
   *     getErrHelpWidth()
   *     // functions based on what is being written out
   *     outputError(str, write) // used for displaying errors, and not used for displaying help
   *
   * @param {Object} [configuration] - configuration options
   * @return {Command|Object} `this` command for chaining, or stored configuration
   */

  configureOutput(configuration) {
    if (configuration === undefined) return this._outputConfiguration;
    Object.assign(this._outputConfiguration, configuration);
    return this;
  }

  /**
   * Display the help or a custom message after an error occurs.
   *
   * @param {boolean|string} [displayHelp]
   * @return {Command} `this` command for chaining
   */
  showHelpAfterError(displayHelp = true) {
    if (typeof displayHelp !== 'string') displayHelp = !!displayHelp;
    this._showHelpAfterError = displayHelp;
    return this;
  }

  /**
   * Display suggestion of similar commands for unknown commands, or options for unknown options.
   *
   * @param {boolean} [displaySuggestion]
   * @return {Command} `this` command for chaining
   */
  showSuggestionAfterError(displaySuggestion = true) {
    this._showSuggestionAfterError = !!displaySuggestion;
    return this;
  }

  /**
   * Add a prepared subcommand.
   *
   * See .command() for creating an attached subcommand which inherits settings from its parent.
   *
   * @param {Command} cmd - new subcommand
   * @param {Object} [opts] - configuration options
   * @return {Command} `this` command for chaining
   */

  addCommand(cmd, opts) {
    if (!cmd._name) {
      throw new Error(`Command passed to .addCommand() must have a name
- specify the name in Command constructor or using .name()`);
    }
    opts = opts || {};
    if (opts.isDefault) this._defaultCommandName = cmd._name;
    if (opts.noHelp || opts.hidden) cmd._hidden = true; // modifying passed command due to existing implementation

    this.commands.push(cmd);
    cmd.parent = this;
    return this;
  }

  /**
   * Factory routine to create a new unattached argument.
   *
   * See .argument() for creating an attached argument, which uses this routine to
   * create the argument. You can override createArgument to return a custom argument.
   *
   * @param {string} name
   * @param {string} [description]
   * @return {Argument} new argument
   */

  createArgument(name, description) {
    return new Argument$1(name, description);
  }

  /**
   * Define argument syntax for command.
   *
   * The default is that the argument is required, and you can explicitly
   * indicate this with <> around the name. Put [] around the name for an optional argument.
   *
   * @example
   * program.argument('<input-file>');
   * program.argument('[output-file]');
   *
   * @param {string} name
   * @param {string} [description]
   * @param {Function|*} [fn] - custom argument processing function
   * @param {*} [defaultValue]
   * @return {Command} `this` command for chaining
   */
  argument(name, description, fn, defaultValue) {
    const argument = this.createArgument(name, description);
    if (typeof fn === 'function') {
      argument.default(defaultValue).argParser(fn);
    } else {
      argument.default(fn);
    }
    this.addArgument(argument);
    return this;
  }

  /**
   * Define argument syntax for command, adding multiple at once (without descriptions).
   *
   * See also .argument().
   *
   * @example
   * program.arguments('<cmd> [env]');
   *
   * @param {string} names
   * @return {Command} `this` command for chaining
   */

  arguments(names) {
    names.trim().split(/ +/).forEach(detail => {
      this.argument(detail);
    });
    return this;
  }

  /**
   * Define argument syntax for command, adding a prepared argument.
   *
   * @param {Argument} argument
   * @return {Command} `this` command for chaining
   */
  addArgument(argument) {
    const previousArgument = this._args.slice(-1)[0];
    if (previousArgument && previousArgument.variadic) {
      throw new Error(`only the last argument can be variadic '${previousArgument.name()}'`);
    }
    if (argument.required && argument.defaultValue !== undefined && argument.parseArg === undefined) {
      throw new Error(`a default value for a required argument is never used: '${argument.name()}'`);
    }
    this._args.push(argument);
    return this;
  }

  /**
   * Override default decision whether to add implicit help command.
   *
   *    addHelpCommand() // force on
   *    addHelpCommand(false); // force off
   *    addHelpCommand('help [cmd]', 'display help for [cmd]'); // force on with custom details
   *
   * @return {Command} `this` command for chaining
   */

  addHelpCommand(enableOrNameAndArgs, description) {
    if (enableOrNameAndArgs === false) {
      this._addImplicitHelpCommand = false;
    } else {
      this._addImplicitHelpCommand = true;
      if (typeof enableOrNameAndArgs === 'string') {
        this._helpCommandName = enableOrNameAndArgs.split(' ')[0];
        this._helpCommandnameAndArgs = enableOrNameAndArgs;
      }
      this._helpCommandDescription = description || this._helpCommandDescription;
    }
    return this;
  }

  /**
   * @return {boolean}
   * @api private
   */

  _hasImplicitHelpCommand() {
    if (this._addImplicitHelpCommand === undefined) {
      return this.commands.length && !this._actionHandler && !this._findCommand('help');
    }
    return this._addImplicitHelpCommand;
  }

  /**
   * Add hook for life cycle event.
   *
   * @param {string} event
   * @param {Function} listener
   * @return {Command} `this` command for chaining
   */

  hook(event, listener) {
    const allowedValues = ['preSubcommand', 'preAction', 'postAction'];
    if (!allowedValues.includes(event)) {
      throw new Error(`Unexpected value for event passed to hook : '${event}'.
Expecting one of '${allowedValues.join("', '")}'`);
    }
    if (this._lifeCycleHooks[event]) {
      this._lifeCycleHooks[event].push(listener);
    } else {
      this._lifeCycleHooks[event] = [listener];
    }
    return this;
  }

  /**
   * Register callback to use as replacement for calling process.exit.
   *
   * @param {Function} [fn] optional callback which will be passed a CommanderError, defaults to throwing
   * @return {Command} `this` command for chaining
   */

  exitOverride(fn) {
    if (fn) {
      this._exitCallback = fn;
    } else {
      this._exitCallback = err => {
        if (err.code !== 'commander.executeSubCommandAsync') {
          throw err;
        }
      };
    }
    return this;
  }

  /**
   * Call process.exit, and _exitCallback if defined.
   *
   * @param {number} exitCode exit code for using with process.exit
   * @param {string} code an id string representing the error
   * @param {string} message human-readable description of the error
   * @return never
   * @api private
   */

  _exit(exitCode, code, message) {
    if (this._exitCallback) {
      this._exitCallback(new CommanderError$1(exitCode, code, message));
      // Expecting this line is not reached.
    }

    process$1.exit(exitCode);
  }

  /**
   * Register callback `fn` for the command.
   *
   * @example
   * program
   *   .command('serve')
   *   .description('start service')
   *   .action(function() {
   *      // do work here
   *   });
   *
   * @param {Function} fn
   * @return {Command} `this` command for chaining
   */

  action(fn) {
    const listener = args => {
      // The .action callback takes an extra parameter which is the command or options.
      const expectedArgsCount = this._args.length;
      const actionArgs = args.slice(0, expectedArgsCount);
      if (this._storeOptionsAsProperties) {
        actionArgs[expectedArgsCount] = this; // backwards compatible "options"
      } else {
        actionArgs[expectedArgsCount] = this.opts();
      }
      actionArgs.push(this);
      return fn.apply(this, actionArgs);
    };
    this._actionHandler = listener;
    return this;
  }

  /**
   * Factory routine to create a new unattached option.
   *
   * See .option() for creating an attached option, which uses this routine to
   * create the option. You can override createOption to return a custom option.
   *
   * @param {string} flags
   * @param {string} [description]
   * @return {Option} new option
   */

  createOption(flags, description) {
    return new Option$1(flags, description);
  }

  /**
   * Add an option.
   *
   * @param {Option} option
   * @return {Command} `this` command for chaining
   */
  addOption(option) {
    const oname = option.name();
    const name = option.attributeName();

    // store default value
    if (option.negate) {
      // --no-foo is special and defaults foo to true, unless a --foo option is already defined
      const positiveLongFlag = option.long.replace(/^--no-/, '--');
      if (!this._findOption(positiveLongFlag)) {
        this.setOptionValueWithSource(name, option.defaultValue === undefined ? true : option.defaultValue, 'default');
      }
    } else if (option.defaultValue !== undefined) {
      this.setOptionValueWithSource(name, option.defaultValue, 'default');
    }

    // register the option
    this.options.push(option);

    // handler for cli and env supplied values
    const handleOptionValue = (val, invalidValueMessage, valueSource) => {
      // val is null for optional option used without an optional-argument.
      // val is undefined for boolean and negated option.
      if (val == null && option.presetArg !== undefined) {
        val = option.presetArg;
      }

      // custom processing
      const oldValue = this.getOptionValue(name);
      if (val !== null && option.parseArg) {
        try {
          val = option.parseArg(val, oldValue);
        } catch (err) {
          if (err.code === 'commander.invalidArgument') {
            const message = `${invalidValueMessage} ${err.message}`;
            this.error(message, {
              exitCode: err.exitCode,
              code: err.code
            });
          }
          throw err;
        }
      } else if (val !== null && option.variadic) {
        val = option._concatValue(val, oldValue);
      }

      // Fill-in appropriate missing values. Long winded but easy to follow.
      if (val == null) {
        if (option.negate) {
          val = false;
        } else if (option.isBoolean() || option.optional) {
          val = true;
        } else {
          val = ''; // not normal, parseArg might have failed or be a mock function for testing
        }
      }

      this.setOptionValueWithSource(name, val, valueSource);
    };
    this.on('option:' + oname, val => {
      const invalidValueMessage = `error: option '${option.flags}' argument '${val}' is invalid.`;
      handleOptionValue(val, invalidValueMessage, 'cli');
    });
    if (option.envVar) {
      this.on('optionEnv:' + oname, val => {
        const invalidValueMessage = `error: option '${option.flags}' value '${val}' from env '${option.envVar}' is invalid.`;
        handleOptionValue(val, invalidValueMessage, 'env');
      });
    }
    return this;
  }

  /**
   * Internal implementation shared by .option() and .requiredOption()
   *
   * @api private
   */
  _optionEx(config, flags, description, fn, defaultValue) {
    if (typeof flags === 'object' && flags instanceof Option$1) {
      throw new Error('To add an Option object use addOption() instead of option() or requiredOption()');
    }
    const option = this.createOption(flags, description);
    option.makeOptionMandatory(!!config.mandatory);
    if (typeof fn === 'function') {
      option.default(defaultValue).argParser(fn);
    } else if (fn instanceof RegExp) {
      // deprecated
      const regex = fn;
      fn = (val, def) => {
        const m = regex.exec(val);
        return m ? m[0] : def;
      };
      option.default(defaultValue).argParser(fn);
    } else {
      option.default(fn);
    }
    return this.addOption(option);
  }

  /**
   * Define option with `flags`, `description` and optional
   * coercion `fn`.
   *
   * The `flags` string contains the short and/or long flags,
   * separated by comma, a pipe or space. The following are all valid
   * all will output this way when `--help` is used.
   *
   *     "-p, --pepper"
   *     "-p|--pepper"
   *     "-p --pepper"
   *
   * @example
   * // simple boolean defaulting to undefined
   * program.option('-p, --pepper', 'add pepper');
   *
   * program.pepper
   * // => undefined
   *
   * --pepper
   * program.pepper
   * // => true
   *
   * // simple boolean defaulting to true (unless non-negated option is also defined)
   * program.option('-C, --no-cheese', 'remove cheese');
   *
   * program.cheese
   * // => true
   *
   * --no-cheese
   * program.cheese
   * // => false
   *
   * // required argument
   * program.option('-C, --chdir <path>', 'change the working directory');
   *
   * --chdir /tmp
   * program.chdir
   * // => "/tmp"
   *
   * // optional argument
   * program.option('-c, --cheese [type]', 'add cheese [marble]');
   *
   * @param {string} flags
   * @param {string} [description]
   * @param {Function|*} [fn] - custom option processing function or default value
   * @param {*} [defaultValue]
   * @return {Command} `this` command for chaining
   */

  option(flags, description, fn, defaultValue) {
    return this._optionEx({}, flags, description, fn, defaultValue);
  }

  /**
  * Add a required option which must have a value after parsing. This usually means
  * the option must be specified on the command line. (Otherwise the same as .option().)
  *
  * The `flags` string contains the short and/or long flags, separated by comma, a pipe or space.
  *
  * @param {string} flags
  * @param {string} [description]
  * @param {Function|*} [fn] - custom option processing function or default value
  * @param {*} [defaultValue]
  * @return {Command} `this` command for chaining
  */

  requiredOption(flags, description, fn, defaultValue) {
    return this._optionEx({
      mandatory: true
    }, flags, description, fn, defaultValue);
  }

  /**
   * Alter parsing of short flags with optional values.
   *
   * @example
   * // for `.option('-f,--flag [value]'):
   * program.combineFlagAndOptionalValue(true);  // `-f80` is treated like `--flag=80`, this is the default behaviour
   * program.combineFlagAndOptionalValue(false) // `-fb` is treated like `-f -b`
   *
   * @param {Boolean} [combine=true] - if `true` or omitted, an optional value can be specified directly after the flag.
   */
  combineFlagAndOptionalValue(combine = true) {
    this._combineFlagAndOptionalValue = !!combine;
    return this;
  }

  /**
   * Allow unknown options on the command line.
   *
   * @param {Boolean} [allowUnknown=true] - if `true` or omitted, no error will be thrown
   * for unknown options.
   */
  allowUnknownOption(allowUnknown = true) {
    this._allowUnknownOption = !!allowUnknown;
    return this;
  }

  /**
   * Allow excess command-arguments on the command line. Pass false to make excess arguments an error.
   *
   * @param {Boolean} [allowExcess=true] - if `true` or omitted, no error will be thrown
   * for excess arguments.
   */
  allowExcessArguments(allowExcess = true) {
    this._allowExcessArguments = !!allowExcess;
    return this;
  }

  /**
   * Enable positional options. Positional means global options are specified before subcommands which lets
   * subcommands reuse the same option names, and also enables subcommands to turn on passThroughOptions.
   * The default behaviour is non-positional and global options may appear anywhere on the command line.
   *
   * @param {Boolean} [positional=true]
   */
  enablePositionalOptions(positional = true) {
    this._enablePositionalOptions = !!positional;
    return this;
  }

  /**
   * Pass through options that come after command-arguments rather than treat them as command-options,
   * so actual command-options come before command-arguments. Turning this on for a subcommand requires
   * positional options to have been enabled on the program (parent commands).
   * The default behaviour is non-positional and options may appear before or after command-arguments.
   *
   * @param {Boolean} [passThrough=true]
   * for unknown options.
   */
  passThroughOptions(passThrough = true) {
    this._passThroughOptions = !!passThrough;
    if (!!this.parent && passThrough && !this.parent._enablePositionalOptions) {
      throw new Error('passThroughOptions can not be used without turning on enablePositionalOptions for parent command(s)');
    }
    return this;
  }

  /**
    * Whether to store option values as properties on command object,
    * or store separately (specify false). In both cases the option values can be accessed using .opts().
    *
    * @param {boolean} [storeAsProperties=true]
    * @return {Command} `this` command for chaining
    */

  storeOptionsAsProperties(storeAsProperties = true) {
    this._storeOptionsAsProperties = !!storeAsProperties;
    if (this.options.length) {
      throw new Error('call .storeOptionsAsProperties() before adding options');
    }
    return this;
  }

  /**
   * Retrieve option value.
   *
   * @param {string} key
   * @return {Object} value
   */

  getOptionValue(key) {
    if (this._storeOptionsAsProperties) {
      return this[key];
    }
    return this._optionValues[key];
  }

  /**
   * Store option value.
   *
   * @param {string} key
   * @param {Object} value
   * @return {Command} `this` command for chaining
   */

  setOptionValue(key, value) {
    return this.setOptionValueWithSource(key, value, undefined);
  }

  /**
    * Store option value and where the value came from.
    *
    * @param {string} key
    * @param {Object} value
    * @param {string} source - expected values are default/config/env/cli/implied
    * @return {Command} `this` command for chaining
    */

  setOptionValueWithSource(key, value, source) {
    if (this._storeOptionsAsProperties) {
      this[key] = value;
    } else {
      this._optionValues[key] = value;
    }
    this._optionValueSources[key] = source;
    return this;
  }

  /**
    * Get source of option value.
    * Expected values are default | config | env | cli | implied
    *
    * @param {string} key
    * @return {string}
    */

  getOptionValueSource(key) {
    return this._optionValueSources[key];
  }

  /**
    * Get source of option value. See also .optsWithGlobals().
    * Expected values are default | config | env | cli | implied
    *
    * @param {string} key
    * @return {string}
    */

  getOptionValueSourceWithGlobals(key) {
    // global overwrites local, like optsWithGlobals
    let source;
    getCommandAndParents(this).forEach(cmd => {
      if (cmd.getOptionValueSource(key) !== undefined) {
        source = cmd.getOptionValueSource(key);
      }
    });
    return source;
  }

  /**
   * Get user arguments from implied or explicit arguments.
   * Side-effects: set _scriptPath if args included script. Used for default program name, and subcommand searches.
   *
   * @api private
   */

  _prepareUserArgs(argv, parseOptions) {
    if (argv !== undefined && !Array.isArray(argv)) {
      throw new Error('first parameter to parse must be array or undefined');
    }
    parseOptions = parseOptions || {};

    // Default to using process.argv
    if (argv === undefined) {
      argv = process$1.argv;
      // @ts-ignore: unknown property
      if (process$1.versions && process$1.versions.electron) {
        parseOptions.from = 'electron';
      }
    }
    this.rawArgs = argv.slice();

    // make it a little easier for callers by supporting various argv conventions
    let userArgs;
    switch (parseOptions.from) {
      case undefined:
      case 'node':
        this._scriptPath = argv[1];
        userArgs = argv.slice(2);
        break;
      case 'electron':
        // @ts-ignore: unknown property
        if (process$1.defaultApp) {
          this._scriptPath = argv[1];
          userArgs = argv.slice(2);
        } else {
          userArgs = argv.slice(1);
        }
        break;
      case 'user':
        userArgs = argv.slice(0);
        break;
      default:
        throw new Error(`unexpected parse option { from: '${parseOptions.from}' }`);
    }

    // Find default name for program from arguments.
    if (!this._name && this._scriptPath) this.nameFromFilename(this._scriptPath);
    this._name = this._name || 'program';
    return userArgs;
  }

  /**
   * Parse `argv`, setting options and invoking commands when defined.
   *
   * The default expectation is that the arguments are from node and have the application as argv[0]
   * and the script being run in argv[1], with user parameters after that.
   *
   * @example
   * program.parse(process.argv);
   * program.parse(); // implicitly use process.argv and auto-detect node vs electron conventions
   * program.parse(my-args, { from: 'user' }); // just user supplied arguments, nothing special about argv[0]
   *
   * @param {string[]} [argv] - optional, defaults to process.argv
   * @param {Object} [parseOptions] - optionally specify style of options with from: node/user/electron
   * @param {string} [parseOptions.from] - where the args are from: 'node', 'user', 'electron'
   * @return {Command} `this` command for chaining
   */

  parse(argv, parseOptions) {
    const userArgs = this._prepareUserArgs(argv, parseOptions);
    this._parseCommand([], userArgs);
    return this;
  }

  /**
   * Parse `argv`, setting options and invoking commands when defined.
   *
   * Use parseAsync instead of parse if any of your action handlers are async. Returns a Promise.
   *
   * The default expectation is that the arguments are from node and have the application as argv[0]
   * and the script being run in argv[1], with user parameters after that.
   *
   * @example
   * await program.parseAsync(process.argv);
   * await program.parseAsync(); // implicitly use process.argv and auto-detect node vs electron conventions
   * await program.parseAsync(my-args, { from: 'user' }); // just user supplied arguments, nothing special about argv[0]
   *
   * @param {string[]} [argv]
   * @param {Object} [parseOptions]
   * @param {string} parseOptions.from - where the args are from: 'node', 'user', 'electron'
   * @return {Promise}
   */

  async parseAsync(argv, parseOptions) {
    const userArgs = this._prepareUserArgs(argv, parseOptions);
    await this._parseCommand([], userArgs);
    return this;
  }

  /**
   * Execute a sub-command executable.
   *
   * @api private
   */

  _executeSubCommand(subcommand, args) {
    args = args.slice();
    let launchWithNode = false; // Use node for source targets so do not need to get permissions correct, and on Windows.
    const sourceExt = ['.js', '.ts', '.tsx', '.mjs', '.cjs'];
    function findFile(baseDir, baseName) {
      // Look for specified file
      const localBin = path.resolve(baseDir, baseName);
      if (fs.existsSync(localBin)) return localBin;

      // Stop looking if candidate already has an expected extension.
      if (sourceExt.includes(path.extname(baseName))) return undefined;

      // Try all the extensions.
      const foundExt = sourceExt.find(ext => fs.existsSync(`${localBin}${ext}`));
      if (foundExt) return `${localBin}${foundExt}`;
      return undefined;
    }

    // Not checking for help first. Unlikely to have mandatory and executable, and can't robustly test for help flags in external command.
    this._checkForMissingMandatoryOptions();
    this._checkForConflictingOptions();

    // executableFile and executableDir might be full path, or just a name
    let executableFile = subcommand._executableFile || `${this._name}-${subcommand._name}`;
    let executableDir = this._executableDir || '';
    if (this._scriptPath) {
      let resolvedScriptPath; // resolve possible symlink for installed npm binary
      try {
        resolvedScriptPath = fs.realpathSync(this._scriptPath);
      } catch (err) {
        resolvedScriptPath = this._scriptPath;
      }
      executableDir = path.resolve(path.dirname(resolvedScriptPath), executableDir);
    }

    // Look for a local file in preference to a command in PATH.
    if (executableDir) {
      let localFile = findFile(executableDir, executableFile);

      // Legacy search using prefix of script name instead of command name
      if (!localFile && !subcommand._executableFile && this._scriptPath) {
        const legacyName = path.basename(this._scriptPath, path.extname(this._scriptPath));
        if (legacyName !== this._name) {
          localFile = findFile(executableDir, `${legacyName}-${subcommand._name}`);
        }
      }
      executableFile = localFile || executableFile;
    }
    launchWithNode = sourceExt.includes(path.extname(executableFile));
    let proc;
    if (process$1.platform !== 'win32') {
      if (launchWithNode) {
        args.unshift(executableFile);
        // add executable arguments to spawn
        args = incrementNodeInspectorPort(process$1.execArgv).concat(args);
        proc = childProcess.spawn(process$1.argv[0], args, {
          stdio: 'inherit'
        });
      } else {
        proc = childProcess.spawn(executableFile, args, {
          stdio: 'inherit'
        });
      }
    } else {
      args.unshift(executableFile);
      // add executable arguments to spawn
      args = incrementNodeInspectorPort(process$1.execArgv).concat(args);
      proc = childProcess.spawn(process$1.execPath, args, {
        stdio: 'inherit'
      });
    }
    if (!proc.killed) {
      // testing mainly to avoid leak warnings during unit tests with mocked spawn
      const signals = ['SIGUSR1', 'SIGUSR2', 'SIGTERM', 'SIGINT', 'SIGHUP'];
      signals.forEach(signal => {
        // @ts-ignore
        process$1.on(signal, () => {
          if (proc.killed === false && proc.exitCode === null) {
            proc.kill(signal);
          }
        });
      });
    }

    // By default terminate process when spawned process terminates.
    // Suppressing the exit if exitCallback defined is a bit messy and of limited use, but does allow process to stay running!
    const exitCallback = this._exitCallback;
    if (!exitCallback) {
      proc.on('close', process$1.exit.bind(process$1));
    } else {
      proc.on('close', () => {
        exitCallback(new CommanderError$1(process$1.exitCode || 0, 'commander.executeSubCommandAsync', '(close)'));
      });
    }
    proc.on('error', err => {
      // @ts-ignore
      if (err.code === 'ENOENT') {
        const executableDirMessage = executableDir ? `searched for local subcommand relative to directory '${executableDir}'` : 'no directory for search for local subcommand, use .executableDir() to supply a custom directory';
        const executableMissing = `'${executableFile}' does not exist
 - if '${subcommand._name}' is not meant to be an executable command, remove description parameter from '.command()' and use '.description()' instead
 - if the default executable name is not suitable, use the executableFile option to supply a custom name or path
 - ${executableDirMessage}`;
        throw new Error(executableMissing);
        // @ts-ignore
      } else if (err.code === 'EACCES') {
        throw new Error(`'${executableFile}' not executable`);
      }
      if (!exitCallback) {
        process$1.exit(1);
      } else {
        const wrappedError = new CommanderError$1(1, 'commander.executeSubCommandAsync', '(error)');
        wrappedError.nestedError = err;
        exitCallback(wrappedError);
      }
    });

    // Store the reference to the child process
    this.runningCommand = proc;
  }

  /**
   * @api private
   */

  _dispatchSubcommand(commandName, operands, unknown) {
    const subCommand = this._findCommand(commandName);
    if (!subCommand) this.help({
      error: true
    });
    let hookResult;
    hookResult = this._chainOrCallSubCommandHook(hookResult, subCommand, 'preSubcommand');
    hookResult = this._chainOrCall(hookResult, () => {
      if (subCommand._executableHandler) {
        this._executeSubCommand(subCommand, operands.concat(unknown));
      } else {
        return subCommand._parseCommand(operands, unknown);
      }
    });
    return hookResult;
  }

  /**
   * Invoke help directly if possible, or dispatch if necessary.
   * e.g. help foo
   *
   * @api private
   */

  _dispatchHelpCommand(subcommandName) {
    if (!subcommandName) {
      this.help();
    }
    const subCommand = this._findCommand(subcommandName);
    if (subCommand && !subCommand._executableHandler) {
      subCommand.help();
    }

    // Fallback to parsing the help flag to invoke the help.
    return this._dispatchSubcommand(subcommandName, [], [this._helpLongFlag]);
  }

  /**
   * Check this.args against expected this._args.
   *
   * @api private
   */

  _checkNumberOfArguments() {
    // too few
    this._args.forEach((arg, i) => {
      if (arg.required && this.args[i] == null) {
        this.missingArgument(arg.name());
      }
    });
    // too many
    if (this._args.length > 0 && this._args[this._args.length - 1].variadic) {
      return;
    }
    if (this.args.length > this._args.length) {
      this._excessArguments(this.args);
    }
  }

  /**
   * Process this.args using this._args and save as this.processedArgs!
   *
   * @api private
   */

  _processArguments() {
    const myParseArg = (argument, value, previous) => {
      // Extra processing for nice error message on parsing failure.
      let parsedValue = value;
      if (value !== null && argument.parseArg) {
        try {
          parsedValue = argument.parseArg(value, previous);
        } catch (err) {
          if (err.code === 'commander.invalidArgument') {
            const message = `error: command-argument value '${value}' is invalid for argument '${argument.name()}'. ${err.message}`;
            this.error(message, {
              exitCode: err.exitCode,
              code: err.code
            });
          }
          throw err;
        }
      }
      return parsedValue;
    };
    this._checkNumberOfArguments();
    const processedArgs = [];
    this._args.forEach((declaredArg, index) => {
      let value = declaredArg.defaultValue;
      if (declaredArg.variadic) {
        // Collect together remaining arguments for passing together as an array.
        if (index < this.args.length) {
          value = this.args.slice(index);
          if (declaredArg.parseArg) {
            value = value.reduce((processed, v) => {
              return myParseArg(declaredArg, v, processed);
            }, declaredArg.defaultValue);
          }
        } else if (value === undefined) {
          value = [];
        }
      } else if (index < this.args.length) {
        value = this.args[index];
        if (declaredArg.parseArg) {
          value = myParseArg(declaredArg, value, declaredArg.defaultValue);
        }
      }
      processedArgs[index] = value;
    });
    this.processedArgs = processedArgs;
  }

  /**
   * Once we have a promise we chain, but call synchronously until then.
   *
   * @param {Promise|undefined} promise
   * @param {Function} fn
   * @return {Promise|undefined}
   * @api private
   */

  _chainOrCall(promise, fn) {
    // thenable
    if (promise && promise.then && typeof promise.then === 'function') {
      // already have a promise, chain callback
      return promise.then(() => fn());
    }
    // callback might return a promise
    return fn();
  }

  /**
   *
   * @param {Promise|undefined} promise
   * @param {string} event
   * @return {Promise|undefined}
   * @api private
   */

  _chainOrCallHooks(promise, event) {
    let result = promise;
    const hooks = [];
    getCommandAndParents(this).reverse().filter(cmd => cmd._lifeCycleHooks[event] !== undefined).forEach(hookedCommand => {
      hookedCommand._lifeCycleHooks[event].forEach(callback => {
        hooks.push({
          hookedCommand,
          callback
        });
      });
    });
    if (event === 'postAction') {
      hooks.reverse();
    }
    hooks.forEach(hookDetail => {
      result = this._chainOrCall(result, () => {
        return hookDetail.callback(hookDetail.hookedCommand, this);
      });
    });
    return result;
  }

  /**
   *
   * @param {Promise|undefined} promise
   * @param {Command} subCommand
   * @param {string} event
   * @return {Promise|undefined}
   * @api private
   */

  _chainOrCallSubCommandHook(promise, subCommand, event) {
    let result = promise;
    if (this._lifeCycleHooks[event] !== undefined) {
      this._lifeCycleHooks[event].forEach(hook => {
        result = this._chainOrCall(result, () => {
          return hook(this, subCommand);
        });
      });
    }
    return result;
  }

  /**
   * Process arguments in context of this command.
   * Returns action result, in case it is a promise.
   *
   * @api private
   */

  _parseCommand(operands, unknown) {
    const parsed = this.parseOptions(unknown);
    this._parseOptionsEnv(); // after cli, so parseArg not called on both cli and env
    this._parseOptionsImplied();
    operands = operands.concat(parsed.operands);
    unknown = parsed.unknown;
    this.args = operands.concat(unknown);
    if (operands && this._findCommand(operands[0])) {
      return this._dispatchSubcommand(operands[0], operands.slice(1), unknown);
    }
    if (this._hasImplicitHelpCommand() && operands[0] === this._helpCommandName) {
      return this._dispatchHelpCommand(operands[1]);
    }
    if (this._defaultCommandName) {
      outputHelpIfRequested(this, unknown); // Run the help for default command from parent rather than passing to default command
      return this._dispatchSubcommand(this._defaultCommandName, operands, unknown);
    }
    if (this.commands.length && this.args.length === 0 && !this._actionHandler && !this._defaultCommandName) {
      // probably missing subcommand and no handler, user needs help (and exit)
      this.help({
        error: true
      });
    }
    outputHelpIfRequested(this, parsed.unknown);
    this._checkForMissingMandatoryOptions();
    this._checkForConflictingOptions();

    // We do not always call this check to avoid masking a "better" error, like unknown command.
    const checkForUnknownOptions = () => {
      if (parsed.unknown.length > 0) {
        this.unknownOption(parsed.unknown[0]);
      }
    };
    const commandEvent = `command:${this.name()}`;
    if (this._actionHandler) {
      checkForUnknownOptions();
      this._processArguments();
      let actionResult;
      actionResult = this._chainOrCallHooks(actionResult, 'preAction');
      actionResult = this._chainOrCall(actionResult, () => this._actionHandler(this.processedArgs));
      if (this.parent) {
        actionResult = this._chainOrCall(actionResult, () => {
          this.parent.emit(commandEvent, operands, unknown); // legacy
        });
      }

      actionResult = this._chainOrCallHooks(actionResult, 'postAction');
      return actionResult;
    }
    if (this.parent && this.parent.listenerCount(commandEvent)) {
      checkForUnknownOptions();
      this._processArguments();
      this.parent.emit(commandEvent, operands, unknown); // legacy
    } else if (operands.length) {
      if (this._findCommand('*')) {
        // legacy default command
        return this._dispatchSubcommand('*', operands, unknown);
      }
      if (this.listenerCount('command:*')) {
        // skip option check, emit event for possible misspelling suggestion
        this.emit('command:*', operands, unknown);
      } else if (this.commands.length) {
        this.unknownCommand();
      } else {
        checkForUnknownOptions();
        this._processArguments();
      }
    } else if (this.commands.length) {
      checkForUnknownOptions();
      // This command has subcommands and nothing hooked up at this level, so display help (and exit).
      this.help({
        error: true
      });
    } else {
      checkForUnknownOptions();
      this._processArguments();
      // fall through for caller to handle after calling .parse()
    }
  }

  /**
   * Find matching command.
   *
   * @api private
   */
  _findCommand(name) {
    if (!name) return undefined;
    return this.commands.find(cmd => cmd._name === name || cmd._aliases.includes(name));
  }

  /**
   * Return an option matching `arg` if any.
   *
   * @param {string} arg
   * @return {Option}
   * @api private
   */

  _findOption(arg) {
    return this.options.find(option => option.is(arg));
  }

  /**
   * Display an error message if a mandatory option does not have a value.
   * Called after checking for help flags in leaf subcommand.
   *
   * @api private
   */

  _checkForMissingMandatoryOptions() {
    // Walk up hierarchy so can call in subcommand after checking for displaying help.
    for (let cmd = this; cmd; cmd = cmd.parent) {
      cmd.options.forEach(anOption => {
        if (anOption.mandatory && cmd.getOptionValue(anOption.attributeName()) === undefined) {
          cmd.missingMandatoryOptionValue(anOption);
        }
      });
    }
  }

  /**
   * Display an error message if conflicting options are used together in this.
   *
   * @api private
   */
  _checkForConflictingLocalOptions() {
    const definedNonDefaultOptions = this.options.filter(option => {
      const optionKey = option.attributeName();
      if (this.getOptionValue(optionKey) === undefined) {
        return false;
      }
      return this.getOptionValueSource(optionKey) !== 'default';
    });
    const optionsWithConflicting = definedNonDefaultOptions.filter(option => option.conflictsWith.length > 0);
    optionsWithConflicting.forEach(option => {
      const conflictingAndDefined = definedNonDefaultOptions.find(defined => option.conflictsWith.includes(defined.attributeName()));
      if (conflictingAndDefined) {
        this._conflictingOption(option, conflictingAndDefined);
      }
    });
  }

  /**
   * Display an error message if conflicting options are used together.
   * Called after checking for help flags in leaf subcommand.
   *
   * @api private
   */
  _checkForConflictingOptions() {
    // Walk up hierarchy so can call in subcommand after checking for displaying help.
    for (let cmd = this; cmd; cmd = cmd.parent) {
      cmd._checkForConflictingLocalOptions();
    }
  }

  /**
   * Parse options from `argv` removing known options,
   * and return argv split into operands and unknown arguments.
   *
   * Examples:
   *
   *     argv => operands, unknown
   *     --known kkk op => [op], []
   *     op --known kkk => [op], []
   *     sub --unknown uuu op => [sub], [--unknown uuu op]
   *     sub -- --unknown uuu op => [sub --unknown uuu op], []
   *
   * @param {String[]} argv
   * @return {{operands: String[], unknown: String[]}}
   */

  parseOptions(argv) {
    const operands = []; // operands, not options or values
    const unknown = []; // first unknown option and remaining unknown args
    let dest = operands;
    const args = argv.slice();
    function maybeOption(arg) {
      return arg.length > 1 && arg[0] === '-';
    }

    // parse options
    let activeVariadicOption = null;
    while (args.length) {
      const arg = args.shift();

      // literal
      if (arg === '--') {
        if (dest === unknown) dest.push(arg);
        dest.push(...args);
        break;
      }
      if (activeVariadicOption && !maybeOption(arg)) {
        this.emit(`option:${activeVariadicOption.name()}`, arg);
        continue;
      }
      activeVariadicOption = null;
      if (maybeOption(arg)) {
        const option = this._findOption(arg);
        // recognised option, call listener to assign value with possible custom processing
        if (option) {
          if (option.required) {
            const value = args.shift();
            if (value === undefined) this.optionMissingArgument(option);
            this.emit(`option:${option.name()}`, value);
          } else if (option.optional) {
            let value = null;
            // historical behaviour is optional value is following arg unless an option
            if (args.length > 0 && !maybeOption(args[0])) {
              value = args.shift();
            }
            this.emit(`option:${option.name()}`, value);
          } else {
            // boolean flag
            this.emit(`option:${option.name()}`);
          }
          activeVariadicOption = option.variadic ? option : null;
          continue;
        }
      }

      // Look for combo options following single dash, eat first one if known.
      if (arg.length > 2 && arg[0] === '-' && arg[1] !== '-') {
        const option = this._findOption(`-${arg[1]}`);
        if (option) {
          if (option.required || option.optional && this._combineFlagAndOptionalValue) {
            // option with value following in same argument
            this.emit(`option:${option.name()}`, arg.slice(2));
          } else {
            // boolean option, emit and put back remainder of arg for further processing
            this.emit(`option:${option.name()}`);
            args.unshift(`-${arg.slice(2)}`);
          }
          continue;
        }
      }

      // Look for known long flag with value, like --foo=bar
      if (/^--[^=]+=/.test(arg)) {
        const index = arg.indexOf('=');
        const option = this._findOption(arg.slice(0, index));
        if (option && (option.required || option.optional)) {
          this.emit(`option:${option.name()}`, arg.slice(index + 1));
          continue;
        }
      }

      // Not a recognised option by this command.
      // Might be a command-argument, or subcommand option, or unknown option, or help command or option.

      // An unknown option means further arguments also classified as unknown so can be reprocessed by subcommands.
      if (maybeOption(arg)) {
        dest = unknown;
      }

      // If using positionalOptions, stop processing our options at subcommand.
      if ((this._enablePositionalOptions || this._passThroughOptions) && operands.length === 0 && unknown.length === 0) {
        if (this._findCommand(arg)) {
          operands.push(arg);
          if (args.length > 0) unknown.push(...args);
          break;
        } else if (arg === this._helpCommandName && this._hasImplicitHelpCommand()) {
          operands.push(arg);
          if (args.length > 0) operands.push(...args);
          break;
        } else if (this._defaultCommandName) {
          unknown.push(arg);
          if (args.length > 0) unknown.push(...args);
          break;
        }
      }

      // If using passThroughOptions, stop processing options at first command-argument.
      if (this._passThroughOptions) {
        dest.push(arg);
        if (args.length > 0) dest.push(...args);
        break;
      }

      // add arg
      dest.push(arg);
    }
    return {
      operands,
      unknown
    };
  }

  /**
   * Return an object containing local option values as key-value pairs.
   *
   * @return {Object}
   */
  opts() {
    if (this._storeOptionsAsProperties) {
      // Preserve original behaviour so backwards compatible when still using properties
      const result = {};
      const len = this.options.length;
      for (let i = 0; i < len; i++) {
        const key = this.options[i].attributeName();
        result[key] = key === this._versionOptionName ? this._version : this[key];
      }
      return result;
    }
    return this._optionValues;
  }

  /**
   * Return an object containing merged local and global option values as key-value pairs.
   *
   * @return {Object}
   */
  optsWithGlobals() {
    // globals overwrite locals
    return getCommandAndParents(this).reduce((combinedOptions, cmd) => Object.assign(combinedOptions, cmd.opts()), {});
  }

  /**
   * Display error message and exit (or call exitOverride).
   *
   * @param {string} message
   * @param {Object} [errorOptions]
   * @param {string} [errorOptions.code] - an id string representing the error
   * @param {number} [errorOptions.exitCode] - used with process.exit
   */
  error(message, errorOptions) {
    // output handling
    this._outputConfiguration.outputError(`${message}\n`, this._outputConfiguration.writeErr);
    if (typeof this._showHelpAfterError === 'string') {
      this._outputConfiguration.writeErr(`${this._showHelpAfterError}\n`);
    } else if (this._showHelpAfterError) {
      this._outputConfiguration.writeErr('\n');
      this.outputHelp({
        error: true
      });
    }

    // exit handling
    const config = errorOptions || {};
    const exitCode = config.exitCode || 1;
    const code = config.code || 'commander.error';
    this._exit(exitCode, code, message);
  }

  /**
   * Apply any option related environment variables, if option does
   * not have a value from cli or client code.
   *
   * @api private
   */
  _parseOptionsEnv() {
    this.options.forEach(option => {
      if (option.envVar && option.envVar in process$1.env) {
        const optionKey = option.attributeName();
        // Priority check. Do not overwrite cli or options from unknown source (client-code).
        if (this.getOptionValue(optionKey) === undefined || ['default', 'config', 'env'].includes(this.getOptionValueSource(optionKey))) {
          if (option.required || option.optional) {
            // option can take a value
            // keep very simple, optional always takes value
            this.emit(`optionEnv:${option.name()}`, process$1.env[option.envVar]);
          } else {
            // boolean
            // keep very simple, only care that envVar defined and not the value
            this.emit(`optionEnv:${option.name()}`);
          }
        }
      }
    });
  }

  /**
   * Apply any implied option values, if option is undefined or default value.
   *
   * @api private
   */
  _parseOptionsImplied() {
    const dualHelper = new DualOptions(this.options);
    const hasCustomOptionValue = optionKey => {
      return this.getOptionValue(optionKey) !== undefined && !['default', 'implied'].includes(this.getOptionValueSource(optionKey));
    };
    this.options.filter(option => option.implied !== undefined && hasCustomOptionValue(option.attributeName()) && dualHelper.valueFromOption(this.getOptionValue(option.attributeName()), option)).forEach(option => {
      Object.keys(option.implied).filter(impliedKey => !hasCustomOptionValue(impliedKey)).forEach(impliedKey => {
        this.setOptionValueWithSource(impliedKey, option.implied[impliedKey], 'implied');
      });
    });
  }

  /**
   * Argument `name` is missing.
   *
   * @param {string} name
   * @api private
   */

  missingArgument(name) {
    const message = `error: missing required argument '${name}'`;
    this.error(message, {
      code: 'commander.missingArgument'
    });
  }

  /**
   * `Option` is missing an argument.
   *
   * @param {Option} option
   * @api private
   */

  optionMissingArgument(option) {
    const message = `error: option '${option.flags}' argument missing`;
    this.error(message, {
      code: 'commander.optionMissingArgument'
    });
  }

  /**
   * `Option` does not have a value, and is a mandatory option.
   *
   * @param {Option} option
   * @api private
   */

  missingMandatoryOptionValue(option) {
    const message = `error: required option '${option.flags}' not specified`;
    this.error(message, {
      code: 'commander.missingMandatoryOptionValue'
    });
  }

  /**
   * `Option` conflicts with another option.
   *
   * @param {Option} option
   * @param {Option} conflictingOption
   * @api private
   */
  _conflictingOption(option, conflictingOption) {
    // The calling code does not know whether a negated option is the source of the
    // value, so do some work to take an educated guess.
    const findBestOptionFromValue = option => {
      const optionKey = option.attributeName();
      const optionValue = this.getOptionValue(optionKey);
      const negativeOption = this.options.find(target => target.negate && optionKey === target.attributeName());
      const positiveOption = this.options.find(target => !target.negate && optionKey === target.attributeName());
      if (negativeOption && (negativeOption.presetArg === undefined && optionValue === false || negativeOption.presetArg !== undefined && optionValue === negativeOption.presetArg)) {
        return negativeOption;
      }
      return positiveOption || option;
    };
    const getErrorMessage = option => {
      const bestOption = findBestOptionFromValue(option);
      const optionKey = bestOption.attributeName();
      const source = this.getOptionValueSource(optionKey);
      if (source === 'env') {
        return `environment variable '${bestOption.envVar}'`;
      }
      return `option '${bestOption.flags}'`;
    };
    const message = `error: ${getErrorMessage(option)} cannot be used with ${getErrorMessage(conflictingOption)}`;
    this.error(message, {
      code: 'commander.conflictingOption'
    });
  }

  /**
   * Unknown option `flag`.
   *
   * @param {string} flag
   * @api private
   */

  unknownOption(flag) {
    if (this._allowUnknownOption) return;
    let suggestion = '';
    if (flag.startsWith('--') && this._showSuggestionAfterError) {
      // Looping to pick up the global options too
      let candidateFlags = [];
      let command = this;
      do {
        const moreFlags = command.createHelp().visibleOptions(command).filter(option => option.long).map(option => option.long);
        candidateFlags = candidateFlags.concat(moreFlags);
        command = command.parent;
      } while (command && !command._enablePositionalOptions);
      suggestion = suggestSimilar(flag, candidateFlags);
    }
    const message = `error: unknown option '${flag}'${suggestion}`;
    this.error(message, {
      code: 'commander.unknownOption'
    });
  }

  /**
   * Excess arguments, more than expected.
   *
   * @param {string[]} receivedArgs
   * @api private
   */

  _excessArguments(receivedArgs) {
    if (this._allowExcessArguments) return;
    const expected = this._args.length;
    const s = expected === 1 ? '' : 's';
    const forSubcommand = this.parent ? ` for '${this.name()}'` : '';
    const message = `error: too many arguments${forSubcommand}. Expected ${expected} argument${s} but got ${receivedArgs.length}.`;
    this.error(message, {
      code: 'commander.excessArguments'
    });
  }

  /**
   * Unknown command.
   *
   * @api private
   */

  unknownCommand() {
    const unknownName = this.args[0];
    let suggestion = '';
    if (this._showSuggestionAfterError) {
      const candidateNames = [];
      this.createHelp().visibleCommands(this).forEach(command => {
        candidateNames.push(command.name());
        // just visible alias
        if (command.alias()) candidateNames.push(command.alias());
      });
      suggestion = suggestSimilar(unknownName, candidateNames);
    }
    const message = `error: unknown command '${unknownName}'${suggestion}`;
    this.error(message, {
      code: 'commander.unknownCommand'
    });
  }

  /**
   * Set the program version to `str`.
   *
   * This method auto-registers the "-V, --version" flag
   * which will print the version number when passed.
   *
   * You can optionally supply the  flags and description to override the defaults.
   *
   * @param {string} str
   * @param {string} [flags]
   * @param {string} [description]
   * @return {this | string} `this` command for chaining, or version string if no arguments
   */

  version(str, flags, description) {
    if (str === undefined) return this._version;
    this._version = str;
    flags = flags || '-V, --version';
    description = description || 'output the version number';
    const versionOption = this.createOption(flags, description);
    this._versionOptionName = versionOption.attributeName();
    this.options.push(versionOption);
    this.on('option:' + versionOption.name(), () => {
      this._outputConfiguration.writeOut(`${str}\n`);
      this._exit(0, 'commander.version', str);
    });
    return this;
  }

  /**
   * Set the description.
   *
   * @param {string} [str]
   * @param {Object} [argsDescription]
   * @return {string|Command}
   */
  description(str, argsDescription) {
    if (str === undefined && argsDescription === undefined) return this._description;
    this._description = str;
    if (argsDescription) {
      this._argsDescription = argsDescription;
    }
    return this;
  }

  /**
   * Set the summary. Used when listed as subcommand of parent.
   *
   * @param {string} [str]
   * @return {string|Command}
   */
  summary(str) {
    if (str === undefined) return this._summary;
    this._summary = str;
    return this;
  }

  /**
   * Set an alias for the command.
   *
   * You may call more than once to add multiple aliases. Only the first alias is shown in the auto-generated help.
   *
   * @param {string} [alias]
   * @return {string|Command}
   */

  alias(alias) {
    if (alias === undefined) return this._aliases[0]; // just return first, for backwards compatibility

    /** @type {Command} */
    let command = this;
    if (this.commands.length !== 0 && this.commands[this.commands.length - 1]._executableHandler) {
      // assume adding alias for last added executable subcommand, rather than this
      command = this.commands[this.commands.length - 1];
    }
    if (alias === command._name) throw new Error('Command alias can\'t be the same as its name');
    command._aliases.push(alias);
    return this;
  }

  /**
   * Set aliases for the command.
   *
   * Only the first alias is shown in the auto-generated help.
   *
   * @param {string[]} [aliases]
   * @return {string[]|Command}
   */

  aliases(aliases) {
    // Getter for the array of aliases is the main reason for having aliases() in addition to alias().
    if (aliases === undefined) return this._aliases;
    aliases.forEach(alias => this.alias(alias));
    return this;
  }

  /**
   * Set / get the command usage `str`.
   *
   * @param {string} [str]
   * @return {String|Command}
   */

  usage(str) {
    if (str === undefined) {
      if (this._usage) return this._usage;
      const args = this._args.map(arg => {
        return humanReadableArgName(arg);
      });
      return [].concat(this.options.length || this._hasHelpOption ? '[options]' : [], this.commands.length ? '[command]' : [], this._args.length ? args : []).join(' ');
    }
    this._usage = str;
    return this;
  }

  /**
   * Get or set the name of the command.
   *
   * @param {string} [str]
   * @return {string|Command}
   */

  name(str) {
    if (str === undefined) return this._name;
    this._name = str;
    return this;
  }

  /**
   * Set the name of the command from script filename, such as process.argv[1],
   * or require.main.filename, or __filename.
   *
   * (Used internally and public although not documented in README.)
   *
   * @example
   * program.nameFromFilename(require.main.filename);
   *
   * @param {string} filename
   * @return {Command}
   */

  nameFromFilename(filename) {
    this._name = path.basename(filename, path.extname(filename));
    return this;
  }

  /**
   * Get or set the directory for searching for executable subcommands of this command.
   *
   * @example
   * program.executableDir(__dirname);
   * // or
   * program.executableDir('subcommands');
   *
   * @param {string} [path]
   * @return {string|Command}
   */

  executableDir(path) {
    if (path === undefined) return this._executableDir;
    this._executableDir = path;
    return this;
  }

  /**
   * Return program help documentation.
   *
   * @param {{ error: boolean }} [contextOptions] - pass {error:true} to wrap for stderr instead of stdout
   * @return {string}
   */

  helpInformation(contextOptions) {
    const helper = this.createHelp();
    if (helper.helpWidth === undefined) {
      helper.helpWidth = contextOptions && contextOptions.error ? this._outputConfiguration.getErrHelpWidth() : this._outputConfiguration.getOutHelpWidth();
    }
    return helper.formatHelp(this, helper);
  }

  /**
   * @api private
   */

  _getHelpContext(contextOptions) {
    contextOptions = contextOptions || {};
    const context = {
      error: !!contextOptions.error
    };
    let write;
    if (context.error) {
      write = arg => this._outputConfiguration.writeErr(arg);
    } else {
      write = arg => this._outputConfiguration.writeOut(arg);
    }
    context.write = contextOptions.write || write;
    context.command = this;
    return context;
  }

  /**
   * Output help information for this command.
   *
   * Outputs built-in help, and custom text added using `.addHelpText()`.
   *
   * @param {{ error: boolean } | Function} [contextOptions] - pass {error:true} to write to stderr instead of stdout
   */

  outputHelp(contextOptions) {
    let deprecatedCallback;
    if (typeof contextOptions === 'function') {
      deprecatedCallback = contextOptions;
      contextOptions = undefined;
    }
    const context = this._getHelpContext(contextOptions);
    getCommandAndParents(this).reverse().forEach(command => command.emit('beforeAllHelp', context));
    this.emit('beforeHelp', context);
    let helpInformation = this.helpInformation(context);
    if (deprecatedCallback) {
      helpInformation = deprecatedCallback(helpInformation);
      if (typeof helpInformation !== 'string' && !Buffer.isBuffer(helpInformation)) {
        throw new Error('outputHelp callback must return a string or a Buffer');
      }
    }
    context.write(helpInformation);
    this.emit(this._helpLongFlag); // deprecated
    this.emit('afterHelp', context);
    getCommandAndParents(this).forEach(command => command.emit('afterAllHelp', context));
  }

  /**
   * You can pass in flags and a description to override the help
   * flags and help description for your command. Pass in false to
   * disable the built-in help option.
   *
   * @param {string | boolean} [flags]
   * @param {string} [description]
   * @return {Command} `this` command for chaining
   */

  helpOption(flags, description) {
    if (typeof flags === 'boolean') {
      this._hasHelpOption = flags;
      return this;
    }
    this._helpFlags = flags || this._helpFlags;
    this._helpDescription = description || this._helpDescription;
    const helpFlags = splitOptionFlags(this._helpFlags);
    this._helpShortFlag = helpFlags.shortFlag;
    this._helpLongFlag = helpFlags.longFlag;
    return this;
  }

  /**
   * Output help information and exit.
   *
   * Outputs built-in help, and custom text added using `.addHelpText()`.
   *
   * @param {{ error: boolean }} [contextOptions] - pass {error:true} to write to stderr instead of stdout
   */

  help(contextOptions) {
    this.outputHelp(contextOptions);
    let exitCode = process$1.exitCode || 0;
    if (exitCode === 0 && contextOptions && typeof contextOptions !== 'function' && contextOptions.error) {
      exitCode = 1;
    }
    // message: do not have all displayed text available so only passing placeholder.
    this._exit(exitCode, 'commander.help', '(outputHelp)');
  }

  /**
   * Add additional text to be displayed with the built-in help.
   *
   * Position is 'before' or 'after' to affect just this command,
   * and 'beforeAll' or 'afterAll' to affect this command and all its subcommands.
   *
   * @param {string} position - before or after built-in help
   * @param {string | Function} text - string to add, or a function returning a string
   * @return {Command} `this` command for chaining
   */
  addHelpText(position, text) {
    const allowedValues = ['beforeAll', 'before', 'after', 'afterAll'];
    if (!allowedValues.includes(position)) {
      throw new Error(`Unexpected value for position to addHelpText.
Expecting one of '${allowedValues.join("', '")}'`);
    }
    const helpEvent = `${position}Help`;
    this.on(helpEvent, context => {
      let helpStr;
      if (typeof text === 'function') {
        helpStr = text({
          error: context.error,
          command: context.command
        });
      } else {
        helpStr = text;
      }
      // Ignore falsy value when nothing to output.
      if (helpStr) {
        context.write(`${helpStr}\n`);
      }
    });
    return this;
  }
};

/**
 * Output help information if help flags specified
 *
 * @param {Command} cmd - command to output help for
 * @param {Array} args - array of options to search for help flags
 * @api private
 */

function outputHelpIfRequested(cmd, args) {
  const helpOption = cmd._hasHelpOption && args.find(arg => arg === cmd._helpLongFlag || arg === cmd._helpShortFlag);
  if (helpOption) {
    cmd.outputHelp();
    // (Do not have all displayed text available so only passing placeholder.)
    cmd._exit(0, 'commander.helpDisplayed', '(outputHelp)');
  }
}

/**
 * Scan arguments and increment port number for inspect calls (to avoid conflicts when spawning new command).
 *
 * @param {string[]} args - array of arguments from node.execArgv
 * @returns {string[]}
 * @api private
 */

function incrementNodeInspectorPort(args) {
  // Testing for these options:
  //  --inspect[=[host:]port]
  //  --inspect-brk[=[host:]port]
  //  --inspect-port=[host:]port
  return args.map(arg => {
    if (!arg.startsWith('--inspect')) {
      return arg;
    }
    let debugOption;
    let debugHost = '127.0.0.1';
    let debugPort = '9229';
    let match;
    if ((match = arg.match(/^(--inspect(-brk)?)$/)) !== null) {
      // e.g. --inspect
      debugOption = match[1];
    } else if ((match = arg.match(/^(--inspect(-brk|-port)?)=([^:]+)$/)) !== null) {
      debugOption = match[1];
      if (/^\d+$/.test(match[3])) {
        // e.g. --inspect=1234
        debugPort = match[3];
      } else {
        // e.g. --inspect=localhost
        debugHost = match[3];
      }
    } else if ((match = arg.match(/^(--inspect(-brk|-port)?)=([^:]+):(\d+)$/)) !== null) {
      // e.g. --inspect=localhost:1234
      debugOption = match[1];
      debugHost = match[3];
      debugPort = match[4];
    }
    if (debugOption && debugPort !== '0') {
      return `${debugOption}=${debugHost}:${parseInt(debugPort) + 1}`;
    }
    return arg;
  });
}

/**
 * @param {Command} startCommand
 * @returns {Command[]}
 * @api private
 */

function getCommandAndParents(startCommand) {
  const result = [];
  for (let command = startCommand; command; command = command.parent) {
    result.push(command);
  }
  return result;
}
command.Command = Command$1;

(function (module, exports) {
	const {
	  Argument
	} = argument;
	const {
	  Command
	} = command;
	const {
	  CommanderError,
	  InvalidArgumentError
	} = error;
	const {
	  Help
	} = help;
	const {
	  Option
	} = option;

	// @ts-check

	/**
	 * Expose the root command.
	 */

	exports = module.exports = new Command();
	exports.program = exports; // More explicit access to global command.
	// Implicit export of createArgument, createCommand, and createOption.

	/**
	 * Expose classes
	 */

	exports.Argument = Argument;
	exports.Command = Command;
	exports.CommanderError = CommanderError;
	exports.Help = Help;
	exports.InvalidArgumentError = InvalidArgumentError;
	exports.InvalidOptionArgumentError = InvalidArgumentError; // Deprecated
	exports.Option = Option; 
} (commander$1, commander$1.exports));

var commanderExports = commander$1.exports;
var commander = /*@__PURE__*/getDefaultExportFromCjs(commanderExports);

// wrapper to provide named exports for ESM.
const {
  program,
  createCommand,
  createArgument,
  createOption,
  CommanderError,
  InvalidArgumentError,
  InvalidOptionArgumentError,
  // deprecated old name
  Command,
  Argument,
  Option,
  Help
} = commander;

function validateRootPath(root) {
  try {
    require$$0.accessSync(path$6.join(root, "package.json"), require$$0.constants.F_OK);
  } catch (err) {
    throw new Error(`\u6307\u5B9A\u7684\u76EE\u5F55${path$6.resolve(root)}\u4E2D\u4E0D\u5B58\u5728package.json\u6587\u4EF6`);
  }
}
function validateJsonOption(jsonPath) {
  if (!jsonPath.endsWith(".json")) {
    throw new Error("\u8BF7\u8F93\u5165\u6B63\u786E\u7684json\u6587\u4EF6\u8DEF\u5F84, \u5982./data.json");
  }
  try {
    require$$0.writeFileSync(jsonPath, "");
  } catch (err) {
    throw new Error("json\u6587\u4EF6\u8DEF\u5F84\u683C\u5F0F\u9519\u8BEF\u6216\u4E0D\u5B58\u5728");
  }
}
function validateDepthOption(depth) {
  if (!/^[1-9]+[0-9]*]*$/.test(depth)) {
    throw new Error("depth\u53C2\u6570\u5FC5\u987B\u4E3A\u6B63\u6574\u6570");
  }
}

function parseConfig(root, options) {
  const config = {
    root: "",
    saveJson: false,
    jsonPath: "",
    depthLimit: -1
  };
  if (root) {
    validateRootPath(root);
    config.root = path$6.resolve(root);
  } else {
    config.root = path$6.resolve("./");
  }
  if (options.depth) {
    validateDepthOption(options.depth);
    config.depthLimit = Number.parseInt(options.depth);
  }
  if (options.json) {
    validateJsonOption(options.json);
    if (path$6.isAbsolute(options.json)) {
      config.jsonPath = options.json;
    } else {
      config.jsonPath = path$6.join(path$6.resolve("./"), options.json);
    }
    config.saveJson = true;
  }
  return config;
}

function getDirname(importMetaUrl) {
  const __filename = fileURLToPath$1(importMetaUrl);
  return dirname(__filename);
}

function execute(config) {
  let analyzer;
  if (config.depthLimit === -1) {
    warning("The maximum recursion depth is not set, and all package dependencies of the project will be generated");
  }
  measureExecutionTime(() => {
    analyzer = AnalyzerFactory(config.root, config.depthLimit);
  }, (time) => success(`Completed reading the project package data, taking ${time} ms`));
  if (config.saveJson) {
    measureExecutionTime(() => {
      saveJsonData(config.jsonPath, analyzer.dependencyGraph.exportToJson());
    }, () => success(`Successfully saved dependency relationships to${config.jsonPath}`));
  } else {
    createDataServer(analyzer, path$6.join(getDirname(import.meta.url), "../ui"));
  }
}
function createProgram() {
  const program = new Command();
  program.name("npa-cli1").description("ttttVisualize nodes_ Modules dependency structure").version("2.3.2");
  program.command("analyze").description("\u5206\u6790\u6307\u5B9A\u76EE\u5F55\u7684\u5305\u4F9D\u8D56\u5173\u7CFB").argument("[root-path]", "\u89E3\u6790\u7684\u9879\u76EE\u6839\u76EE\u5F55").option("--depth <depth>", "\u5411\u4E0B\u9012\u5F52\u5206\u6790\u7684\u5C42\u6B21\u6DF1\u5EA6").option("--json <json-path>", "\u5C06\u4F9D\u8D56\u5173\u7CFB\u4EE5json\u5F62\u5F0F\u5B58\u50A8\u5230\u6307\u5B9A\u7684\u6587\u4EF6").action((root, options) => {
    try {
      execute(parseConfig(root, options));
    } catch (err) {
      error$2(program, err.message);
    }
  });
  return program;
}
function main() {
  createProgram().parse(process.argv);
}

export { main };
